/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is Civil Netizen
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2005 Civil Engines Research.  All Rights 
 * Reserved.  
 *
 * Portions created by Netscape Copyright (C) 1998 Netscape 
 * Communications Corporation. All Rights Reserved.
 *
 * Portions created by the Mozilla Foundation are Copyright (C) 
 * 2003 Mozilla Foundation. All Rights Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/


const NETIZEN_CONTENT_HANDLER_CONTRACTID       = "@civilengines.com/appshell/content-handler;1";
const NETIZEN_DEFAULT_COMMANDLINE_CONTRACTID   = "@civilengines.com/appshell/default-commandline;1";

var ScriptLoader =
{
    globalScriptsLoaded : false,
    
    loadGlobalToolkitScripts : function()
    {
        if(this.globalScriptsLoaded == false)
        {
            var scriptLoader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
                .getService(Components.interfaces.mozIJSSubScriptLoader);
            
            if(scriptLoader)
            {
                scriptLoader.loadSubScript("chrome://global/content/utils/nsLocalPreferences.js");
                scriptLoader.loadSubScript("chrome://global/content/utils/ceTraceLogService.js");
                scriptLoader.loadSubScript("chrome://global/content/utils/ceComponentConnector.js");
            }
        }
        
        this.globalScriptsLoaded = true;
        
        return;
    }
};


//@line 79 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/base/src/ceNetizenContentHandler.js"


var ContentHandlerUtils = 
{
    shouldLoadURI : function(uri) 
    {
        if(uri && !uri.schemeIs("chrome"))
        {
            return true;
        }
        
        Syslog.logMessage("Netizen Content Handler : Preventing load of XUL chrome");
        
        return false;
    },

    resolveURIInternal : function(commandLine, urlArgument) 
    {
        var uri = commandLine.resolveURI(urlArgument);

        if(!(uri instanceof Interfaces.nsIFileURL)) 
        {
            return uri;
        }

        try 
        {
            if(uri.file.exists())
            {
                return uri;
            }
        }        
        catch(exception) 
        {
            Components.utils.reportError(exception);
        }

        try 
        {
            var urifixup = do_GetService("@mozilla.org/docshell/urifixup;1",
                                         Interfaces.nsIURIFixup            );

            uri = urifixup.createFixupURI(urlArgument, 0);
        }
        catch(exception) 
        {
            Components.utils.reportError(exception);
        }
        
        return uri;
    },

    getMostRecentNetizenWindow : function()
    {
        var windowMediator = do_GetService("@mozilla.org/appshell/window-mediator;1",
                                           Interfaces.nsIWindowMediator             );

        var recentWindow   = null;
        
//@line 159 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/base/src/ceNetizenContentHandler.js"
        var windowList = windowMediator.getZOrderDOMWindowEnumerator("urn:civil:windowtype:netizen", 
                                                                     true                          );
        if(!windowList.hasMoreElements())
        {
            return null;
        }
        
        recentWindow = windowList.getNext();
        while(!recentWindow.toolbar.visible) 
        {
            if(!windowList.hasMoreElements()) 
            {
                return null;
            }
            
            recentWindow = windowList.getNext();
        }
//@line 177 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/base/src/ceNetizenContentHandler.js"
        
        return recentWindow;
    },
    
    openWindow : function(parent, url, target, features, args) 
    {
        var windowWatcher = do_GetService("@mozilla.org/embedcomp/window-watcher;1",
                                          Interfaces.nsIWindowWatcher              );        
        var argString;
        if(args) 
        {
            argString = do_CreateInstance("@mozilla.org/supports-string;1",
                                          Interfaces.nsISupportsString    );
            
            argString.data = args;
        }

        Syslog.logMessage("Netizen Content Handler : Open Window = " + url);
        Syslog.logMessage("Netizen Content Handler : target      = " + target);
        Syslog.logMessage("Netizen Content Handler : features    = " + features);
        Syslog.logMessage("Netizen Content Handler : windowArgs  = " + args);

        return windowWatcher.openWindow(parent, url, target, features, argString);
    }
};



/////////////////////////////////////////////////////////////////////////////////
//
// ceNetizenContentHandler Implementation
//
/////////////////////////////////////////////////////////////////////////////////


function ceNetizenContentHandler()
{
    ScriptLoader.loadGlobalToolkitScripts();
}


ceNetizenContentHandler.prototype =
{
    defaultChromeURL : null,
    defaultFeatures  : null,

    get chromeURL() 
    {
        if(this.defaultChromeURL) 
        {
            return this.defaultChromeURL;
        }

        this.defaultChromeURL = LocalPreferences.getCharPref("browser.chromeURL");
        
        return this.defaultChromeURL;
    },


    get defaultArgs() 
    {
        return "about:blank";
    },


    getFeatures : function(commandLine) 
    {
        if(this.defaultFeatures === null) 
        {
            this.defaultFeatures = 
                "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar,dialog=no"

            try 
            {
                var width  = commandLine.handleFlagWithParam("width", false);
                var height = commandLine.handleFlagWithParam("height", false);

                if(width)
                {
                    this.defaultFeatures += ",width=" + width;
                }
                
                if(height)
                {
                    this.defaultFeatures += ",height=" + height;
                }
            }            
            catch(exception)
            {
            }
        }

        return this.defaultFeatures;
    },


    loadApplicationURI : function(uri)
    {
        Syslog.logMessage("Netizen Content Handler : Attempting to load URI to an existing window");

        if(!ContentHandlerUtils.shouldLoadURI(uri))
        {
            return;
        }

        netizenWindow = ContentHandlerUtils.getMostRecentNetizenWindow();
        if(!netizenWindow)
        {
            Syslog.logMessage("Netizen Content Handler : Loading URI to New Window");

            /* Could not load into existing window so open a new one */
            ContentHandlerUtils.openWindow(null                         , 
                                           this.chromeURL               ,
                                           "_blank"                     ,
                                           this.getFeatures(commandLine),
                                           uri.spec                     );   

            return;
        }

        var netizenNavigation = do_QueryInterface(netizenWindow, Interfaces.nsIInterfaceRequestor);
        netizenNavigation     = netizenNavigation.getInterface(Interfaces.nsIWebNavigation);

        var rootItem          = do_QueryInterface(netizenNavigation, Interfaces.nsIDocShellTreeItem);
        rootItem              = rootItem.rootTreeItem;

        var rootWindow        = do_QueryInterface(rootItem, Interfaces.nsIInterfaceRequestor)
        rootWindow            = rootWindow.getInterface(nsIDOMWindow);
        
        var rootChromeWindow  = do_QueryInterface(rootWindow, Interfaces.nsIDOMChromeWindow);
        
        if(!rootWindow || !rootChromeWindow)
        {
            Syslog.logMessage("Netizen Content Handler : Unable to Acquire Root Window");
        }
        
        return;
    },
    
    QueryInterface : function(iid) 
    {
        if (!iid.equals(Interfaces.nsISupports)           &&
            !iid.equals(Interfaces.nsICommandLineHandler)  )
        {
            throw Components.errors.NS_ERROR_NO_INTERFACE;
        }
        
        return this;
    },


    helpInfo : "\t-url\t\t\tLoad Netizen URL.\n",


    handle : function(commandLine)
    {
        var urlList = [];

        try 
        {
            var argument;
            while((argument = commandLine.handleFlagWithParam("url", false))) 
            {
                urlList.push(ContentHandlerUtils.resolveURIInternal(commandLine, argument));
            }
        }
        catch(exception) 
        {
            Components.utils.reportError(exception);
            Syslog.logException(exception);
        }


        var count = commandLine.length;

        for(var i = 0; i < count; ++i) 
        {
            var currentArg = commandLine.getArgument(i);

            if(currentArg.match(/^-/)) 
            {
                Components.utils.reportError("Warning: unrecognized command line flag " + 
                                             currentArg + "\n"                          );
                ++i;
            } 
            else 
            {
                try 
                {
                    urlList.push(ContentHandlerUtils.resolveURIInternal(commandLine, currentArg));
                }
                catch(exception) 
                {
                    Components.utils.reportError("Error opening URI '"        + 
                                                 currentArg                   + 
                                                 "' from the command line : " +
                                                 exception + "\n"             );
                }
            }
        }
        
        if(urlList.length) 
        {
            if(commandLine.state != Interfaces.nsICommandLine.STATE_INITIAL_LAUNCH &&
               urlList.length == 1                                                  )
            {
                try 
                {
                    this.loadApplicationURI(urlList[0]);

                    return;
                }
                catch(exception) 
                {
                    Syslog.logException(exception);
                }
            }

            var specList = [];

            for(var uri in urlList) 
            {
                if(ContentHandlerUtils.shouldLoadURI(urlList[uri]))
                {
                    specList.push(urlList[uri].spec);
                }
            }
            
            if(specList.length) 
            {
                Syslog.logMessage("Netizen Content Handler : Loading URL(s) to New Window");

                ContentHandlerUtils.openWindow(null                         , 
                                               this.chromeURL               ,
                                               "_blank"                     ,
                                               this.getFeatures(commandLine),
                                               specList.join("|")           );
                
                return;
            }
            
        }
        else if(!commandLine.preventDefault) 
        {
            Syslog.logMessage("Netizen Content Handler : Opening Default Application Window");

            ContentHandlerUtils.openWindow(null                         ,
                                           this.chromeURL               , 
                                           "_blank"                     ,
                                           this.getFeatures(commandLine),
                                           this.defaultArgs             );
        }

        return;
    }    
};



/////////////////////////////////////////////////////////////////////////////////
//
// nsIFactory Implementation
//
/////////////////////////////////////////////////////////////////////////////////


var ceNetizenContentHandlerModule = 
{
    moduleObjects :
    {
        ceNetizenContentHandler :
        {
            CID        : Components.ID("{7a6bfb99-8f2b-4f0c-ad6a-fe8153a484c2}"),
            contractID : NETIZEN_CONTENT_HANDLER_CONTRACTID,
            className  : "ceNetizenContentHandler",
            factory    : 
            {
                createInstance : function(outer, iid) 
                {
                    if(outer != null)
                    {
                        throw Components.results.NS_ERROR_NO_AGGREGATION;
                    }
                    
                    var netizenContentHandler = new ceNetizenContentHandler();
                    return netizenContentHandler.QueryInterface(iid);
                }       
            }
        },
    },
    
    registerSelf : function(componentManager, fileSpec, location, type) 
    {
        componentManager = componentManager.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    
        for(var key in this.moduleObjects) 
        {
            var moduleObject = this.moduleObjects[key];

            componentManager.registerFactoryLocation(moduleObject.CID       , 
                                                     moduleObject.className , 
                                                     moduleObject.contractID,
                                                     fileSpec               , 
                                                     location               ,
                                                     type                   );
        }


        var categoryManager = Components.classes["@mozilla.org/categorymanager;1"]
            .getService(Components.interfaces.nsICategoryManager);

        categoryManager.addCategoryEntry("command-line-handler"            ,
                                         "y-default"                       ,
                                         NETIZEN_CONTENT_HANDLER_CONTRACTID,
                                         true                              ,
                                         true                              );
        return;
    },

    unregisterSelf : function(componentManager, fileSpec, location, type) 
    {
        componentManager = componentManager.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    
        for(var key in this.moduleObjects) 
        {
            var moduleObject = this.moduleObjects[key];

            componentManager.unregisterFactoryLocation(moduleObject.CID       , 
                                                       moduleObject.className , 
                                                       moduleObject.contractID,
                                                       fileSpec               , 
                                                       location               ,
                                                       type                   );
        }

        
        var categoryManager = Components.classes["@mozilla.org/categorymanager;1"]
            .getService(Components.interfaces.nsICategoryManager);

        categoryManager.deleteCategoryEntry("command-line-handler",
                                            "y-default"           ,
                                            true                  );

        return;        
    },
    
    getClassObject : function(componentManager, cid, iid)
    {
        if(!iid.equals(Components.interfaces.nsIFactory))
        {
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        }
        
        for(var key in this.moduleObjects)
        {
            if(cid.equals(this.moduleObjects[key].CID))
            {
                return this.moduleObjects[key].factory;
            }
        }
            
        throw Components.results.NS_ERROR_NO_INTERFACE;
    },
  

    canUnload : function(componentManager)
    {
        return true;
    }
};



function NSGetModule(componentManager, fileSpec)
{
    return ceNetizenContentHandlerModule;
}
