/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is the Mozilla Update Service.
 *
 * The Initial Developer of the Original Code is Google Inc
 *
 * Portions created by the Mozilla Foundation are Copyright (C) 
 * 2006 Mozilla Foundation. All Rights Reserved.
 *
 * Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2006 Civil Engines Research.  All Rights 
 * Reserved.  
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s):
 *   Darin Fisher <darin@meer.net> (Original Author)
 *   Paolo de Dios <paolo.dedios@civilengines.com>
 *
 *
 ***** END License Block *****/

/* Civil Engines specific fork of nsPostUpdateWin.js */
const PREF_GENERAL_USERAGENT_LOCALE                    = "general.useragent.locale";
const NETIZEN_BRAND_PROPERTIES_URI                     = "chrome://global/locale/brand.properties";

const DIRECTORY_SERVICE_KEY_APP_DIRECTORY              = "XCurProcD";
const DIRECTORY_SERVICE_KEY_COMPONENTS_DIRECTORY       = "ComsD";
const DIRECTORY_SERVICE_KEY_EXECUTABLE_FILE            = "XREExeF";

const IO_SERVICE_MODE_RDONLY                           = 0x01;
const IO_SERVICE_MODE_WRONLY                           = 0x02;
const IO_SERVICE_MODE_CREATE                           = 0x08;
const IO_SERVICE_MODE_APPEND                           = 0x10;
const IO_SERVICE_MODE_TRUNCATE                         = 0x20;

const INSTALLER_LOG_PREFIX_INSTALLING                  = "installing: ";
const INSTALLER_LOG_PREFIX_CREATE_FOLDER               = "create folder: ";
const INSTALLER_LOG_PREFIX_CREATE_REGISTRY_KEY         = "create registry key: ";
const INSTALLER_LOG_PREFIX_STORE_REGISTRY_VALUE_STRING = "store registry value string: ";

var InstallLogWriter = null;

var ScriptLoader     =
{
    globalScriptsLoaded : false,
    
    loadGlobalToolkitScripts : function()
    {
        if(this.globalScriptsLoaded == false)
        {
            var scriptLoader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
                .getService(Components.interfaces.mozIJSSubScriptLoader);
            
            if(scriptLoader)
            {
                scriptLoader.loadSubScript("chrome://global/content/utils/nsLocalPreferences.js");
                scriptLoader.loadSubScript("chrome://global/content/utils/ceTraceLogService.js");
                scriptLoader.loadSubScript("chrome://global/content/utils/ceComponentConnector.js");
            }
        }
        
        this.globalScriptsLoaded = true;
        
        return;
    }
};



var PostUpdateUtils  =
{
    getFile : function(key)
    {
        var directoryService = do_GetService("@mozilla.org/file/directory_service;1",
                                             Components.interfaces.nsIProperties    );

        return directoryService.get(key, Components.interfaces.nsIFile);
    },


    getFileRelativeTo : function(directory, relativePath) 
    {
        var file = directory.clone().QueryInterface(Components.interfaces.nsILocalFile);

        file.setRelativeDescriptor(directory, relativePath);
        return file;
    },


    /* Creates a new file object given a native file path.
     * @param   path
     *          The native file path.
     * @return  nsILocalFile object for the given native file path.
     */
    newFile : function(path) 
    {
        var file = do_CreateInstance("@mozilla.org/file/local;1"       ,
                                     Components.interfaces.nsILocalFile);

        file.initWithPath(path);

        return file;
    },

    openFileInputStream : function(file) 
    {
        var fileInputStream = do_CreateInstance("@mozilla.org/network/file-input-stream;1",
                                                Components.interfaces.nsIFileInputStream  );

        fileInputStream.init(file, IO_SERVICE_MODE_RDONLY, 0, 0);

        return fileInputStream;
    },

    openFileOutputStream : function(file, flags) 
    {
        var fileOutputStream = do_CreateInstance("@mozilla.org/network/file-output-stream;1",
                                                 Components.interfaces.nsIFileOutputStream  );

        fileOutputStream.init(file, flags, 0644, 0);

        return fileOutputStream;
    },

    getUserAgentLocale : function()
    {
        try
        {
            return LocalPreferences.getLocalizedUnicharPref(PREF_GENERAL_USERAGENT_LOCALE);
        }
        catch(exception)
        {
        }
        
        return LocalPreferences.getCharPref(PREF_GENERAL_USERAGENT_LOCALE);
    }
};



/////////////////////////////////////////////////////////////////////////////////
//
// ceInstallLogWriter Implementation
//
/////////////////////////////////////////////////////////////////////////////////


function ceInstallLogWriter()
{
    ScriptLoader.loadGlobalToolkitScripts();
}



ceInstallLogWriter.prototype =
{
    outputStream : null,

    writeLine : function(stringData) 
    {
        stringData = stringData + "\n";

        this.outputStream.write(stringData, stringData.length);

        return;
    },

    makeLeafName : function(index) 
    {
        return "install_wizard" + index + ".log";    
    },

    /* This function creates an empty install wizard log file and returns
     * a reference to the resulting nsIFile.
     */
    getInstallLogFile : function() 
    {            
        var file = PostUpdateUtils.getFile(DIRECTORY_SERVICE_KEY_APP_DIRECTORY);
 
        file.append("uninstall");
        if(!file.exists())
        {
            return null;
        }
        
        file.append("x");

        var counter = 0;

        do 
        {
            file.leafName = this.makeLeafName(++counter);
        
        }while (file.exists());

        if(counter == 1)
        {
            return null;
        }
        
        file.leafName = this.makeLeafName(counter - 1);
        
        return file;
    },

    /* Return the update.log file.  Use last-update.log file in case the
     * updates/0 directory has already been cleaned out (see bug 311302).
     */
    getUpdateLogFile : function() 
    {
        var file = PostUpdateUtils.getFile(DIRECTORY_SERVICE_KEY_APP_DIRECTORY);

        file.append("updates");
        file.append("0");
        file.append("update.log");
    
        if(file.exists())
        {
            return file;
        }
        
        file = PostUpdateUtils.getFile(DIRECTORY_SERVICE_KEY_APP_DIRECTORY);

        file.append("updates");
        file.append("last-update.log");

        if(file.exists())
        {
            return file;
        }
        
        return null;
    },


    /* Read update.log to extract information about files that were
     * newly added for this update.
     */
    readUpdateLog : function(logFile, entries)
    {
        var appDirectory = PostUpdateUtils.getFile(DIRECTORY_SERVICE_KEY_APP_DIRECTORY);
        var stream       = null;

        try 
        {
            stream = PostUpdateUtils.openFileInputStream(logFile).
            stream = do_QueryInterface(stream, Components.interfaces.nsILineInputStream);

            var dirs = {};
            var line = {};

            while(stream.readLine(line)) 
            {
                var data = line.value.split(" ");

                if(data[0] == "EXECUTE" && data[1] == "ADD") 
                {
                    var file   = PostUpdateUtils.getFileRelativeTo(appDirectory, data[2]);
                    
                    var parent = file.parent;
                    
                    while(!parent.equals(appDirectory)) 
                    {
                        dirs[parent.path] = true;
                        parent = parent.parent;
                    }
                    
                    entries.files.push(file.path);
                }
            }

            for(var directory in dirs)
            {
                entries.dirs.push(directory);
            }
            
            /* Sort the directories so that subdirectories are deleted first. */
            entries.dirs.sort();
        } 
        finally 
        {
            if(stream)
            {
                stream.close();
            }
        }
        

        return;
    },

    /* This function initializes the log writer and is responsible for
     * translating 'update.log' to the install wizard format.
     */
    begin : function() 
    {
        var installLog = this.getInstallLogFile();
        var updateLog  = this.getUpdateLogFile();

        if(!installLog || !updateLog)
        {
            return;
        }
        
        var entries = 
        { 
            dirs  : [], 
            files : [] 
        };

        this.readUpdateLog(updateLog, entries);

        var outputMode    = IO_SERVICE_MODE_WRONLY | IO_SERVICE_MODE_APPEND;
        this.outputStream = PostUpdateUtils.openFileOutputStream(installLog, outputMode);
        

        this.writeLine("\nUPDATE [" + new Date().toUTCString() + "]");

        var counter = 0;

        /* The log file is processed in reverse order, so list directories before
         * files to ensure that the directories will be deleted.
         */
        for(counter = 0; counter < entries.dirs.length; ++counter)
        {
            this.writeLine(INSTALLER_LOG_PREFIX_CREATE_FOLDER + entries.dirs[counter]);
        }

        for(counter = 0; counter < entries.files.length; ++counter)
        {
            this.writeLine(INSTALLER_LOG_PREFIX_INSTALLING + entries.files[counter]);
        }
      
        return;
    },


    /* This function records the creation of a registry key. */
    registryKeyCreated : function(keyPath) 
    {
        if(!this.outputStream)
        {
            return;
        }
        
        this.writeLine(INSTALLER_LOG_PREFIX_CREATE_REGISTRY_KEY + keyPath + " []");

        return;
    },

    /* This function records the creation of a registry key value. */
    registryKeyValueSet : function(keyPath, valueName) 
    {
        if(!this.outputStream)
        {
            return;
        }
        
        this.writeLine(INSTALLER_LOG_PREFIX_STORE_REGISTRY_VALUE_STRING + 
                       keyPath + " [" + valueName + "]"                 );

        return;
    },


    end : function() 
    {
        if(!this.outputStream)
        {
            return;
        }

        this.outputStream.close();
        this.outputStream = null;
    }
};


/////////////////////////////////////////////////////////////////////////////////
//
// ceRegKeyWrapper Implementation
//
/////////////////////////////////////////////////////////////////////////////////

var RegKeyUtils = 
{
    /* This function creates a heirarchy of registry keys.  If any of the
     * keys or values already exist, then they will be updated instead.
     * @param rootKey
     *        The root registry key from which to create the new registry
     *        keys.
     * @param data
     *        A JS object with properties: "name", "values", and "children"
     *        as defined above.  All children of this key will be created.
     */
    createRegistryKeys : function(rootKey, data) 
    {
        var key;

        try 
        {
            key = rootKey.createChild(data.name, rootKey.ACCESS_WRITE);
            var i;

            if("values" in data) 
            {
                for(i = 0; i < data.values.length; i += 2)
                {
                    key.writeStringValue(data.values[i], data.values[i + 1]); 
                }
            }
            
            if("children" in data) 
            {
                for (i = 0; i < data.children.length; ++i)
                {
                    RegKeyUtils.createRegistryKeys(key, data.children[i]);
                }
            }
            
            key.close();
        } 
        catch(exception) 
        {
            Syslog.logException(exception);
            if(key)
            {
                key.close();
            }
        }
                
        return;
    },

    /* This function deletes the specified registry key and optionally all of its
     * children.
     * @param rootKey
     *        The parent nsIwindowRegKey of the key to delete.
     * @param name
     *        The name of the key to delete.
     * @param recurse
     *        Pass true to also delete all children of the named key.  Take care!
     */
    deleteRegistryKey : function(rootKey, name, recurse) 
    {
        if(!rootKey.hasChild(name)) 
        {
            Syslog.logMessage("Windows Post Update: Root Key does not have child: \"" + name + "\"");
            return;
        }

        if(recurse) 
        {
            var key = rootKey.openChild(name, rootKey.ACCESS_ALL);
            try 
            {
                for(var i = key.childCount - 1; i >= 0; --i)
                {
                    RegKeyUtils.deleteRegistryKey(key, key.getChildName(i), true);
                }
                
            } 
            finally 
            {
                key.close();
            }
        }

        rootKey.removeChild(name);
    
        return;
    },

    /* This method walks the registry looking for the registry keys of
     * the previous version of the application.
     */
    locateOldInstall : function(key, ourInstallDir) 
    {
        var result     = null;
        var childKey   = null;
        var productKey = null;
        var mainKey    = null;

        try 
        {
            for(var i = 0; i < key.childCount; ++i) 
            {
                var childName = key.getChildName(i);
                childKey = key.openChild(childName, key.ACCESS_READ);
               
                if(childKey.hasValue("CurrentVersion")) 
                {
                    for(var j = 0; j < childKey.childCount; ++j) 
                    {
                        var productVer = childKey.getChildName(j); 

                        productKey = childKey.openChild(productVer, key.ACCESS_READ);
                        
                        if(productKey.hasChild("Main")) 
                        {
                            mainKey = productKey.openChild("Main", key.ACCESS_READ);
                            
                            var installDir = mainKey.readStringValue("Install Directory");
                            var menuPath   = mainKey.readStringValue("Program Folder Path");

                            mainKey.close();
                            
                            if(PostUpdateUtils.newFile(installDir).equals(ourInstallDir))
                            {
                                result = new Object();

                                result.fullName          = childName;
                                result.versionWithLocale = productVer;
                                result.version           = productVer.split(" ")[0];
                                result.menuPath          = menuPath;
                            }
                        }

                        productKey.close();

                        if(result)
                        {
                            break;
                        }
                    }
                    
                }

                childKey.close();
                
                if(result)
                {
                    break;
                }                
            }
        } 
        catch(exception) 
        {
            result = null;
            
            if(childKey)
            {
                childKey.close();
            }

            if(productKey)
            {
                productKey.close();
            }

            if(mainKey)
            {
                mainKey.close();
            }
        }
        
        return result;
    },


    /* Delete registry keys left-over from the previous version of the app
     * installed at our location.
     */
    deleteOldRegKeys : function(key, info) 
    {
        RegKeyUtils.deleteRegistryKey(key, info.fullName + " " + info.version, true);

        var productKey = key.openChild(info.fullName, key.ACCESS_ALL);
        var productCount;
        
        try 
        {
            RegKeyUtils.deleteRegistryKey(productKey, info.versionWithLocale, true);
            productCount = productKey.childCount;
        } 
        finally 
        {
            productKey.close();
        }

        if(productCount == 0)
        {
            key.removeChild(info.fullName);  
        }
        
        return;
    },
    

    /* The installer sets various registry keys and values that may need to be
     * updated.
     *
     * This operation is a bit tricky since we do not know the previous value of
     * brandFullName.  As a result, we must walk the registry looking for an
     * existing key that references the same install directory.  We assume that
     * the value of vendorShortName does not change across updates.
     */
    updateRegistry : function(rootKey) 
    {
        Syslog.logMessage("Windows Post Update : Updating Registry");
        
        var ourInstallDir   = PostUpdateUtils.getFile(DIRECTORY_SERVICE_KEY_APP_DIRECTORY);
        
        var applicationInfo = do_GetService("@mozilla.org/xre/app-info;1"      ,
                                            Components.interfaces.nsIXULAppInfo);

        var stringBundleService  = do_GetService("@mozilla.org/intl/stringbundle;1"          ,
                                                 Components.interfaces.nsIStringBundleService);


        applicationInfo     = do_QueryInterface(applicationInfo                    ,  
                                                Components.interfaces.nsIXULRuntime);

        var versionInfo     = applicationInfo.version;
        var releaseString   = applicationInfo.releaseString;
                
        var brandBundle     = stringBundleService.createBundle(NETIZEN_BRAND_PROPERTIES_URI);
        var brandFullName   = brandBundle.GetStringFromName("brandFullName");
        var brandWebsite    = brandBundle.GetStringFromName("brandWebsite");
        var vendorWebsite   = brandBundle.GetStringFromName("vendorWebsite");
       
        var vendorShortName = LocalPreferences.getCharPref("app.update.vendorName.override");
        if(!vendorShortName)
        {
            vendorShortName = brandBundle.GetStringFromName("vendorShortName");
        }
        
        var key = new ceRegKeyWrapper();

        var oldInstall;

        try 
        {
            key.open(rootKey, "SOFTWARE\\" + vendorShortName, key.ACCESS_READ);
            oldInstall = RegKeyUtils.locateOldInstall(key, ourInstallDir);
        } 
        finally 
        {
            key.close();
        }

        if(!oldInstall) 
        {
            Syslog.logMessage("Windows Post Update : No Existing Keys Found");
            return;
        }

        if(oldInstall.fullName == brandFullName &&
           oldInstall.version  == releaseString  ) 
        {
            Syslog.logMessage("Windows Post Update : Registry is Up to Date");
            return;
        }

        /* 1. Delete the old Registry Keys */
        try 
        {
            key.open(rootKey, "SOFTWARE\\" + vendorShortName, key.ACCESS_READ);
            RegKeyUtils.deleteOldRegKeys(key, oldInstall);
        }
        finally 
        {
            key.close();
        }

        /* 2. Create/Update the new Registry Keys */
        var versionWithLocale = releaseString + " (" + PostUpdateUtils.getUserAgentLocale() + ")";
        var installPath       = ourInstallDir.path + "\\";
        var pathToExe         = PostUpdateUtils.getFile(DIRECTORY_SERVICE_KEY_EXECUTABLE_FILE).path;

        var Key_main = 
        {
            name     : "Main",
            values   : ["Install Directory"  , installPath        ,
                        "PathToExe"          , pathToExe          ,
                        "Program Folder Path", oldInstall.menuPath]
        };

        var Key_uninstall = 
        {
            name     : "Uninstall",
            values   : ["Description"         , brandFullName + " (" + releaseString + ")",
                        "Uninstall Log Folder", installPath   + "uninstall" ]
        };

        var Key_versionWithLocale = 
        {
            name     : versionWithLocale,
            children : [Key_main, Key_uninstall ]
        };

        var Key_name = 
        {
            name     : brandFullName,
            values   : ["CurrentVersion", versionWithLocale],
            children : [Key_versionWithLocale]
        };

        var Key_brand = 
        {
            name     : vendorShortName,
            children : [Key_name]
        };


        try 
        {
            key.open(rootKey, "SOFTWARE", key.ACCESS_READ);
            RegKeyUtils.createRegistryKeys(key, Key_brand);
        
        } 
        finally 
        {
            key.close();
        }
        
        if(rootKey != ceRegKeyWrapper.prototype.ROOT_KEY_LOCAL_MACHINE)
        {
            return;
        }
        

        /* 3. Create Registry Keys for Add/Remove Programs control panel */
        const uninstallRoot = "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall";

        try 
        {
            key.open(rootKey, uninstallRoot, key.ACCESS_READ);

            var oldName = oldInstall.fullName + " (" + oldInstall.version + ")";
            RegKeyUtils.deleteRegistryKey(key, oldName, false);
        } 
        finally 
        {
            key.close();
        }


        var nameWithVersion = brandFullName + " (" + releaseString + ")";
        var uninstaller     = PostUpdateUtils.getFile(DIRECTORY_SERVICE_KEY_APP_DIRECTORY);

        uninstaller.append("uninstall");
        uninstaller.append("uninstall.exe");

        var uninstallString = uninstaller.path + " /ua \"" + versionWithLocale + "\"";
        Key_uninstall = 
        {
            name    : nameWithVersion,
            values  : ["Comment"        , brandFullName      ,
                       "DisplayIcon"    , pathToExe + ",0"   ,
                       "DisplayName"    , nameWithVersion    ,
                       "DisplayVersion" , versionWithLocale  , 
                       "InstallLocation", ourInstallDir.path ,
                       "Publisher"      , vendorShortName    ,
                       "UninstallString", uninstallString    ,
                       "URLInfoAbout"   , vendorWebsite      ,
                       "URLUpdateInfo"  , brandWebsite       ]
        };


        var child = null;
        
        try 
        {
            key.open(rootKey, uninstallRoot, key.ACCESS_READ);

            RegKeyUtils.createRegistryKeys(key, Key_uninstall);

            /* Create additional DWORD keys for NoModify and NoRepair */
            child = key.openChild(nameWithVersion, key.ACCESS_WRITE);
            child.writeIntValue("NoModify", 1);
            child.writeIntValue("NoRepair", 1);
        } 
        finally 
        {
            if(child)
            {
                child.close();
            }
            
            key.close();
        }

        return;
    }    
};



function ceRegKeyWrapper()
{
    ScriptLoader.loadGlobalToolkitScripts();

    if(arguments.length == 3) 
    {
        this.key  = arguments[0];
        this.root = arguments[1];
        this.path = arguments[2];
    } 
    else 
    {
        this.key = do_CreateInstance("@mozilla.org/windows-registry-key;1" ,
                                     Components.interfaces.nsIWindowsRegKey);
    }

    return;
}


ceRegKeyWrapper.prototype = 
{
    key  : null,
    root : null,
    path : null,

    ACCESS_READ            : Components.interfaces.nsIWindowsRegKey.ACCESS_READ,
    ACCESS_WRITE           : Components.interfaces.nsIWindowsRegKey.ACCESS_WRITE,
    ACCESS_ALL             : Components.interfaces.nsIWindowsRegKey.ACCESS_ALL,
    
    ROOT_KEY_CURRENT_USER  : Components.interfaces.nsIWindowsRegKey.ROOT_KEY_CURRENT_USER,
    ROOT_KEY_LOCAL_MACHINE : Components.interfaces.nsIWindowsRegKey.ROOT_KEY_LOCAL_MACHINE,


    close : function() 
    {
        Syslog.logMessage("Windows Post Update : Closing Key = " + this.path);
        
        this.key.close();
        this.root = null;
        this.path = null;
    
        return;
    },


    open : function(rootKey, path, mode) 
    {
        Syslog.logMessage("Windows Post Update : Opening Key = " + path);
        
        this.key.open(rootKey, path, mode);

        this.root = rootKey;
        this.path = path;

        return;
    },

    openChild : function(path, mode) 
    {
        var child = this.key.openChild(path, mode);

        return (new ceRegKeyWrapper(child, this.root, this.path + "\\" + path));
    },


    createChild : function(path, mode) 
    {
        var child = this.key.createChild(path, mode);
        var key   = new ceRegKey(child, this.root, this.path + "\\" + path);

        if(InstallLogWriter)
        {
            InstallLogWriter.registryKeyCreated(key.toString());
        }
        
        return key;
    },

    readStringValue : function(name) 
    {
        return this.key.readStringValue(name);
    },


    writeStringValue : function(name, value) 
    {
        this.key.writeStringValue(name, value);

        if(InstallLogWriter)
        {
            InstallLogWriter.registryKeyValueSet(this.toString(), name);
        }

        return;
    },


    writeIntValue: function(name, value) 
    {
        this.key.writeIntValue(name, value);

        if(InstallLogWriter)
        {
            InstallLogWriter.registryKeyValueSet(this.toString(), name);
        }
        
        return;
    },
    

    hasValue : function(name) 
    {
        return this.key.hasValue(name);
    },


    hasChild : function(name) 
    {
        return this.key.hasChild(name);
    },


    get childCount() 
    {
        return this.key.childCount;
    },

    getChildName : function(index) 
    {
        return this.key.getChildName(index);
    },

    removeChild : function(name) 
    {
        this.key.removeChild(name);
    },


    toString : function() 
    {
        var root;

        switch(this.root) 
        {
          case this.ROOT_KEY_LOCAL_MACHINE:
          {
              root = "HKEY_LOCAL_MACHINE";
              break;
          }
          case this.ROOT_KEY_CURRENT_USER:
          {
              root = "HKEY_CURRENT_USER";
              break;
          }
          default:
          {
              Syslog.logMessage("Windows Post Update : Unknown Root Key");
              return "";
          }
          
        }
        
        return root + "\\" + this.path;
    }
};



/////////////////////////////////////////////////////////////////////////////////
//
// cePostUpdateWin Implementation
//
/////////////////////////////////////////////////////////////////////////////////

function cePostUpdateWin()
{
    ScriptLoader.loadGlobalToolkitScripts();
    
    return;
}


cePostUpdateWin.prototype =
{
    QueryInterface : function(iid)
    {
        if(!iid.equals(Components.interfaces.nsIRunnable) &&
           !iid.equals(Components.interfaces.nsISupports)  )
        {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }

        return this;
    },

    run : function()
    {
        try 
        {
            /* We use a global object here for the install log writer, so that the
             * registry updating code can easily inform it of registry keys that it
             * may create.
             */
            InstallLogWriter = new ceInstallLogWriter();

            try 
            {
                InstallLogWriter.begin();

                RegKeyUtils.updateRegistry(ceRegKeyWrapper.prototype.ROOT_KEY_CURRENT_USER);
                RegKeyUtils.updateRegistry(ceRegKeyWrapper.prototype.ROOT_KEY_LOCAL_MACHINE);
            } 
            finally 
            {
                InstallLogWriter.end();
                InstallLogWriter = null;
            }
        } 
        catch(exception) 
        {
            Syslog.logException(exception);
        }

        return;
    }
};



/////////////////////////////////////////////////////////////////////////////////
//
// nsIFactory Implementation
//
/////////////////////////////////////////////////////////////////////////////////

var cePostUpdateWinModule = 
{
    moduleObjects :
    {
        cePostUpdateWin :
        {
            CID        : Components.ID("{86218d4a-e8a2-42ba-9823-2a7073d24081}"),
            contractID : "@civilengines.com/updates/post-update;1",
            className  : "cePostUpdateWin",
            factory    : 
            {
                createInstance : function(outer, iid) 
                {
                    if(outer != null)
                    {
                        throw Components.results.NS_ERROR_NO_AGGREGATION;
                    }
                    
                    var postUpdateWin = new cePostUpdateWin();
                    return postUpdateWin.QueryInterface(iid);
                }       
            }
        },
    },
    
    registerSelf : function(componentManager, fileSpec, location, type) 
    {
        componentManager = componentManager.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    
        for(var key in this.moduleObjects) 
        {
            var moduleObject = this.moduleObjects[key];

            componentManager.registerFactoryLocation(moduleObject.CID       , 
                                                     moduleObject.className , 
                                                     moduleObject.contractID,
                                                     fileSpec               , 
                                                     location               ,
                                                     type                   );
        }

        return;
    },

    unregisterSelf : function(componentManager, fileSpec, location, type) 
    {
        componentManager = componentManager.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    
        for(var key in this.moduleObjects) 
        {
            var moduleObject = this.moduleObjects[key];

            componentManager.unregisterFactoryLocation(moduleObject.CID       , 
                                                       moduleObject.className , 
                                                       moduleObject.contractID,
                                                       fileSpec               , 
                                                       location               ,
                                                       type                   );
        }
    },
    
    getClassObject : function(componentManager, cid, iid)
    {
        if(!iid.equals(Components.interfaces.nsIFactory))
        {
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        }
        
        for(var key in this.moduleObjects)
        {
            if(cid.equals(this.moduleObjects[key].CID))
            {
                return this.moduleObjects[key].factory;
            }
        }
            
        throw Components.results.NS_ERROR_NO_INTERFACE;
    },
  

    canUnload : function(componentManager)
    {
        return true;
    }
};



function NSGetModule(componentManager, fileSpec)
{
    return cePostUpdateWinModule;
}
