/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is the Mozilla Update Service.
 *
 * The Initial Developer of the Original Code is Ben Goodger
 *
 * Portions created by the Mozilla Foundation are Copyright (C) 
 * 2006 Mozilla Foundation. All Rights Reserved.
 *
 * Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2006 Civil Engines Research.  All Rights 
 * Reserved.  
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s):
 *   Ben Goodger <ben@mozilla.org> (Original Author)
 *   Darin Fisher <darin@meer.net>
 *   Ben Turner <bent.mozilla@gmail.com>
 *   Paolo de Dios <paolo.dedios@civilengines.com>
 *
 *
 ***** END License Block *****/

const PREF_APP_UPDATE_ENABLED                      = "app.update.enabled";
const PREF_APP_UPDATE_AUTO                         = "app.update.auto";
const PREF_APP_UPDATE_MODE                         = "app.update.mode";
const PREF_APP_UPDATE_SILENT                       = "app.update.silent";
const PREF_APP_UPDATE_INTERVAL                     = "app.update.interval";
const PREF_APP_UPDATE_TIMER                        = "app.update.timer";
const PREF_APP_UPDATE_LOG_BRANCH                   = "app.update.log.";
const PREF_APP_UPDATE_URL                          = "app.update.url";
const PREF_APP_UPDATE_URL_OVERRIDE                 = "app.update.url.override";
const PREF_APP_UPDATE_URL_DETAILS                  = "app.update.url.details";
const PREF_APP_UPDATE_CHANNEL                      = "app.update.channel";
const PREF_APP_UPDATE_SHOW_INSTALLED_UI            = "app.update.showInstalledUI";
const PREF_APP_UPDATE_LASTUPDATETIME_FMT           = "app.update.lastUpdateTime.%ID%";
const PREF_APP_EXTENSIONS_VERSION                  = "app.extensions.version";
const PREF_APP_UPDATE_INCOMPATIBLE_MODE            = "app.update.incompatible.mode";

const PREF_GENERAL_USERAGENT_LOCALE                = "general.useragent.locale";

/* Civil Engines specific fork of nsUpdateService.js */
const NETIZEN_UPDATE_TOPIC_VERSION_AVAILABLE       = "urn:runtime:status:update:available";
const NETIZEN_UPDATE_PROMPT_DIALOG_URI             = "chrome://netizen/content/updates/updateWizard.xul";
const NETIZEN_BRAND_PROPERTIES_URI                 = "chrome://global/locale/brand.properties";
const NETIZEN_UPDATES_PROPERTIES_URI               = "chrome://netizen/locale/updates.properties";
const UPDATE_NS_URI                                = "http://www.mozilla.org/2005/app-update";

const DIRECTORY_SERVICE_KEY_APP_DIRECTORY          = "XCurProcD";
const DIRECTORY_SERVICE_UPDATES_DIRECTORY          = "updates";
const DIRECTORY_SERVICE_UPDATE_STATUS_FILE         = "update.status";
const DIRECTORY_SERVICE_UPDATE_ARCHIVE_FILE        = "update.mar";
const DIRECTORY_SERVICE_UPDATE_LOG_FILE            = "update.log"
const DIRECTORY_SERVICE_UPDATES_DATASOURCE         = "updates.xml";
const DIRECTORY_SERVICE_ACTIVE_UPDATE_FILE         = "active-update.xml";
const DIRECTORY_SERVICE_TEST_FILE                  = "update.test";
const DIRECTORY_SERVICE_LAST_LOG_FILE              = "last-update.log";

const IO_SERVICE_MODE_RDONLY                       = 0x01;
const IO_SERVICE_MODE_WRONLY                       = 0x02;
const IO_SERVICE_MODE_CREATE                       = 0x08;
const IO_SERVICE_MODE_APPEND                       = 0x10;
const IO_SERVICE_MODE_TRUNCATE                     = 0x20;

const IO_SERVICE_FILE_PERMISSION                   = 0644;
const IO_SERVICE_DIRECTORY_PERMISSION              = 0755;

const UPDATE_SERVICE_TOPIC_STARTUP                 = "profile-after-change";
const UPDATE_SERVICE_TOPIC_SHUTDOWN                = "xpcom-shutdown";

const UPDATE_SERVICE_STATE_NONE                    = null;
const UPDATE_SERVICE_STATE_DOWNLOADING             = "downloading";
const UPDATE_SERVICE_STATE_PENDING                 = "pending";
const UPDATE_SERVICE_STATE_APPLYING                = "applying";
const UPDATE_SERVICE_STATE_SUCCEEDED               = "succeeded";
const UPDATE_SERVICE_STATE_DOWNLOAD_FAILED         = "download-failed";
const UPDATE_SERVICE_STATE_FAILED                  = "failed";

const UPDATE_SERVICE_READ_ERROR                    = 6;
const UPDATE_SERVICE_WRITE_ERROR                   = 7;

const UPDATE_SERVICE_DOWNLOAD_CHUNK_SIZE           = 131072;
const UPDATE_SERVICE_DOWNLOAD_BACKGROUND_INTERVAL  = 45;
const UPDATE_SERVICE_DOWNLOAD_FOREGROUND_INTERVAL  = 0;

var ApplicationInfo                                = null;
var ComponentABIString                             = null;

var ScriptLoader =
{
    globalScriptsLoaded : false,
    
    loadGlobalToolkitScripts : function()
    {
        if(this.globalScriptsLoaded == false)
        {
            var scriptLoader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
                .getService(Components.interfaces.mozIJSSubScriptLoader);
            
            if(scriptLoader)
            {
                scriptLoader.loadSubScript("chrome://global/content/utils/nsLocalPreferences.js");
                scriptLoader.loadSubScript("chrome://global/content/utils/ceTraceLogService.js");
                scriptLoader.loadSubScript("chrome://global/content/utils/ceComponentConnector.js");
            }
        }
        
        this.globalScriptsLoaded = true;
        
        return;
    }
};


var UpdateServiceUtils = 
{
    binaryToHex : function(input)
    {
        var result = "";

        for(var i = 0; i < input.length; ++i)
        {
            var hex = input.charCodeAt(i).toString(16);
            if(hex.length == 1)
            {
                hex = "0" + hex;
            }
            
            result += hex;
        }

        return result;
    },


    getURLSpecFromFile : function(file) 
    {
        var fileProtocolHandler = do_QueryInterface(IOService.getProtocolHandler("file")        ,
                                                    Components.interfaces.nsIFileProtocolHandler);

        return fileProtocolHandler.getURLSpecFromFile(file);
    },

    /* Gets the specified directory at the specified hierarchy under a 
     * Directory Service key. 
     * @param   key
     *          The Directory Service Key to start from
     * @param   pathArray
     *          An array of path components to locate beneath the directory 
     *          specified by |key|
     * @return  nsIFile object for the location specified. If the directory
     *          requested does not exist, it is created, along with any
     *          parent directories that need to be created.
     */
    getDirectory : function(key, pathArray) 
    {
        return this.getDirectoryInternal(key, pathArray, true);
    },

    /* Gets the specified directory at the speciifed hierarchy under a 
     * Directory Service key. 
     * @param   key
     *          The Directory Service Key to start from
     * @param   pathArray
     *          An array of path components to locate beneath the directory 
     *          specified by |key|
     * @return  nsIFile object for the location specified. If the directory
     *          requested does not exist, it is NOT created.
     */
    getDirectoryNoCreate : function(key, pathArray) 
    {
        return this.getDirectoryInternal(key, pathArray, false);
    },


    /* Gets the specified directory at the speciifed hierarchy under a 
     * Directory Service key. 
     * @param   key
     *          The Directory Service Key to start from
     * @param   pathArray
     *          An array of path components to locate beneath the directory 
     *          specified by |key|
     * @param   shouldCreate
     *          true if the directory hierarchy specified in |pathArray|
     *          should be created if it does not exist,
     *          false otherwise.
     * @return  nsIFile object for the location specified. 
     */
    getDirectoryInternal : function(key, pathArray, shouldCreate)
    {
        var fileLocator = do_GetService("@mozilla.org/file/directory_service;1",
                                        Components.interfaces.nsIProperties    );
        
        var directory   = fileLocator.get(key, Components.interfaces.nsIFile);
        for (var i = 0; i < pathArray.length; ++i) 
        {
            directory.append(pathArray[i]);
            if(shouldCreate && !dir.exists())
            {
                directory.create(Components.interfaces.nsILocalFile.DIRECTORY_TYPE,
                                 IO_SERVICE_DIRECTORY_PERMISSION                  );
            }
        }
        
        return directory;
    },


    /* Gets the file at the speciifed hierarchy under a Directory Service key.
     * @param   key
     *          The Directory Service Key to start from
     * @param   pathArray
     *          An array of path components to locate beneath the directory 
     *          specified by |key|. The last item in this array must be the
     *          leaf name of a file.
     * @return  nsIFile object for the file specified. The file is NOT created
     *          if it does not exist, however all required directories along 
     *          the way are.
     */
    getFile : function(key, pathArray)
    {
        var file = this.getDirectory(key, pathArray.slice(0, -1));
        if(file)
        {
            file.append(pathArray[pathArray.length - 1]);
        }
        
        return file;
    },


    closeSafeOutputStream : function(fileOutputStream) 
    {
        if(fileOutputStream instanceof Components.interfaces.nsISafeOutputStream)
        {
            try 
            {
                fileOutputStream.finish();
            }
            catch(exception) 
            {
                fileOutputStream.close();
            }
        }
        else
        {
            fileOutputStream.close();
        }

        return;
    },

    /* Returns human readable status text from the updates.properties bundle
     * based on an error code
     * @param   code
     *          The error code to look up human readable status text for
     * @param   defaultCode
     *          The default code to look up should human readable status text
     *          not exist for |code|
     * @returns A human readable status text string
     */
    getStatusTextFromCode : function(code, defaultCode) 
    {
        var stringBundleService = do_GetService("@mozilla.org/intl/stringbundle;1"          ,
                                                Components.interfaces.nsIStringBundleService);

        var updateBundle = stringBundleService.createBundle(NETIZEN_UPDATES_PROPERTIES_URI);

        var reason = updateBundle.GetStringFromName("updates.checker_error-" + defaultCode);

        try 
        {
            reason = updateBundle.GetStringFromName("updates.checker_error-" + code);
        }
        catch(exception) 
        {
        }

        return reason;
    },


    getUpdatesDirectory : function() 
    {
        var fileLocator  = do_GetService("@mozilla.org/file/directory_service;1",
                                         Components.interfaces.nsIProperties    );

        var appDirectory = fileLocator.get(DIRECTORY_SERVICE_KEY_APP_DIRECTORY,
                                           Components.interfaces.nsIFile      );
        appDirectory.append(DIRECTORY_SERVICE_UPDATES_DIRECTORY);
        appDirectory.append("0");
        if(!appDirectory.exists())
        {
            appDirectory.create(Components.interfaces.nsILocalFile.DIRECTORY_TYPE,
                                IO_SERVICE_DIRECTORY_PERMISSION                  );
        }
        
        return appDirectory;
    },
    
    /* Reads the update state from the update.status file in the specified
     * directory.
     * @param   dir
     *          The dir to look for an update.status file in
     * @returns The status value of the update.
     */
    readStatusFile : function(directory) 
    {
        var statusFile = directory.clone();

        statusFile.append(DIRECTORY_SERVICE_UPDATE_STATUS_FILE);
        
        Syslog.logMessage("Auto Update : Reading Status File: " + statusFile.path);

        return this.readStringFromFile(statusFile) || UPDATE_SERVICE_STATE_NONE;
    },

    /* Writes the current update operation/state to a file in the patch 
     * directory, indicating to the patching system that operations need
     * to be performed.
     * @param   dir
     *          The patch directory where the update.status file should be 
     *          written.
     * @param   state
     *          The state value to write.
     */
    writeStatusFile : function(directory, state)
    {
        var statusFile = directory.clone();
     
        statusFile.append(DIRECTORY_SERVICE_UPDATE_STATUS_FILE);
        
        this.writeStringToFile(statusFile, state);

        return;
    },

    /* Reads a string of text from a file.  A trailing newline will be removed
     * before the result is returned.  This function only works with ASCII text.
     */
    readStringFromFile : function(file) 
    {
        var fileInputStream = do_CreateInstance("@mozilla.org/network/file-input-stream;1",
                                                Components.interfaces.nsIFileInputStream  );
        var modeFlags       = IO_SERVICE_MODE_RDONLY;
        if(!file.exists())
        {
            return null;
        }

        fileInputStream.init(file, modeFlags, IO_SERVICE_FILE_PERMISSION, 0);

        var scriptableInputStream = do_CreateInstance("@mozilla.org/scriptableinputstream;1"        ,
                                                      Components.interfaces.nsIScriptableInputStream);
        scriptableInputStream.init(fileInputStream);

        var text = scriptableInputStream.read(scriptableInputStream.available());
        scriptableInputStream.close();
        
        if(text[text.length - 1] == "\n")
        {
            text = text.slice(0, -1);
        }

        return text;
    },

    /* Writes a string of text to a file.  A newline will be appended to the data
     * written to the file.  This function only works with ASCII text.
     */
    writeStringToFile : function(file, text) 
    {
        var fileOutputStream = do_CreateInstance("@mozilla.org/network/safe-file-output-stream;1",
                                                 Components.interfaces.nsIFileOutputStream       );
        
        var modeFlags = IO_SERVICE_MODE_WRONLY | IO_SERVICE_MODE_CREATE | IO_SERVICE_MODE_TRUNCATE;

        if(!file.exists()) 
        {
            file.create(Components.interfaces.nsILocalFile.NORMAL_FILE_TYPE, 
                        IO_SERVICE_FILE_PERMISSION                         );
        }

        fileOutputStream.init(file, modeFlags, IO_SERVICE_FILE_PERMISSION, 0);

        text += "\n";
        
        fileOutputStream.write(text, text.length);    

        this.closeSafeOutputStream(fileOutputStream);
        
        return;
    },


    cleanupUpdatesDirectory : function() 
    {
        var updatesDirectory;

        try 
        {
            updatesDirectory = this.getUpdatesDirectory();
        }
        catch(exception)
        {
            return;
        }
        
        var directoryEntries = updatesDirectory.directoryEntries;

        while(directoryEntries.hasMoreElements())
        {
            var file = do_QueryInterface(directoryEntries.getNext()   ,
                                         Components.interfaces.nsIFile);
            
            /* Preserve the last update log file for debugging purposes */
            if(file.leafName == DIRECTORY_SERVICE_UPDATE_LOG_FILE) 
            {
                try 
                {
                    var topLevelDirectory = file.parent.parent;
                    var logFile           = topLevelDirectory.clone();

                    logFile.append(DIRECTORY_SERVICE_LAST_LOG_FILE);
                    
                    if(logFile.exists())
                    {
                        logFile.remove(false);
                    }

                    file.copyTo(topLevelDirectory, DIRECTORY_SERVICE_LAST_LOG_FILE);
                }
                catch(exception) 
                {
                    Syslog.logMessage("Auto Update : Failed to copy file: " + file.path);
                }
            }

            /* Now, recursively remove this file.  The recusive removal is really
             * only needed on Mac OSX because this directory will contain a copy of
             * updater.app, which is itself a directory.
             */
            try 
            {
                file.remove(true);
            }
            catch(exception) 
            {
                Syslog.logMessage("Auto Update : Failed to remove file: " + file.path);
            }
        }


        try 
        {
            updatesDirectory.remove(false);
        } 
        catch(exception) 
        {
            Syslog.logMessage("Auto Update : Failed to remove update directory: " + updateDir.path);
            Syslog.logException(exception);
            
            throw exception;
        }
    
        return;
    },

    /* Clean up updates list and the updates directory. */
    cleanupActiveUpdate : function() 
    {
        /* Move the update from the Active Update list into the Past Updates list. */
        var updateManager = do_GetService("@civilengines.com/updates/update-manager;1",
                                          Components.interfaces.nsIUpdateManager      );
        
        updateManager.activeUpdate = null;
        updateManager.saveUpdates();

        this.cleanupUpdatesDirectory();
    },


    getUserAgentLocale : function()
    {
        try
        {
            return LocalPreferences.getLocalizedUnicharPref(PREF_GENERAL_USERAGENT_LOCALE);
        }
        catch(exception)
        {
        }
        
        return LocalPreferences.getCharPref(PREF_GENERAL_USERAGENT_LOCALE);
    },

    /* Determines whether or there are installed addons which are incompatible 
     * with this update.
     * @param   update
     *          The update to check compatibility against
     * @returns true if there are no addons installed that are incompatible with
     *          the specified update, false otherwise.
     */
    isCompatible : function(update)
    {
        return true;
    },

    /* Shows a prompt for an update, provided there are no incompatible addons.
     * If there are, kick off an update check and see if updates are available
     * that will resolve the incompatibilities.
     * @param   update
     *          The available update to show
     */
    showPromptIfNoIncompatibilities : function(update)
    {
        var prompter = do_CreateInstance("@civilengines.com/updates/update-prompt;1",
                                         Components.interfaces.nsIUpdatePrompt      );
        
        prompter.showUpdateAvailable(update);
    },

    /* Read the update channel from defaults only.  We do this to ensure that
     * the channel is tightly coupled with the application and does not apply
     * to other instances of the application that may use the same profile.
     */
    getUpdateChannel : function()
    {
        var defaultPrefBranch = LocalPreferences.getDefaultBranch(null);   
        var updateChannel     = "official";

        try 
        {
            updateChannel = defaultPrefBranch.getCharPref(PREF_APP_UPDATE_CHANNEL);
        } 
        catch(exception) 
        {
            Syslog.logMessage(exception);
        }

        return updateChannel;
    }
    
};


/////////////////////////////////////////////////////////////////////////////////
//
// nsISimpleEnumerator Implementation
//
/////////////////////////////////////////////////////////////////////////////////

function ceArrayEnumerator(items) 
{
    this.index = 0;

    if(items) 
    {
        for (var i = 0; i < items.length; ++i) 
        {
            if(!items[i])
            {
                items.splice(i, 1);
            }
        }
    }

    this.contents = items;

    return;
}




ceArrayEnumerator.prototype = 
{
    index    : 0,
    contents : [],
  
    hasMoreElements : function() 
    {
        return this.index < this.contents.length;
    },
  
    getNext : function()
    {
        return this.contents[this.index++];      
    }
};



/////////////////////////////////////////////////////////////////////////////////
//
// nsIUpdatePatch Implementation
//
/////////////////////////////////////////////////////////////////////////////////

function ceUpdatePatch(patchElement)
{
    this.properties = {};

    for(var i = 0; i < patchElement.attributes.length; ++i) 
    {
        var attr = patchElement.attributes.item(i);

        attr.QueryInterface(Components.interfaces.nsIDOMAttr);
        
        switch(attr.name) 
        {
          case "selected":
          {
              this.selected = attr.value == "true";
              break;
          }
          default:
          {
              this[attr.name] = attr.value;
              break;
          }
        }
    }

    return;
}


ceUpdatePatch.prototype =
{
    properties : null,

    QueryInterface: function(iid)
    {
        if(!iid.equals(Components.interfaces.nsIUpdatePatch)         &&
           !iid.equals(Components.interfaces.nsIPropertyBag)         && 
           !iid.equals(Components.interfaces.nsIWritablePropertyBag) && 
           !iid.equals(Components.interfaces.nsISupports)             )
        {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        
        return this;
    },

    getProperty : function(name) 
    {
        if(name in this.properties && this.properties[name].present)
        {
            return this.properties[name].data;
        }
        
        throw Components.results.NS_ERROR_FAILURE;
    },

    setProperty : function(name, value) 
    {
        this.properties[name] = { data: value, present: true };
        return;
    },
  

    deleteProperty : function(name)
    {
        if(name in this.properties)
        {
            this.properties[name].present = false;
        }
        else
        {
            throw Components.results.NS_ERROR_FAILURE;
        }
        
        return;
    },

    get enumerator() 
    {
        var propertyList = [];
        
        for(var prop in this.properties)
        {
            propertyList.push(this.properties[p].data);
        }

        return new ceArrayEnumerator(propertyList);
    },

  
    get statusFileExists() 
    {
        var statusFile = UpdateServiceUtils.getUpdatesDirectory();

        statusFile.append(DIRECTORY_SERVICE_UPDATE_STATUS_FILE);

        return statusFile.exists();
    },

    get state() 
    {
        if(!this.statusFileExists)
        {
            return UPDATE_SERVICE_STATE_NONE;
        }

        return this.properties.state;
    },


    set state(val) 
    {
        this.properties.state = val;
    },


    serialize : function(updates) 
    {
        var patch = updates.createElementNS(UPDATE_NS_URI, "patch");

        patch.setAttribute("type"        , this.type);
        patch.setAttribute("URL"         , this.URL);
        patch.setAttribute("hashFunction", this.hashFunction);
        patch.setAttribute("hashValue"   , this.hashValue);
        patch.setAttribute("size"        , this.size);
        patch.setAttribute("selected"    , this.selected);
        patch.setAttribute("state"       , this.state);
        
        for(var prop in this.properties)
        {
            if(this.properties[prop].present)
            {
                patch.setAttribute(prop, this.properties[prop].data);
            }
        }
        
        return patch; 
    }
};



/////////////////////////////////////////////////////////////////////////////////
//
// nsIUpdate Implementation
//
/////////////////////////////////////////////////////////////////////////////////

function ceUpdateInfo(updateElement)
{
    this.properties       = {};
    this.patches          = [];
    this.installDate      = 0;
    this.isCompleteUpdate = false;
    this.channel          = "official";
    
    if(!updateElement)
    {
        return;
    }
    
    for(var i = 0; i < updateElement.childNodes.length; ++i)
    {
        var patchElement = updateElement.childNodes.item(i);
                
        if(patchElement.nodeType  != Components.interfaces.nsIDOMNode.ELEMENT_NODE ||
           patchElement.localName != "patch"                                        )
        {
            continue;
        }
        
        patchElement.QueryInterface(Components.interfaces.nsIDOMElement);
     
        this.patches.push(new ceUpdatePatch(patchElement));
    }


    for(var i = 0; i < updateElement.attributes.length; ++i) 
    {
        var attr = updateElement.attributes.item(i);

        attr.QueryInterface(Components.interfaces.nsIDOMAttr);

        if(attr.name == "installDate" && attr.value) 
        {
            this.installDate = parseInt(attr.value);
        }
        else if(attr.name == "isCompleteUpdate")
        {
            this.isCompleteUpdate = attr.value == "true";
        }
        else if(attr.name == "isSecurityUpdate")
        {
            this.isSecurityUpdate = attr.value == "true";
        }
        else if(attr.name == "detailsURL")
        {
            this.moreDetailsURL = attr.value;
        }
        else if(attr.name == "channel")
        {
            this.channel = attr.value;
        }
        else
        {
            this[attr.name] = attr.value;
        }
    }

    /* The Update Name is either the string provided by the <update>
     * element, or the string: "[App Name] [Update App Version]"
     */
    var updateName = "";

    if(updateElement.hasAttribute("name"))
    {
        updateName = updateElement.getAttribute("name");
    }
    else 
    {
        var stringBundleService = do_GetService("@mozilla.org/intl/stringbundle;1"          ,
                                                Components.interfaces.nsIStringBundleService);

        var brandBundle  = stringBundleService.createBundle(NETIZEN_BRAND_PROPERTIES_URI);
        var updateBundle = stringBundleService.createBundle(NETIZEN_UPDATES_PROPERTIES_URI);
        var appName      = brandBundle.GetStringFromName("brandShortName");
    
        updateName = updateBundle.formatStringFromName("updates.updateName", [appName, this.version], 2);
    }

    this.name = updateName;
    
    return;
}


ceUpdateInfo.prototype =
{
    updateState : "",
    errorCode   : 0,
    properties  : null,

    QueryInterface: function(iid)
    {
        if(!iid.equals(Components.interfaces.nsIUpdate)              &&
           !iid.equals(Components.interfaces.nsIPropertyBag)         && 
           !iid.equals(Components.interfaces.nsIWritablePropertyBag) && 
           !iid.equals(Components.interfaces.nsISupports)             )
        {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        
        return this;
    },

    get patchCount() 
    {
        return this.patches.length;
    },


    getPatchAt : function(index) 
    {
        return this.patches[index];
    },


    set state(newState) 
    {
        if(this.selectedPatch)
        {
            this.selectedPatch.state = newState;
        }
        
        this.updateState = newState;

        return newState;
    },
    
    get state() 
    {
        if(this.selectedPatch)
        {
            return this.selectedPatch.state;
        }

        return this.updateState;
    },

    get selectedPatch() 
    {
        for(var i = 0; i < this.patchCount; ++i) 
        {
            if(this.patches[i].selected)
            {
                return this.patches[i];
            }
        }
        
        return null;
    },

    get detailsURL() 
    {
        if(!this.moreDetailsURL) 
        {
            try 
            {
                /* Try using a default details URL supplied by the distribution
                 * if the update XML does not supply one.
                 */
                return LocalPreferences.getLocalizedUnicharPref(PREF_APP_UPDATE_URL_DETAILS);
            }
            catch(exception) 
            {
            }
        }

        return this.moreDetailsURL || "";
    },

    getProperty : function(name) 
    {
        if(name in this.properties && this.properties[name].present)
        {
            return this.properties[name].data;
        }
        
        throw Components.results.NS_ERROR_FAILURE;
    },

    setProperty : function(name, value) 
    {
        this.properties[name] = { data: value, present: true };
        return;
    },
  

    deleteProperty : function(name)
    {
        if(name in this.properties)
        {
            this.properties[name].present = false;
        }
        else
        {
            throw Components.results.NS_ERROR_FAILURE;
        }
        
        return;
    },

    get enumerator() 
    {
        var propertyList = [];
        
        for(var prop in this.properties)
        {
            propertyList.push(this.properties[p].data);
        }

        return new ceArrayEnumerator(propertyList);
    },

    serialize : function(updatesDocument)
    {
        var update = updatesDocument.createElementNS(UPDATE_NS_URI, "update");

        update.setAttribute("type"            , this.type);
        update.setAttribute("name"            , this.name);
        update.setAttribute("version"         , this.version);
        update.setAttribute("extensionVersion", this.extensionVersion);
        update.setAttribute("detailsURL"      , this.detailsURL);
        update.setAttribute("licenseURL"      , this.licenseURL);
        update.setAttribute("serviceURL"      , this.serviceURL);
        update.setAttribute("installDate"     , this.installDate);
        update.setAttribute("statusText"      , this.statusText);
        update.setAttribute("buildID"         , this.buildID);
        update.setAttribute("isCompleteUpdate", this.isCompleteUpdate);
        update.setAttribute("channel"         , this.channel);

        updatesDocument.documentElement.appendChild(update);
    
        for(var prop in this.properties) 
        {
            if(this.properties[prop].present)
            {
                update.setAttribute(prop, this.properties[prop].data);
            }
        }
            
        for(var i = 0; i < this.patchCount; ++i)
        {
            update.appendChild(this.getPatchAt(i).serialize(updatesDocument));
        }
        
        return update;
    }

};


/////////////////////////////////////////////////////////////////////////////////
//
// nsIUpdateService Implementation
//
/////////////////////////////////////////////////////////////////////////////////


function ceUpdateService()
{
    ScriptLoader.loadGlobalToolkitScripts();
        
    ApplicationInfo = do_GetService("@mozilla.org/xre/app-info;1"      ,
                                    Components.interfaces.nsIXULAppInfo);

    ApplicationInfo = do_QueryInterface(ApplicationInfo                    ,  
                                        Components.interfaces.nsIXULRuntime);    

    try
    {
        ComponentABIString = ApplicationInfo.componentABI;
    }
    catch(exception)
    {
        Syslog.logException(exception);
    }

    if("@mozilla.org/xpcom/mac-utils;1" in Components.classes)
    {
        var macUtils = do_GetService("@mozilla.org/xpcom/mac-utils;1" ,
                                     Components.interfaces.nsIMacUtils);
        if(macUtils.isUniversalBinary)
        {
            ComponentABIString = "universal-gcc4";
        }        
    }

    var buildTarget       = ApplicationInfo.osTarget + "_" + ComponentABIString;    
        
    Syslog.logMessage("Auto Update : Update Service Loaded");
    Syslog.logMessage("Auto Update : Application Name     = " + ApplicationInfo.name);
    Syslog.logMessage("Auto Update : Application Release  = " + ApplicationInfo.releaseString);
    Syslog.logMessage("Auto Update : Application Version  = " + ApplicationInfo.version);
    Syslog.logMessage("Auto Update : Application Build ID = " + ApplicationInfo.buildID);
    Syslog.logMessage("Auto Update : Application Target   = " + buildTarget);
    Syslog.logMessage("Auto Update : Application Locale   = " + UpdateServiceUtils.getUserAgentLocale());
    Syslog.logMessage("Auto Update : Update Channel       = " + UpdateServiceUtils.getUpdateChannel());

    this.loadObservers();
    
    return;
}


ceUpdateService.prototype = 
{
    downloader    : null,
    updateChecker : null,

    loadObservers : function()
    {
        ObserverService.addObserver(this, NETIZEN_UPDATE_TOPIC_VERSION_AVAILABLE, false);        

        /* Start the update timer only after a profile has been selected so that the
         * appropriate values for the update check are read from the user's profile.
         */  
        ObserverService.addObserver(this, UPDATE_SERVICE_TOPIC_STARTUP, false);


        /* Observe xpcom-shutdown to unhook pref branch observers above to avoid 
         * shutdown leaks.
         */
        ObserverService.addObserver(this, UPDATE_SERVICE_TOPIC_SHUTDOWN, false);

        return;
    },


    unloadObservers : function()
    {
        ObserverService.removeObserver(this, NETIZEN_UPDATE_TOPIC_VERSION_AVAILABLE);
        ObserverService.removeObserver(this, "xpcom-shutdown");

        return;
    },


    start : function()
    {
        this.performPostUpdateProcessing();
        
        var timerManager   = do_GetService("@civilengines.com/updates/timer-manager;1",
                                           Components.interfaces.nsIUpdateTimerManager);

        var updateInterval = LocalPreferences.getIntPref(PREF_APP_UPDATE_INTERVAL, 86400);
        
        timerManager.registerTimer("background-update-timer", this, updateInterval);
        
        var updateManager  = do_GetService("@civilengines.com/updates/update-manager;1",
                                           Components.interfaces.nsIUpdateManager      );

        var activeUpdate   = updateManager.activeUpdate;
        if(activeUpdate)
        {
            var status = this.downloadUpdate(activeUpdate, true);
            if(status == UPDATE_SERVICE_STATE_NONE)
            {
                UpdateServiceUtils.cleanupActiveUpdate();
            }
        }
        else
        {
            Syslog.logMessage("Auto Update : No Active Updates");
        }
        
        return;
    },
    
    /* Perform post-processing on updates lingering in the updates directory
     * from a previous session - either report install failures (and
     * optionally attempt to fetch a different version if appropriate) or 
     * notify the user of install success.
     */
    performPostUpdateProcessing : function()
    {
        if(!this.canUpdate)
        {
            return;
        }
        
        var status = UpdateServiceUtils.readStatusFile(UpdateServiceUtils.getUpdatesDirectory());
        if(status == "null")
        {
            status = null;
        }
        
        Syslog.logMessage("Auto Update : Post Update Status = " + status);
        
        if(status == UPDATE_SERVICE_STATE_DOWNLOADING)
        {
            Syslog.logMessage("Auto Update : Resuming patch download");
        }
        else if(status != null)
        {
            /* null status means the update.status file is not present, because either:
             * 1) no update was performed, and so there's no UI to show
             * 2) an update was attempted but failed during checking, transfer or 
             *    verification, and was cleaned up at that point, and UI notifying of
             *    that error was shown at that stage.
             */ 
            var updateManager = do_GetService("@civilengines.com/updates/update-manager;1",
                                              Components.interfaces.nsIUpdateManager      );

            var prompter      = do_CreateInstance("@civilengines.com/updates/update-prompt;1",
                                                  Components.interfaces.nsIUpdatePrompt      );
            
            var shouldCleanup = true;
            var update        = updateManager.activeUpdate;
            if(!update)
            {
                update = new ceUpdateInfo(null);
            }

            update.state = status;

            var stringBundleService = do_GetService("@mozilla.org/intl/stringbundle;1"          ,
                                                    Components.interfaces.nsIStringBundleService);

            var updateBundle = stringBundleService.createBundle(NETIZEN_UPDATES_PROPERTIES_URI);

            if(status == UPDATE_SERVICE_STATE_SUCCEEDED)
            {
                update.statusText = updateBundle.GetStringFromName("updates.installSuccess");
        
                /* Dig through the update history to find the patch that was just
                 * installed and update its metadata.
                 */
                for(var i = 0; i < updateManager.updateCount; ++i) 
                {
                    var updateManagerUpdate = updateManager.getUpdateAt(i);
                    if(updateManagerUpdate && updateManagerUpdate.version == update.version &&
                       updateManagerUpdate.buildID == update.buildID                         )
                    {
                        updateManagerUpdate.statusText = update.statusText;
                        break;
                    }
                }

                Syslog.logMessage("Auto Update : Install Succeeded, Showing UI");

                prompter.showUpdateInstalled(update);
                
                if("@civilengines.com/updates/post-update;1" in Components.classes) 
                {
                    var postUpdateProcessor = null;
                    postUpdateProcessor = do_CreateInstance("@civilengines.com/updates/post-update;1",
                                                            Components.interfaces.nsIRunnable        );
                    if(postUpdateProcessor)
                    {
                        postUpdateProcessor.run();
                    }                    
                }
                
                
                UpdateServiceUtils.cleanupActiveUpdate();
            }
            else
            {
                /* If we hit an error, then the error code will be included in the
                 * status string following a colon.  If we had an I/O error, then we
                 * assume that the patch is not invalid, and we restage the patch so
                 * that it can be attempted again the next time we restart.
                 */
                var ary = status.split(": ");

                update.state = ary[0];

                if(update.state == UPDATE_SERVICE_STATE_FAILED && ary[1]) 
                {
                    update.errorCode = ary[1];

                    if(update.errorCode == UPDATE_SERVICE_WRITE_ERROR) 
                    {
                        prompter.showUpdateError(update);
                        
                        update.state = UPDATE_SERVICE_STATE_PENDING;
                        
                        UpdateServiceUtils.writeStatusFile(UpdateServiceUtils.getUpdatesDirectory(),
                                                           update.state                            );
                        return;
                    }
                }
                
                UpdateServiceUtils.cleanupActiveUpdate();


                update.statusText = updateBundle.GetStringFromName("updates.patchApplyFailure");
                var oldType = update.selectedPatch ? update.selectedPatch.type : "complete";

                if(update.selectedPatch && oldType == "partial") 
                {
                    /* Partial patch application failed, try downloading the complete
                     * update in the background instead.
                     */
                    Syslog.logMessage("Auto Update : Install of Partial Patch failed");
                    Syslog.logMessage("Auto Update : Downloading Complete Patch");

                    var status = this.downloadUpdate(update, true);
                    if(status == UPDATE_SERVICE_STATE_NONE)
                    {
                        UpdateServiceUtils.cleanupActiveUpdate();
                    }
                    
                }
                else 
                {
                    Syslog.logMessage("Auto Update : Install of Complete/Partial Patch Failed");
                }

                update.QueryInterface(Components.interfaces.nsIWritablePropertyBag);
                update.setProperty("patchingFailed", oldType);

                prompter.showUpdateError(update);
            }
        }
        else
        {
            Syslog.logMessage("Auto Update : Status Unavailable, No Updates");
        }
        
        
        return;
    },
    
    /* Determine whether or not an update requires user confirmation before it
     * can be installed.
     * @param   update
     *          The update to be installed
     * @returns true if a prompt UI should be shown asking the user if they want
     *          to install the update, false if the update should just be 
     *          silently downloaded and installed.
     */
    shouldPrompt: function(update)
    {
        /* There are two possible outcomes here:
         * 1. download and install the update automatically
         * 2. alert the user about the presence of an update before doing anything
         *
         * The outcome we follow is determined as follows:
         * 
         * Update Type      Mode      Incompatible    Outcome
         * Major            0         Yes or No       Auto Install
         * Major            1         No              Auto Install
         * Major            1         Yes             Notify and Confirm
         * Major            2         Yes or No       Notify and Confirm
         * Minor            0         Yes or No       Auto Install
         * Minor            1         No              Auto Install
         * Minor            1         Yes             Notify and Confirm
         * Minor            2         No              Auto Install
         * Minor            2         Yes             Notify and Confirm
         *
         * In addition, if there is a license associated with an update, regardless
         * of type it must be agreed to. 
         *
         * If app.update.enabled is set to false, an update check is not performed
         * at all, and so none of the decision making above is entered into.
         */
        update.QueryInterface(Components.interfaces.nsIPropertyBag);
        
        try 
        {
            var licenseAccepted = update.getProperty("licenseAccepted") == "true";
        }
        catch(exception) 
        {
            licenseAccepted = false;
        }

        if(update.licenseURL && !licenseAccepted) 
        {
            Syslog.logMessage("Update Requires License Acceptance");
            return true;
        }


        var updateEnabled = LocalPreferences.getBoolPref(PREF_APP_UPDATE_ENABLED, true);
        if(!updateEnabled) 
        {
            Syslog.logMessage("Auto Update Disabled");
            return false;
        }
    
        /* User has turned off automatic download and install */
        var autoEnabled = LocalPreferences.getBoolPref(PREF_APP_UPDATE_AUTO, true);
        if(!autoEnabled) 
        {
            Syslog.logMessage("Auto Install Disabled : Prompting User");
            return true;
        }

        var updateMode = LocalPreferences.getIntPref(PREF_APP_UPDATE_MODE, 1);
        switch(updateMode)
        {
          case 1:
          {
              /* Mode 1 is do not prompt only if there are no incompatibilities releases */
              Syslog.logMessage("Prompting if there are incompatibilities");
              return !UpdateServiceUtils.isCompatible(update);
          }
          case 2:
          {
              /* Mode 2 is do not prompt only if there are no incompatibilities for 
               * minor updates only
               */
              Syslog.logMessage("Prompting if release is major or if there are incompatibilities");
              
              return update.type == "minor" ? !UpdateServiceUtils.isCompatible(update) : true;
          }
        }
        
        Syslog.logMessage("Prompt Disabled : User Chose to Ignore Incompatiblities");
        return false;
    },
    
    /* Determine which of the specified updates should be installed and
     * begin the download/installation process, optionally prompting the
     * user for permission if required.
     * @param   updates
     *          An array of available updates
     */
    selectAndInstallUpdate : function(updates) 
    {
        /* Don't prompt if there's an active update - the user is already 
         * aware and is downloading, or performed some user action to prevent
         * notification.
         */
        var updateManager = do_GetService("@civilengines.com/updates/update-manager;1",
                                          Components.interfaces.nsIUpdateManager      );
        if(updateManager.activeUpdate)
        {
            return;
        }

        var update = this.selectUpdate(updates, updates.length);
        if(!update)
        {
            return;
        }
        
        if(this.shouldPrompt(update))
        {
            UpdateServiceUtils.showPromptIfNoIncompatibilities(update);
        }
        else 
        {
            var status = this.downloadUpdate(update, true);
            if(status == UPDATE_SERVICE_STATE_NONE)
            {
                UpdateServiceUtils.cleanupActiveUpdate();
            }
        }
      
        return;
    },


    runBackgroundChecker : function()
    {
        /* If a download is in progress, then do nothing. */
        if(this.isDownloading || this.downloader && this.downloader.patchIsStaged)
        {
            Sylog.logMessage("Auto Update : Update Download Already In Progress");
            return;
        }
        
        var updateService  = this;

        var updateListener = 
        {
            onProgress: function(request, position, totalSize) 
            {
                return;
            },
      
            onCheckComplete: function(request, updates, updateCount) 
            {
                Syslog.logMessage("Auto Update : Update Check Complete");
                updateService.selectAndInstallUpdate(updates);

                return;
            },

            onError: function(request, update) 
            { 
                Syslog.logMessage("Auto Update : Error occurred during background update");
                Syslog.logMessage(update.statusText);
                return;
            }
        }

        this.backgroundChecker.checkForUpdates(updateListener, false);

        return;
    },


    QueryInterface : function(iid)
    {
        if(!iid.equals(Components.interfaces.nsIUpdateService) &&
           !iid.equals(Components.interfaces.nsITimerCallback) && 
           !iid.equals(Components.interfaces.nsIObserver)      && 
           !iid.equals(Components.interfaces.nsISupports)       )
        {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        
        return this;
    },


    observe : function(subject, topic, data)
    {
        switch(topic) 
        {
          case UPDATE_SERVICE_TOPIC_STARTUP :
          {
              ObserverService.removeObserver(this, UPDATE_SERVICE_TOPIC_STARTUP);
              this.start();

              break;
          }
          case NETIZEN_UPDATE_TOPIC_VERSION_AVAILABLE :
          { 
              Syslog.logMessage("Auto Update : New Update (" + data + ") Available" );

              this.runBackgroundChecker();

              break;
          }
          case UPDATE_SERVICE_TOPIC_SHUTDOWN :
          {
              this.unloadObservers();
                            
              ApplicationInfo     = null;
              ObserverService     = null;

              break;
          }          
        }

        return;
    },


    notify : function(timer)
    {
        /* If a download is in progress, then do nothing. */
        if(this.isDownloading || this.downloader && this.downloader.patchIsStaged)
        {
            return;
        }

        var updateObserver = do_GetService("@civilengines.com/updates/update-observer;1",
                                           Components.interfaces.nsIUpdateObserver      );
        
        updateObserver.checkUpdateAvailability();

        return;
    },


    /* Determine which of the specified updates should be installed.
     * @param   updates
     *          An array of available updates
     */
    selectUpdate: function(updates) 
    {
        if(updates.length == 0)
        {
            return null;
        }
        
        /* Choose the newest of the available minor and major updates. */ 
        var majorUpdate = null;
        var minorUpdate = null;
        var newestMinor = updates[0];
        var newestMajor = updates[0];

        var versionComparator = do_GetService("@mozilla.org/xpcom/version-comparator;1",
                                              Components.interfaces.nsIVersionComparator);
        for(var i = 0; i < updates.length; ++i) 
        {
            if(updates[i].type == "major" && 
               versionComparator.compare(newestMajor.version, updates[i].version) <= 0)
            {
                majorUpdate = newestMajor = updates[i];
            }
            
            if(updates[i].type == "minor" && 
               versionComparator.compare(newestMinor.version, updates[i].version) <= 0)
            {
                minorUpdate = newestMinor = updates[i];
            }
        }
        
        /* If there's a major update, always try and fetch that one first, 
         * otherwise fall back to the newest minor update.
         */
        return majorUpdate || minorUpdate;
    },


    get backgroundChecker() 
    {
        if(!this.updateChecker) 
        {
            this.updateChecker = new ceUpdateChecker();
        }

        return this.updateChecker;
    },


    get canUpdate() 
    {
        try 
        {
            var appDirFile = UpdateServiceUtils.getFile(DIRECTORY_SERVICE_KEY_APP_DIRECTORY,
                                                        [DIRECTORY_SERVICE_TEST_FILE]      );
            if(!appDirFile.exists()) 
            {
                appDirFile.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 
                                  IO_SERVICE_FILE_PERMISSION                    );

                appDirFile.remove(false);
            }

            var updateDir     = UpdateServiceUtils.getUpdatesDirectory();
            var updateDirFile = updateDir.clone();

            updateDirFile.append(DIRECTORY_SERVICE_TEST_FILE);

            if(!updateDirFile.exists()) 
            {
                updateDirFile.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE,
                                     IO_SERVICE_FILE_PERMISSION                    );

                updateDirFile.remove(false);
            }
        }
        catch(exception) 
        {
            Syslog.logMessage("Auto Update : Cannot Perform Update, Permission Denied");
            return false;
        }

        /* If the administrator has locked the app update functionality 
         * OFF - this is not just a user setting, so disable the manual
         * UI too.
         */
        var enabled = LocalPreferences.getBoolPref(PREF_APP_UPDATE_ENABLED, true);

        if(!enabled && LocalPreferences.prefIsLocked(PREF_APP_UPDATE_ENABLED))
        {            
            return false;
        }
        
        if(!ComponentABIString)
        {            
            return false;
        }
        
        return true;
    },

    addDownloadListener: function(listener)
    {
        if(!this.downloader) 
        {
            Syslog.logMessage("Auto Update : addDownloadListener failed : Downloader Not Available");
            return;
        }

        this.downloader.addDownloadListener(listener);
    },
  
    
    removeDownloadListener: function(listener)
    {
        if(!this.downloader)
        {
            Syslog.logMessage("Auto Update : removeDownloadListener failed : Downloader Not Available");
            return;
        }
        
        this.downloader.removeDownloadListener(listener);
    },


    get isDownloading()
    {
        return this.downloader && this.downloader.isBusy;
    },
    

    downloadUpdate : function(update, background)
    {
        if(!update)
        {
            throw Components.results.NS_ERROR_NULL_POINTER;
        }

        if(this.isDownloading)
        {
            if(update.isCompleteUpdate == this.downloader.isCompleteUpdate &&
               background == this.downloader.background                     )
            {
                return UpdateServiceUtils.readStatusFile(UpdateServiceUtils.getUpdatesDirectory());
            }
            
            this.downloader.cancel();
        }
        
        this.downloader = new ceUpdateDownloader(background);

        return this.downloader.downloadUpdate(update);
    },
    

    pauseDownload : function()
    {
        if(this.isDownloading)
        {
            this.downloader.cancel();
        }
        
        return;
    },
    
    
};


/////////////////////////////////////////////////////////////////////////////////
//
// nsIUpdateManager Implementation
//
/////////////////////////////////////////////////////////////////////////////////

function ceUpdateManager()
{
    ScriptLoader.loadGlobalToolkitScripts();

    var xmlFile = UpdateServiceUtils.getFile(DIRECTORY_SERVICE_KEY_APP_DIRECTORY   ,
                                             [DIRECTORY_SERVICE_ACTIVE_UPDATE_FILE]);
    
    var updateList = this.loadXMLFileIntoArray(xmlFile);
    if(updateList.length > 0)
    {
        this.currentUpdate = updateList[0];
    }
    
    return;
}



ceUpdateManager.prototype =
{
    updates       : null,
    currentUpdate : null,

    /* Loads an updates.xml formatted file into an array of nsIUpdate items.
     * @param   file
     *          A nsIFile for the updates.xml file
     * @returns The array of nsIUpdate items held in the file.
     */
    loadXMLFileIntoArray : function(file)
    {
        if(!file.exists())
        {
            Syslog.logMessage("Auto Update : Cannot load XML file; does not exist");
            return [];
        }

        var result     = [];
        var fileStream = do_CreateInstance("@mozilla.org/network/file-input-stream;1",
                                           Components.interfaces.nsIFileInputStream  );

        fileStream.init(file, IO_SERVICE_MODE_RDONLY, IO_SERVICE_FILE_PERMISSION, 0);

        try 
        {
            var parser = do_CreateInstance("@mozilla.org/xmlextras/domparser;1",
                                           Components.interfaces.nsIDOMParser  );

            var doc    = parser.parseFromStream(fileStream            , 
                                                "UTF-8"               , 
                                                fileStream.available(), 
                                                "text/xml"            );
      
            var updateCount = doc.documentElement.childNodes.length;

            for(var i = 0; i < updateCount; ++i) 
            {
                var updateElement = doc.documentElement.childNodes.item(i);
                if(updateElement.nodeType  != Components.interfaces.nsIDOMNode.ELEMENT_NODE ||
                   updateElement.localName != "update"                                       )
                {
                    continue;
                }
                
                updateElement.QueryInterface(Components.interfaces.nsIDOMElement);
                result.push(new ceUpdateInfo(updateElement));
            }
        }
        catch(exception) 
        {
            Syslog.logException(exception);
        }

        fileStream.close();

        return result;
    },

    ensureUpdates : function() 
    {
        if(!this.updates) 
        {
            var xmlFile  = UpdateServiceUtils.getFile(DIRECTORY_SERVICE_KEY_APP_DIRECTORY  ,
                                                     [DIRECTORY_SERVICE_UPDATES_DATASOURCE]);

            this.updates = this.loadXMLFileIntoArray(xmlFile);
            

            var active = this.currentUpdate;
            if(active)
            {
                this.addUpdate(active);
            }
        }
      
        return;
    },

               
    /* Add an update to the Updates list. If the item already exists in the list,
     * replace the existing value with the new value.
     * @param   update
     *          The nsIUpdate object to add.
     */
    addUpdate : function(update)
    {
        if(!update)
        {
            return;
        }
        
        if(this.updates) 
        {
            for(var i = 0; i < this.updates.length; ++i)
            {
                if(this.updates[i].version == update.version &&
                   this.updates[i].buildID == update.buildID  ) 
                {
                    this.updates[i] = update;
                    return;
                }
            }
        }

        if(update)
        { 
            this.updates = [update].concat(this.updates);
        }
        
        return;
    },
               
    /* Serializes an array of updates to an XML file
     * @param   updates
     *          An array of nsIUpdate objects
     * @param   file
     *          The nsIFile object to serialize to
     */
    writeUpdatesToXMLFile : function(updates, file) 
    {
        var fileOutputStream = do_CreateInstance("@mozilla.org/network/safe-file-output-stream;1",
                                                 Components.interfaces.nsIFileOutputStream       );
        
        var modeFlags = IO_SERVICE_MODE_WRONLY | IO_SERVICE_MODE_CREATE | IO_SERVICE_MODE_TRUNCATE;
        if(!file.exists())
        { 
            file.create(Components.interfaces.nsILocalFile.NORMAL_FILE_TYPE,
                        IO_SERVICE_FILE_PERMISSION                         );
        }
        
        fileOutputStream.init(file, modeFlags, IO_SERVICE_FILE_PERMISSION, 0);
    
        try 
        {
            var parser = do_CreateInstance("@mozilla.org/xmlextras/domparser;1",
                                           Components.interfaces.nsIDOMParser  );
            
            const EMPTY_UPDATES_DOCUMENT = 
                "<?xml version=\"1.0\"?>" +
                "<updates xmlns=\"http://www.mozilla.org/2005/app-update\"></updates>";

            var doc = parser.parseFromString(EMPTY_UPDATES_DOCUMENT, "text/xml");

            for(var i = 0; i < updates.length; ++i)
            {
                if(updates[i])
                {
                    doc.documentElement.appendChild(updates[i].serialize(doc));
                }                
            }

            var serializer = do_CreateInstance("@mozilla.org/xmlextras/xmlserializer;1",
                                               Components.interfaces.nsIDOMSerializer  );
            
            serializer.serializeToStream(doc.documentElement, fileOutputStream, null);
        }
        catch(exception) 
        {
            Syslog.logException(exception);
        }
        
        UpdateServiceUtils.closeSafeOutputStream(fileOutputStream);

        return;
    },

    QueryInterface : function(iid) 
    {
        if(!iid.equals(Components.interfaces.nsIUpdateManager) &&
           !iid.equals(Components.interfaces.nsISupports)       )
        {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        
        return this;
    },


    saveUpdates : function()
    {
        var xmlFile = UpdateServiceUtils.getFile(DIRECTORY_SERVICE_KEY_APP_DIRECTORY   ,
                                                 [DIRECTORY_SERVICE_ACTIVE_UPDATE_FILE]);

        this.writeUpdatesToXMLFile([this.currentUpdate], xmlFile);
        if(this.updates) 
        {
            var datasourceFile = UpdateServiceUtils.getFile(DIRECTORY_SERVICE_KEY_APP_DIRECTORY  ,
                                                            [DIRECTORY_SERVICE_UPDATES_DATASOURCE]);

            this.writeUpdatesToXMLFile(this.updates, datasourceFile);
        }

        return;
    },

    getUpdateAt : function(index)
    {
        this.ensureUpdates();
        return this.updates[index];
    },
  

    get updateCount() 
    {
        this.ensureUpdates();
        return this.updates.length;
    },

    get activeUpdate()
    {
        if(this.currentUpdate &&
           this.currentUpdate.channel != UpdateServiceUtils.getUpdateChannel()) 
        {
            /* User switched channels, clear out any old active
             * updates and remove partial downloads
             */
            this.currentUpdate = null;
      
            /* Destroy the updates directory, since we're done with it. */
            UpdateServiceUtils.cleanupUpdatesDirectory();
        }

        return this.currentUpdate;
    },

    set activeUpdate(activeUpdate)
    {
        this.addUpdate(activeUpdate);

        this.currentUpdate = activeUpdate;

        if(!activeUpdate) 
        {
            /* If |activeUpdate| is null, we have updated both lists - the active list
             * and the history list, so we want to write both files.
             */
            this.saveUpdates();
        }
        else
        {
            var xmlFile = UpdateServiceUtils.getFile(DIRECTORY_SERVICE_KEY_APP_DIRECTORY   ,
                                                     [DIRECTORY_SERVICE_ACTIVE_UPDATE_FILE]);            

            this.writeUpdatesToXMLFile([this.currentUpdate], xmlFile);
        }
        
        return activeUpdate;
    }
  
};



/////////////////////////////////////////////////////////////////////////////////
//
// nsIInterfaceRequestor Implementation
//
/////////////////////////////////////////////////////////////////////////////////

function ceUpdateChannelEventSink()
{
}

ceUpdateChannelEventSink.prototype = 
{
    QueryInterface : function(iid)
    {
        if(!iid.equals(Components.interfaces.nsIBadCertListener)    &&
           !iid.equals(Components.interfaces.nsIInterfaceRequestor) &&
           !iid.equals(Components.interfaces.nsISupports)            )
        {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        
        return this;
    },


    getInterface : function(iid) 
    {
        if(iid.equals(Components.interfaces.nsIBadCertListener) ||
           iid.equals(Components.interfaces.nsIChannelEventSink)||
           iid.equals(Components.interfaces.nsIHttpEventSink)   ||
           iid.equals(Components.interfaces.nsIPrompt)           )
        {
            return this;
        }
        
        Components.returnCode = Components.results.NS_ERROR_NO_INTERFACE;
        return null;
    },

    confirmUnknownIssuer: function(socketInfo, cert, certAddType) 
    {
        return false;
    },

    confirmMismatchDomain: function(socketInfo, targetURL, cert) 
    {
        return false;
    },
    
    confirmCertExpired: function(socketInfo, cert) 
    {
        return false;
    },

    notifyCrlNextupdate: function(socketInfo, targetURL, cert)
    {
        return;
    },

    onRedirect : function(httpChannel, newChannel)
    {
        return;
    },
    
    onChannelRedirect : function(oldChannel, newChannel, redirectFlags)
    {
        return;
    }    
};


/////////////////////////////////////////////////////////////////////////////////
//
// nsIUpdateChecker Implementation
//
/////////////////////////////////////////////////////////////////////////////////


function ceUpdateChecker()
{
    ScriptLoader.loadGlobalToolkitScripts();
}


ceUpdateChecker.prototype =
{
    request    : null,
    callback   : null,
    isEnabled  : true,

    /* Returns an array of nsIUpdate objects discovered by the update check */
    get updates() 
    {
        Syslog.logMessage("Auto Update : Checking Update XML");

        var updatesElement = this.request.responseXML.documentElement;

        if(!updatesElement) 
        {
            Syslog.logMessage("Auto Update: Empty Update Document");
            return [];
        }
        
        if(updatesElement.nodeName != "updates") 
        {
            Syslog.logMessage("Auto Update : Unexpected Update Document Element");
            throw Components.results.NS_ERROR_FAILURE;
        }
        
        var updateList = [];

        for(var i = 0; i < updatesElement.childNodes.length; ++i) 
        {        
            var updateElement = updatesElement.childNodes.item(i);

            if(updateElement.nodeType  != Components.interfaces.nsIDOMNode.ELEMENT_NODE ||
               updateElement.localName != "update"                                       )
            {
                continue;
            }
            
            updateElement.QueryInterface(Components.interfaces.nsIDOMElement);
            
            Syslog.logMessage("Auto Update : Loading Update Element");

            var update = new ceUpdateInfo(updateElement);

            update.serviceURL = this.updateURL;
            update.channel    = UpdateServiceUtils.getUpdateChannel();
            
            updateList.push(update);
        }
        
        return updateList;
    },

    /* When progress associated with the XMLHttpRequest is received.
     * @param   event
     *          The nsIDOMLSProgressEvent for the load.
     */
    onProgress : function(event)
    {
        Syslog.logMessage("Auto Update : Update Progress " + event.position + "/" + event.totalSize);

        this.callback.onProgress(event.target, event.position, event.totalSize);

        return;
    },

    /* The XMLHttpRequest succeeded and the document was loaded.
     * @param   event
     *          The nsIDOMLSEvent for the load
     */
    onLoad : function(event)
    {
        Syslog.logMessage("Auto Update : Update document downloaded ");
    
        /* Analyze the resulting DOM and determine the set of updates to install */
        try 
        {
            var updates = this.updates;
      
            Syslog.logMessage("Auto Update : " + updates.length + " Update(s) Available");
      
            /* Notify the Update Service of the updates */
            this.callback.onCheckComplete(event.target, updates, updates.length);
        }
        catch(exception) 
        {
            Syslog.logMessage("Auto Update : There was an error with the update request");
            
            var update = new ceUpdateInfo(null);
            update.statusText = UpdateServiceUtils.getStatusTextFromCode(404, 404);

            this.callback.onError(event.target, update);
        }
        
        this.request = null;
        
        return;
    },


    /* There was an error of some kind during the XMLHttpRequest
     * @param   event
     *          The nsIDOMLSEvent for the load
     */
    onError : function(event)
    {
        Syslog.logMessage("Auto Update : Error during load of Update Request");
    
        var request = event.target;
        var status  = null;
        
        try 
        {
            status = request.status;
        }
        catch(exception) 
        {
            var requestObject = request.channel.QueryInterface(Components.interfaces.nsIRequest);
            status = requestObject.status;
        }
        
        /* If we can't find an error string specific to this status code, 
         * just use the 200 message from above, which means everything 
         * "looks" fine but there was probably an XML error or a bogus file.
         */
        var update = new ceUpdateInfo(null);
        update.statusText = UpdateServiceUtils.getStatusTextFromCode(status, 200);

        this.callback.onError(request, update);

        this.request = null;
        
        return;
    },

    
    QueryInterface : function(iid)
    {
        if(!iid.equals(Components.interfaces.nsIUpdateChecker) &&
           !iid.equals(Components.interfaces.nsISupports)       )
        {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        
        return this;
    },


    get enabled()
    {
        var updateService = do_GetService("@civilengines.com/updates/update-service;1",
                                          Components.interfaces.nsIUpdateService      );

        return LocalPreferences.getBoolPref(PREF_APP_UPDATE_ENABLED, true) && 
               updateService.canUpdate                                     && 
               this.isEnabled;
    },


    get updateURL()
    {
        var defaultPrefBranch = LocalPreferences.getDefaultBranch(null);   

        /* Use the override URL if specified */
        var updateURL = LocalPreferences.getCharPref(PREF_APP_UPDATE_URL_OVERRIDE, null);
        
        /* Otherwise, construct the update URL from component parts. */
        if(!updateURL) 
        {
            try 
            {
                updateURL = defaultPrefBranch.getCharPref(PREF_APP_UPDATE_URL);
            } 
            catch(exception) 
            {
                Syslog.logException(exception);
            }
        }

        if(!updateURL || updateURL == "") 
        {
            Syslog.logMessage("Auto Update : Update URL Not Defined");
            return null;
        }


        var updateChannel = UpdateServiceUtils.getUpdateChannel();
        var buildTarget   = ApplicationInfo.osTarget + "_" + ComponentABIString;
        
        updateURL = updateURL.replace(/%PRODUCT%/g     , ApplicationInfo.name);
        updateURL = updateURL.replace(/%VERSION%/g     , ApplicationInfo.version);
        updateURL = updateURL.replace(/%BUILD_ID%/g    , ApplicationInfo.buildID);
        updateURL = updateURL.replace(/%BUILD_TARGET%/g, buildTarget);
        updateURL = updateURL.replace(/%LOCALE%/g      , UpdateServiceUtils.getUserAgentLocale());
        updateURL = updateURL.replace(/%CHANNEL%/g     , updateChannel);
        updateURL = updateURL.replace(/\+/g            , "%2B");

        return updateURL;        
    },

    
    checkForUpdates: function(listener, force)
    {
        if(!listener)
        {
            throw Components.results.NS_ERROR_NULL_POINTER;
        }
        
        if(!this.updateURL || (!this.enabled && !force))
        {
            return;
        }

        Syslog.logMessage("Auto Update : Checking for Updates...");
        
        this.request = do_CreateInstance("@mozilla.org/xmlextras/xmlhttprequest;1",
                                         Components.interfaces.nsIXMLHttpRequest  );
        
        this.request.open("GET", this.updateURL, true);
        this.request.channel.notificationCallbacks = new ceUpdateChannelEventSink();

        this.request.overrideMimeType("text/xml");
        this.request.setRequestHeader("Cache-Control", "no-cache");
        
        var updateChecker = this;

        this.request.onerror     = function(event) 
        {
            updateChecker.onError(event);    
        };

        this.request.onload      = function(event) 
        { 
            updateChecker.onLoad(event);     
        };

        this.request.onprogress  = function(event) 
        { 
            updateChecker.onProgress(event); 
        };
        
        Syslog.logMessage("Auto Update : Sending request to " + this.updateURL);

        /* Fire off the XMLHttpRequst request */
        this.request.send(null);
        
        this.callback = listener;
        
        return;
    },


    stopChecking : function(duration) 
    {
        Syslog.logMessage("Auto Update : Stopping Updates Checks...");

        /* Always stop the current check */
        if(this.request)
        {
            this.request.abort();
        }
        
        switch(duration) 
        {
          case Components.interfaces.nsIUpdateChecker.CURRENT_SESSION:
          {
              this.isEnabled = false;
              break;
          }          
          case Components.interfaces.nsIUpdateChecker.ANY_CHECKS:
          {
              this.isEnabled = false;
              LocalPreferences.setBoolPref(PREF_APP_UPDATE_ENABLED, this.isEnabled);
              break;
          }
        }

        return;
    }
};



/////////////////////////////////////////////////////////////////////////////////
//
// ceUpdateDownloader Implementation
//
/////////////////////////////////////////////////////////////////////////////////

function ceUpdateDownloader(background)
{
    ScriptLoader.loadGlobalToolkitScripts();

    this.background = background;
}


ceUpdateDownloader.prototype = 
{
    background        : null,
    patch             : null,
    update            : null,
    request           : null,
    isCompleteUpdate  : false,
    downloadListeners : [],


    get isBusy()
    {
        return this.request != null;
    },


    get patchIsStaged() 
    {
        var updatesDirectory = UpdateServiceUtils.getUpdatesDirectory();
        
        return UpdateServiceUtils.readStatusFile(updatesDirectory) == UPDATE_SERVICE_STATE_PENDING;
    },


    /* Adds a listener to the download process
     * @param   listener
     *          A download listener, implementing nsIRequestObserver and
     *          nsIProgressEventSink
     */
    addDownloadListener : function(listener)
    {
        for (var i = 0; i < this.downloadListeners.length; ++i) 
        {
            if(this.downloadListeners[i] == listener)
            {
                return;
            }            
        }

        this.downloadListeners.push(listener);

        return;
    },
  

    /* Removes a download listener
     * @param   listener
     *          The listener to remove.
     */
    removeDownloadListener: function(listener) 
    {
        for(var i = 0; i < this.downloadListeners.length; ++i)
        {
            if(this.downloadListeners[i] == listener)
            {
                this.downloadListeners.splice(i, 1);
                return;
            }
        }
        
        return;
    },

    /* Download and stage the given update.
     * @param   update
     *          A nsIUpdate object to download a patch for. Cannot be null.
     */
    downloadUpdate : function(update)
    {
        if(!update)
        {
            throw Components.results.NS_ERROR_NULL_POINTER;
        }
        
        Syslog.logMessage("Auto Update : Initiating Update Download");
        
        var updatesDirectory = UpdateServiceUtils.getUpdatesDirectory();

        this.update = update;

        /* This function may return null, which indicates that there 
         * are no patches to download.
         */
        this.patch = this.selectPatch(update, updatesDirectory);
        if(!this.patch) 
        {
            Syslog.logMessage("Auto Update : No Updates to Download");
            return UpdateServiceUtils.readStatusFile(updatesDirectory);
        }

        this.isCompleteUpdate = (this.patch.type == "complete");

        var patchFile = updatesDirectory.clone();
        patchFile.append(DIRECTORY_SERVICE_UPDATE_ARCHIVE_FILE);

        var updateURI = IOService.newURI(this.patch.URL, null, null);

        this.request  = do_CreateInstance("@mozilla.org/network/incremental-download;1",
                                          Components.interfaces.nsIIncrementalDownload );
        
        Syslog.logMessage("Auto Update : Downloading Update from " + 
                          updateURI.spec                           + 
                          " to "                                   +
                          patchFile.path                           );

        var downloadInterval = this.background ? 
            UPDATE_SERVICE_DOWNLOAD_BACKGROUND_INTERVAL :
            UPDATE_SERVICE_DOWNLOAD_FOREGROUND_INTERVAL;

        this.request.init(updateURI                         , 
                          patchFile                         , 
                          UPDATE_SERVICE_DOWNLOAD_CHUNK_SIZE,
                          downloadInterval                  );

        this.request.start(this, null);

        UpdateServiceUtils.writeStatusFile(updatesDirectory, UPDATE_SERVICE_STATE_DOWNLOADING);

        this.patch.QueryInterface(Components.interfaces.nsIWritablePropertyBag);
        this.patch.state  = UPDATE_SERVICE_STATE_DOWNLOADING;

        var updateManager = do_GetService("@civilengines.com/updates/update-manager;1",
                                          Components.interfaces.nsIUpdateManager      );

        updateManager.saveUpdates();

        return UPDATE_SERVICE_STATE_DOWNLOADING;
    },
    

    verifyDownload : function()
    {
        if(!this.request)
        {
            return false;
        }
        
        var destination = this.request.destination;

        /* Ensure that the file size matches the expected file size */
        if(destination.fileSize != this.patch.size)
        {
            Syslog.logMessage("Auto Update : Verification Failure; File size mismatch");
            return false;
        }
        
        var fileStream = do_CreateInstance("@mozilla.org/network/file-input-stream;1",
                                           Components.interfaces.nsIFileInputStream  );
        
        fileStream.init(destination, IO_SERVICE_MODE_RDONLY, IO_SERVICE_FILE_PERMISSION, 0);

        try 
        {
            var hash = do_CreateInstance("@mozilla.org/security/hash;1"     ,
                                         Components.interfaces.nsICryptoHash);

            var hashFunction = Components.interfaces.nsICryptoHash[this.patch.hashFunction.toUpperCase()];
            if(hashFunction == undefined)
            {
                throw Components.results.NS_ERROR_UNEXPECTED;
            }
            
            hash.init(hashFunction);
            hash.updateFromStream(fileStream, -1);

            /* NOTE: For now, we assume that the format of patch.hashValue is hex
             * encoded binary (such as what is typically output by programs like
             * sha1sum).  In the future, this may change to base64 depending on how
             * we choose to compute these hashes.
             */
            digest = UpdateServiceUtils.binaryToHex(hash.finish(false));
        } 
        catch(exception) 
        {
            Syslog.logMessage("Auto Update : Failed to compute hash of update archive");
            digest = "";
        }

        fileStream.close();

        return digest == this.patch.hashValue.toLowerCase();
    },


    /* Return the first UpdatePatch with the given type.
     * @param   type
     *          The type of the patch ("complete" or "partial")
     * @returns A nsIUpdatePatch object matching the type specified
     */
    getPatchOfType : function(update, type) 
    {
        for(var i = 0; i < update.patchCount; ++i) 
        {
            var patch = update.getPatchAt(i);
            if(patch && patch.type == type)
            {
                return patch;
            }
            
        }
        
        return null;
    },

    /* Select the patch to use given the current state of updateDirectory and 
     * the given set of update patches. Given an update to download, we will 
     * always try to download the patch for a partial update over the patch for
     * a full update.
     *
     * @param   update
     *          A nsIUpdate object to select a patch from
     * @param   updateDir
     *          A nsIFile representing the update directory
     * @returns A nsIUpdatePatch object to download
     */
    selectPatch: function(update, updateDirectory) 
    {
        /* Look to see if any of the patches in the Update object has been
         * pre-selected for download, otherwise we must figure out which one
         * to select ourselves. 
         */
        var selectedPatch = update.selectedPatch;        
        var state         = UpdateServiceUtils.readStatusFile(updateDirectory);
        
        /* If this is a patch that we know about, then select it.  If it is a patch
         * that we do not know about, then remove it and use our default logic.
         */

        var useComplete = false;
        if(selectedPatch) 
        {
            Syslog.logMessage("Auto Update : Found existing patch [state="+ state + "]");

            switch(state) 
            {
              case UPDATE_SERVICE_STATE_DOWNLOADING: 
              {
                  Syslog.logMessage("Auto Update : Resuming Download");
                  return selectedPatch;
              }
              case UPDATE_SERVICE_STATE_PENDING:
              {
                  Syslog.logMessage("Auto Update : Update Already Downloaded and Staged");
                  return null;
              }
              default:
              {
                  /* Something went wrong when we tried to apply the previous patch.
                   * Try the complete patch next time.
                   */
                  if(update && selectedPatch.type == "partial") 
                  {
                      useComplete = true;
                  } 
                  else 
                  {
                      Syslog.logMessage("Auto Update : Failed to Apply Complete Patch");
                      UpdateServiceUtils.writeStatusFile(updateDirectory, UPDATE_SERVICE_STATE_NONE);
                      return null;
                  }
              }
              
            }

            selectedPatch = null;
        }

        /* If we were not able to discover an update from a previous download, we 
         * select the best patch from the given set.
         */
        var partialPatch = this.getPatchOfType(update, "partial");
        if(!useComplete)
        {        
            selectedPatch = partialPatch;
        }
        
        if(!selectedPatch) 
        {
            if(partialPatch)
            {
                partialPatch.selected = false;
            }

            selectedPatch = this.getPatchOfType(update, "complete");
        }
        
        updateDirectory = UpdateServiceUtils.getUpdatesDirectory();
        
        selectedPatch.selected = true;

        update.isCompleteUpdate = useComplete;
        
        /* Reset the Active Update object on the Update Manager and flush the
         * Active Update DB.
         */ 
        var updateManager = do_GetService("@civilengines.com/updates/update-manager;1",
                                          Components.interfaces.nsIUpdateManager      );
        
        updateManager.activeUpdate = update;
        
        return selectedPatch;
    },
    

    QueryInterface : function(iid) 
    {
        if(!iid.equals(Components.interfaces.nsIRequestObserver)    &&
           !iid.equals(Components.interfaces.nsIProgressEventSink)  &&
           !iid.equals(Components.interfaces.nsIInterfaceRequestor) &&
           !iid.equals(Components.interfaces.nsISupports)            )
        {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        
        return this;
    },


    getInterface : function(iid) 
    {
        /* The network request may require proxy authentication, so provide the
         * default nsIAuthPrompt if requested.
         */
        if(iid.equals(Components.interfaces.nsIAuthPrompt)) 
        {
            var authPrompt = do_CreateInstance("@mozilla.org/network/default-auth-prompt;1",
                                               Components.interfaces.nsIAuthPrompt         );
            
            return authPrompt.QueryInterface(iid);
        }

        Components.returnCode = Components.results.NS_ERROR_NO_INTERFACE;
        return null;
    },


    cancel : function() 
    {
        if (this.request && 
            this.request instanceof Components.interfaces.nsIRequest) 
        {
            const NS_BINDING_ABORTED = 0x804b0002;
            this.request.cancel(NS_BINDING_ABORTED);
        }

        return;
    },


    /* When the async request begins
     * @param   request
     *          The nsIRequest object for the transfer
     * @param   context
     *          Additional data
     */
    onStartRequest : function(request, context) 
    {
        request.QueryInterface(Components.interfaces.nsIIncrementalDownload);

        Syslog.logMessage("Auto Update : Download Request Started for " + request.URI.spec);
    
        var listenerCount = this.downloadListeners.length;
        
        for(var i = 0; i < listenerCount; ++i)
        {
            this.downloadListeners[i].onStartRequest(request, context);
        }

        return;
    },
    

    /* When data transfer ceases
     * @param   request
     *          The nsIRequest object for the transfer
     * @param   context
     *          Additional data
     * @param   status
     *          Status code containing the reason for the cessation.
     */
    onStopRequest: function(request, context, status) 
    {
        const NS_BINDING_ABORTED = 0x804b0002;
        const NS_ERROR_ABORT     = 0x80004004;
        const NS_BINDING_FAILED  = 2152398849;

        request.QueryInterface(Components.interfaces.nsIIncrementalDownload);

        Syslog.logMessage("Auto Update : Download Request Stopped for " + 
                          request.URI.spec + ", STATUS : " + status     );

        var state              = this.patch.state;
        var shouldShowPrompt   = false;
        var deleteActiveUpdate = false;
        
        if(Components.isSuccessCode(status))
        {
            if(this.verifyDownload())
            {
                state = UPDATE_SERVICE_STATE_PENDING;
        
                /* We only need to explicitly show the prompt if this is a backround
                 * download, since otherwise some kind of UI is already visible and 
                 * that UI will notify.
                 */ 
                if(this.background)
                {
                    shouldShowPrompt = true;
                }
                
                /* Indicate to the updater.exe we're ready to APPLY the UPDATE */
                UpdateServiceUtils.writeStatusFile(UpdateServiceUtils.getUpdatesDirectory(), state);

                this.update.installDate = (new Date()).getTime();
                this.update.statusText = "Install Pending";
            }
            else 
            {
                Syslog.logMessage("Auto Update : Download Verification Failed");
                state = UPDATE_SERVICE_STATE_DOWNLOAD_FAILED;
        
                var stringBundleService = do_GetService("@mozilla.org/intl/stringbundle;1"          ,
                                                        Components.interfaces.nsIStringBundleService);

                var brandBundle    = stringBundleService.createBundle(NETIZEN_BRAND_PROPERTIES_URI);
                var updateBundle   = stringBundleService.createBundle(NETIZEN_UPDATES_PROPERTIES_URI);
                var brandShortName = brandBundle.GetStringFromName("brandShortName");
                
                this.update.statusText = updateBundle.formatStringFromName("updates.verificationError", 
                                                                           [brandShortName]           , 
                                                                           1                          );
                
                status = Components.results.NS_ERROR_UNEXPECTED;
                
                var message = UpdateServiceUtils.getStatusTextFromCode("verification_failed", 
                                                                       "verification_failed");
                this.update.statusText = message;
                
                if(this.update.isCompleteUpdate)
                {
                    deleteActiveUpdate = true;
                }
                
                /* Destroy the updates directory, since we're done with it. */
                UpdateServiceUtils.cleanupUpdatesDirectory();
            }      
        }
        else if (status != NS_BINDING_ABORTED && status != NS_ERROR_ABORT)
        {
            Syslog.logMessage("Auto Update : Non-Verification Failure");

            state = UPDATE_SERVICE_STATE_DOWNLOAD_FAILED;
      
            this.update.statusText = UpdateServiceUtils.getStatusTextFromCode(status           , 
                                                                              NS_BINDING_FAILED);
      
            /* Destroy the updates directory, since we're done with it. */
            UpdateServiceUtils.cleanupUpdatesDirectory();
            
            deleteActiveUpdate = true;
        }
        
        Syslog.logMessage("Auto Update : Setting State to => " + state);

        this.patch.state = state;

        var updateManager = do_GetService("@civilengines.com/updates/update-manager;1",
                                          Components.interfaces.nsIUpdateManager      );

        if(deleteActiveUpdate)
        {
            this.update.installDate = (new Date()).getTime();
            updateManager.activeUpdate = null;
        }
        
        updateManager.saveUpdates();

        var listenerCount = this.downloadListeners.length;
        
        for(var i = 0; i < listenerCount; ++i)
        {
            this.downloadListeners[i].onStopRequest(request, context, status);
        }

        /* Cleanup request object */
        this.request = null;
        
        if(state == UPDATE_SERVICE_STATE_DOWNLOAD_FAILED)
        {
            if(!this.update.isCompleteUpdate) 
            {
                /* If we were downloading a patch and the patch verification phase 
                 * failed, log this and then commence downloading the complete update.
                 */
                Syslog.logMessage("Auto Update : Verification of patch failed, downloading full update");

                this.update.isCompleteUpdate = true;
                
                var status = this.downloadUpdate(this.update);
                if(status == UPDATE_SERVICE_STATE_NONE)
                {
                    /* This will reset the "state" property on this.update
                     *  if a new download initiates.
                     */
                    UpdateServiceUtils.cleanupActiveUpdate();
                }
            }
            else
            {
                /* If this was ever a foreground download, and now there is no UI active
                 * (e.g. because the user closed the download window) and there was an
                 * error, we must notify now. Otherwise we can keep the failure to 
                 * ourselves since the user won't be expecting it. 
                 */
                var foregroundDownload = false;
                
                try 
                {
                    this.update.QueryInterface(Components.interfaces.nsIPropertyBag);
                    foregroundDownload = this.update.getProperty("foregroundDownload");
                }
                catch(exception) 
                {
                    Syslog.logException(exception);
                }
                
                if(foregroundDownload == "true")
                {
                    var prompter = do_CreateInstance("@civilengines.com/updates/update-prompt;1",
                                                     Components.interfaces.nsIUpdatePrompt      );

                    this.update.QueryInterface(Components.interfaces.nsIWritablePropertyBag);
                    this.update.setProperty("downloadFailed", "true");

                    prompter.showUpdateError(this.update);
                }
            }
            
        }
        

        /* Do this after *everything* else, since it will likely
         * cause the app to shut down. 
         */
        if(shouldShowPrompt)
        {
            /* Notify the user that an update has been downloaded and is ready for 
             * installation (i.e. that they should restart the application). We do
             * not notify on failed update attempts.
             */
            var prompter = do_CreateInstance("@civilengines.com/updates/update-prompt;1",
                                             Components.interfaces.nsIUpdatePrompt      );
            
            prompter.showUpdateDownloaded(this.update);
        }
        
        return;
    },
    

    /* When new data has been downloaded
     * @param   request
     *          The nsIRequest object for the transfer
     * @param   context
     *          Additional data
     * @param   progress
     *          The current number of bytes transferred
     * @param   maxProgress
     *          The total number of bytes that must be transferred
     */
    onProgress : function(request, context, progress, maxProgress) 
    {
        request.QueryInterface(Components.interfaces.nsIIncrementalDownload);

        Syslog.logMessage("Auto Update : " + request.URI.spec + ", " + progress + "/" + maxProgress);
    
        var listenerCount = this.downloadListeners.length;
        
        for(var i = 0; i < listenerCount; ++i)
        {
            var listener = this.downloadListeners[i];

            if(listener instanceof Components.interfaces.nsIProgressEventSink)
            {
                listener.onProgress(request, context, progress, maxProgress);
            }
        }

        return;
    },


    /* When we have new status text
     * @param   request
     *          The nsIRequest object for the transfer
     * @param   context
     *          Additional data
     * @param   status
     *          A status code
     * @param   statusText
     *          Human readable version of |status|
     */
    onStatus : function(request, context, status, statusText) 
    {
        request.QueryInterface(Components.interfaces.nsIIncrementalDownload);

        Syslog.logMessage("Auto Update : " + request.URI.spec + ", STATUS : " + statusText);
    
        var listenerCount = this.downloadListeners.length;
        
        for(var i = 0; i < listenerCount; ++i)
        {
            var listener = this.downloadListeners[i];

            if(listener instanceof Components.interfaces.nsIProgressEventSink)
            {
                listener.onStatus(request, context, status, statusText);
            }
        }

        return;
    }    
};



/////////////////////////////////////////////////////////////////////////////////
//
// nsIUpdateTimerManager Implementation
//
/////////////////////////////////////////////////////////////////////////////////

function ceUpdateTimerManager()
{
    ScriptLoader.loadGlobalToolkitScripts();

    ObserverService.addObserver(this, "xpcom-shutdown", false);
    
    var timerInterval = LocalPreferences.getIntPref(PREF_APP_UPDATE_TIMER, 600000);
    
    this.timer = do_CreateInstance("@mozilla.org/timer;1", Components.interfaces.nsITimer);
    this.timer.initWithCallback(this, timerInterval, Components.interfaces.nsITimer.TYPE_REPEATING_SLACK);

    Syslog.logMessage("Auto Update : UpdateTimer Manager (period = " + timerInterval + " msecs)");
    
    return;
}


ceUpdateTimerManager.prototype =
{
    timer  : null,
    timers : { },
    
    QueryInterface : function(iid)
    {
        if(!iid.equals(Components.interfaces.nsIUpdateTimerManager) &&
           !iid.equals(Components.interfaces.nsITimerCallback)      &&
           !iid.equals(Components.interfaces.nsIObserver)           &&
           !iid.equals(Components.interfaces.nsISupports)            )
        {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }

        return this;
    },


    observe: function(subject, topic, data) 
    {
        if(topic == "xpcom-shutdown") 
        {
            ObserverService.removeObserver(this, "xpcom-shutdown");
            
            this.timer  = null;
            this.timers = null;
        }

        return;
    },


    notify : function(timer) 
    {
        Syslog.logMessage("Auto Update : Checking Update Timers");

        for(var timerId in this.timers)
        {
            var timerData      = this.timers[timerId];
            var lastUpdateTime = timerData.lastUpdateTime;
            var now            = Math.round(Date.now() / 1000);
            
            /* Fudge the lastUpdateTime by some random increment of the update 
             * check interval (e.g. some random slice of 10 minutes) so that when
             * the time comes to check, we offset each client request by a random
             * amount so they don't all hit at once. app.update.timer is in milliseconds,
             * whereas app.update.lastUpdateTime is in seconds
             */
            var timerInterval  = LocalPreferences.getIntPref(PREF_APP_UPDATE_TIMER, 600000);
            lastUpdateTime    += Math.round(Math.random() * timerInterval / 1000);
            
            if((now - lastUpdateTime) > timerData.interval                          &&
               timerData.callback instanceof Components.interfaces.nsITimerCallback  ) 
            {
                Syslog.logMessage("Auto Update : Notifying Update Timer Callback");

                timerData.callback.notify(timer);
                timerData.lastUpdateTime = now;

                var preference = PREF_APP_UPDATE_LASTUPDATETIME_FMT.replace(/%ID%/, timerId);
                LocalPreferences.setIntPref(preference, now);
            }
        }
        
        return;
    },
    

    registerTimer: function(timerId, callback, intervalInSeconds)
    {
        Syslog.logMessage("Auto Update : Registering Update Check Timer");

        var preference     = PREF_APP_UPDATE_LASTUPDATETIME_FMT.replace(/%ID%/, timerId);
        var now            = Math.round(Date.now() / 1000);
        var lastUpdateTime = null;

        if(LocalPreferences.prefHasUserValue(preference))
        {
            lastUpdateTime = LocalPreferences.getIntPref(preference);
        } 
        else 
        {
            LocalPreferences.setIntPref(preference, now);
            lastUpdateTime = now;
        }

        this.timers[timerId] = 
        {
            callback       : callback, 
            interval       : intervalInSeconds,
            lastUpdateTime : lastUpdateTime 
        }; 
        
        return;
    }    
};




/////////////////////////////////////////////////////////////////////////////////
//
// nsIUpdatePrompt Implementation
//
/////////////////////////////////////////////////////////////////////////////////

function ceUpdatePrompt()
{
    ScriptLoader.loadGlobalToolkitScripts();
}


ceUpdatePrompt.prototype = 
{
    get enabled() 
    {
        return !LocalPreferences.getBoolPref(PREF_APP_UPDATE_SILENT, false);
    },

    /* Show the Update Checking UI
     * @param   parent
     *          A parent window, can be null
     * @param   uri
     *          The URI string of the dialog to show
     * @param   name
     *          The Window Name of the dialog to show, in case it is already open
     *          and can merely be focused
     * @param   page
     *          The page of the wizard to be displayed, if one is already open.
     * @param   update
     *          An update to pass to the UI in the window arguments. 
     *          Can be null
     */
    showUpdateUI : function(parent, uri, features, name, page, update) 
    {
        var supportsArray = null;

        if(update) 
        {
            supportsArray = do_CreateInstance("@mozilla.org/supports-array;1"       ,
                                              Components.interfaces.nsISupportsArray);

            supportsArray.AppendElement(update);
        }
        
        var windowMediator = do_GetService("@mozilla.org/appshell/window-mediator;1",
                                           Components.interfaces.nsIWindowMediator  );

        var recentWindow   = windowMediator.getMostRecentWindow(name);
        if(recentWindow) 
        {
            if(page && "setCurrentPage" in win)
            {
                recentWindow.setCurrentPage(page);
            }
            
            recentWindow.focus();
        }
        else 
        {
            var openFeatures = "chrome,centerscreen,dialog=no,resizable=no,titlebar,toolbar=no";

            if(features)
            {
                openFeatures += "," + features;
            }

            var windowWatcher = do_GetService("@mozilla.org/embedcomp/window-watcher;1",
                                              Components.interfaces.nsIWindowWatcher   );

            windowWatcher.openWindow(parent, uri, "", openFeatures, supportsArray);
        }
    },

    QueryInterface : function(iid)
    {
        if(!iid.equals(Components.interfaces.nsIUpdatePrompt) &&
           !iid.equals(Components.interfaces.nsISupports)      )
        {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }

        return this;
    },

    checkForUpdates : function() 
    {
        this.showUpdateUI(null                            , 
                          NETIZEN_UPDATE_PROMPT_DIALOG_URI,
                          null                            , 
                          "urn:civil:windowtype:updates"  , 
                          null                            ,
                          null                            );
        return;
    },


    showUpdateAvailable : function(update) 
    {
        if(this.enabled) 
        {
            this.showUpdateUI(null                            , 
                              NETIZEN_UPDATE_PROMPT_DIALOG_URI,
                              null                            ,
                              "urn:civil:windowtype:updates"  , 
                              "updatesAvailablePage"          ,
                              update                          );
        }

        return;
    },

    showUpdateDownloaded : function(update) 
    {
        if(this.enabled) 
        {
            this.showUpdateUI(null                            , 
                              NETIZEN_UPDATE_PROMPT_DIALOG_URI,
                              null                            ,
                              "urn:civil:windowtype:updates"  , 
                              "updatesDownloadedPage"         ,
                              update                          );
        }
        
        return;
    },


    showUpdateInstalled : function(update) 
    {
        var showUpdateInstalledUI = LocalPreferences.getBoolPref(PREF_APP_UPDATE_SHOW_INSTALLED_UI,
                                                                 true                             );

        if(this.enabled && showUpdateInstalledUI) 
        {
            this.showUpdateUI(null                            , 
                              NETIZEN_UPDATE_PROMPT_DIALOG_URI,
                              null                            ,
                              "urn:civil:windowtype:updates"  , 
                              "updatesInstalledPage"          ,
                              update                          );
        }
        
        return;
    },


    showUpdateError : function(update) 
    {
        if(this.enabled) 
        {
            if(update.state == UPDATE_SERVICE_STATE_FAILED && 
               update.errorCode == IO_SERVICE_WRITE_ERROR   )
            {
                var stringBundleService = do_GetService("@mozilla.org/intl/stringbundle;1"          ,
                                                        Components.interfaces.nsIStringBundleService);
                
                var updateBundle  = stringBundleService.createBundle(NETIZEN_UPDATES_PROPERTIES_URI);
                var title         = updateBundle.GetStringFromName("updates.updaterIOErrorTitle");
                var text          = updateBundle.formatStringFromName("updates.updaterIOErrorText",
                                                                      [ApplicationInfo.name]      ,
                                                                      1                           );


                var windowWatcher = do_GetService("@mozilla.org/embedcomp/window-watcher;1",
                                                  Components.interfaces.nsIWindowWatcher   );

                windowWatcher.getNewPrompter(null).alert(title, text);
            } 
            else 
            {
                this.showUpdateUI(null                            , 
                                  NETIZEN_UPDATE_PROMPT_DIALOG_URI,
                                  null                            ,
                                  "urn:civil:windowtype:updates"  , 
                                  "updatesErrorPage"              ,
                                  update                          );
            }
        }

        return;
    },

    showUpdateHistory: function(parent) 
    {
        Syslog.logMessage("Auto Update : Error => Update History Disabled");
        return;
    },

};



/////////////////////////////////////////////////////////////////////////////////
//
// nsIFactory Implementation
//
/////////////////////////////////////////////////////////////////////////////////

var ceUpdateServiceModule = 
{
    moduleObjects :
    {
        ceUpdateService : 
        {
            CID        : Components.ID("{71debeb6-8d75-4881-88f7-f270806c190e}"),
            contractID : "@civilengines.com/updates/update-service;1",
            className  : "ceUpdateService",
            factory    : 
            {
                createInstance : function(outer, iid) 
                {
                    if(outer != null)
                    {
                        throw Components.results.NS_ERROR_NO_AGGREGATION;
                    }
                    
                    var updateService = new ceUpdateService();
                    return updateService.QueryInterface(iid);
                }
            }        
        },

        ceUpdateChecker :
        {
            CID        : Components.ID("{971e2ea0-5ed2-4d11-9f85-c2d861b2b447}"),
            contractID : "@civilengines.com/updates/update-checker;1",
            className  : "ceUpdateChecker",
            factory    : 
            {
                createInstance : function(outer, iid) 
                {
                    if(outer != null)
                    {
                        throw Components.results.NS_ERROR_NO_AGGREGATION;
                    }
                    
                    var updateChecker = new ceUpdateChecker();
                    return updateChecker.QueryInterface(iid);
                }       
            }
        },
        
        ceUpdatePrompt :
        {
            CID        : Components.ID("{e068f520-a44e-4589-a848-dce3a49ad333}"),
            contractID : "@civilengines.com/updates/update-prompt;1",
            className  : "ceUpdatePrompt",
            factory    : 
            {
                createInstance : function(outer, iid) 
                {
                    if(outer != null)
                    {
                        throw Components.results.NS_ERROR_NO_AGGREGATION;
                    }
                    
                    var updatePrompt = new ceUpdatePrompt();
                    return updatePrompt.QueryInterface(iid);
                }       
            }
        },

        ceUpdateTimers :
        {
            CID        : Components.ID("{ca8d5009-11ba-4baf-a155-c1f70ffa8145}"),
            contractID : "@civilengines.com/updates/timer-manager;1",
            className  : "ceUpdateTimerManager",
            factory    : 
            {
                createInstance : function(outer, iid) 
                {
                    if(outer != null)
                    {
                        throw Components.results.NS_ERROR_NO_AGGREGATION;
                    }
                    
                    var updateTimerManager = new ceUpdateTimerManager();
                    return updateTimerManager.QueryInterface(iid);
                }       
            }
        },

        ceUpdateManager :
        {
            CID        : Components.ID("{8b787fb6-ff26-46c5-929d-8490469a9218}"),
            contractID : "@civilengines.com/updates/update-manager;1",
            className  : "ceUpdateManager",
            factory    : 
            {
                createInstance : function(outer, iid) 
                {
                    if(outer != null)
                    {
                        throw Components.results.NS_ERROR_NO_AGGREGATION;
                    }
                    
                    var updateManager = new ceUpdateManager();
                    return updateManager.QueryInterface(iid);
                }       
            }
        }
    },

    
    registerSelf : function(componentManager, fileSpec, location, type) 
    {
        componentManager = componentManager.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    
        for(var key in this.moduleObjects) 
        {
            var moduleObject = this.moduleObjects[key];

            componentManager.registerFactoryLocation(moduleObject.CID       , 
                                                     moduleObject.className , 
                                                     moduleObject.contractID,
                                                     fileSpec               , 
                                                     location               ,
                                                     type                   );
        }
        
        var categoryManager = Components.classes["@mozilla.org/categorymanager;1"]
            .getService(Components.interfaces.nsICategoryManager);
        
        categoryManager.addCategoryEntry("app-startup"                                , 
                                         this.moduleObjects.ceUpdateService.className ,
                                         "service," + 
                                         this.moduleObjects.ceUpdateService.contractID, 
                                         true                                         , 
                                         true                                         , 
                                         null                                         );
        
        return;
    },

    unregisterSelf : function(componentManager, fileSpec, location, type) 
    {
        componentManager = componentManager.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    
        for(var key in this.moduleObjects) 
        {
            var moduleObject = this.moduleObjects[key];

            componentManager.unregisterFactoryLocation(moduleObject.CID       , 
                                                       moduleObject.className , 
                                                       moduleObject.contractID,
                                                       fileSpec               , 
                                                       location               ,
                                                       type                   );
        }
    },
    
    getClassObject : function(componentManager, cid, iid)
    {
        if(!iid.equals(Components.interfaces.nsIFactory))
        {
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        }
        
        for(var key in this.moduleObjects)
        {
            if(cid.equals(this.moduleObjects[key].CID))
            {
                return this.moduleObjects[key].factory;
            }
        }
            
        throw Components.results.NS_ERROR_NO_INTERFACE;
    },
  

    canUnload : function(componentManager)
    {
        return true;
    }
};



function NSGetModule(componentManager, fileSpec)
{
    return ceUpdateServiceModule;
}
