/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is the Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2003 Civil Engines Research.  All Rights 
 * Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/

var NS_OBSERVER_SERVICE_CONTRACTID         = "@mozilla.org/observer-service;1";
var NS_IOSERVICE_CONTRACTID                = "@mozilla.org/network/io-service;1";
var NS_RDFSERVICE_CONTRACTID               = "@mozilla.org/rdf/rdf-service;1";
var NS_RDFCONTAINERUTILS_CONTRACTID        = "@mozilla.org/rdf/container-utils;1";
var NS_DIRECTORYSERVICE_CONTRACTID         = "@mozilla.org/file/directory_service;1";

var Interfaces    = Components.interfaces;
var Classes       = Components.classes;
var Results       = Components.results;
var XPCOMWrapper  =
{
    createInstance : function(contractID, componentInterface)
    {
        var objectClass    = Components.classes[contractID].createInstance();
        var objectInstance = null;

        switch(typeof(componentInterface))
        {
          case "object":
          {   
              objectInstance = objectClass.QueryInterface(componentInterface);
              break;
          }
          case "string":
          {
              objectInstance = objectClass.QueryInterface(Components.interfaces[componentInterface]);
              break;
          }
          default :
          {
              objectInstance = null;
              break;
          }
        }

        return objectInstance;
    },
      

    getService  : function(contractID, componentInterface)
    {
        var serviceClass    = Components.classes[contractID];
        var serviceInstance = null;
        
        switch(typeof(componentInterface))
        {
          case "object":
          {   
              serviceInstance = serviceClass.getService(componentInterface);
              break;
          }
          case "string":
          {
              serviceInstance = serviceClass.getService(Components.interfaces[componentInterface]);
              break;
          }
          default:
          {
              serviceInstance = null;
              break;
          }
        }

        return serviceInstance;
    }, 

    queryInterface : function(componentInstance, componentInterface)
    {
        switch(typeof(componentInterface))
        {
          case "string" :
          {
              var interfaceObject = Components.interfaces[componentInterface];
              return componentInstance.QueryInterface(interfaceObject);
          }
          default :
          {
              return componentInstance.QueryInterface(componentInterface)
          }
        }
    },


    cloneObject : function(objectInstance)
    {
        var objectClone = new Object();
        
        for(var prop in objectInstance)
        {
            objectClone[prop] = objectInstance[prop];
        }
        
        return objectClone;
    },


    renameObjectProperty : function(objectInstance, oldName, newName)
    {
        if(oldName == newName)
        {
            return;
        }
        
        objectInstance[newName] = objectInstance[oldName];
        
        delete objectInstance[oldName];    
    }
};



function do_CreateInstance(contractID, componentInterface)
{
    var objectInstance = null;
    
    try
    {
        objectInstance = XPCOMWrapper.createInstance(contractID, componentInterface);
    }
    catch(exception)
    {
        Syslog.logException(exception);
        return null;
    }
        
    return objectInstance;
}



function do_GetService(contractID, componentInterface)
{
    var serviceInstance = null;

    try
    {
        serviceInstance = XPCOMWrapper.getService(contractID, componentInterface);        
    }
    catch(exception)
    {
        Syslog.logException(exception);
        return null;
    }
    
    return serviceInstance;
}




function do_QueryInterface(componentInstance, componentInterface)
{
    var interfaceObject = null;
    
    try
    {
        interfaceObject = XPCOMWrapper.queryInterface(componentInstance, componentInterface);
    }
    catch(exception)
    {
        Syslog.logException(exception);
        return null;
    }

    return interfaceObject;
}





function getInterfaces(classInstance)
{
    var interfaceList = new Object();

    for(var index in Components.interfaces)
    {
        try
        {
            var componentInterface = Components.interfaces[index];

            classInstance.QueryInterface(componentInterface);
            
            /* If the query interface succeeds, then add
             * it to the list of interfaces
             */
            interfaceList[index]   = componentInterface;
        }
        catch(exception)
        {
            Syslog.logException(exception);
            return null;
        }
    }

    return interfaceList; 
}




function getCIDFromContractID(contractID)
{
    var classID = null;

    if(!contractID)
    {
        throw Components.results.NS_ERROR_XPC_NOT_ENOUGH_ARGS;
        return null;
    }
    
    if(typeof(Components.classes[contractID]) == "undefined")
    {
        throw Components.results.NS_ERROR_INVALID_ARGUMENT;
        return null;
    }
    
    if(Components.classes[contractID].valid)
    {
        classID = Components.classes[contractID].number;
    }
    
    return classID;
}




function getContractIDFromCID(classID)
{
    var contractID = null;
    var result     = {};
    
    if(!classID)
    {
        throw Components.results.NS_ERROR_XPC_NOT_ENOUGH_ARGS;
        return null;
    }

    try
    {
        if(typeof(Components.ID(classID)) == "undefined")
        {
            contractID = null;
        }            
        else
        {
            if(Components.ID(classID).valid)
            {
                contractID = Components.manager.CLSIDToContractID(Components.ID(classID), 
                                                                  result                );
            }
        }
    }
    catch(exception)
    { 
        contractID = null; 
    }

    try
    {
        
        if(typeof(Components.classesByID[classID]) == "undefined")
        {
            return null;
        }
        
        if(!contractID)
        {
            for(list in Components.classes)
            {
                if(typeof(Components.classes[list]) != "undefined")
                {
                    
                    if(Components.classesByID[classID].equals(Components.classes[list]))
                    {
                        contractID = list;
                        break;
                    }
                }
            }
        }
    }
    catch(exception)
    { 
        Syslog.logException(exception);
        contractID = null; 
    }

    return contractID;
}



function getInterfaceListFromContractId(contractID)
{

    var resultSet = new Array();
    var instance  = null;
    var list      = null;
    
    if(!contractID)
    {
        throw Components.results.NS_ERROR_XPC_NOT_ENOUGH_ARGS;
        return null;
    }

    if(typeof(Components.classes[contractID]) == "undefined")
    {
        return nsnull;
    }
    
    
    instance = Components.classes[contractID].getService();
    
    for(list in Components.interfaces)
    {
        try
        {
            if(typeof(Components.interfaces[list]) != "undefined")
            {
                if(typeof(instance.QueryInterface(Components.interfaces[list])) == "object")
                {
                    resultSet.push(list);
                }
            }
        }
        catch(exception)
        {
            Syslog.logException(exception);
            return null;
        }
    }
    
    return (resultSet != "" ? resultSet : null);
}



function getInterfaceListFromCID(classID)
{
    var contractID = null;

    if(!classID)
    {
        throw Components.results.NS_ERROR_XPC_NOT_ENOUGH_ARGS;
        return null;
    }
    
    contractID = getContractIDFromCID(classID);

    return (typeof(contractID) !='undefined') ? 
            getInterfaceListFromContractID(contractID) : null;
}




function getComponentError(errorNumber)
{
    var errorResult = null;
    
    for(var list in Components.results)
    {
        if(errorNumber == Components.results[list])
        {
            errorResult = list;
            break;
        }
    }
    
    return errorResult;
}

var ObserverService    = do_GetService(NS_OBSERVER_SERVICE_CONTRACTID, Interfaces.nsIObserverService);
var IOService          = do_GetService(NS_IOSERVICE_CONTRACTID, Interfaces.nsIIOService);
var RDFService         = do_GetService(NS_RDFSERVICE_CONTRACTID, Interfaces.nsIRDFService);
var RDFContainerUtils  = do_GetService(NS_RDFCONTAINERUTILS_CONTRACTID, Interfaces.nsIRDFContainerUtils);
