/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is the Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2003 Civil Engines Research.  All Rights 
 * Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/


function ceDateTimeConverter()
{
}


ceDateTimeConverter.prototype =
{
    ISO_DATETIME_FORMAT      : '%s-%s-%s%s%s:%s:%s%s'                                         ,
	ISO_DATETIME_PATTERN     : /(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2})(\.(\d{1,3}))?/,
	ISO_MILLISECOND_PATTERN  : /\.\d{1,3}$/                                                   ,
	ISO_TIME_DELIMITER       : 'T'                                                            ,
	ISO_TIME_SPACER          : ' '                                                            ,
    ISO_DATETIME_ZERO        : '000-00-00T00:00:00'                                           ,
    UTC_DATETIME_ZERO        : 0                                                              ,

    getLeadingZero : function(value)
    {
        var leadingZero = (value < 10) ? '0' : '';
        return leadingZero + value;
    },
    

    getTrailingMilliseconds : function(milliseconds)
    {
        return (milliseconds != "00") ? "." + this.getLeadingZero(milliseconds) : "";
    },
    

    getISODateTime : function(dateTime, formatISO8601)
    {
        try
        {
            var milliseconds = dateTime.getMilliseconds();
            if(milliseconds < 100)
            {
                milliseconds = '0' + milliseconds;
                
                return sprintf(this.ISO_DATETIME_FORMAT                                      ,
                               dateTime.getFullYear()                                        ,
                               this.getLeadingZero(dateTime.getMonth() + 1)                  ,
                               this.getLeadingZero(dateTime.getDate())                       ,
                               formatISO8601 ? this.ISO_TIME_SPACER : this.ISO_TIME_DELIMITER,
                               this.getLeadingZero(dateTime.getHours())                      ,
                               this.getLeadingZero(dateTime.getMinutes())                    ,
                               this.getLeadingZero(dateTime.getSeconds())                    ,
                               this.getTrailingMilliseconds(milliseconds)                    );
            }
            
        }
        catch(exception)
        {
            return this.ISO_DATETIME_ZERO;
        }
    },


    convertISOToUTCTime : function(isoDateTime)
    {
        try
        {
            var isoSet = isoDateTime.match(this.ISO_DATETIME_PATTERN);
            return Date.UTC(isoSet[1]     ,
                            isoSet[2]     ,
                            isoSet[3]     ,
                            isoSet[4]     ,
                            isoSet[5]     ,
                            isoSet[6]     ,
                            isoSet[8] || 0);
            
        }
        catch(exception)
        {
            return this.UTC_DATE_ZERO;
        }
    },

    convertUTCToISOTime : function(utcDateTime, formatISO8601)
    {
        try 
        {
            var utcDate      = new Date(utcDateTime);
            var milliseconds = utcDate.getUTCMilliseconds();
            
            if(milliseconds < 100)
            {
                milliseconds = '0' + milliseconds;
            }
            
            return sprintf(this.ISO_DATETIME_FORMAT                                      ,
                           utcDate.getUTCFullYear()                                      ,
                           this.getLeadingZero(utcDate.getUTCMonth())                    ,
                           this.getLeadingZero(utcDate.getUTCDate())                     ,
                           formatISO8601 ? this.ISO_TIME_SPACER : this.ISO_TIME_DELIMITER,
                           this.getLeadingZero(utcDate.getUTCHours())                    ,
                           this.getLeadingZero(utcDate.getUTCMinutes())                  ,
                           this.getLeadingZero(utcDate.getUTCSeconds())                  ,
                           this.getTrailingMilliseconds(utcDate.getUTCMilliseconds())    );
        }
        catch(exception)
        {
            return this.ISO_DATETIME_ZERO;
        }
    },


    formatTimeCounter : function(seconds, format)
    {
        seconds     = parseInt(seconds);
        
        var minutes = parseInt(seconds / 60);
        seconds     = seconds % 60;
        
        var hours   = parseInt(minutes / 60);
        minutes     = minutes % 60;
        
        var days    = parseInt(hours / 24);
        hours       = hours % 24;
        
        if(!format)
        {
            var formatArray = new Array();
            
            if(days > 0)
            {
                formatArray.push(days    + " days"   );
            }
            
            if(hours > 0)
            {
                formatArray.push(hours   + " hours"  );
            }
            
            if(minutes > 0)
            {
                formatArray.push(minutes + " minutes");
            }
            
            if(seconds > 0)
            {
                formatArray.push(seconds + " seconds");
            }
            
            format = formatArray.join(", ");
        }
        else
        {
            format = format.replace("%d", days   );
            format = format.replace("%h", hours  );
            format = format.replace("%m", minutes);
            format = format.replace("%s", seconds);
        }
        
        return format;
    }
    
};

const DateTimeConverter = new ceDateTimeConverter();

