/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is the Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2003 Civil Engines Research.  All Rights 
 * Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/


const NS_DIRECTORY_DEFAULT_PERMISSIONS         = 0766;


function ceDirectoryObject(pathName)
{
    if(!pathName)
    {
        return null;
    }
    
    return this.initWithPath(pathName);
}



ceDirectoryObject.prototype = new ceFilesystemObject();

ceDirectoryObject.create = function(permissions)
{
    if(!this.__path__)
    {
        throw Components.results.NS_ERROR_NOT_INITIALIZED;
    }
    
    if(this.exists())
    {
        throw Components.results.NS_ERROR_FILE_ALREADY_EXISTS;
    }
    
    if(typeof(permisions) == "number")
    {
        var result = 0;
        
        try
        {
            result = this.fileObject.create(NS_DIRECTORYFILE_TYPE_ID, permissions);
        }
        catch(exception)
        {
            return Components.returnCode;
        }
        
        return result;
    }
    
    throw Components.results.NS_ERROR_INVALID_ARG;
}



ceDirectoryObject.prototype.contents = function()
{
    if(!this.exists())
    {
        throw Components.results.NS_ERROR_FILE_TARGET_DOES_NOT_EXIST;
    }
    
    var fileList    = new Array();
    var localFile   = null;
        
    try
    {
        if(!this.isDirectory())
        {
            throw Components.results.NS_ERROR_FILE_NOT_DIRECTORY;
        }
    
        var fileEntries = this.fileObject.directoryEntries;  
        while(fileEntries.hasMoreElements())
        {
            localFile = do_QueryInterface(fileEntries.getNext()             ,
                                          Components.interfaces.nsILocalFile);
            if(localFile.isFile())
            {
                fileList.push(new ceFileObject(localFile.path));
            }
            
            if(localFile.isDirectory())
            {
                fileList.push(new ceDirectoryObject(localFile.path));
            }   
        }
    }
    catch(exception)
    {
        return null;
    }
    
    return fileList;
}


ceDirectoryObject.prototype.remove = function(recursiveRemove)
{
    if(!this.fileObject || !this.__path__)
    {
        throw Components.results.NS_ERROR_NOT_INITIALIZED;
    }

    var result = 0;
    
    try
    {
        if(!this.exists())
        {
            throw Components.results.NS_ERROR_FILE_TARGET_DOES_NOT_EXIST;
        }
        
        if(!this.isDirectory())
        {
            throw Components.results.NS_ERROR_FILE_NOT_DIRECTORY;
        }
        
        result = this.fileObject.remove(recursiveRemove);
    }
    catch(exception)
    {
        return Components.returnCode;
    }
    
    return result;
}
