/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2005 Civil Engines Research.  All Rights 
 * Reserved.  
 *
 * Portions created by Netscape Copyright (C) 1998 Netscape 
 * Communications Corporation. All Rights Reserved.
 *
 * Portions created by the Mozilla Foundation are Copyright (C) 
 * 2003 Mozilla Foundation. All Rights Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Adam Michela  <adam@axentric.com> (original author)
 *     http://www.axentric.com/aside/fat
 *
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/


const CE_FADE_EFFECTS_FADE_TIMEOUT_DEFAULT         = 50;
const CE_FADE_EFFECTS_OPACITY_LEVEL_MAX            = 1.00;
const CE_FADE_EFFECTS_OPACITY_LEVEL_MIN            = 0.00;
const CE_FADE_EFFECTS_OPACITY_INCREMENT_DEFAULT    = 0.05;
const CE_FADE_EFFECTS_OPACITY_INITIAL_VALUE        = 0;

const CE_FADE_EFFECTS_CROSSFADE_FRAMES_PER_SECOND  = 30;
const CE_FADE_EFFECTS_CROSSFADE_DURATION_DEFAULT   = 3000;


function ceFadeEffects()
{
}


ceFadeEffects.prototype = 
{
    fadeTimeout       : CE_FADE_EFFECTS_FADE_TIMEOUT_DEFAULT       ,
    fadeUpperBounds   : CE_FADE_EFFECTS_OPACITY_LEVEL_MAX          ,
    fadeLowerBounds   : CE_FADE_EFFECTS_OPACITY_LEVEL_MIN          ,
    fadeIncrement     : CE_FADE_EFFECTS_OPACITY_INCREMENT_DEFAULT  ,
    currentFadeValue  : CE_FADE_EFFECTS_OPACITY_INITIAL_VALUE      ,

    framesPerSecond   : CE_FADE_EFFECTS_CROSSFADE_FRAMES_PER_SECOND,
    crossFadeDuration : CE_FADE_EFFECTS_CROSSFADE_DURATION_DEFAULT ,


    fadeInTimerCallback : function(fadeEffectsObject, targetElement) 
    {
        if(fadeEffectsObject.currentFadeValue > fadeEffectsObject.fadeUpperBounds)
        {
            return;
        }
        
        fadeEffectsObject.currentFadeValue += fadeEffectsObject.fadeIncrement;
        
        var opacityProperty = "opacity: " + fadeEffectsObject.currentFadeValue + " !important;";
        targetElement.setAttribute("style", opacityProperty);
        
        window.setTimeout(fadeEffectsObject.fadeInTimerCallback, 
                          fadeEffectsObject.fadeTimeout        , 
                          fadeEffectsObject                    ,
                          targetElement                        );
        return;
    },


    fadeElementIn : function(targetElement)
    {
        this.currentFadeValue = this.fadeLowerBounds;
        window.setTimeout(this.fadeInTimerCallback, this.fadeTimeout, this, targetElement);
        
        return;
    },


    fadeOutTimerCallback : function(fadeEffectsObject, targetElement) 
    {
        if(fadeEffectsObject.currentFadeValue <= fadeEffectsObject.fadeLowerBounds)
        {
            return;
        }
        
        if(fadeEffectsObject.fadeIncrement >= fadeEffectsObject.currentFadeValue)
        {
            fadeEffectsObject.currentFadeValue = 0.00;
        }
        else
        {
            fadeEffectsObject.currentFadeValue -= fadeEffectsObject.fadeIncrement;
        }
        
        var opacityProperty = "opacity: " + fadeEffectsObject.currentFadeValue + " !important;";
        targetElement.setAttribute("style", opacityProperty);
        
        window.setTimeout(fadeEffectsObject.fadeOutTimerCallback, 
                          fadeEffectsObject.fadeTimeout         , 
                          fadeEffectsObject                     ,
                          targetElement                         );
        return;
    },


    fadeElementOut : function(targetElement)
    {
        this.currentFadeValue = this.fadeUpperBounds;
        window.setTimeout(this.fadeOutTimerCallback, this.fadeTimeout, this, targetElement);
        
        return;
    },


	rgbColorToHex : function(rValue, gValue, bValue) 
	{
		rValue = rValue.toString(16); 
        if(rValue.length == 1)
        {
            rValue = '0' + rValue;
        }
        
		gValue = gValue.toString(16);
        if(gValue.length == 1)
        {
            gValue = '0' + gValue;
        }
        
		bValue = bValue.toString(16);
        if(bValue.length == 1)
        {
            bValue = '0' + bValue;
        }
        
		return "#" + rValue + gValue + bValue;
	},


	setElementBackgroundColor : function(targetElement, colorHex)
	{
        if(targetElement)
        {
            targetElement.style.backgroundColor = colorHex;
        }
        
        return;
	},


	getElementBackgroundColor : function(targetElement)
	{
        var backgroundColor;
        
		while(targetElement)
		{
			if(window.getComputedStyle)
            { 
                var computedStyle = window.getComputedStyle(targetElement, null);
                backgroundColor   = computedStyle.getPropertyValue("background-color");
            }
            
            if(targetElement.currentStyle)
            {
                backgroundColor = targetElement.currentStyle.backgroundColor;
            }
            
            if((backgroundColor != "" && backgroundColor != "transparent")) 
            { 
                break; 
            }

            targetElement = targetElement.parentNode;
        }

		if(backgroundColor == undefined     || 
           backgroundColor == ""            || 
           backgroundColor == "transparent"  )
        {
            backgroundColor = "#FFFFFF";
        }

		var rgbValue = backgroundColor.match(/rgb\s*\(\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*(\d{1,3})\s*\)/);
		if(rgbValue)
        {
            backgroundColor = this.rgbColorToHex(parseInt(rgbValue[1]),
                                                 parseInt(rgbValue[2]),
                                                 parseInt(rgbValue[3]));
        }
        
		return backgroundColor;
	},


	crossFadeElement : function(targetElement, fromColor, toColor) 
	{
		if(!fromColor || fromColor == "#")
        {
            fromColor = "#ffffce";
        }
        
		if(!toColor)
        {
            toColor = this.getElementBackgroundColor(targetElement);
		}
        
		var framesCount     = Math.round(this.framesPerSecond * (this.crossFadeDuration / 1000));
		var intervalCount   = this.crossFadeDuration / framesCount;
		var timeoutDelay    = intervalCount;
		var frameIndex      = 0;
		
		if(fromColor.length < 7) 
        {
            fromColor += fromColor.substr(1,3);
        }
        
		if(toColor.length   < 7)
        {
            toColor   += toColor.substr(1,3);
		}
        
		var rValueFrom = parseInt(fromColor.substr(1,2), 16);
		var gValueFrom = parseInt(fromColor.substr(3,2), 16);
		var bValueFrom = parseInt(fromColor.substr(5,2), 16);

		var rValueTo   = parseInt(toColor.substr(1,2), 16);
		var gValueTo   = parseInt(toColor.substr(3,2), 16);
		var bValueTo   = parseInt(toColor.substr(5,2), 16);
		
		var rValue     = 0;
        var gValue     = 0;
        var bValue     = 0;
        var hexValue;

		while(frameIndex < framesCount)
		{
			rValue   = Math.floor(rValueFrom * ((framesCount - frameIndex) / framesCount) + 
                                  rValueTo   * (frameIndex/framesCount)                   );

			gValue   = Math.floor(gValueFrom * ((framesCount - frameIndex) / framesCount) + 
                                  gValueTo   * (frameIndex / framesCount)                 );

			bValue   = Math.floor(bValueFrom * ((framesCount - frameIndex) / framesCount) +
                                  bValueTo   * (frameIndex / framesCount)                 );

			hexValue = this.rgbColorToHex(rValue, gValue, bValue);
            
			window.setTimeout(this.setElementBackgroundColor, timeoutDelay, targetElement, hexValue);

			frameIndex++;

			timeoutDelay = intervalCount * frameIndex; 
		}

        window.setTimeout(this.setElementBackgroundColor, timeoutDelay, targetElement, toColor);
        
        return;
	}
};
