/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is the Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2003 Civil Engines Research.  All Rights 
 * Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/


const NS_FILE_INPUTSTREAM_CONTRACTID      = "@mozilla.org/scriptableinputstream;1";
const NS_FILE_OUTPUTSTREAM_CONTRACTID     = "@mozilla.org/network/file-output-stream;1";
const NS_FILETRANSPORTSERVICE_CONTRACTID  = "@mozilla.org/network/file-transport-service;1";
const NS_EXTERNAL_PROTOCOL_CONTRACTID     = "@mozilla.org/uriloader/external-protocol-service;1"


const NS_FILE_READ_MODE                   = "r";
const NS_FILE_WRITE_MODE                  = "w";
const NS_FILE_APPEND_MODE                 = "a";


const NS_FILE_READ_MODE_MASK              = 0x01;
const NS_FILE_WRITE_MODE_MASK             = 0x08;
const NS_FILE_APPEND_MODE_MASK            = 0x10;


const NS_FILE_RDONLY_FLAG                 = 0x01;
const NS_FILE_WRONLY_FLAG                 = 0x02;
const NS_FILE_RW_FLAG                     = 0x04;
const NS_FILE_CREATE_FILE_FLAG            = 0x08;
const NS_FILE_APPEND_FLAG                 = 0x10;
const NS_FILE_TRUNCATE_FLAG               = 0x20;
const NS_FILE_SYNC_FLAG                   = 0x40;
const NS_FILE_EXCL_FLAG                   = 0x80;

const NS_FILE_DEFAULT_PERMISSIONS         = 0644;


function ceFileObject(filePath)
{
    if(filePath)
    {
        this.createFileObject(filePath);
    }

    return;
}


ceFileObject.prototype                = new ceFilesystemObject();
ceFileObject.prototype.mode           = null;
ceFileObject.prototype.fileChannel    = null;
ceFileObject.prototype.fileTransport  = null;
ceFileObject.prototype.uri            = null;
ceFileObject.prototype.inputStream    = null;
ceFileObject.prototype.outputStream   = null;
ceFileObject.prototype.lineBuffer     = null;
ceFileObject.prototype.position       = 0;



function eofReached()
{
    if(!this.fileObject || !this.inputStream)
    {
        throw Components.results.NS_ERROR_NOT_INITIALIZED;
    }
    
    if(this.lineBufferLength > 0        || 
       this.inputStream.available() > 0  )
    {
        return false;
    }
    
    return true;
}



function localFileSize()
{
    if(!this.fileObject)
    {
        throw Components.results.NS_ERROR_NOT_INITIALIZED;
    }
    
    if(!this.__path__)
    {
        throw Components.results.NS_ERROR_FILE_INVALID_PATH;
    }
    
    if(!this.exists())
    {
        throw Components.results.NS_ERROR_FILE_NOT_FOUND;
    }
    
    this.reset();

    var result = 0;
    
    try
    {
        result = this.fileObject.fileSize;
    }
    catch(exception)
    {
        return Components.returnCode;
    }
    
    return result;
}




function localFileExtension()
{
    if(!this.fileObject)
    {
        throw Components.results.NS_ERROR_NOT_INITIALIZED;
    }
    
    if(!this.__path__)
    {
        throw Components.results.NS_ERROR_FILE_INVALID_PATH;
    }
    
    if(!this.exists())
    {
        throw Components.results.NS_ERROR_FILE_NOT_FOUND;
    }
    
    this.reset();

    var result = null;
    
    try
    {
        var leafName = this.fileObject.leafName;
        var dotIndex = leafName.lastIndexOf('.');
        
        result = (dotIndex >= 0) ? leafName.substring(dotIndex + 1) : "";
    }
    catch(exception)
    {
        return Components.returnCode;
    }
    
    return result;
}



ceFileObject.prototype.__defineGetter__("eof"      , eofReached        );
ceFileObject.prototype.__defineGetter__("size"     , localFileSize     );
ceFileObject.prototype.__defineGetter__("extension", localFileExtension);



ceFileObject.prototype.create = function()
{
    if(!this.fileObject)
    {
        throw Components.results.NS_ERROR_NOT_INITIALIZED;
    }
    
    if(this.exists())
    {
        throw Components.results.NS_ERROR_FILE_ALREADY_EXISTS;
    }
    
    var result = 0;
    
    try
    {
        result = this.fileObject.create(NS_LOCALFILE_TYPE_ID       ,
                                        NS_FILE_DEFAULT_PERMISSIONS);
    }
    catch(exception)
    {
        return Components.returnCode;
    }
    
    return result;
}




ceFileObject.prototype.remove = function()
{
    if(!this.fileObject)
    {
        throw Components.results.NS_ERROR_NOT_INITIALIZED;
    }
    
    if(!this.__path__)
    {
        throw Components.results.NS_ERROR_FILE_INVALID_PATH;
    }
    
    this.close();
    
    var result = 0;
    
    try
    {
        result = this.fileObject.remove(false);
    }
    catch(exception)
    {
        return Components.returnCode;
    }
    
    return result;
}



ceFileObject.prototype.open = function(modeString, permission)
{
    if(!this.__path__ || !this.fileObject)
    {
        throw Components.results.NS_ERROR_NOT_INITIALIZED;
    }

    if(this.exists() && this.fileObject.isDirectory())
    {
        throw Components.results.NS_ERROR_FILE_IS_DIRECTORY;
    }
    
    if(this.mode)
    {
        throw Components.results.NS_ERROR_NOT_INITIALIZED;
    }
    
    this.close();
    
    if(!this.uri)
    {
        if(!this.exists())
        {
            this.create();
        }
        
        this.uri = IOService.newFileURI(this.fileObject);
    }
    
    if(!modeString)
    {
        modeString = NS_FILE_READ_MODE;
    }
    
    this.reset();
    
    switch(modeString)
    {
      case NS_FILE_WRITE_MODE  :
      case NS_FILE_APPEND_MODE :
      {
          try
          {
              if(!this.fileChannel)
              {
                  this.fileChannel = IOService.newChannelFromURI(this.uri);
              }         
          }
          catch(exception)
          {
              return Components.results.NS_ERROR_FAILURE;
          }
          
          if(!permission)
          {
              permission = NS_FILE_DEFAULT_PERMISSION;
          }
          
          try
          {
              var filestream = null;
              var offset     = 0;
              if(mode == NS_FILE_WRITE_MODE)
              {
                  this.mode = NS_FILE_WRITE_MODE;
                  
                  filestream = do_CreateInstance(NS_FILE_OUTPUTSTREAM_CONTRACTID          ,
                                                 Components.interfaces.nsIFileOutputStream);
                  filestream.init(this.fileObject                              ,
                                  (NS_FILE_WRONLY_FLAG | NS_FILE_TRUNCATE_FLAG),
                                  0x0004                                       ,
                                  null                                         );
                  
                  this.outputStream = filestream;
              }
              else
              {
                  this.mode = NS_FILE_APPEND_MODE;
                  filestream = do_CreateInstance(NS_FILE_OUTPUTSTREAM_CONTRACTID          ,
                                                 Components.interfaces.nsIFileOutputStream);
                  filestream.init(this.fileObject                        ,
                                  (NS_FILE_RW_FLAG | NS_FILE_APPEND_FLAG),
                                  0x0004                                 ,
                                  null                                   );
                  
                  this.outputStream = filestream;
              }
          }
          catch(exception)
          {
              return Components.results.NS_ERROR_FAILURE;
          }
          
          break;
      }
      case NS_FILE_READ_MODE   :
      {
          if(!this.exists())
          {
              throw Components.results.NS_ERROR_FILE_NOT_FOUND;
          }
          
          this.mode = NS_FILE_READ_MODE;
          
          try
          {
              this.fileChannel  = IOService.newChannelFromURI(this.uri);
              this.inputStream  = do_CreateInstance(NS_FILE_INPUTSTREAM_CONTRACTID                ,
                                                    Components.interfaces.nsIScriptableInputStream);
               
              this.inputStream.init(this.fileChannel.open());
              this.lineBuffer   = new Array();
          }
          catch(exception)
          {
              return Components.results.NS_ERROR_FAILURE;
          }
          
          break;
      }
      default  :
      {
          throw Components.results.NS_ERROR_INVALID_ARG;
      }
    }
    

    return Components.results.NS_OK;
}




ceFileObject.prototype.read = function(readSize)
{
    if(!this.fileObject || !this.inputStream)
    {
        throw Components.results.NS_ERROR_NOT_INITIALIZED;
    }

    if(this.mode != NS_FILE_READ_MODE)
    {
        this.close();
        return Components.results.NS_ERROR_NOT_AVAILABLE;
    }
    
    var readBuffer = null;
    
    try
    {
        readBuffer = this.inputStream.read(readSize);
    }
    catch(exception)
    {
        return null;
    }
    
    return readBuffer;
}





ceFileObject.prototype.readLine = function()
{
    if(!this.fileObject || !this.inputStream)
    {
        throw Components.results.NS_ERROR_NOT_INITIALIZED;
    }

    if(this.mode != NS_FILE_READ_MODE)
    {
        this.close();
        return Components.results.NS_ERROR_NOT_AVAILABLE;
    }
    
    var readBuffer = null;
    var tempBuffer = null;
    
    try
    {
        if(this.lineBuffer.length < 2)
        {
            readBuffer = this.inputStream.read(1024);
            this.position = this.position + 1024;
            
            if(this.position > this.fileObject.fileSize)
            {
                this.position = this.fileObject.fileSize;
            }
            
            if(readBuffer)
            {
                if(this.lineBuffer.length == 1)
                {
                    tempBuffer = this.lineBuffer.shift();
                    readBuffer = tempBuffer + readBuffer;
                }   
            }
        }
    }
    catch(exception)
    {
        return Components.returnCode;
    }
    
    return this.lineBuffer.shift();
}



ceFileObject.prototype.write = function(writeBuffer)
{
    if(!this.fileObject || !this.outputStream)
    {
        throw Components.results.NS_ERROR_NOT_INITIALIZED;
    }

    if(this.mode == NS_FILE_READ_MODE)
    {
        this.close();
        return Components.results.NS_ERROR_FILE_READ_ONLY;
    }
        
    var bytesWritten = 0;
    
    try
    {
        bytesWritten = this.outputStream.write(writeBuffer, writeBuffer.length);
    }
    catch(exception)
    {
        return Components.returnCode;
    }
    
    return bytesWritten;
}



ceFileObject.prototype.flush = function()
{
    if(!this.fileObject || !this.outputStream)
    {
        throw Components.results.NS_ERROR_NOT_INITIALIZED;
    }

    if(this.mode == NS_FILE_READ_MODE)
    {
        this.close();
        return Components.results.NS_ERROR_FILE_READ_ONLY;
    }
        
    try
    {
        this.outputStream.flush();
    }
    catch(exception)
    {
        return Components.returnCode;
    }
    
    return Components.results.NS_OK;
}



ceFileObject.prototype.copy = function(destination)
{
    if(!destination)
    {
        throw Components.results.NS_ERROR_INVALID_ARG;
    }
    
    if(!this.fileObject)
    {
        throw Components.results.NS_ERROR_NOT_INITIALIZED;
    }


    if(!this.exists())
    {
        throw Components.results.NS_ERROR_FILE_NOT_FOUND;
    }
   
    try
    {
        var nsLocalFile = Components.Constructor(NS_LOCALFILE_CONTRACTID,
                                                 "nsILocalFile"         ,
                                                 "initWithPath"         );
        
        var destinationFile = new nsLocalFile(destination);
        var copyName        = null;
        var directory       = null;
        
        if(destinationFile.equals(this.fileObject))
        {
            return Components.results.NS_ERROR_FILE_COPY_OR_MOVE_FAILED;
        }
        
        if(destinationFile.exists())
        {
            return Components.results.NS_ERROR_FILE_ALREADY_EXISTS;
        }
        
        if(this.fileObject.isDirectory())
        {
            return Components.results.NS_ERROR_FILE_IS_DIRECTORY;
        }
        
        if(!destinationFile.exists())
        {
            copyName  = destinationFile.leafName;
            directory = destinationFile.parent;
            
            if(!directory.exists())
            {
                return Components.results.NS_ERROR_FILE_NOT_FOUND;
            }
            
            if(!directory.isDirectory())
            {
                return Components.results.NS_ERROR_FILE_DESTINATION_NOT_DIR;
            }
        }
        
        if(!directory)
        {
            directory = destinationFile;
            if(destinationFile.equals(this.fileObject))
            {
                return Components.results.NS_ERROR_FILE_COPY_OR_MOVE_FAILED;
            }
        }
        
        this.fileObject.copyTo(directory, copyName);
    }
    catch(exception)
    {
        return Components.returnCode;
    }
    
    return Components.results.NS_OK;
}



ceFileObject.prototype.close = function()
{
    if(this.fileChannel)
    {
        delete this.fileChannel;
    }

    if(this.inputStream)
    {
        delete this.inputStream;
    }

    if(this.fileTransport)
    {
        delete this.fileTransport;
    }

    if(this.mode)
    {
        this.mode = null;
    }
    
    if(this.outputStream)
    {
        this.outputStream.flush();
        this.outputStream.close();
        delete this.outputStream;
    }
    
    if(this.lineBuffer)
    {
        this.lineBuffer = null;
    }
    
    this.position = 0;
    
    return Components.results.NS_OK;
}



ceFileObject.prototype.openExternal = function()
{
    if(!this.fileObject)
    {
        throw Components.results.NS_ERROR_NOT_INITIALIZED;
    }

    var fileURI         = IOService.newFileURI(this.fileObject);
    var protocolService = do_GetService(NS_EXTERNAL_PROTOCOL_CONTRACTID      ,
                                        Interfaces.nsIExternalProtocolService);
    if(protocolService)
    {
        protocolService.loadURL(fileURI);
    }
}
