/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is the Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2003 Civil Engines Research.  All Rights 
 * Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/


const NS_LOCALFILE_CONTRACTID           = "@mozilla.org/file/local;1";
const NS_STANDARDURL_CONTRACTID         = '@mozilla.org/network/standard-url;1';

const NS_LOCALFILE_TYPE_ID              = 0;
const NS_DIRECTORYFILE_TYPE_ID          = 1;


function ceFilesystemObject()
{
}



ceFilesystemObject.prototype =
{
    __path__   :  null,   /* nsILocalFile path     */
    fileObject :  null,   /* nsILocalFile instance */


    /* Initialize getter for property ceFilesystemObject.pathUrl */
    get pathUrl()
    {
        return FSUtils.convertPathToUrl(this.pathName);
    },
    

    /* Initialize getter for property ceFilesystemObject.pathName */
    get pathName()
    {
        if(!this.fileObject)
        {
            throw Components.results.NS_ERROR_NOT_INITIALIZED;
        }
        
        var pathName = null;
        try
        {
            pathName = this.fileObject.path;
        }
        catch(exception)
        {
            return null;
        }
        
        return pathName;
    },
    
    /* Initialize getter for property ceFilesystemObject.leafName */
    get leafName()
    {
        if(!this.fileObject)
        {
            throw Components.results.NS_ERROR_NOT_INITIALIZED;
        }
        
        var leaf = null;
        try
        {
            leaf = this.fileObject.leafName;
        }
        catch(exception)
        {
            return null;
        }
        
        return leaf;
    },

    /* Initialize setter for property ceFilesystemObject.leafName */
    set leafName(leafNameString)
    {
        if(!this.fileObject)
        {
            throw Components.results.NS_ERROR_NOT_INITIALIZED;
        }
        
        if(leafNameString)
        {
            var leafResult
            try
            {
                leafResult = this.fileObject.leafName = leafNameString;
            }
            catch(exception)
            {
                return null;
            }
            
            return leafResult;
        }
        
        return null;
    },
    
    getParentDirectory : function()
    {
        if(!this.fileObject)
        {
            throw Components.results.NS_ERROR_NOT_INITIALIZED;
        }
        
        var directoryInstance = null;
        
        try
        {
            if(this.fileObject.parent.isDirectory())
            {
                directoryInstance = new ceDirectoryObject(this.fileObject.parent.path);
            }
        }
        catch(exception)
        {
            return null;
        }
        
        return directoryInstance;
    },


    getPermissions : function()
    {
        if(!this.fileObject)
        {
            throw Components.results.NS_ERROR_NOT_INITIALIZED;
        }

        if(!this.exists())
        {
            throw Components.results.NS_ERROR_NOT_INITIALIZED;
        }
        
        var permissions = 0;
        
        try
        {
            permissions = this.fileObject.permissions;
        }
        catch(exception)
        {
            return 0;
        }
        
        return permissions;
    },


    setPermissions : function(permission)
    {
        if(!this.fileObject)
        {
            throw Components.results.NS_ERROR_NOT_INITIALIZED;
        }

        if(!this.exists())
        {
            throw Components.results.NS_ERROR_NOT_INITIALIZED;
        }
        
        var returnResult = null;
        
        try
        {
            returnResult = this.fileObject.permissions = permission;
        }
        catch(exception)
        {
            return null;
        }
        
        return returnResult;
    },
    
    getDateModified : function()
    {
        if(!this.fileObject)
        {
            throw Components.results.NS_ERROR_NOT_INITIALIZED;
        }

        if(!this.exists())
        {
            throw Components.results.NS_ERROR_NOT_INITIALIZED;
        }

        return new Date(this.fileObject.lastModifiedTime);
    },

    
    createFileObject : function(initialPath)
    {
        if(initialPath)    
        {
            this.fileObject = FSUtils.createLocalFile(initialPath);
            return this.fileObject;
        }
        
        return null;
    },
    
    initWithPath : function (pathArgument)
    {
        var fileURL    = null;
        var resultPath = null;
        
        if(typeof(pathArgument) == "object")
        {
            for(var i = 0; i < pathArgument.length; i++)
            {
                if(pathArgument[i].search(/^file:/) == 0)
                {
                    try
                    {
                        fileURL          = do_CreateInstance(NS_STANDARDURL_CONTRACTID   ,
                                                             Components.interfaces.nsIURL);
                        fileURL.spec     = pathArguments[i];
                        pathArguments[i] = fileURL.path;
                    }
                    catch(exception)
                    {
                        return null;
                    }
                    
                }
                
            }
        }
        else
        {
            if(pathArgument.search(/^file:/) == 0)
            {
                try
                {
                    fileURL          = do_CreateInstance(NS_STANDARDURL_CONTRACTID   ,
                                                         Components.interfaces.nsIURL);
                    fileURL.spec     = pathArguments;
                    pathArguments    = fileURL.path;
                }
                catch(exception)
                {
                    return null;
                }
            }
        }
        
        try
        {
            if(typeof(pathArgument) == "object")
            {
                this.createFileObject(pathArgument[0] ? pathArgument[0] : this.__path__);
                if(pathArgument.length > 1)
                {
                    for(var j = 0; j < pathArgument.length; j++)
                    {
                        this.fileObject.append(pathArgument[j]);
                    }
                }
                
                if(pathArgument[0] || this.__path__)
                {
                    this.__path__  = this.fileObject.path;
                    resultPath     = this.__path__;
                }
            }
            else
            {
                this.createFileObject(pathArgument[0] ? pathArgument[0] : this.__path__);
                if(this.fileObject && this.fileObject.path)
                {
                    this.__path__  = this.fileObject.path;
                    resultPath     = this.__path__;
                }
            }
        }
        catch(exception)
        {
            return null;
        }
        
        return resultPath;
    },
    
    exists : function()
    {
        if(!this.fileObject)
        {
            throw Components.results.NS_ERROR_NOT_INITIALIZED;
        }
        
        var doesExist = false;
        try
        {
            doesExist = this.fileObject.exists();
        }
        catch(exception)
        {
            return false;
        }
        
        return doesExist;
    },

    reset : function()
    {
        if(!this.fileObject)
        {
            throw Components.results.NS_ERROR_NOT_INITIALIZED;
        }

        var wasReset = false;
        
        if(this.__path__)
        {
            delete this.fileObject;
            try
            {
                this.createFileObject(this.__path__);
                wasReset = true;
            }
            catch(exception)
            {
                return false;

            }            
        }
        
        return wasReset;
    },
    
    cloneFileObject : function()
    {
        if(!this.fileObject)
        {
            throw Components.results.NS_ERROR_NOT_INITIALIZED;
        }

        var fileObjectClone = null;
        
        try
        {
            fileObjectClone = this.fileObject.clone();
        }
        catch(exception)
        {
            return null;
        }
        
        return fileObjectClone;
    },
    
    
    moveToNewPath : function(destinationPath)
    {
        if(!this.fileObject)
        {
            throw Components.results.NS_ERROR_NOT_INITIALIZED;
        }

        if(!destinationPath || !this.__path__)
        {
            return false;
        }
        
        var newPathName = null;
        try
        {
            var newFileObject = FSUtils.createLocalFile(destinationPath);
            if(newFileObject.exists() && !newFileObject.isDirectory())
            {
                Syslog.logMessage("Destination file exists : " + destinationPath);
                return false;
            }
            
            if(newFileObject.equals(this.fileObject))
            {
                return false;
            }
            
            if(!newFileObject.exists && newFileObject.parent.exists())
            {
                newPathName = newFileObject.leafName;
            }
            
            if(newFileObject.equals(this.fileObject.parent) && !newPathName)
            {
                return false;
            }
            
            var parentDirectory = newFileObject.parent;
            if(parentDirectory.exists() && parentDirectory.isDirectory())
            {
                this.fileObject.moveTo(parentDirectory, newPathName);
                this.__path__ = fileObject.path;
                this.reset();
                
                delete parentDirectory();
                return true;
            }
            else
            {
                Syslog.logMessage("Destination does not exist : " + destinationPath);
                return false;
            }            
        }
        catch(exception)
        {
            Syslog.logMessage("Error creating file");
            return false;
        }
        
        return true;
    },
    
    appendPath : function(leafName)
    {
        if(!this.fileObject)
        {
            throw Components.results.NS_ERROR_NOT_INITIALIZED;
        }

        if(!leafName || !this.__path__)
        {
            return null;
        }

        var newPath = null;
        try
        {
            this.fileObject.append(leafName);
            this.__path__ = this.pathName;
            newPath       = this.__path__;
        }
        catch(exception)
        {
            return null;
        }
        
        return newPath;
    },
    

    appendRelativePath : function(relativePathName)
    {
        if(!this.fileObject)
        {
            throw Components.results.NS_ERROR_NOT_INITIALIZED;
        }

        if(!relativePathName || !this.__path__)
        {
            return null;
        }

        var newPath = null;
        try
        {
            this.fileObject.appendRelativePath(leafName);
            this.__path__ = this.pathName;
            newPath       = this.__path__;
        }
        catch(exception)
        {
            return null;
        }
        
        return newPath;
    },


    isDirectory : function()
    {                  
        try
        {
            return this.fileObject.isDirectory();
        }
        catch(exception)
        {
            return false;
        }
    },


    isFile : function()
    {                  
        try
        {
            return this.fileObject.isFile();
        }
        catch(exception)
        {
            return false;
        }
    },

    
    isWritable : function()
    {                  
        try
        {
            return this.fileObject.isWritable();
        }
        catch(exception)
        {
            return false;
        }
    },


    isReadable : function()
    {                  
        try
        {
            return this.fileObject.isReadable();
        }
        catch(exception)
        {
            return false;
        }
    },


    isExecutable : function()
    {                  
        try
        {
            return this.fileObject.isExecutable();
        }
        catch(exception)
        {
            return false;
        }
    },


    isHidden : function()
    {                  
        try
        {
            return this.fileObject.isHidden();
        }
        catch(exception)
        {
            return false;
        }
    },


    isSpecial : function()
    {                  
        try
        {
            return this.fileObject.isSpecial();
        }
        catch(exception)
        {
            return false;
        }
    },


    isSymbolicLink : function()
    {                  
        try
        {
            return this.fileObject.isSymLink();
        }
        catch(exception)
        {
            return false;
        }
    }
    
};

