/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is the Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2003 Civil Engines Research.  All Rights 
 * Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/


function ceFilesystemUtils()
{
}


ceFilesystemUtils.prototype =
{
    nsLocalFile : Components.Constructor(NS_LOCALFILE_CONTRACTID,
                                         "nsILocalFile"         ,
                                         "initWithPath"         ),

    createLocalFile : function(pathString)
    {
        return (new this.nsLocalFile(pathString));
    },
    

    convertUrlToPath : function(urlString)
    {
        if(!urlString)
        {
            return Components.results.NS_ERROR_INVALID_ARG;
        }
        
        var urlPath      = urlString;
        var pathResult   = null;
        var file         = null;
        
        if(/^file:/.test(urlPath))
        {
            try
            {
                var uri    = do_CreateInstance(NS_STANDARDURL_CONTRACTID   ,
                                               Components.interfaces.nsIURI);
                uri.spec   = urlPath;
                pathResult = uri.path;
                file       = do_CreateInstance(NS_LOCALFILE_CONTRACTID           ,
                                               Components.interfaces.nsILocalFile);
                
                try
                {
                    file.initWithPath(pathResult);
                    pathResult = unescape(file.path);
                
                    return pathResult;
                }
                catch(exception)
                {
                }
                
            
                try
                {
                    file.initWithPath(pathResult.replace(/^\//,"").replace(/\//g,"\\"));
                    pathResult = unescape(file.path);
                    
                    return pathResult;
                }
                catch(exception)
                {
                }
            }
            catch(exception)
            {
                return Components.returnCode;
            }
        }
        
        return pathResult;
    },
    

    convertPathToUrl : function(pathString)
    {
        return (pathString ?
                'file:///' + pathString.
                replace(/\\/g, "\/")   .
                replace(/^\s*\/?/, "") .
                replace(/\ /g, "%20")  :'');
    },
    

    exists : function(pathString)
    {
        if(!pathString)
        {
            return Components.results.NS_ERROR_INVALID_ARG;
        }
        
        var doesExist = false;
        try
        {
            doesExist = (this.createLocalFile(pathString)).exists();
        }
        catch(exception)
        {
            return Components.returnCode;
        }
        
        return doesExist;
    },


    remove : function(pathString, recursive)
    {
        if(!pathString)
        {
            return Components.results.NS_ERROR_INVALID_ARG;
        }
        
        if(!this.exists(pathString))
        {
            return Components.results.NS_ERROR_FILE_TARGET_DOES_NOT_EXIST;
        }
                
        try
        {
            var fileInstance = this.createLocalFile(pathString);
            
            if(fileInstance.isDirectory())
            {
                return Components.results.NS_ERROR_INVALID_ARG;
            }
            
            if(typeof(recursive) == "boolean")
            {
                fileInstance.remove(recursive);
            }
            else
            {
                fileInstance.remove(false);
            }
        }
        catch(exception)
        {
            return Components.returnCode;
        }
        
        return Components.results.NS_OK;
    },


    copy : function(sourceFilePath, destFilePath)
    {
        if(!sourceFilePath || !destFilePath)
        {
            return Components.results.NS_ERROR_INVALID_ARG;
        }
        
        if(!this.exists(sourceFilePath))
        {
            return Components.results.NS_ERROR_INVALID_ARG;
        }
        
        try
        {
            var fileSource = this.createLocalFile(sourceFilePath);
            var fileDest   = this.createLocalFile(destFilePath);
            var copyName   = fileSource.leafName;
            
            if(fileSource.isDirectory())
            {
                return Components.results.NS_ERROR_FILE_COPY_OR_MOVE_FAILED;
            }
            
            if(!this.exists(destFilepath) || !fileDest.isDirectory())
            {
                copyName = fileDest.leafName;
                fileDest = this.createLocalFile(fileDest.path.replace(copyName, ''));
                
                if(!this.exists(fileDest.path))
                {
                    return Components.results.NS_ERROR_FILE_ALREADY_EXISTS;
                }
                
                if(!fileDest.isDirectory())
                {
                    return Components.results.NS_ERROR_FILE_INVALID_PATH;
                }
            }
            
            if(this.exists(this.append(fileDest.path, copyName)))
            {
                return Components.results.NS_ERROR_FILE_ALREADY_EXISTS;
            }
            
            fileSource.copyTo(fileDest, copyName);
        }
        catch(exception)
        {
            return Components.returnCode;
        }
        
        return Components.results.NS_OK;
    },
    

    getLeafName : function(pathString)
    {
        if(!pathString)
        {
            return Components.results.NS_ERROR_INVALID_ARG;
        }
        
        if(!this.exists(pathString))
        {
            return Components.results.NS_ERROR_FILE_NOT_FOUND;
        }
        
        var leaf = null;
        try
        {
            var fileInstance = this.createLocalFile(pathString);
            leaf = fileInstance.leafName;
        }
        catch(exception)
        {
            return Components.returnCode;
        }
        
        return leaf;
    },


    append : function(pathString, filename)
    {
        if(!pathString || !filename)
        {
            return Components.results.NS_ERROR_INVALID_ARG;
        }
        
        if(!this.exists(pathString))
        {
            return Components.results.NS_ERROR_FILE_NOT_FOUND;
        }
        
        var result = null;
        try
        {
            var fileInstance = this.createLocalFile(pathString);
            if(fileInstance.exists() && !fileInstance.isDirectory())
            {
                return Components.results.NS_ERROR_INVALID_ARG;
            }
            
            fileInstance.append(filename);
            result = fileInstance.path;
        }
        catch(exception)
        {
            return Components.returnCode;
        }
        
        return result;
    },

    getPermissions : function(pathString)
    {
        if(!pathString)
        {
            return Components.results.NS_ERROR_INVALID_ARG;
        }
        
        if(!this.exists(pathString))
        {
            return Components.results.NS_ERROR_FILE_NOT_FOUND;
        }
        
        var permissions = 0;
        try
        {
            var fileInstance = this.createLocalFile(pathString);
            permissions = fileInstance.permissions;
        }
        catch(exception)
        {
            return Components.returnCode;
        }
        
        return permissions;
    },

    
    getDateModified : function(pathString)
    {
        if(!pathString)
        {
            return Components.results.NS_ERROR_INVALID_ARG;
        }
        
        if(!this.exists(pathString))
        {
            return Components.results.NS_ERROR_FILE_NOT_FOUND;
        }
        
        var dateModified = null;
        try
        {
            var fileInstance = this.createLocalFile(pathString);
            dateModified = new Date((fileInstance.lastModifiedTime).toLocaleString());
        }
        catch(exception)
        {
            return Components.returnCode;
        }

        return dateModified;
    },


    getFileSize : function(pathString)
    {
        if(!pathString)
        {
            return Components.results.NS_ERROR_INVALID_ARG;
        }
        
        if(!this.exists(pathString))
        {
            return Components.results.NS_ERROR_FILE_NOT_FOUND;
        }
        
        var fileSize = 0;
        try
        {
            var fileInstance = this.createLocalFile(pathString);
            fileSize = fileInstance.fileSize;
        }
        catch(exception)
        {
            return Components.returnCode;
        }

        return fileSize;
    },

    
    getFileExtension : function(pathString)
    {
        if(!pathString)
        {
            return Components.results.NS_ERROR_INVALID_ARG;
        }
        
        if(!this.exists(pathString))
        {
            return Components.results.NS_ERROR_FILE_NOT_FOUND;
        }

        var extension = null;
        
        try
        {
            var fileInstance = this.createLocalFile(pathString);
            leafName         = fileInstance.leafName;
            
            var dotIndex     = leafName.lastIndexOf('.');
            extension        = (dotIndex >= 0) ? leafName.substring(dotIndex + 1) : "";
        }
        catch(exception)
        {
            return Components.returnCode;
        }

        return extension;
        
    },
    
    getParentDirectory : function(pathString)
    {
        if(!pathString)
        {
            return Components.results.NS_ERROR_INVALID_ARG;
        }
        
        var parentDir = null;
        
        try
        {
            var fileInstance = this.createLocalFile(pathString);
            if(!fileInstance.exists())
            {
                return Components.results.NS_ERROR_FILE_NOT_FOUND;
            }
            
            if(fileInstance.isFile())
            {
                parentDir = fileInstance.parent.path;
            }
            else if(fileInstance.isDirectory())
            {
                parentDir = fileInstance.path;
            }            
        }
        catch(exception)
        {
            return Components.returnCode;
        }

        return parentDir;
    },


    ensureUniqueness : function(localFile)
    {
        var targetFile     = localFile;
        var collisionCount = 0;
        
        while(targetFile.exists())
        {
            collisionCount++;
            if(collisionCount == 1)
            {
                if(targetFile.leafName.match(/\.[^\.]{1,3}\.(gz|bz2|Z)$/i))
                {
                    targetFile.leafName = targetFile.leafName.replace(/\.[^\.]{1,3}\.(gz|bz2|Z)$/i, 
                                                                      "(2)$&"                     );
                }
                else
                {
                    targetFile.leafName = targetFile.leafName.replace(/(\.[^\.]*)?$/, "(2)$&");
                }
            }
            else
            {
                targetFile.leafName = targetFile.leafName.replace(/^(.*\()\d+\)/                 , 
                                                                  "$1" + (collisionCount+1) + ")");
            }
        }
        
        return;
    }
};


const FSUtils = new ceFilesystemUtils();
