/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is the Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2003 Civil Engines Research.  All Rights 
 * Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/


function ceRDFNode()
{
    this.node       = null;
    this.dataSource = null;
}


ceRDFNode.prototype =
{
    getLabel : function()
    {
        return this.node.ValueUTF8;
    },

    /* The argument can either be a ceRDFNode or RegExp object.
     * A regexp pattern will match the string value returned by
     * the XPCOM rdf engine
     */
    equals : function(rdfNode)
    {
        if(rdfNode instanceof RegExp)
        {
            return this.node.ValueUTF8.match(rdfNode);
        }
        
        return this.node.EqualsNode(rdfNode.node);
    },


    getSource : function(property)
    {
        var propertyNode = RDFFactory.getResourceComponent(property);
        var sourceNode   = this.dataSource.GetSource(propertyNode, this.node, true);
        
        if(!sourceNode)
        {
            return null;
        }
        
        return RDFFactory.wrapNodeComponent(sourceNode);
    },


    getSources : function(property)
    {
        var propertyNode = RDFFactory.getResourceComponent(property);
        var enumeration  = this.dataSource.GetSources(propertyNode, this.node, true);
        
        return new ceRDFNodeEnumerator(enumeration, this.dataSource);
    },


    getPropertiesFromSources : function()
    {
        var enumeration = this.dataSource.ArcLabelsIn(this.node);
        return new ceRDFNodeEnumerator(enumeration, this.dataSource);
    },


    hasPropertyFromSource : function(property)
    {
        var propertyNode = RDFFactory.getResourceComponent(property);
        return this.dataSource.hasArcIn(this.node, propertyNode);
    },


    modifySource : function(oldSource, newSource, property)
    {
        var oldSourceNode = RDFFactory.getResourceComponent(oldSource);
        var newSourceNode = RDFFactory.getResourceComponent(newSource);
        var propertyNode  = RDFFactory.getResourceComponent(property);
        
        this.dataSource.Move(oldSourceNode, newSourceNode, propertyNode, this.node);
    }
};


/* A statement represents an RDF graph 3-tuple assertion
 * consisting of: 
 *
 *  [ statement ] --> [ property ] --> [ target ]
 *
 * The RDF data model expects that the statement
 * collection is comprised of:
 *
 *  [ ceRDFNode     ] --> [ceRDFNode    ] --> [ceRDFNode    ]
 *  [ ceRDFResource ] --> [ceRDFResource] --> [ceRDFResource]
 *  [ ceRDFResource ] --> [ceRDFResource] --> [ceRDFLiteral ]
 *
 * or regular expressions when used to match or equate 
 * rdf model patterns
 *
 * i.e
 *
 *  [ RegExp        ] --> [ceRDFResource] --> [ RegExp      ]
 *  [ RegExp        ] --> [ RegExp      ] --> [ceRDFLiteral ]
 *
 */
function ceRDFStatement(source, property, target)
{
    this.source    = null;
    this.property  = null;
    this.target    = null;

    if(source)
    {
        this.source   = source;
    }
    
    if(property)
    {
        this.property = property;        
    }
    
    if(target)
    {
        this.target   = target;
    }
}



ceRDFStatement.prototype = 
{
    getSource   : function()
    {
        return this.source;
    },

    getProperty : function()
    {
        return this.property;
    },

    getTarget   : function()
    {
        return this.target;
    },

    equals      : function(statement)
    {
        if(!this.source.equals(statement.source))
        {
            return false;
        }

        if(!this.property.equals(statement.property))
        {
            return false;
        }

        if(!this.target.equals(statement.target))
        {
            return false;
        }
        
        return true;
    }
};


