/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is the Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2003 Civil Engines Research.  All Rights 
 * Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/



function ceRDFResource(resourceURI, rdfDataModel)
{
    this.uri        = null;
    this.node       = null;
    this.dataSource = null;
    this.container  = null;

    if(resourceURI)
    {
        this.uri    = resourceURI;
        this.node   = RDFService.GetResource(this.uri);
    }
 
    if(rdfDataModel)
    {
        this.dataSource = rdfDataModel.getDatasource();
    }
}



ceRDFResource.prototype = new ceRDFNode();


ceRDFResource.prototype.constructor   = ceRDFResource;


ceRDFResource.prototype.initContainer = function()
{
    if(this.container != null)
    {
        return true;
    }
    
    this.container = do_CreateInstance(NS_RDFCONTAINER_CONTRACTID           , 
                                       Components.interfaces.nsIRDFContainer);

    if(!this.container)
    {
        return false;
    }
    
    try
    {
        this.container.Init(this.dataSource, this.node);
    }
    catch(exception)
    {
        return false;
    }
    
    return true;
}



ceRDFResource.prototype.createTarget = function(property, target)
{
    var propertyNode = RDFFactory.getResourceComponent(property);
    var targetNode   = RDFFactory.getLiteralComponent(target);
    
    this.dataSource.Assert(this.node, propertyNode, targetNode, true);
}



ceRDFResource.prototype.deleteTarget = function(property, target)
{
    var propertyNode = RDFFactory.getResourceComponent(property);
    var targetNode   = RDFFactory.getLiteralComponent(target);
    
    this.dataSource.Unassert(this.node, propertyNode, targetNode, true);
}



ceRDFResource.prototype.modifyTarget = function(property, oldTarget, newTarget)
{
    var propertyNode  = RDFFactory.getResourceComponent(property);
    var oldTargetNode = RDFFactory.getLiteralComponent(oldTarget);
    var newTargetNode = RDFFactory.getLiteralComponent(newTarget);
    
    this.dataSource.Change(this.node, propertyNode, oldTargetNode, newTargetNode);
}



ceRDFResource.prototype.replaceTargetWithProperty = function(property, newTarget)
{
    var propertyNode  = RDFFactory.getResourceComponent(property);
    var targetNode    = RDFFactory.getLiteralComponent(newTarget);
    
    var oldTargetNode = this.dataSource.GetTarget(this.node, propertyNode, true);
    
    if(oldTargetNode != null)
    {
        this.dataSource.Change(this.node, propertyNode, oldTargetNode, targetNode);
    }
    else
    {
        this.dataSource.Assert(this.node, propertyNode, targetNode, true);
    }
}



ceRDFResource.prototype.replaceSourceUsingProperty = function(property, newSource)
{
    var sourceNode    = RDFFactory.getResourceComponent(newSource);
    var propertyNode  = RDFFactory.getResourceComponent(property);
    
    var oldSourceNode = this.dataSource.GetSource(propertyNode, this.node, true);
    
    if(oldSourceNode != null)
    {
        this.dataSource.Move(oldSourceNode, sourceNode, propertyNode, this.node);
    }
    else
    {
        this.dataSource.Assert(sourceNode, propertyNode, this.node, true);
    }
}



ceRDFResource.prototype.hasTarget = function(property, target)
{
    var propertyNode  = RDFFactory.getResourceComponent(property);
    var targetNode    = RDFFactory.getLiteralComponent(newTarget);

    return this.dataSource.HasAssertion(this.node, propertyNode, targetNode, true);
}



ceRDFResource.prototype.getPropertiesToTargets = function()
{
    var enumeration = this.dataSource.ArcLabelsOut(this.node);
    return new ceRDFNodeEnumerator(enumeration, this.dataSource);
}



ceRDFResource.prototype.hasPropertyToTarget = function(property)
{
    var propertyNode = RDFFactory.getResourceComponent(property);
    return this.dataSource.hasArcOut(this.node, propertyNode);
}



ceRDFResource.prototype.getTarget = function(property)
{
    var propertyNode = RDFFactory.getResourceComponent(property);
    var targetNode   = this.dataSource.GetTarget(this.node, propertyNode, true);
    
    if(!targetNode)
    {
        return null;
    }
    
    return RDFFactory.wrapNodeComponent(targetNode);
}


ceRDFResource.prototype.getTargets = function(property)
{
    var propertyNode = RDFFactory.getResourceComponent(property);
    var enumeration  = this.dataSource.GetTargets(this.node, propertyNode, true);
    
    return new ceRDFNodeEnumerator(enumeration, this.dataSource);
}



ceRDFResource.prototype.convertToBagContainer = function()
{
    this.container = RDFContainerUtils.MakeBag(this.dataSource, this.node);
}



ceRDFResource.prototype.convertToSeqContainer = function()
{
    this.container = RDFContainerUtils.MakeSeq(this.dataSource, this.node);
}


ceRDFResource.prototype.convertToAltContainer = function()
{
    this.container = RDFContainerUtils.MakeAlt(this.dataSource, this.node);
}


ceRDFResource.prototype.isBagContainer = function()
{
    return RDFContainerUtils.IsBag(this.dataSource, this.node);
}


ceRDFResource.prototype.isSeqContainer = function()
{
    return RDFContainerUtils.IsBag(this.dataSource, this.node);
}


ceRDFResource.prototype.isAltContainer = function()
{
    return RDFContainerUtils.IsBag(this.dataSource, this.node);
}


ceRDFResource.prototype.isContainer = function()
{
    return RDFContainerUtils.IsContainer(this.dataSource, this.node);
}



ceRDFResource.prototype.getNumberOfChildern = function()
{
    if(this.initContainer())
    {
        var enumeration = this.container.GetElements();
        return new ceRDFNodeEnumerator(enumeration, this.dataSource);
    }
    
    return null;
}



ceRDFResource.prototype.appendChild = function(childNode, overwrite)
{
    if(this.initContainer())
    {
        var childResource = null;
        if(typeof(childNode) == "string")
        {
            childResource        = RDFService.GetResource(childNode);
            childNode            = new ceRDFResource();
            childNode.node       = childResource;
            childNode.dataSource = this.dataSource;
        }
        else
        {
            childResource.node = childNode.node;
        }
        
        if(!overwrite && this.container.IndexOf(childResource) >= 0)
        {
            return childNode;
        }
        
        this.container.AppendElement(childResource);
        return childNode;
    }
    
    return null;
}


ceRDFResource.prototype.insertChild = function(childNode, index)
{
    if(this.initContainer())
    {
        var childResource = null;
        if(typeof(childNode) == "string")
        {
            childResource        = RDFService.GetResource(childNode);
            childNode            = new ceRDFResource();
            childNode.node       = childResource;
            childNode.dataSource = this.dataSource;
        }
        else
        {
            childResource.node = childNode.node;
        }
        
        this.container.InsertElementAt(childResource, index, true);
        return childNode;
    }
    
    return null;
}



ceRDFResource.prototype.removeChild = function(childNode)
{
    if(this.initContainer())
    {
        var childResource = null;
        if(typeof(childNode) == "string")
        {
            childResource        = RDFService.GetResource(childNode);
            childNode            = new ceRDFResource();
            childNode.node       = childResource;
            childNode.dataSource = this.dataSource;
        }
        else
        {
            childResource.node = childNode.node;
        }
        
        this.container.RemoveElement(childResource, true);
        return childNode;
    }
    
    return null;
}



ceRDFResource.prototype.removeChildAt = function(index)
{
    if(this.initContainer())
    {
        var childNode = this.container.RemoveElementat(index, true);
        return RDFFactory.wrapNodeComponent(childNode);
    }
    
    return null;
}



ceRDFResource.prototype.indexOfChild = function(childNode)
{
    if(this.initContainer())
    {
        return this.container.IndexOf(childNode);
    }
    
    return null;
}
