/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is the Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2003 Civil Engines Research.  All Rights 
 * Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/


const NS_RDFDATASOURCE_CONTRACTID         = "@mozilla.org/rdf/datasource;1?name=";
const NS_RDFINMEMORYDATASOURCE_CONTRACTID = NS_RDFDATASOURCE_CONTRACTID + "in-memory-datasource";
const NS_RDFXMLDATASOURCE_CONTRACTID      = NS_RDFDATASOURCE_CONTRACTID + "xml-datasource"


function ceRDFResourceFactory()
{
    this.rdfService = RDFService;
    this.rdfUtils   = RDFContainerUtils;
}


ceRDFResourceFactory.prototype =
{
    getResource : function(uri)
    {
        var resourceNode = new ceRDFResource(uri);
        return resourceNode;
    },


    getAnonymousResource : function()
    {
        var anonResourceNode = RDFService.GetAnonymousResource();
        var resourceNode     = new ceRDFResource();
        
        resourceNode.node = anonResourceNode;

        return resourceNode;
    },
    

    isAnonymousResource : function(resourceNode)
    {
        return this.rdfService.isAnonymousResource(resourceNode.node);
    },
    


    getLiteral : function(literalValue)
    {
        var literalNode = ceRDFLiteral(literalValue);
        return literalNode;
    },
   

    createDatasourceOfType : function(type, datasourceInterface)
    {
        var initialInterface = (datasourceInterface) ? datasourceInterface :
            Components.interfaces.nsIRDFDataSource;
        
        return do_CreateInstance(NS_RDFDATASOURCE_CONTRACTID + type, initialInterface);
    },
    
    createMemoryDatasource : function()
    {
        return do_CreateInstance(NS_RDFINMEMORYDATASOURCE_CONTRACTID   ,
                                 Components.interfaces.nsIRDFDataSource);
    },
   

    createXMLDatasource : function()
    {
        return do_CreateInstance(NS_RDFXMLDATASOURCE_CONTRACTID       ,
                                 Components.interfaces.nsIRDFDataSource);
    },
        

    getResourceComponent : function(value)
    {
        if(typeof(value) == "string") 
        {
            return this.rdfService.GetResource(value);
        }
        
        return value.node;
    },


    getLiteralComponent : function(value)
    {
        if(typeof(value) == "string") 
        {
            return this.rdfService.GetLiteral(value);
        }
        
        return value.node;
    },


    wrapNodeComponent : function(rdfComponent, rdfDatasource)
    {
        var resourceValue   = nsnull;

        if(rdfComponent != null)
        {
            rdfComponent = do_QueryInterface(rdfComponent, Components.interfaces.nsIRDFResource);
            if(rdfComponent)
            {
                resourceValue   = new ceRDFResource();
            }
            else
            {    
                rdfComponent = do_QueryInterface(rdfComponent, Components.interfaces.nsIRDFLiteral);
                resourceValue   = (rdfComponent) ? new ceRDFLiteral() : null;   
            }
            
            if(resourceValue && rdfComponent)
            {
                resourceValue.node       = rdfComponent;
                resourceValue.dataSource = rdfDatasource;
            }
        }
        
        return resourceValue;
    },

    
    calculateModelUnion : function(firstDataModel, secondDataModel)
    {
        var modelUnion   = firstDataModel.cloneDataModel();
        var statementSet = secondDataModel.getStatementCollection();
        
        for(var i = 0, setlength = statementSet.length; i < setlength; i++)
        {
            var statementElement = statementSet[i];
            modelUnion.addStatementSet(statementElement);
        }
        
        return modelUnion;
    },


    calculateModelSubtraction : function(firstDataModel, secondDataModel)
    {
        var modelSubtraction  = firstDataModel.cloneDataModel();
        var statementSet      = secondDataModel.getStatementCollection();
        
        for(var i = 0, setlength = statementSet.length; i < setlength; i++)
        {
            var statementElement = statementSet[i];
            modelSubtraction.removeStatementSet(statementElement);
        }
        
        return modelSubtraction;
    },

    
    calculateModelIntersection : function(firstDataModel, secondDataModel)
    {
        var modelIntersection  = firstDataModel.cloneDataModel();
        var statementSet       = secondDataModel.getStatementCollection();
        var statementElement   = null;
        
        for(var i = 0, set1length = statementSet.length; i < set1length; i++)
        {
            statementElement = statementSet[i];
            if(!firstDataModel.hasStatementSet(statement))
            {
                modelIntersection.removeStatementSet(statementElement);
            }
        }
        
        statementSet = modelIntersection.getStatementCollection();

        for(var j = 0, set2length = statementSet.length; j < set2length; j++)
        {
            statementElement = statementSet[j];
            if(!secondDataModel.hasStatementSet(statement))
            {
                modelIntersection.removeStatementSet(statementElement);
            }
        }

        return modelIntersection;
    }
};


const RDFFactory = new ceRDFResourceFactory;
