/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is the Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2003 Civil Engines Research.  All Rights 
 * Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/


const STANDARD_URL_CONTRACTID            = "@mozilla.org/network/standard-url;1";
const STANDARD_URL_SCHEME_SEPARATOR      = "://";


/* nsIURI components
 * ----------------- 
 * This interface follows Tim Berners-Lee's URI spec (RFC2396) [1], where the
 * basic URI components are defined as such:
 *  
 *      ftp://username:password@hostname:portnumber/pathname
 *      \ /   \               / \      / \        /\       /
 *       +     +-------------+   +----+   +------+  +-----+
 *       |            |             |        |         |
 *       |            |             |        |        Path
 *       |            |             |       Port         
 *       |            |            Host      /
 *       |         UserPass                 /
 *     Scheme                              /
 *        \                               /
 *         +-----------------------------+
 *                       |
 *                    PrePath
 *
 * nsIURL components
 * -----------------
 * This interfaces provides convenience methods that further
 * break down the path portion of nsIURI
 *
 * http://directory/fileBaseName.fileExtension?query
 * http://directory/fileBaseName.fileExtension#ref
 * http://directory/fileBaseName.fileExtension;param
 *        \         \                       /
 *         \         +---------------------+
 *          \                  |          /
 *           \              fileName     /
 *            +-------------------------+
 *                       |
 *                   filePath
 *
 */
function ceStandardURL(urlSpecString, urlQueryString)
{
    this.queryParameters     = { /* map of parameter names to values */ };
    this.queryParameterNames = [ /* array of parameter names         */ ];
    this.parameterIndex      = 0;

    this.standardURL = do_CreateInstance(STANDARD_URL_CONTRACTID             , 
                                         Components.interfaces.nsIStandardURL);
    
    this.standardURL.init(Components.interfaces.nsIStandardURL.URLTYPE_AUTHORITY,
                          0                                                     ,
                          urlSpecString                                         ,
                          null                                                  ,
                          null                                                  );
    
    this.standardURL.mutable = true;

    if(urlQueryString)
    {
        var url = do_QueryInterface(this.standardURL, Components.interfaces.nsIURL);

        if(typeof(urlQueryString) == "object")
        {
            if(urlQueryString["toQueryString"] == undefined)
            {
                throw Components.results.NS_ERROR_INVALID_ARG;
            }
            
            url.query = urlQueryString.toQueryString();
        }
        else
        {            
            url.query = urlQueryString;
        }
    }
    
    
    this.parseQuery();
}


ceStandardURL.prototype = 
{
    QueryInterface : function(iid)
    {
        if(iid.equals(Interfaces.nsIURI))
        {
            return this.URI;
        }
        
        if(iid.equals(Interfaces.nsIURL))
        {
            return this.URL;
        }

        if(iid.equals(Interfaces.nsISupports))
        {
            return this;
        }
        
        throw Results.NS_NOINTERFACE;
    },

    get URI()
    {
        return do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
    },

    get URL()
    {
        return do_QueryInterface(this.standardURL, Components.interfaces.nsIURL);
    },

    
    get baseSpec()
    {
        return (this.scheme + STANDARD_URL_SCHEME_SEPARATOR + this.host + this.filePath);
    },
    

    get spec()
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        return uri.spec;
    },

    set spec(specString)
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        uri.spec = specString;
    },


    get prePath()
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        return uri.prePath;
    },

    get scheme()
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        return uri.scheme;
    },

    set scheme(schemeString)
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        uri.scheme = schemeString;
    },

    get userPass()
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        return uri.userPass;
    },

    set userPass(userPassString)
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        uri.userPass = userPassString;
    },

    get username()
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        return uri.username;
    },

    set username(usernameString)
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        uri.username = usernameString;
    },


    get password()
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        return uri.password;
    },

    set password(passwordString)
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        uri.password = passwordString;
    },


    get host()
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        return uri.host;
    },

    set host(hostString)
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        uri.host = hostString;
    },


    get hostPort()
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        return uri.hostPort;
    },

    set hostPort(hostPortString)
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        uri.hostPort = hostPortString;
    },

    get host()
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        return uri.host;
    },

    set host(hostString)
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        uri.host = hostString;
    },

    get port()
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        return uri.port;
    },

    set port(portNumber)
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        uri.port = portNumber;
    },

    get path()
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        return uri.path;
    },

    set path(pathString)
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        uri.path = pathString;
    },

    schemeIs : function(schemeString)
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        return uri.schemeIs(schemeString);
    },

    resolve : function(relativePathString)
    {
        var uri = do_QueryInterface(this.standardURL, Components.interfaces.nsIURI);
        return uri.resolve(relativePathString);
    },
                      

    get filePath()
    {
        var url = do_QueryInterface(this.standardURL, Components.interfaces.nsIURL);
        return url.filePath;
    },

    set filePath(pathString)
    {
        var url = do_QueryInterface(this.standardURL, Components.interfaces.nsIURL);
        url.filePath = pathString;
    },

    get param()
    {
        var url = do_QueryInterface(this.standardURL, Components.interfaces.nsIURL);
        return url.param;
    },
    
    set param(paramString)
    {
        var url = do_QueryInterface(this.standardURL, Components.interfaces.nsIURL);
        url.param = paramString;
    },

    get query()
    {
        var url = do_QueryInterface(this.standardURL, Components.interfaces.nsIURL);
        return url.query;
    },

    set query(queryString)
    {
        var url = do_QueryInterface(this.standardURL, Components.interfaces.nsIURL);
        url.query = queryString;
    },

    get ref()
    {
        var url = do_QueryInterface(this.standardURL, Components.interfaces.nsIURL);
        return url.ref;
    },

    set ref(refString)
    {
        var url = do_QueryInterface(this.standardURL, Components.interfaces.nsIURL);
        url.ref = refString;
    },

    get directory()
    {
        var url = do_QueryInterface(this.standardURL, Components.interfaces.nsIURL);
        return url.directory;
    },

    set directory(dirString)
    {
        var url = do_QueryInterface(this.standardURL, Components.interfaces.nsIURL);
        url.directory = dirString;
    },

    get fileName()
    {
        var url = do_QueryInterface(this.standardURL, Components.interfaces.nsIURL);
        return url.fileName;
    },

    set fileName(nameString)
    {
        var url = do_QueryInterface(this.standardURL, Components.interfaces.nsIURL);
        url.fileName = nameString;
    },

    get fileBaseName()
    {
        var url = do_QueryInterface(this.standardURL, Components.interfaces.nsIURL);
        return url.fileBaseName;
    },

    set fileBaseName(baseNameString)
    {
        var url = do_QueryInterface(this.standardURL, Components.interfaces.nsIURL);
        url.fileBaseName = baseNameString;
    },

    get fileExtension()
    {
        var url = do_QueryInterface(this.standardURL, Components.interfaces.nsIURL);
        return url.fileExtension;
    },

    set fileExtension(extensionString)
    {
        var url = do_QueryInterface(this.standardURL, Components.interfaces.nsIURL);
        url.fileExtension = extensionString;
    },

    parseQuery : function()
    {
        var queryString = this.query;
        if(!queryString)
        {
            return;
        }
             
        if(queryString.length < 2)
        {
            return;
        }

        var pairs;     
        pairs = queryString.split(/[&;]/);
     
        for(var i = 0; i < pairs.length; ++i)
        {
            var pair;
            var name;
            var value;
            
            pair = pairs[i].split(/=/);
            
            name = decodeURIComponent(pair[0]);
            if(pair[1] == undefined)
            {
                continue;
            }
            else
            {
                value = decodeURIComponent(pair[1]);
            }
            
            if(this.queryParameters[name] == undefined) 
            {
                this.queryParameters[name] = [value];
                this.queryParameterNames.push(name);
            }
            else
            {
                this.queryParameters[name].push(value);
            }
        }   

        return;
    },

    getParameterValue : function(paramName) 
    {
        if(arguments.length)
        {
            if(this.queryParameters[paramName] == undefined)
            {
                return null;
            }
            else
            {
                return this.queryParameters[paramName][0];
            }
        }
        else 
        {
            if(this.parameterIndex == this.queryParameterNames.length) 
            {
                this.parameterIndex = 0;
                return null;
            }
            else
            {
                return this.queryParameterNames[this.parameterIndex++];
            }
        }

        return;
    },
    
    getAllParameterValues : function(paramName) 
    {
        if(arguments.length)
        {
            if(this.queryParameters[paramName] == undefined)
            {
                return null;
            }
            else
            {
                return Connector.cloneObject(this.queryParameters[paramName]);
            }
        }
        else
        {
            return this.queryParameterNames.clone();
        }

        return;
    },


    getQueryParameters : function()
    {
        return new ceStandardURLQuery(this.queryParameters);
    }
    
};



/* ceStandardURLQuery 
 * ------------------
 * The StandardURLQuery object is a helper object that should be
 * used in conjunction with StandardURL to help form URLs with
 * query strings.
 *
 * Usage:
 * 1. Create the query string object and populate with query 
 *    parametes as name value pairs.
 *
 *    var queryStringObject = new ceStandardURLQuery();
 *    if(queryStringObject)
 *    {
 *       queryStringObject.addQueryParameter("term"  , "Civil Netizen");
 *       queryStringObject.addQueryParameter("viewid", "230adf9e"     );
 *    }
 *
 * 2. Append the query string to a URL.  They StandardURLQuery object
 *    returns query string name and value parameters as escaped strings
 *
 *    var requestURL = new ceStandardURL("http://www.civilengines.com/search");
 *    if(requestURL)
 *    {
 *       requestURL.query = queryStringObject.toQueryString();
 *    }
 *
 * 3. The resulting URL will be:
 *    http://www.civilengines.com/search?term=Civil%20Netizen&viewid=230adf9e
 */
function ceStandardURLQuery(queryParameterMap)
{
    this.queryParameters = { /* map of parameter names to values */ };

    if(queryParameterMap)
    {
        this.queryParameters = queryParameterMap;
    }
 
    return;
}


ceStandardURLQuery.prototype = 
{
    queryParameters   : null,

    addQueryParameter : function(parameterName, parameterValue)
    {
        if(this.queryParameters[parameterName] == undefined) 
        {
            this.queryParameters[parameterName] = [parameterValue];
        }
        else
        {
            this.queryParameters[parameterName].push(parameterValue);
        }
        
        return;
    },
    
    getParameterValue : function(parameterName) 
    {
        if(arguments.length)
        {
            if(this.queryParameters[parameterName] == undefined)
            {
                return null;
            }
            else
            {
                return this.queryParameters[parameterName][0];
            }
        }

        return null;
    },
    
    getAllParameterValues : function(parameterName) 
    {
        if(arguments.length)
        {
            if(this.queryParameters[parameterName] == undefined)
            {
                return null;
            }
            else
            {
                return Connector.cloneObject(this.queryParameters[parameterName]);
            }
        }

        return null;
    },

    toQueryString : function()
    {
        var queryString   = "";
        
        for(var parameterName in this.queryParameters)
        {
            if(this.queryParameters[parameterName][0] == undefined)
            {
                continue;
            }
                                       
            if(queryString != "")
            {
                queryString += ("&"                                                       +
                                parameterName                                             +
                                "="                                                       +
                                encodeURIComponent(this.queryParameters[parameterName][0]));
            }
            else
            {
                queryString += (parameterName                                             +
                                "="                                                       +
                                encodeURIComponent(this.queryParameters[parameterName][0]));
            }            
        }
        
        return queryString;
    }
};
