/***** BEGIN License Block *****
 * Copyright (C) 2004, 2005 Metaha.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *         http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Portions created by Civil Engines Copyright (C) 2005 Civil Engines 
 * Research. All Rights Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com>
 *
 *
 ***** END License Block *****/



function ceTemplateConfiguration()
{
    this.statementDefs = 
    {
        "if"      : { delta:  1, prefix: "if(", suffix: "){", paramMin: 1                       },
        "else"    : { delta:  0, prefix: "} else {"                                             },
        "elseif"  : { delta:  0, prefix: "} else if(", suffix: ") {", paramDefault: "true"      },
        "/if"     : { delta: -1, prefix: "}"                                                    },
        "var"     : { delta:  0, prefix: "var ", suffix: ";"                                    },
        "for"     : { delta:  1, paramMin: 3, prefixFunc: this.forPrefix                        },
        "forelse" : { delta:  0, prefix: this.forElsePrefix, suffix: "){", paramDefault: "true" },
        "/for"    : { delta: -1, prefix: this.forEndPrefix                                      },
        "macro"   : { delta:  1, prefixFunc : this.macroPrefix                                  },
        "/macro"  : { delta: -1, prefix: this.macroEndPrefix                                    }
    };
    

    this.modifierDefs =
    {
        "eat"        : this.eatModifier       ,
        "escape"     : this.escapeModifier    ,
        "capitalize" : this.capitalizeModifier,
        "default"    : this.defaultModifier
    };
    
    return;
}


ceTemplateConfiguration.prototype =
{
    forElsePrefix  :  "} } if (__LENGTH_STACK__[__LENGTH_STACK__.length - 1] == 0) { if (",
    forEndPrefix   : "} }; delete __LENGTH_STACK__[__LENGTH_STACK__.length - 1];"         ,
    macroEndPrefix : " return __OUTPUT_array.join(''); };"                                ,
    statementTags  : "forelse|for|if|elseif|else|var|macro",

    eatModifier : function(stringParam)
    {
        return "";
    },
    

    escapeModifier : function(stringParam)
    { 
        return String(stringParam).replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;"); 
    },


    capitalizeModifier : function(stringParam)
    { 
        return String(stringParam).toUpperCase(); 
    },


    defaultModifier : function(stringParam, defaultValue)
    {
        return stringParam != null ? stringParam : defaultValue; 
    },


    forPrefix : function(statementParts, state, templateName, templateConfiguration)
    {
        if(statementParts[2] != "in")
        {
            Syslog.logMessage("Illegal 'for' statememt " + statementParts.join(' '));
            throw Results.NS_ERROR_FAILURE;
        }
        
        var iterVar = statementParts[1];
        var listVar = "__LIST__" + iterVar;

        return [ "var "                                                                    , 
                 listVar                                                                   , 
                 " = "                                                                     , 
                 statementParts[3]                                                         , 
                 ";"                                                                       ,
                 "var __LENGTH_STACK__;"                                                   ,
                 "if (typeof(__LENGTH_STACK__) == 'undefined' || !__LENGTH_STACK__.length)",
                 "__LENGTH_STACK__ = new Array();"                                         , 
                 "__LENGTH_STACK__[__LENGTH_STACK__.length] = 0;"                          , 
                 "if(("                                                                    , 
                 listVar                                                                   , 
                 ") != null) { "                                                           ,
                 "var "                                                                    , 
                 iterVar                                                                   , 
                 "_counter = 0;"                                                           ,
                 "for(var "                                                                ,
                 iterVar                                                                   ,
                 "_index in "                                                              ,
                 listVar                                                                   , 
                 ") { "                                                                    ,
                 iterVar                                                                   , 
                 "_counter++;"                                                             ,
                 "if (typeof("                                                             , 
                 listVar                                                                   ,
                 "["                                                                       ,
                 iterVar                                                                   , 
                 "_index]) == 'function') {continue;}"                                     ,
                 "__LENGTH_STACK__[__LENGTH_STACK__.length - 1]++;"                        ,
                 "var "                                                                    , 
                 iterVar                                                                   ,
                 " = "                                                                     ,
                 listVar                                                                   ,
                 "["                                                                       ,
                 iterVar                                                                   ,
                 "_index];" 
            ].join("");
    },

    macroPrefix : function(statementParts, state, templateName, templateConfiguration)
    {
        var macroName = statementParts[1].split('(')[0];

        return [ "var "                                                                    , 
                 macroName                                                                 , 
                 " = function"                                                             , 
                 statementParts.slice(1).join(' ').substring(macroName.length)             ,
                 "{var __OUTPUT_array = [];"                                               ,
                 " var __OUTPUT = { write: function(m) { if(m) __OUTPUT_array.push(m);} };" 
            ].join('');
    }
};



function ceTemplateParser()
{
}


ceTemplateParser.prototype = 
{
    parse : function(templateContent, templateName, templateConfig) 
    {
        templateContent = this.cleanWhiteSpace(templateContent);

        var endStatementPrev = -1;
        var state            = { stack: [], line: 1 };
        var functionText     = 
        [
            "var __template_function_text__ = function(__OUTPUT, __CONTEXT, __FLAGS){ with(__CONTEXT){" 
        ];

        while(endStatementPrev + 1 < templateContent.length) 
        {
            var beginStatement = endStatementPrev;

            beginStatement = templateContent.indexOf("{", beginStatement + 1);

            while(beginStatement >= 0)
            {
                var endStatement = templateContent.indexOf('}', beginStatement + 1);
                var statement    = templateContent.substring(beginStatement, endStatement);
                var blockRegex   = statement.match(/^\{(cdata|minify|eval)/);
                if(blockRegex) 
                {
                    var blockType        = blockRegEx[1]; 
                    var blockMarkerBegin = beginStatement + blockType.length + 1;
                    var blockMarkerEnd   = templateContent.indexOf('}', blockMarkerBegin);
                    
                    if(blockMarkerEnd >= 0)
                    {
                        var blockMarker;
                        
                        if(blockMarkerEnd - blockMarkerBegin <= 0)
                        {
                            blockMarker = "{/" + blockType + "}";
                        } 
                        else 
                        {
                            blockMarker = templateContent.substring(blockMarkerBegin + 1, 
                                                                    blockMarkerEnd      );
                        }                        
                        
                        var blockEnd = templateContent.indexOf(blockMarker, blockMarkerEnd + 1);
                        if(blockEnd >= 0) 
                        {
                            var substringText = templateContent.substring(endStatementPrev + 1, 
                                                                          beginStatement      );
                            
                            this.emitSectionText(substringText, functionText);
                            
                            var blockText = templateContent.substring(blockMarkerEnd + 1, blockEnd);
                            if(blockType == 'cdata') 
                            {
                                this.emitText(blockText, functionText);
                            } 
                            else if(blockType == 'minify')
                            {
                                this.emitText(this.scrubWhiteSpace(blockText), functionText);
                            } 
                            else if(blockType == 'eval')
                            {
                                if(blockText != null && blockText.length > 0)
                                {
                                    var functionOutput = "__OUTPUT.write( (function() { " + 
                                                         blockText                        + 
                                                         " })() );";
                                    
                                    functionText.push(functionOutput);
                                }
                            }

                            endStatementPrev = blockEnd + blockMarker.length - 1;
                            beginStatement   = endStatementPrev;
                        }
                    }
                    
                } 
                else if(templateContent.charAt(beginStatement - 1) != '$' &&
                        templateContent.charAt(beginStatement - 1) != '\\' ) 
                {
                    /* Closing tag name is offset at 2 to skip "/" */
                    var offset        = (templateContent.charAt(beginStatement + 1) == '/' ? 2 : 1);

                    /* 10 is larger than the maximum statement tag length */
                    var substringText = templateContent.substring(beginStatement + offset     , 
                                                                  beginStatement + 10 + offset);
                    
                    if(substringText.search(templateConfig.statementTags) == 0)
                    {
                        break;
                    }
                }

                beginStatement = templateContent.indexOf("{", beginStatement + 1);
            }

            if(beginStatement < 0)
            {
                break;
            }
            
            var endStatement = templateContent.indexOf("}", beginStatement + 1);
            if(endStatement < 0)
            {
                break;
            }
            
            this.emitSectionText(templateContent.substring(endStatementPrev + 1, beginStatement), 
                                 functionText                                                   );

            this.emitStatement(templateContent.substring(beginStatement, endStatement + 1), 
                               state                                                      ,
                               functionText                                               ,
                               templateName                                               ,
                               templateConfig                                             );

            endStatementPrev = endStatement;
        }

        this.emitSectionText(templateContent.substring(endStatementPrev + 1), functionText);
        if(state.stack.length != 0)
        {
            Syslog.logMessage("Unmatched Statements : " + templateName );
            Syslog.logMessage("  line      => " + state.line           );
            Syslog.logMessage("  statement => " + state.stack.join(","));

            throw Results.NS_ERROR_FAILURE;
        }
        
        functionText.push("}}; __template_function_text__");

        return functionText.join("");
    },
    

    emitStatement : function(statementString, state, functionText, templateName, templateConfig) 
    {
        var parts     = statementString.slice(1, -1).split(' ');
        var statement = templateConfig.statementDefs[parts[0]];

        if(statement == null)
        {
            this.emitSectionText(statementString, functionText);
            return;
        }

        if(statement.delta < 0)
        {
            if(state.stack.length <= 0)
            {
                Syslog.logMessage("Closing Tag Mismatch : " + templateName);
                Syslog.logMessage("  line      => " + state.line     );
                Syslog.logMessage("  statement => " + statementString);
                
                throw Results.NS_ERROR_FAILURE;
            }
            
            state.stack.pop();
        } 

        if(statement.delta > 0)
        {
            state.stack.push(statementString);
        }
        
        if(statement.paramMin != null && statement.paramMin >= parts.length)
        {
            Syslog.logMessage("Missing Parameters : " + templateName);
            Syslog.logMessage("  line      => " + state.line     );
            Syslog.logMessage("  statement => " + statementString);

            throw Results.NS_ERROR_FAILURE;
        }
        
        if(statement.prefixFunc != null)
        {
            functionText.push(statement.prefixFunc(parts, state, templateName, templateConfig));
        }
        else
        { 
            functionText.push(statement.prefix);
        }
        
        if(statement.suffix != null) 
        {
            if(parts.length <= 1) 
            {
                if(statement.paramDefault != null)
                {
                    functionText.push(statement.paramDefault);
                }    
            }
            else 
            {
                for(var i = 1; i < parts.length; i++)
                {
                    if(i > 1)
                    {
                        functionText.push(' ');
                    }
                    
                    functionText.push(parts[i]);
                }
            }
            
            functionText.push(statement.suffix);
        }

        return;
    },


    emitSectionText : function(text, functionText) 
    {
        if(text.length <= 0)
        {
            return;
        }
        
        /* nlPrefix => index to the first non-newline in the prefix
         * nlSuffix => index to the first non-space/tab in the suffix
         */
        var nlPrefix = 0;
        var nlSuffix = text.length - 1;

        while(nlPrefix < text.length && (text.charAt(nlPrefix) == '\n'))
        {
            nlPrefix++;
        }
        
        while(nlSuffix >= 0 && (text.charAt(nlSuffix) == ' ' || text.charAt(nlSuffix) == '\t'))
        {
            nlSuffix--;
        }
        
        if(nlSuffix < nlPrefix)
        {
            nlSuffix = nlPrefix;
        }
        
        if(nlPrefix > 0)
        {
            functionText.push('if (__FLAGS.keepWhitespace == true) __OUTPUT.write("');
            var substringText = text.substring(0, nlPrefix).replace('\n', '\\n');

            if(substringText.charAt(substringText.length - 1) == '\n')
            {
            	substringText = substringText.substring(0, substringText.length - 1);
            }
            
            functionText.push(substringText);
            functionText.push('");');
        }

        var lines = text.substring(nlPrefix, nlSuffix + 1).split('\n');
        for(var i = 0; i < lines.length; i++) 
        {
            this.emitSectionTextLine(lines[i], functionText);
            if(i < lines.length - 1)
            {
                functionText.push('__OUTPUT.write("\\n");\n');
            }
        }

        if(nlSuffix + 1 < text.length)
        {
            functionText.push('if(__FLAGS.keepWhitespace == true) __OUTPUT.write("');

            var substringText = text.substring(nlSuffix + 1).replace('\n', '\\n');
      
            if(substringText.charAt(substringText.length - 1) == '\n')
            {
            	substringText = substringText.substring(0, substringText.length - 1);
            }
            
            functionText.push(substringText);
            functionText.push('");');
        }
    
        return;
    },


    emitSectionTextLine : function(line, functionText)
    {
        var endMarkPrev = '}';
        var endExprPrev = -1;
    
        while(endExprPrev + endMarkPrev.length < line.length)
        {
            var beginMark = "${";
            var endMark   = "}";
            var beginExpr = line.indexOf(beginMark, endExprPrev + endMarkPrev.length);
            if(beginExpr < 0)
            {
                break;
            }
            
            if(line.charAt(beginExpr + 2) == '%') 
            {
                beginMark = "${%";
                endMark   = "%}";
            }

            var endExpr = line.indexOf(endMark, beginExpr + beginMark.length);
            if(endExpr < 0)
            {
                break;
            }
            
            this.emitText(line.substring(endExprPrev + endMarkPrev.length, beginExpr), functionText);

            /* Example: expressionArray == 'firstName|default:"John Doe"|capitalize'.split('|') */
            var expressionArray = 
                line.substring(beginExpr + beginMark.length, endExpr).replace(/\|\|/g, "#@@#").split('|');
            
            for(var expression in expressionArray) 
            {
                if(expressionArray[expression].replace)
                {
                    expressionArray[expression] = expressionArray[expression].replace(/#@@#/g, '||');
                }
            }
            
            functionText.push('__OUTPUT.write(');
            this.emitExpression(expressionArray, expressionArray.length - 1, functionText); 
            functionText.push(');');
            
            endExprPrev = endExpr;
            endMarkPrev = endMark;
        }
        
        this.emitText(line.substring(endExprPrev + endMarkPrev.length), functionText); 
        return;
    },


    emitText : function(text, functionText)
    {
        if(text == null || text.length <= 0)
        {
            return;
        }
        
        text = text.replace(/\\/g, '\\\\');
        text = text.replace(/\n/g, '\\n' );
        text = text.replace(/\"/g,  '\\"');

        functionText.push('__OUTPUT.write("');
        functionText.push(text);
        functionText.push('");');
        
        return;
    },


    emitExpression : function(expressionArray, index, functionText)
    {
        /* Example: foo|a:x|b:y1,y2|c:z1,z2 is emitted as c(b(a(foo,x),y1,y2),z1,z2)
         *          expressionArray  == [firstName,capitalize,default:"John Doe"]
         *          expression       == 'default:"John Doe"'
         */
        var expression = expressionArray[index];
        if(index <= 0)
        {
            functionText.push(expression);
            return;
        }

        var parts = expression.split(':');

        functionText.push('__MODIFIERS__["');
        functionText.push(parts[0]);
        functionText.push('"](');

        this.emitExpression(expressionArray, index - 1, functionText);
        if(parts.length > 1)
        {
            functionText.push(',');
            functionText.push(parts[1]);
        }

        functionText.push(')');
        return;
    },


    cleanWhiteSpace : function(result)
    {
        result = result.replace(/\t/g  , "    ");
        result = result.replace(/\r\n/g, "\n"  );
        result = result.replace(/\r/g  , "\n"  );
        result = result.replace(/^(\s*\S*(\s+\S+)*)\s*$/, '$1');

        return result;
    },

    scrubWhiteSpace : function(result) 
    {
        result = result.replace(/^\s+/g, "" );
        result = result.replace(/\s+$/g, "" );
        result = result.replace(/\s+/g , " ");
        result = result.replace(/^(\s*\S*(\s+\S+)*)\s*$/, '$1');

        return result;
    }
};




function ceTemplateObject(templateName, templateContent, functionText, functionScript, templateConfig)
{
    this.name           = templateName;
    this.source         = templateContent; 
    this.functionText   = functionText;
    this.functionScript = functionScript;
    this.templateConfig = templateConfig;
    
    return;
}


ceTemplateObject.prototype = 
{
    process : function(context, flags) 
    {
        if(context == null)
        {
            context = {};
        }
        
        if(context.__MODIFIERS__ == null)
        {
            context.__MODIFIERS__ = {};
        }
        
        
        if(context.defined == null)
        {
            context.defined = function(variable) 
            { 
                return (context[variable] != undefined); 
            };
        }
        
        for(var modifier in this.templateConfig.modifierDefs) 
        {
            if(context.__MODIFIERS__[modifier] == null)
            {
                context.__MODIFIERS__[modifier] = this.templateConfig.modifierDefs[modifier];
            }
        }

        /* Default Flags
         * ------------- 
         * flags.keepWhiteSpace  = true/false;
         * flags.throwExceptions = true/false;
         */
        if(flags == null)
        {
            flags = {};
        }
        
        var resultArray  = [];
        var resultOutput = 
        { 
            write: function(resultObject) { resultArray.push(resultObject); } 
        };
        

        try 
        {
            this.functionScript(resultOutput, context, flags);
        } 
        catch(exception) 
        {
            Syslog.logException(exception);

            if(flags.throwExceptions == true)
            {
                throw exception;
            }
            
            return;
        }
        
        return resultArray.join("");
    }
}



function ceTemplateEngine()
{
    this.templateParserConfig = new ceTemplateConfiguration();
    this.templateParser       = new ceTemplateParser();

    String.prototype.templateProcess = function(context, flags) 
    {
        if(TemplateEngine)
        {
            var template = TemplateEngine.parseTemplate(this, null);
            if(template != null)
            {
                return template.process(context, flags);
            }
            
            return this;
        }
    };
    
    return;     
}



ceTemplateEngine.prototype =
{
    parseTemplate : function(templateContent, templateName, templateConfig)
    {
        if(templateConfig == null)
        {
            templateConfig = this.templateParserConfig;
        }
        
        var functionText   = this.templateParser.parse(templateContent, templateName, templateConfig);
        var functionScript = eval(functionText, templateName, 1);
        if(!functionScript)
        {
            return null;
        }
        
        return new ceTemplateObject(templateName   , 
                                    templateContent, 
                                    functionText   , 
                                    functionScript , 
                                    templateConfig );
    },


    parseTemplateById : function(elementId, documentContext, templateConfig) 
    {
        if(documentContext == null)
        {
            documentContext = document;
        }
        
        var element = documentContext.getElementById(elementId);
        var content = null;        
        if(content == null)
        {
            /* 1. Assume that the element is an HTML element and
             *    the template content is contained as an innerHTML
             */
            content = element.innerHTML;
        }
        
        if(content == null)
        {
            /* 2. Assume that element is the parent XML element with 
             *    a CDATA section containing the template content
             */
            var cdataElement = element.firstChild;
            if(cdataElement)
            {
                content = cdataElement.data;
            }            
        }

        if(content == null)
        {
            /* 3. Assume the element is a XUL label or description 
             *    element and try retrieving the content from the value
             *    property
             */
            content = element.value;
        }

        content = content.replace(/&lt;/g, "<").replace(/&gt;/g, ">");
        Syslog.logMessage("content = " + content);
        
        return this.parseTemplate(content, elementId, templateConfig);
    },

    
    processTemplateById : function(elementId, context, documentContext, flags, templateConfig) 
    {
        var template = this.parseTemplateById(elementId, documentContext, templateConfig);
        if(template)
        {
            return template.process(context, flags);    
        }

        return null;
    },

    processTemplate : function(templateContent, context, flags, templateName, templateConfig) 
    {
        var template = this.parseTemplate(templateContent, templateName, templateConfig);
        if(template)
        {
            return template.process(context, flags);    
        }

        return null;
    }
};



var TemplateEngine = new ceTemplateEngine();
