/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is the mozilla.org code
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation. Portions created by the Initial Developer are Copyright
 * (C) 1998 the Initial Developer. All Rights Reserved.
 *
 * Portions created by Civil Engines Research LLC are Copyright (C)
 * 2001-2003 Civil Engines Research.  All Rights 
 * Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Ben Goodger <ben@netscape.com> (Original Author)
 *   Paolo de Dios <paolo.dedios@civilengines.com>
 *
 *
 ***** END License Block *****/


function nsLocalPreferences()
{
    this.prefService = Components.classes["@mozilla.org/preferences-service;1"]
        .getService(Components.interfaces.nsIPrefBranch2);
}


nsLocalPreferences.prototype = 
{
    prefService : null,

    getDefaultBranch : function(branchRoot)
    {
        var prefService = this.prefService.QueryInterface(Components.interfaces.nsIPrefService);
        return prefService.getDefaultBranch(branchRoot);
    },
    
    getPrefType : function(prefString)
    {
        var prefType = null;
        
        try
        {
            this.prefService.GetPrefType(prefString);
        }
        catch(exception)
        {
            return "undefined";
        }
        
        return prefType;
    },
    
    setBoolPref: function(prefName, prefValue)
    {
        try 
        {
            this.prefService.setBoolPref(prefName, prefValue);
        }
        catch(exception)
        {
            return Components.results.NS_ERROR_FAILURE;
        }

        return  Components.results.NS_OK;
    },
  

    getBoolPref: function(prefName, defVal)
    {
        try
        {
            return this.prefService.getBoolPref(prefName);
        }
        catch(exception)
        {
            return defVal != undefined ? defVal : null;
        }

        return null;
    },
  

    setUnicharPref: function(prefName, prefValue)
    {
        try
        {
            var str = Components.classes["@mozilla.org/supports-string;1"]
                .createInstance(Components.interfaces.nsISupportsString);

            str.data = prefValue;
            this.prefService.setComplexValue(prefName                               ,
                                             Components.interfaces.nsISupportsString, 
                                             str                                    );
        }
        catch(exception)
        {
        }
    },
  
    copyUnicharPref: function(prefName, defaultValue)
    {
        try
        {
            return this.prefService.getComplexValue(prefName,
                                                    Components.interfaces.nsISupportsString).data;
        }
        catch(exception)
        {
            return defaultValue != undefined ? defaultValue : null;
        }

        return null;
    },
    
    setIntPref: function(prefName, prefValue)
    {
        try
        {
            this.prefService.setIntPref(prefName, prefValue);
        }
        catch(exception)
        {
            return Components.results.NS_ERROR_FAILURE;
        }

        return Components.results.NS_OK;
    },
    
    getIntPref: function(prefName, defaultValue)
    {
        try
        {
            return this.prefService.getIntPref(prefName);
        }
        catch(exception)
        {
            return defaultValue != undefined ? defaultValue : null;
        }

        return null;
    },
    
    getLocalizedUnicharPref: function(prefName, defaultValue)
    {
        try
        {
            return this.prefService.getComplexValue(prefName                                    ,
                                                    Components.interfaces.nsIPrefLocalizedString).data;
        }
        catch(exception)
        {
            return defaultValue != undefined ? defaultValue : null;
        }

        return null;
    },

    
    getCharPref : function(prefName, defaultValue)
    {
        try
        {
            return this.prefService.getCharPref(prefName);
        }
        catch(exception)
        {
            return defaultValue != undefined ? defaultValue : null;
        }
    },

    resetBranch : function(startingAt)
    {
        try
        {
            this.prefService.resetBranch(startingAt);
        }
        catch(exception)
        {
            return Components.results.NS_ERROR_FAILURE;
        }

        return Components.results.NS_OK;
    },


    clearPref : function(prefString)
    {
        try
        {
            this.prefService.clearUserPref(prefString);
        }
        catch(exception)
        {
            return Components.results.NS_ERROR_FAILURE;
        }

        return Components.results.NS_OK;
    },

    
    save : function(fileObject)
    {
        var fileResource = null;
        switch(typeof(fileObject))
        {
          case "object" :
          {
              if(typeof(fileObject.path) == "string")
              {
                  fileResource = fileObject;
                  break;
              }
          }
          case "string" :
          {
              fileResource = new ceFileObject(fileObject).fileObject;
              break;
          }
        }
        
        try
        {
            this.prefService.savePrefFile(fileObject);
        }
        catch(exception)
        {
            return Components.results.NS_ERROR_FAILURE;
        }
        
        return Components.results.NS_OK;
    },

    lockPref : function(prefString)
    {
        try
        {
            this.prefService.lockPref(prefString);
        }
        catch(exception)
        {
            return Components.results.NS_ERROR_FAILURE;
        }
        
        return Components.results.NS_OK;
    },


    unlockPref : function(prefString)
    {
        try
        {
            this.prefService.unlockPref(prefString);
        }
        catch(exception)
        {
            return Components.results.NS_ERROR_FAILURE;
        }
        
        return Components.results.NS_OK;
    },


    prefHasUserValue : function(prefString)
    {
        try
        {
            return this.prefService.prefHasUserValue(prefString);
        }
        catch(exception)
        {
            return Components.results.NS_ERROR_FAILURE;
        }
        
        return false;
    },

    
    prefIsLocked : function(prefString)
    {
        try
        {
            return this.prefService.prefIsLocked(prefString);
        }
        catch(exception)
        {
            return Components.results.NS_ERROR_FAILURE;
        }
        
        return false;
    }
};


var LocalPreferences = new nsLocalPreferences();
