/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2005 Civil Engines Research.  All Rights 
 * Reserved.  
 *
 * Portions created by Netscape Copyright (C) 1998 Netscape 
 * Communications Corporation. All Rights Reserved.
 *
 * Portions created by the Mozilla Foundation are Copyright (C) 
 * 2003 Mozilla Foundation. All Rights Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Ben Goodger <ben@netscape.com> (Original Author)
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/


/* For panels displayed inside the main dialog, this value will be the child iframe
 * window. For panels displayed in sub-dialogs, those launched by panels of the main
 * dialog, this will be the sub dialog window.
 */ 
var gCurrentWindow    = null;
var gWidgetManager    = null;


/* nsWidgetStateManager(string frameID)
 * 
 * object for managing data in iframe multi-panel dialogs. The frameID parameter is
 * the string name for the default content area used by a particular instance of the
 * widget state manager
 */
function nsWidgetStateManager(aFrameID)
{
    this.dataManager = 
    {
        /* Persisted Data Hash Table
         *  PageID -> ElementID -> Property -> Value
         */
        pageData    : { },
        
        setPageData : function(aPageTag, aDataObject)
        {
            this.pageData[aPageTag] = aDataObject;
        },

        /* Retrieve the hastable for the given page tag */
        getPageData : function(aPageTag)
        {
            if(!(aPageTag in this.pageData))
            {
                this.pageData[aPageTag] = new Object();
            }
            
            if(!('elementIDs' in this.pageData[aPageTag]))
            {
                this.pageData[aPageTag].elementIDs = new Object();
            }
            
            if(!('userData' in this.pageData[aPageTag]))
            {
                this.pageData[aPageTag].userData = new Object();
            }
            
            return this.pageData[aPageTag];
        },

        /* Save the element data for the given page tag */
        setItemData : function(aPageTag, aItemID, aDataObject)
        {
            if(!(aPageTag in this.pageData))
            {
                this.pageData[aPageTag] = new Object();
            }
            
            if(!('elementIDs' in this.pageData[aPageTag]))
            {
                this.pageData[aPageTag].elementIDs = new Object();
            }
            
            if(!('userData' in this.pageData[aPageTag]))
            {
                this.pageData[aPageTag].userData = new Object();
            }

            this.pageData[aPageTag].elementIDs[aItemID] = aDataObject;
        },
        
        /* Retrieve the element data for the given page tag */
        getItemData : function(aPageTag, aItemID)
        {
            if(!(aItemID in this.pageData[aPageTag].elementIDs))
            {
                this.pageData[aPageTag].elementIDs[aItemID] = new Object();
            }
            
            return this.pageData[aPageTag].elementIDs[aItemID];
        }
    };
    

    this.contentID    = aFrameID;
    gWidgetManager    = this;

    /* Element Handlers
     * ----------------
     * Provides default get and set handler functions for supported
     * widgets. Clients can override or add new widgets.
     */
    this.handlers     = 
    {
        menulist        : {  get: this.getMenulist  ,  set: this.setMenulist   },
        radiogroup      : {  get: this.getRadiogroup,  set: this.setRadiogroup },
        checkbox        : {  get: this.getCheckbox  ,  set: this.setCheckbox   },
        textbox         : {  get: this.getTextbox   ,  set: this.setTextbox    },
        listitem        : {  get: this.getListitem  ,  set: this.setListitem   },
        data            : {  get: this.getData      ,  set: this.setData       },
        default_handler : {  get: this.getDefault   ,  set: this.setDefault    }
    };
    
    /* Extra attributes to scan and save. */
    this.attributes   = [];
}


/* nsWidgetStateManager saves widget state for multi-page windows/dialogs/etc. 
 * To use nsWidgetStateManager, the window must contain panels in an <iframe> or
 * <browser>.
 * 
 * nsWidgetStateManager.savePageData() => Save widget information given
 * nsWidgetStateManager.setPageData()  => Set widget information from saved data
 */
nsWidgetStateManager.prototype =
{
    /* Getter property for the target content area in the dom window */
    get contentArea()
    {
        return window.frames[this.contentID];
    },

    containsPageData : function(aPageTag)
    {
        return (aPageTag in this.dataManager.pageData);
    },

    /* Save page data within the window under a page tag name. If a window
     * is not provided, the current content area is assumed to be the
     * target. The page tag is a unique identifier, perhaps a URI, for the
     * page.
     */
    savePageData : function(aPageTag, aWindow)
    {
        gCurrentWindow = aWindow || this.contentArea;
        
        /* Explicit element retrieval: 
         * 1. If the GetFields function is provided for the current window's
         * JS Context then use that function to retrieve elements that
         * should be persisted
         */
        if("GetFields" in gCurrentWindow) 
        {
            /* save page data based on user supplied function in content area */
            var dataObject = gCurrentWindow.GetFields();
            if(dataObject)
            {        
                this.dataManager.pageData[aPageTag].userData = dataObject;
            }
        }
                
        /* Automatic element retrieval. 
         * 1) If an element id array is present in the document, this is
         *    used to build a list of elements to persist.
         * 2) Otherwise, all elements with "wsm_persist" set to "true"
         *    are persisted
         */
        var elements;
        if("_elementIDs" in gCurrentWindow) 
        {
            elements = [];
            for (var i = 0; i < gCurrentWindow._elementIDs.length; i++) 
            {
                var elt = gCurrentWindow.document.getElementById(gCurrentWindow._elementIDs[i]);
                if(elt)
                {
                    elements[elements.length] = elt;
                }
                else 
                {
                    /* Bug #40329. People forget this too often */
                    dump("-*-*-*-*: ERROR: '_elementIDs' in '"                   + 
                         gCurrentWindow.location.href.split('/').pop()           +
                         "' contains a reference to a non-existent element ID '" +
                         gCurrentWindow._elementIDs[i] + "'.\n"                  );
                }
            }
        }
        else
        { 
            elements = gCurrentWindow.document.getElementsByAttribute("wsm_persist", "true");
            if(elements.length < 1)
            {
                elements = gCurrentWindow.document.getElementsByAttribute("wsmpersist", "true");
            }
        }
        
        /* Save the element data using the default or provided handlers */
        for(var ii = 0; ii < elements.length; ii++) 
        {
            var elementID   = elements[ii].id;
            var elementType = elements[ii].localName;
            
            var getFunction = (elementType in this.handlers) ?
                this.handlers[elementType].get : this.handlers.default_handler.get;
            
            this.dataManager.setItemData(aPageTag, elementID, getFunction(elementID));
        }
    },
    
    /* Populate a content area window with the contents of the widget
     * state data identified by the page tag name/id.
     */
    setPageData : function(aPageTag, aWindow)
    {
        gCurrentWindow = aWindow || this.contentArea;
        
        var pageData = this.dataManager.getPageData(aPageTag);
     
        /* If a user defined function is provided to set field
         * information for the content area, then use it
         */
        if("SetFields" in gCurrentWindow)
        {
            gCurrentWindow.SetFields(pageData.userData);
        }

        /* The elementIDs array must be initialized before
         * this function will work
         */
        if(!('elementIDs' in pageData))
        {
            return;
        }
        
        /* Set the widget state data in each page using the 
         * default handler
         */
        for(var elementID in pageData.elementIDs) 
        {
            var element = gCurrentWindow.document.getElementById(elementID);   
            if(element) 
            {
                var elementType = element.localName;
                var setFunction = (elementType in this.handlers) ?
                    this.handlers[elementType].set : this.handlers.default_handler.set;
                
                setFunction(elementID, pageData.elementIDs[elementID]);
            }
        }
    },


    /* Widget Get/Set Function Implementations these can be overridden
     * by the client.  These generic getters and setters are used by
     * the default widget handlers.
     */
    genericSet : function(aElement, aDataObject)
    {
        if(aElement) 
        {
            for(var property in aDataObject) 
            {
                if(property == "localname")
                {
                    continue;
                }
                if(!aDataObject[property] && typeof aDataObject[property] == "boolean")
                {
                    aElement.removeAttribute(property);
                }
                else
                {
                    var data = aDataObject[property];
                    
                    /* prevent text boxes from displaying 'undefined' */
                    if(property == "value" && (!data || data == "undefined"))
                    {
                        continue;
                    }
                    
                    aElement.setAttribute(property, data);
                }
            }
            
            if(!aElement.getAttribute("disabled", "true"))
            {
                aElement.removeAttribute("disabled");
            }
        }
    },

    
    genericGet : function(aElement)
    {
        if(aElement) 
        {
            var dataObject    = new Object();
            var wsmAttributes = aElement.getAttribute("wsm_attributes");
            var attributes    = gWidgetManager.attributes;
            
            if(wsmAttributes != "")
            {
                /* Expect a single space separated list of attributes to persist */
                attributes.push(wsmAttributes.split(" "));
            }
            else
            {
                wsmAttributes = aElement.getAttribute("wsmattributes");
                if(wsmAttributes != "")
                {
                    /* Expect a comma separated list of attributes to persist*/
                    var wsmList = wsmAttributes.split(",");
                    for(var count = 0; count < wsmList.length; ++count)
                    {
                        attributes.push(wsmList[count].replace(/ /g, ""));
                    }
                }
            }
            
            for(var i = 0; i < attributes.length; i++)
            {
                dataObject[attributes[i]] = aElement.getAttribute(attributes[i]);
            }
            
            dataObject.localname = aElement.localName;
            return dataObject;
        }

        return null;
    },


    setMenulist : function(aElementID, aDataObject)
    {
        var element = gCurrentWindow.document.getElementById(aElementID);
        
        gWidgetManager.genericSet(element, aDataObject);
       
        if("value" in aDataObject) 
        {
            try 
            {
                element.value = aDataObject.value;
            }
            catch(exception) 
            {
                dump(aElementID + ", exception: " + exception + "\n");
            }
        }
    },


    getMenulist : function(aElementID)
    {
        var element     = gCurrentWindow.document.getElementById(aElementID);
        var dataObject  = gWidgetManager.genericGet(element);
        
        if(dataObject) 
        {
            dataObject.value = element.getAttribute("value");
            return dataObject;
        }

        return null;
    },


    setRadiogroup : function(aElementID, aDataObject)
    {
        var element = gCurrentWindow.document.getElementById(aElementID);
        gWidgetManager.genericSet(element, aDataObject);
        if("value" in aDataObject)
        {
            element.selectedItem = aDataObject.value;
        }

        if("disabled" in aDataObject)
        {
            element.disabled = aDataObject.disabled;
        }
    },


    getRadiogroup : function(aElementID)
    {
        var element    = gCurrentWindow.document.getElementById(aElementID);
        var dataObject = gWidgetManager.genericGet(element);
        if(dataObject) 
        {
            dataObject.value = element.getAttribute("value");
            return dataObject;
        }
       
        return null;
    },


    setTextbox : function(aElementID, aDataObject)
    {
        var element = gCurrentWindow.document.getElementById(aElementID);
        gWidgetManager.genericSet(element, aDataObject);
    },


    getTextbox : function(aElementID)
    {
        var element    = gCurrentWindow.document.getElementById(aElementID);
        var dataObject = gWidgetManager.genericGet(element);
        if(dataObject) 
        {
            dataObject.value = element.value;
            return dataObject;
        }

        return null;
    },


    setCheckbox : function(aElementID, aDataObject)
    {
        var element = gCurrentWindow.document.getElementById(aElementID);
        gWidgetManager.genericSet(element, aDataObject);
        
        if("checked" in aDataObject && element.hasAttribute("reversed"))
        {
            element.checked = !aDataObject.checked; 
        }
    },
    

    getCheckbox : function(aElementID)
    {
        var element    = gCurrentWindow.document.getElementById(aElementID);
        var dataObject = gWidgetManager.genericGet(element);
        if(dataObject) 
        {
            var checked = element.checked;
            dataObject.checked = element.getAttribute("reversed") == "true" ? !checked : checked;
            return dataObject;
        }
        return null;
    },


    setListitem : function(aElementID, aDataObject)
    {
        var element = gCurrentWindow.document.getElementById(aElementID);
        gWidgetManager.genericSet(element, aDataObject);
    },


    getListitem : function(aElementID)
    {
        var element    = gCurrentWindow.document.getElementById(aElementID);
        var dataObject = gWidgetManager.genericGet(element);
        if(dataObject) 
        {
            if(element.getAttribute("type") == "checkbox")
            {
                dataObject.checked = element.checked;
            }

            return dataObject;
        }
        return null;
    },


    setData : function(aElementID, aDataObject)
    {
        var element = gCurrentWindow.document.getElementById(aElementID);
        gWidgetManager.genericSet(element, aDataObject);
        if("value" in aDataObject)
        {
            element.setAttribute("value", aDataObject.value);
        }
    },
    
    getData : function(aElementID)
    {
        var element = gCurrentWindow.document.getElementById(aElementID);
        var dataObject = gWidgetManager.genericGet(element);
        if(dataObject) 
        {
            dataObject.value = element.getAttribute("value");
            return dataObject;
        }
        return null;
    },
    
    
    setDefault : function(aElementID, aDataObject)
    {
        var element = gCurrentWindow.document.getElementById(aElementID);
        gWidgetManager.genericSet(element, aDataObject);
    },


    getDefault : function(aElementID)
    {
        var element    = gCurrentWindow.document.getElementById(aElementID);
        var dataObject = gWidgetManager.genericGet(element);
        
        return dataObject ? dataObject : null;
    }
};
