/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is the Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2003 Civil Engines Research.  All Rights 
 * Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/

Number.max = function(x, y) 
{
    return (x < y) ? y : x;
}


Number.min = function(x, y) 
{
    return (x > y) ? y : x;
}


Math.mod = function(val, mod) 
{
    if(val < 0)
    {
        while(val < 0)
        {
            val += mod;
        }
        
        return val;
    } 
    else 
    {
        return val % mod;
    }
}



Array.prototype.clone = function()
{
    var i;
    var arrayCopy = [];
    
    for(i = 0; i < this.length; ++i) 
    {
        if (typeof this[i] == "object")
        {
            arrayCopy[i] = this[i].clone();
        }
        else
        {
            arrayCopy[i] = this[i];
        }
    }
    
    return arrayCopy;
}



Array.prototype.arrayIndexOf = function(element)
{
    for(var index in this)
    {
        if(this[index] == element)
        {
            return index;
        }
    }
    
    return -1;
}


Array.prototype.contains = function(element)
{
    return (this.arrayIndexOf(element) != -1);
}



Array.prototype.insertAt = function(index, obj)
{
    this.splice(index, 0, obj);
    return;
}



Array.prototype.removeAt = function(index)
{
    this.splice(index, 1);
    return;
}


Array.prototype.foldr = function(func, start) 
{
    var target = start;

    for(var i = this.length - 1; i > -1; i--) 
    {
        target = func(this[i], target);
    }

    return target;
}


Array.prototype.foldl = function(func, start) 
{
    var target = start;

    for(var i = 0; i < this.length; i++) 
    {
        target = func(this[i], target);
    }

    return target;
}


Array.prototype.exists = function(element) 
{
    for(var i = 0; i < this.length; i++) 
    {
        if(this[i] == element)
        {
            return true;
        }
    }

    return false;
}


Array.prototype.filter = function(func) 
{
    var result = new Array();

    for(var i = 0; i < this.length; i++) 
    {
        if(func(this[i]))
        {
            result.push(this[i]);
        }
    }

    return result;
}


Array.prototype.random = function() 
{
    return this[Math.floor((Math.random() * this.length))];
}


window.getInnerWidth = function() 
{
    if(window.innerWidth) 
    {
        return window.innerWidth;
    } 
    else if(document.body.clientWidth)
    {
        return document.body.clientWidth;
    } 
    else if(document.documentElement.clientWidth) 
    {
        return document.documentElement.clientWidth;
    }

    return;
} 



window.getInnerHeight = function() 
{
    if(window.innerHeight)
    {
        return window.innerHeight;
    } 
    else if(document.body.clientHeight) 
    {
        return document.body.clientHeight;
    } 
    else if(document.documentElement.clientHeight) 
    {
        return document.documentElement.clientHeight;
    }

    return;
} 



function roundNumberTo(number, precision)
{
    return parseInt( Math.round(number * Math.pow(10, precision))) / Math.pow(10, precision);   
}



function randomRange(min, max)
{
    if(typeof(min) == "undefined")
    {
        min = 0;
    }
    
    if(typeof(max) == "undefined")
    {
        max = 1;
    }
    
    return (parseInt(Math.round((Math.random() * (max - min)) + min )));
}



function isBoolean(unknownObject)
{
    return typeof(unknownObject) == "boolean";
}


function isString(unknownObject)
{
    return (typeof(unknownObject) == "string");
}


function isNumber(unknownObject)
{
    return (typeof(unknownObject) == "number" && isFinite(unknownObject));
}


function isFunction(unknownObject)
{
    return typeof(unknownObject) == "function";
}


function isObject(unknownObject)
{
    return (unknownObject && typeof(unknownObject) == "object");
}


function isArray(unknownObject)
{
    return (isObject(unknownObject) && unknownObject.constructor == Array);
}


function isNull(unknownObject)
{
    return (typeof(unknownObject) == "object" && !unknownObject);
}


function isUndefined(unknownObject)
{
    return (typeof(unknownObject) == "undefined");
}




function gatherDescendentText(rootElement) 
{
    var text = "";
    var node = rootElement.firstChild;
    var depth = 1;

    while(node && depth > 0) 
    {
        /* See if this node is text */
        if(node.nodeName == "#text") 
        {
            /* Add this text to our collection. */
            text += " " + node.data;
        } 
        else if(node.nodeType == Node.ELEMENT_NODE    &&
                node.localName.toUpperCase() == "IMG"  ) 
        {
            /* If it has an ALT attribute, use that. */
            var altText = node.getAttribute( "alt" );
            if(altText && altText != "") 
            {
                text = altText;
                break;
            }
        }

        if(node.hasChildNodes()) 
        {
            node = node.firstChild;
            depth++;
        } 
        else 
        {
            if(node.nextSibling) 
            {
                node = node.nextSibling;
            } 
            else 
            {
                node = node.parentNode.nextSibling;
                depth--;
            }
        }
    }
    
    /* Strip leading whitespace  */
    text = text.replace( /^\s+/, "" );

    /* Strip trailing whitespace */
    text = text.replace( /\s+$/, "" );

    /* Compress remaining whitespace */
    text = text.replace( /\s+/g, " " );

    return text;
}
