/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is the Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2003 Civil Engines Research.  All Rights 
 * Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/


/* String Formatting
 * ----------------- 
 * var copyrightString = "Copyright %S %S. All Rights Reserved";
 * var result          = copyrightString.substitute(2004, "Civil Engines"); 
 */
String.prototype.substitute = function(parameters)
{
    var argCount     = arguments.length;
    var resultString = '';
    var formatSet    = this.split('%S');
 
    for(var i = 0, setlen = formatSet.length; i < setlen; i++)
    {
        resultString += formatSet[i] +
            (((i < argCount) && (arguments[i] != undefined)) ? arguments[i] : '');
    }
    
    return resultString;
}



/* Indexed String Formatting
 * ------------------------- 
 * var copyrightString = "Copyright #1 #2. All Rights Reserved";
 * var result          = copyrightString;
 * result = result.replaceIndex(1, 2004); 
 * result = result.replaceIndex(2, "Civil Engines");
 */
String.prototype.replaceIndex = function(index, value)
{
    var indexRegExp = new RegExp("#" + index);
    var resultText  = this;
    
    resultText = resultText.replace(indexRegExp, value);
    return resultText;
}



String.prototype.compare = function(stringToCompareTo)
{
    if(typeof(stringToCompare) != "string")
    {
        return 1;
    }
    
    var counter  = 0;
    var thislen  = this.length;
    var otherlen = stringToCompareTo.length;
    
    while((counter < thislen)                                       && 
          (counter < otherlen)                                      && 
          (this.charAt(counter) == stringToCompare.charAt(counter))  )
    {
        counter++;
    }
    
    /* return result according to ANSI C strcmp semantics */
    if(counter < thislen)
    {
        return (counter < otherlen) ? 
            (this.charCodeAt(counter) - stringToCompare.charCodeAt(counter)) : 1
    }
    
    return (thislen == otherlen) ? 0 : -1;
}



String.prototype.trim = function()
{
    return this.trimStart().trimEnd();
}




String.prototype.trimStart = function()
{                          
    return this.replace(/\s*((\S+\s*)*)/g, "$1");
}



String.prototype.trimEnd = function()
{
    return this.replace(/((\s*\S+)*)\s*/g, "$1");
}



String.prototype.startsWith = function(startString)
{
    return (this.indexOf(startString) == 0);
}



String.prototype.endsWith = function(endString)
{
    return (this.lastIndexOf(endString) == (this.length - endString.length));
}




String.prototype.contains = function(pattern, startIndex)
{
    return this.substr(at, pattern.length) == pattern;
}



String.prototype.splitLines = function(preserveNewLine)
{
    var lineSet       = this.split(/\n/g);
    var newlineEnding = this.charAt(this.length-1) == '\n';

    if(preserveNewLine)
    {
        for(var i = 0,m = lineSet.length - 1; i <= m; i++)
        {
            if((i == m) && !newlineEnding)
            { 
                break;
            }
            
            lineSet[i] += '\n';
        }
    }

    return lineSet;
}



String.prototype.escapeRegExpChars = function()
{
    return this.replace(/\^/g, '\\^').
                replace(/\$/g, '\\$').
                replace(/\+/g, '\\+').
                replace(/\*/g, '\\*').
                replace(/\./g, '\\.').
                replace(/\?/g, '\\?').
                replace(/\[/g, '\\[').
                replace(/\]/g, '\\]');
}




String.prototype.encloseWithQuotations = function()
{
    return '"' + this + '"';
}




String.prototype.encloseWithApostrophes = function()
{
    return "'" + this + "'";
}



String.prototype.convertEntityRefsToUnicode = function()
{
    return this.replace(/&#(x?)([a-fA-F0-9]{2,4});/g, 
                        function(a, x, d)
                        {
                            return '\\u' + ((d.length==2)?'00' :'') + d
                        }                           );
}



String.prototype.unEscapeXMLCharacters = function()
{
    return this.replace(/(&gt;)|(&#x3e;)|(&#x3E;)|(&#62;)/g, '>').
                replace(/(&lt;)|(&#x3c;)|(&#x3C;)|(&#60;)/g, '<').
                replace(/(&apos;)|(&#x27;)|(&#39;)/g       , "'").
                replace(/(&quot;)|(&#x22;)|(&#34;)/g       , '"').
                replace(/(&amp;)|(&#x26;)|(&#38;)/g        , '&');
}



String.prototype.escapeXMLCharacters = function()
{
    var ampRegExp  = new RegExp("&" , "g");
    var quotRegExp = new RegExp("\"", "g");
    var aposRegExp = new RegExp("'" , "g");
    var ltRegExp   = new RegExp("<" , "g");
    var gtRegExp   = new RegExp(">" , "g");

    return this.replace(ampRegExp , '&amp;' ).
                replace(quotRegExp, '&quot;').
                replace(aposRegExp, '&apos;').
                replace(ltRegExp  , '&lt;'  ).
                replace(gtRegExp  , '&gt;'  );
}


