/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2005 Civil Engines Research.  All Rights 
 * Reserved.  
 *
 * Portions created by Netscape Copyright (C) 1998 Netscape 
 * Communications Corporation. All Rights Reserved.
 *
 * Portions created by the Mozilla Foundation are Copyright (C) 
 * 2003 Mozilla Foundation. All Rights Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/

const NETWORK_PROVIDER_OBSERVER_TOPIC_CONNECT      = "urn:network:provider:event:connect";
const NETWORK_PROVIDER_OBSERVER_TOPIC_DISCONNECT   = "urn:network:provider:event:disconnect";
const NETIZEN_LOAD_EXTERNAL_PROTOCOL               = "urn:global:protocol:external:load";


const NETIZEN_DEFAULT_WIDTH_RESOLUTION             = 718;
const NETIZEN_DEFAULT_HEIGHT_RESOLUTION            = 560;
const NETIZEN_MAX_WIDTH_RESOLUTION                 = 800;
const NETIZEN_MAX_HEIGHT_RESOLUTION                = 600;
const NETIZEN_RESOLUTION_SCALE_FACTOR              = 0.65;

var   MainAppBrowser                               = null;
var   ApplicationRegistry                          = null;
var   WidgetStateManager                           = null;
var   ViewManager                                  = null;
var   PropertyCache                                = null;

var NetizenApplication =
{
    startup : function()
    {
//@line 66 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/base/content/netizen.js"
        this.initWindow();
        this.loadStringBundle();
        this.loadServices();
        this.loadControllers();
        this.loadHandlers();

        return;
    },
    
    
    shutdown : function()
    {
//@line 81 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/base/content/netizen.js"
        this.unloadServices();
        this.unloadControllers();
        this.unloadHandlers();
        
        return;
    },

    loadStringBundle : function()
    {
        var netizenStringBundle = document.getElementById("netizenStringBundle");
        if(!netizenStringBundle)
        {
            return;
        }

        this.vendorURL = netizenStringBundle.getString("netizen.web.vendor.url");
        this.updateURL = netizenStringBundle.getString("netizen.web.update.url");
        this.statusURL = netizenStringBundle.getString("netizen.web.status.url");

        return;
    },

    initWindow : function()
    {
        if(!document.documentElement.hasAttribute("width")) 
        {
            var defaultWidth  = 0;
            var defaultHeight = 0;
            if(screen.availWidth <= 600) 
            { 
                document.documentElement.setAttribute("sizemode", "maximized");
            }
            else if((screen.availWidth / 2) > 800)
            {
                defaultWidth  = NETIZEN_MAX_WIDTH_RESOLUTION;
                defaultHeight = NETIZEN_MAX_HEIGHT_RESOLUTION;
            }
            else
            {
                defaultWidth  = NETIZEN_DEFAULT_WIDTH_RESOLUTION;
                defaultHeight = NETIZEN_DEFAULT_HEIGHT_RESOLUTION;
            }
            
            document.documentElement.setAttribute("width" , defaultWidth);
            document.documentElement.setAttribute("height", defaultHeight);
        }

        return;
    },
    

    loadServices : function()
    {
        WidgetStateManager  = new nsWidgetStateManager(NETIZEN_CONTENT_AREA_ID);
        ApplicationRegistry = new Object();
        ViewManager         = new Object();
        PropertyCache       = new Object();

        MainAppBrowser      = document.getElementById(NETIZEN_CONTENT_AREA_ID);
        if(MainAppBrowser)
        {
            var defaultAppURL  = MainAppBrowser.getAttribute("url");
            var defaultAppName = MainAppBrowser.getAttribute("name");

            NetizenWindowController.loadTargetApplication(defaultAppURL, defaultAppName);
        }


        WidgetStateManager.attributes = 
        [
            "ref"      , 
            "hidden"   , 
            "disabled" , 
            "selected" 
        ];
                
        return;
    },
    

    unloadServices : function()
    {
        try
        {
            NetizenPropertyCache.purgeCache();
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }
        
        delete PropertyCache;
        delete ViewManager;
        delete WidgetStateManager;
                
        return;
    },


    loadControllers : function()
    {
        if(!MainAppBrowser)
        {
            MainAppBrowser = document.getElementById(NETIZEN_CONTENT_AREA_ID);
        }

        top.controllers.appendController(NetizenApplicationController);        
        return;
    },


    unloadControllers : function()
    {
        if(!MainAppBrowser)
        {
            MainAppBrowser = document.getElementById(NETIZEN_CONTENT_AREA_ID);
        }

        top.controllers.removeController(NetizenApplicationController);
        NetizenCommandController.onUnload();

        return;
    },



    loadHandlers : function()
    {
        if(!MainAppBrowser)
        {
            MainAppBrowser = document.getElementById(NETIZEN_CONTENT_AREA_ID);
        }

        NetizenNetworkEventListener.registerListener(MainAppBrowser);        
        NetizenApplicationController.loadObservers();
        NetizenStatusObserver.loadObservers();
        
        window.addEventListener("load", NetizenFrameController.onWindowLoad, false);
        return;
    },


    unloadHandlers : function()
    {
        if(!MainAppBrowser)
        {
            MainAppBrowser = document.getElementById(NETIZEN_CONTENT_AREA_ID);
        }

        NetizenStatusObserver.unloadObservers();
        NetizenApplicationController.unloadObservers();
        NetizenNetworkEventListener.unregisterListener(MainAppBrowser);
        
        window.removeEventListener("load", NetizenFrameController.onWindowLoad, false);        
        return;
    },
    

    getNavigationControl : function()
    {
        if(MainAppBrowser)
        {
            return MainAppBrowser.navigationControl;
        }
        
        var netizenBrowser = NetizenWindowController.getNetizenBrowser();
        
        return netizenBrowser.navigationControl;
    }
};




var NetizenApplicationController =
{
    loadObservers : function()
    {
        ObserverService.addObserver(this, NETIZEN_LOAD_EXTERNAL_PROTOCOL, false);
        return;
    },


    unloadObservers : function()
    {
        ObserverService.removeObserver(this, NETIZEN_LOAD_EXTERNAL_PROTOCOL);
        return;
    },

    
    supportsCommand : function(commandId)
    {
        if(commandId == "cmd_netizenAppLoad" || commandId == "cmd_netizenAppUnload")
        {
            return true;
        }
        
        return false;
    },


    isCommandEnabled : function(commandId)
    {
        if(commandId == "cmd_netizenAppLoad" || commandId == "cmd_netizenAppUnload")
        {
            return true;
        }
        
        return false;
    },


    doCommand : function(commandId)
    {
        if(commandId == "cmd_netizenAppLoad")
        {
            this.loadApplication();
        }
        else if(commandId == "cmd_netizenAppUnload")
        {
            this.unloadApplication();
        }
        
        this.onCommandUpdate();
        return;
    },
    

    onCommandUpdate : function()
    {
        var loadIsEnabled   = this.isCommandEnabled("cmd_netizenAppLoad");
        var unloadIsEnabled = this.isCommandEnabled("cmd_netizenAppUnload");
        
        goSetCommandEnabled("cmd_netizenAppLoad"  , loadIsEnabled  );
        goSetCommandEnabled("cmd_netizenAppUnload", unloadIsEnabled);
        
        return;
    },


    loadApplication : function()
    {
        var currentPageURI = NetizenWindowController.getCurrentPageViewURI();
        var netizenBrowser = NetizenWindowController.getNetizenBrowser();
        if(!netizenBrowser)
        {
            return;
        }
        
        var contentWindow  = netizenBrowser.contentWindow;
        
        WidgetStateManager.setPageData(currentPageURI.spec, contentWindow);

        try
        {
            var isHistoryView = false;
            if(netizenBrowser.getAttribute("history"))
            {
                isHistoryView = true;
            }
            
            /* The javascript runtime creates an implicit XPCNativeWrapper for
             * the main content area of the netizen browser object.  All properties
             * defined in that context are considered untrusted by the JS security
             * model and must be explicitly accessed via the wrappedJSObject 
             * property of the wrapper.
             */
            contentWindow.wrappedJSObject.loadApplication(currentPageURI, isHistoryView);
            
//@line 355 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/base/content/netizen.js"
        }
        catch(exception)
        {
            Syslog.logException(exception);
            return;
        }
        
            
        ObserverService.notifyObservers(null                                  , 
                                        NETIZEN_APPLICATION_STATUS_PAGE_LOADED, 
                                        null                                  );

        NetizenNavigationController.onCommandUpdate();
        
        return;
    },


    unloadApplication : function()
    {
        var currentPageURI = NetizenWindowController.getCurrentPageViewURI();
        var netizenBrowser = NetizenWindowController.getNetizenBrowser();
        if(!netizenBrowser)
        {
            return;
        }
        
        var contentWindow  = netizenBrowser.contentWindow;
        
        try
        {
            /* The javascript runtime creates an implicit XPCNativeWrapper for
             * the main content area of the netizen browser object.  All properties
             * defined in that context are considered untrusted by the JS security
             * model and must be explicitly accessed via the wrappedJSObject 
             * property of the wrapper.
             */
            contentWindow.wrappedJSObject.unloadApplication(currentPageURI);

//@line 399 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/base/content/netizen.js"
        }
        catch(exception)
        {
            Syslog.logException(exception);
            return;
        }
        

        ObserverService.notifyObservers(null                                    , 
                                        NETIZEN_APPLICATION_STATUS_PAGE_UNLOADED, 
                                        null                                    );  
        return;
    },


    QueryInterface : function(iid)
    {
        if(iid.equals(Interfaces.nsIObserver) ||
           iid.equals(Interfaces.nsISupports)  )
        {
            return this;
        }
        
        throw Results.NS_NOINTERFACE;
    },

                      
    observe : function(subject, topic, topicdata)
    {
        if(topic == NETIZEN_LOAD_EXTERNAL_PROTOCOL)
        {
            try
            {
                NetizenWindowController.loadExternalURL(topicdata);
            }
            catch(exception)
            {
                Syslog.logException(exception);
            }
        }

        return;
    }
};




var NetizenNetworkEventListener = 
{
    registerListener : function(targetElement)
    {
        if(targetElement)
        {
            targetElement.addEventListener(NETIZEN_NETWORK_EVENT_CONNECT, 
                                           this.onNetworkConnect        , 
                                           false                        );
            

            targetElement.addEventListener(NETIZEN_NETWORK_EVENT_DISCONNECT, 
                                           this.onNetworkDisconnect        , 
                                           false                           );
        }
        
        return;
    },
    
    
    unregisterListener : function(targetElement)
    {
        if(targetElement)
        {
            targetElement.removeEventListener(NETIZEN_NETWORK_EVENT_CONNECT, 
                                              this.onNetworkConnect        , 
                                              false                        );
            

            targetElement.removeEventListener(NETIZEN_NETWORK_EVENT_DISCONNECT, 
                                              this.onNetworkDisconnect        , 
                                              false                           );
        }
        
        return;
    },


    onNetworkConnect : function(eventInfo)
    {
        ObserverService.notifyObservers(null, NETWORK_PROVIDER_OBSERVER_TOPIC_CONNECT, null);
        return;
    },


    onNetworkDisconnect : function(eventInfo)
    {
        ObserverService.notifyObservers(null, NETWORK_PROVIDER_OBSERVER_TOPIC_DISCONNECT, null);
        return;
    },
};



var NetizenStatusObserver =
{
    statusCacheByURI : {},
    statusCacheByKey : {},
    
    loadObservers : function()
    {
        ObserverService.addObserver(this, NETIZEN_APPLICATION_STATUS_IDLE         , false);
        ObserverService.addObserver(this, NETIZEN_APPLICATION_STATUS_BUSY         , false);
        ObserverService.addObserver(this, NETIZEN_APPLICATION_STATUS_PAGE_LOADED  , false);
        ObserverService.addObserver(this, NETIZEN_APPLICATION_STATUS_PAGE_UNLOADED, false);
        
        return;
    },


    unloadObservers : function()
    {
        ObserverService.removeObserver(this, NETIZEN_APPLICATION_STATUS_IDLE         );
        ObserverService.removeObserver(this, NETIZEN_APPLICATION_STATUS_BUSY         );
        ObserverService.removeObserver(this, NETIZEN_APPLICATION_STATUS_PAGE_LOADED  );
        ObserverService.removeObserver(this, NETIZEN_APPLICATION_STATUS_PAGE_UNLOADED);
        
        return;
    },

    QueryInterface : function(iid)
    {
        if(iid.equals(Interfaces.nsIObserver) ||
           iid.equals(Interfaces.nsISupports)  )
        {
            return this;
        }
        
        throw Results.NS_NOINTERFACE;
    },


    setThrobberIdle : function()
    {
        var netizenThrobber = document.getElementById("netizenThrobber");
        if(netizenThrobber)
        {
            netizenThrobber.removeAttribute("busy");
        }
        
        return;
    },


    setThrobberBusy : function()
    {
        var netizenThrobber = document.getElementById("netizenThrobber");
        if(netizenThrobber)
        {
            netizenThrobber.setAttribute("busy", "true");
        }
        
        return;
    },
    

    throbberIsBusy : function()
    {
        var netizenThrobber = document.getElementById("netizenThrobber");
        if(netizenThrobber && netizenThrobber.getAttribute("busy") == "true")
        {
            return true;
        }
        
        return false;
    },


    setApplicationIdle : function(currentPageURI, topicdata)
    {
        var statusContext  = null;

//@line 584 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/base/content/netizen.js"

        statusContext = this.statusCacheByKey[topicdata];
        if(!statusContext && currentPageURI)
        {
            statusContext = this.statusCacheByURI[currentPageURI.spec];
        }
        
        if(statusContext)
        {
            statusContext.isBusy             = false;
            this.statusCacheByKey[topicdata] = null;
        }
        
        if(this.throbberIsBusy() == false)
        {
            return;
        }
        
        if(statusContext)
        {
            this.setThrobberIdle();
        }
            
        return;
    },
    

    setApplicationBusy : function(currentPageURI, topicdata)
    {
        var statusContext  = null;

        if(this.throbberIsBusy() == true)
        {
            return;
        }
         
//@line 625 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/base/content/netizen.js"

        statusContext = this.statusCacheByKey[topicdata];
        if(!statusContext)
        {
            statusContext = this.statusCacheByURI[currentPageURI.spec];
        }
        
        if(!statusContext)
        {
            statusContext = new Object();
        }
        
        statusContext.viewStateURI                 = currentPageURI;
        statusContext.isBusy                       = true;

        this.statusCacheByKey[topicdata]           = statusContext;
        this.statusCacheByURI[currentPageURI.spec] = statusContext;
                
        this.setThrobberBusy();
        
        return;
    },
    

    updateThrobberForURI : function(currentPageURI)
    {
        var statusContext = this.statusCacheByURI[currentPageURI.spec];
        if(statusContext)
        {
            if(statusContext.isBusy)
            {
                this.setApplicationBusy(currentPageURI, null);
            }
            else
            {
                this.setApplicationIdle(currentPageURI, null);
            }
        }

        return;
    },
    
    
    observe : function(subject, topic, topicdata)
    {
        var currentPageURI = NetizenWindowController.getCurrentPageViewURI();
                
//@line 677 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/base/content/netizen.js"
        
        switch(topic)
        {
          case NETIZEN_APPLICATION_STATUS_PAGE_LOADED   :
          {
              this.updateThrobberForURI(currentPageURI);
              break;
          }
          case NETIZEN_APPLICATION_STATUS_PAGE_UNLOADED :
          {
              this.setThrobberIdle();
              break;
          }
          case NETIZEN_APPLICATION_STATUS_IDLE          :
          {
              this.setApplicationIdle(currentPageURI, topicdata);
              break;
          }
          case NETIZEN_APPLICATION_STATUS_BUSY          :
          {
              this.setApplicationBusy(currentPageURI, topicdata);
              break;
          }
        }
        
        return;
    }
};
