/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2005 Civil Engines Research.  All Rights 
 * Reserved.  
 *
 * Portions created by Netscape Copyright (C) 1998 Netscape 
 * Communications Corporation. All Rights Reserved.
 *
 * Portions created by the Mozilla Foundation are Copyright (C) 
 * 2003 Mozilla Foundation. All Rights Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/

const NETIZEN_WINDOW_TYPE_ID                    = "urn:civil:windowtype:netizen";
const NETIZEN_CONTENT_AREA_ID                   = "netizenContentArea";
const NETIZEN_CONTENT_LOAD_DELAY_IN_MS          = 20;
const NETIZEN_LINK_TRIGGER_DELAY_IN_MS          = 10;

const NETIZEN_CONTENT_ORIGIN_PROPERTY           = "origin";
const NETIZEN_CONTENT_VIEWID_PROPERTY           = "viewid";
const NETIZEN_CONTENT_INSTANCE_PROPERTY         = "instance";
const NETIZEN_CONTENT_RESOURCE_PROPERTY         = "res";
const NETIZEN_CONTENT_MODE_PROPERTY             = "mode";
const NETIZEN_CONTENT_REFERRER_PROPERTY         = "referrer";

const NETIZEN_CONTENT_VIEWID_DEFAULT            = "default";
const NETIZEN_TUNNEL_PROVIDER_CONTRACTID        = "@civilengines.com/kernel/tunnel-provider;1"
const WINDOW_MEDIATOR_CONTRACTID                = "@mozilla.org/appshell/window-mediator;1";
const DRAG_SERVICE_CONTRACTID                   = "@mozilla.org/network/io-service;1";
const EXTERNAL_PROTOCOL_SERVICE                 = "@mozilla.org/uriloader/external-protocol-service;1"

const NETIZEN_NETWORK_EVENT_CONNECT             = "urn:netizen:connection:event:connect";
const NETIZEN_NETWORK_EVENT_DISCONNECT          = "urn:netizen:connection:event:disconnect";
const NETIZEN_NETWORK_EVENT_REPAIR              = "urn:netizen:connection:event:repair";

const NETIZEN_NETWORK_STATUS_PROPERTY           = "urn:netizen:property:network:status";
const NETIZEN_NETWORK_ALERT_PROPERTY            = "urn:netizen:property:network:alert";

const NETIZEN_NETWORK_STATUS_DISCONNECTED       = "urn:network:status:disconnected";
const NETIZEN_NETWORK_STATUS_DISCOVERING        = "urn:network:status:discovering";
const NETIZEN_NETWORK_STATUS_CONNECTING         = "urn:network:status:connecting";
const NETIZEN_NETWORK_STATUS_REGISTERING        = "urn:network:status:registering";
const NETIZEN_NETWORK_STATUS_CONNECTED          = "urn:network:status:connected";
const NETIZEN_NETWORK_STATUS_PINGING            = "urn:network:status:pinging";

const NETIZEN_NETWORK_ALERT_FIREWALLED          = "urn:network:alert:firewalled";
const NETIZEN_NETWORK_ALERT_NATDETECTED         = "urn:network:alert:natdetected";
const NETIZEN_NETWORK_ALERT_OPENINTERNET        = "urn:network:alert:openinternet";

const NETIZEN_NETWORK_ALERT_FOLLOWLINK          = "urn:network:alert:event:link";

const NETIZEN_APPLICATION_STATUS_IDLE           = "urn:netizen:status:idle";
const NETIZEN_APPLICATION_STATUS_BUSY           = "urn:netizen:status:busy";
const NETIZEN_APPLICATION_STATUS_PAGE_LOADED    = "urn:netizen:status:pageloaded";
const NETIZEN_APPLICATION_STATUS_PAGE_UNLOADED  = "urn:netizen:status:pageunloaded";


var NetizenWindowController = 
{
    getMostRecentWindow : function(windowType)
    {
        var windowManager    = do_GetService(WINDOW_MEDIATOR_CONTRACTID, Interfaces.nsIWindowMediator);
        var mostRecentWindow = windowManager.getMostRecentWindow(windowType);
        if(mostRecentWindow)
        {
            return mostRecentWindow;
        }
        
        return null;
    },


    getNetizenBrowser : function()
    {
        var netizenWindow  = this.getMostRecentWindow(NETIZEN_WINDOW_TYPE_ID);
        if(!netizenWindow)
        {
            return null;
        }
        
        var netizenBrowser = netizenWindow.document.getElementById(NETIZEN_CONTENT_AREA_ID);
        if(netizenBrowser)
        {
            return netizenBrowser;
        }
        
        return null;
    },


    getPageViewReferringURI : function()
    {
        var netizenBrowser = this.getNetizenBrowser();
        if(!netizenBrowser)
        {
            return null;
        }
        
        var referringURI   = netizenBrowser.referringURI;
        if(!referringURI)
        {
            return null;
        }

        Syslog.logMessage("Referring URI spec = " + referringURI.spec);
        
        return (referringURI && referringURI.spec && referringURI.spec != "about:blank") 
            ? (new ceStandardURL(referringURI.spec)) : null;
    },


    getCurrentPageViewURI : function()
    {
        var netizenBrowser = this.getNetizenBrowser();
        if(!netizenBrowser)
        {
            return null;
        }
        
        var currentURI = netizenBrowser.currentURI;
        if(!currentURI)
        {
            return null;
        }
        
        return (currentURI && currentURI.spec && currentURI.spec != "about:blank") ? 
            (new ceStandardURL(currentURI.spec)) : null;
    },


    getCurrentPageViewParameters : function()
    {
        var currentURI = this.getCurrentPageViewURI();
        if(currentURI)
        {
            return currentURI.getQueryParameters();
        }
        
        return null;
    },
    

    getPageViewProperty : function(parameterName, applicationParams)
    {
        /* If an override set of parameters is not provided, then
         * acquire the current page context for the parameters
         */
        if(!applicationParams)
        {
            applicationParams = this.getCurrentPageViewParameters();
        }
        

        /* Check the page parameters for the property value.  If it is
         * not specified in the page view parameters, check the property
         * cache for the most recent value
         */
        var parameterValue = applicationParams.getParameterValue(parameterName);
        if(!parameterValue)
        {
            parameterValue = NetizenPropertyCache.getPropertyValue(parameterName);
        }


        return parameterValue;
    },
    

    getCurrentPageIndex : function()
    {
        var netizenBrowser    = this.getNetizenBrowser();
        if(!netizenBrowser)
        {
            return 0;
        }
        
        var navigationHistory = netizenBrowser.navigationHistory;
        if(!navigationHistory)
        {
            return 0;
        }
        
        return navigationHistory.index;
    },


    getPageViewURIAtIndex : function(index, moveHistoryIndex)
    {
        var netizenBrowser = this.getNetizenBrowser();
        if(!netizenBrowser)
        {
            return null;
        }
        
        var indexedURI = netizenBrowser.getViewAtIndex(index, moveHistoryIndex);
        if(!indexedURI)
        {
            return null;
        }
        
        return (indexedURI && indexedURI.spec && indexedURI.spec != "about:blank") ? 
            (new ceStandardURL(indexedURI.spec)) : null;
    },


    loadViewWithFlags : function(loadURI, loadFlags)
    {
        var netizenBrowser = this.getNetizenBrowser();
        if(netizenBrowser)
        {
            try
            {
                netizenBrowser.loadViewWithFlags(loadURI, loadFlags);
            }
            catch(exception)
            {
                Syslog.logException(exception);
            }
        }       
        
        return;
    },


    loadViewWithURI : function(loadURI)
    {
        var netizenBrowser = this.getNetizenBrowser();
        if(netizenBrowser)
        {
            try
            {
                netizenBrowser.loadView(loadURI);
            }
            catch(exception)
            {
                Syslog.logException(exception);
            }
        }
        
        return;
    },


    saveCurrentWidgetState : function(targetWindow)
    {
        var widgetManager  = MainAppWindow.WidgetStateManager;
        if(!widgetManager)
        {
            return false;
        }
        
        var currentViewURL = this.getCurrentPageViewURI();    
        var viewBaseURL    = currentViewURL.prePath + currentViewURL.filePath;
        
        var viewParameters = new ceStandardURLQuery();
        var originValue    = currentViewURL.getParameterValue(NETIZEN_CONTENT_ORIGIN_PROPERTY);
        
        viewParameters.addQueryParameter(NETIZEN_CONTENT_ORIGIN_PROPERTY, originValue);
        viewBaseURL += "?" + viewParameters.toQueryString();

//@line 293 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/base/content/netizenAppOverlay.js"
        
        /* Save the current widget state as the most recent application view */
        widgetManager.savePageData(viewBaseURL, targetWindow);
        
        /* Save the current widget state under the history URL */
        widgetManager.savePageData(currentViewURL.spec, targetWindow);
        
        return true;
    },


    hasRecentViewState : function(viewURL)
    {
        var widgetManager = MainAppWindow.WidgetStateManager;
        if(!widgetManager)
        {
            return null;
        }
        
        return widgetManager.containsPageData(viewURL);
    },


    loadApplicationView : function(appViewBaseURL, viewRequestId, appParameters)
    {
        var netizenBrowser  = NetizenWindowController.getNetizenBrowser();
        if(!netizenBrowser)
        {
            return;
        }
        
        /* Application views can be loaded from:
         *
         * 1. The same page; but the view must be treated as a different 
         *    page by the session history. 
         * 2. A different page for the current application
         * 3. A different page from a different application, but using 
         *    the current application as the origin.
         */
        var currentURL      = NetizenWindowController.getCurrentPageViewURI();
        var applicationName = currentURL.getParameterValue(NETIZEN_CONTENT_ORIGIN_PROPERTY);
        
        if(viewRequestId == undefined || viewRequestId == null)
        {
            viewRequestId = NETIZEN_CONTENT_VIEWID_DEFAULT;
        }
        
        /* Construct the application chrome URL */
        var nextViewURL      = appViewBaseURL + "?";
        var viewParameters   = new ceStandardURLQuery();
        var applicationIndex = NetizenWindowController.getCurrentPageIndex() + 1;
        
        viewParameters.addQueryParameter(NETIZEN_CONTENT_ORIGIN_PROPERTY  , applicationName );
        viewParameters.addQueryParameter(NETIZEN_CONTENT_VIEWID_PROPERTY  , viewRequestId   );
        viewParameters.addQueryParameter(NETIZEN_CONTENT_INSTANCE_PROPERTY, applicationIndex);
        
        /* Append the required parameters */
        nextViewURL += viewParameters.toQueryString();
        
        /* Append application specific parameters if specified */
        if(appParameters && appParameters != "")
        {
            nextViewURL += "&" + appParameters;
        }
        
        
        /* Always save the widget state for before switching views */ 
        NetizenWindowController.saveCurrentWidgetState(window);
        
        
        /* Remember the new viewURL for the toolbar shortcut */
        MainAppWindow.ViewManager[applicationName] = nextViewURL;
        
        
        /* Loading a new application view should purge the history
         * so that the next view can be saved as the most current 
         * view.
         */
        if(netizenBrowser.getAttribute("history"))
        {
            netizenBrowser.removeAttribute("history");
        }
        
//@line 379 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/base/content/netizenAppOverlay.js"
        NetizenWindowController.loadViewWithURI(nextViewURL);    
        
        return;
    },


    scheduleApplicationViewLoad : function(appViewBaseURL   , 
                                           requestedViewId  , 
                                           applicationParams, 
                                           loadDelay        )
    {
        if(loadDelay == undefined || loadDelay == 0)
        {
            loadDelay = NETIZEN_CONTENT_LOAD_DELAY_IN_MS;
        }
        
        /* Delay loading of the next application view to allow
         * for the current view to be unloaded properly
         */    
        var timerId = window.setTimeout(NetizenWindowController.loadApplicationView, 
                                        loadDelay                                  ,
                                        appViewBaseURL                             ,
                                        requestedViewId                            ,
                                        applicationParams                          );
//@line 410 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/base/content/netizenAppOverlay.js"

        return timerId;
    },


    switchApplicationMode : function(loadEvent)
    {
        var applicationViewURL = loadEvent.target.getAttribute("target");
        if(!applicationViewURL)
        {
            return;
        }
        
        this.switchApplicationModeToTarget(applicationViewURL);
        
        return;
    },


    switchApplicationModeToTarget : function(applicationViewURL)
    {
        var netizenBrowser  = this.getNetizenBrowser();
        if(!netizenBrowser)
        {
            return;
        }
        
        var applicationName = netizenBrowser.getAttribute("application");
        if(!applicationName)
        {
            return;
        }
        
        applicationViewURL += "?" + NETIZEN_CONTENT_ORIGIN_PROPERTY + "=" + applicationName;
        
        var currentURL      = this.getCurrentPageViewURI();
        var currentViewURL  = currentURL.prePath + currentURL.filePath;
        var viewParameters  = new ceStandardURLQuery();
        var originValue     = currentURL.getParameterValue(NETIZEN_CONTENT_ORIGIN_PROPERTY);
        
        viewParameters.addQueryParameter(NETIZEN_CONTENT_ORIGIN_PROPERTY, originValue);
        currentViewURL += "?" + viewParameters.toQueryString();
        
        
        /* If the currently displayed view is the view being requested,
         * then don't bother reloading the view 
         */
//@line 461 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/base/content/netizenAppOverlay.js"
        if(applicationViewURL == currentViewURL && !netizenBrowser.getAttribute("history"))
        {
//@line 466 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/base/content/netizenAppOverlay.js"
            return;
        }
        
        /* Every time the page is switched, a new view is created and
         * the current view is preserved in history with its corresponding
         * viewstate
         */
        if(!this.hasRecentViewState(applicationViewURL))
        {
            applicationViewURL += "&" + NETIZEN_CONTENT_INSTANCE_PROPERTY + 
                                  "=" + (this.getCurrentPageIndex() + 1);
        }
        
        /* An application switch should purge the history flag since
         * a new view is being created
         */
        netizenBrowser.removeAttribute("history");

        /* Preserve the widget state of the current page */
        this.saveCurrentWidgetState(netizenBrowser.contentWindow);
        
        
        /* Remember the current applicationViewURL for the toolbar shortcut */
        MainAppWindow.ViewManager[applicationName] = applicationViewURL;
        
        this.loadViewWithURI(applicationViewURL);
        
        return;
    },


    loadTargetApplication : function(appBaseURL, applicationName)
    {
        var appParameters = new ceStandardURLQuery();
        
        appParameters.addQueryParameter(NETIZEN_CONTENT_ORIGIN_PROPERTY, 
                                        applicationName                );
        
        appParameters.addQueryParameter(NETIZEN_CONTENT_VIEWID_PROPERTY, 
                                        NETIZEN_CONTENT_VIEWID_DEFAULT );
        
        appParameters.addQueryParameter(NETIZEN_CONTENT_INSTANCE_PROPERTY,
                                        this.getCurrentPageIndex() + 1   );
                

        /* Retrieve the current view for the application, if it exists */
        var applicationURL = appBaseURL + "?" + appParameters.toQueryString();
        

        /* Remember the current viewURL for the toolbar shortcut */
        MainAppWindow.ViewManager[applicationName] = applicationURL;
        
        var netizenBrowser  = this.getNetizenBrowser();
        if(!netizenBrowser)
        {
            return;
        }
        
        /* Set the current application property */
        netizenBrowser.setAttribute("application", applicationName);
        
        this.loadViewWithURI(applicationURL);
        
        return;
    },



    switchApplication : function(loadEvent)
    {
        var appBaseURL = loadEvent.target.getAttribute("target");
        if(!appBaseURL)
        {
            return;
        }
        
        var applicationName = loadEvent.target.getAttribute("application");
        if(!applicationName)
        {
            return;
        }
        
        this.switchApplicationToTarget(appBaseURL, applicationName);
        
        return;
    },



    switchApplicationToTarget : function(appBaseURL, applicationName)
    {
        /* Retrieve the current view for the application, if it exists */
        var applicationURL = null;
        var viewURL        = MainAppWindow.ViewManager[applicationName];
        var viewURLParts   = new ceStandardURL(viewURL);
        if(applicationName == viewURLParts.getParameterValue(NETIZEN_CONTENT_ORIGIN_PROPERTY))
        {
            applicationURL = viewURL;
            
            /* Remember the current viewURL for the toolbar shortcut */
            MainAppWindow.ViewManager[applicationName] = applicationURL;
        }
        
        var currentURL     = this.getCurrentPageViewURI();
        var currentViewURL = currentURL.prePath + currentURL.filePath;
        var viewParameters = new ceStandardURLQuery();
        var originValue    = currentURL.getParameterValue(NETIZEN_CONTENT_ORIGIN_PROPERTY);
        
        viewParameters.addQueryParameter(NETIZEN_CONTENT_ORIGIN_PROPERTY, originValue);
        currentViewURL += "?" + viewParameters.toQueryString();
        
        
        /* If the currently displayed view is the view being requested,
         * then don't bother reloading the view 
         */
        if(applicationURL == currentViewURL)
        {
            return;
        }
        
        /* If a recent view of the application is not available
         * then load a new page with the default view for the application
         */
        if(!applicationURL)
        {
            var appParameters  = new ceStandardURLQuery();
            
            appParameters.addQueryParameter(NETIZEN_CONTENT_ORIGIN_PROPERTY, 
                                            applicationName                );
            
            appParameters.addQueryParameter(NETIZEN_CONTENT_VIEWID_PROPERTY, 
                                            NETIZEN_CONTENT_VIEWID_DEFAULT );
            
            /* Append the default parameters to the application URL */
            applicationURL = appBaseURL + "?" + appParameters.toQueryString();
            
            /* Remember the current viewURL for the toolbar shortcut */
            MainAppWindow.ViewManager[applicationName] = applicationURL;
        }
        
        var netizenBrowser  = this.getNetizenBrowser();
        if(!netizenBrowser)
        {
            return;
        }
        
        /* Save the widget state for the current view only if it is
         * not a page view from session history.  This ensures that
         * any changes to the widget state on a history loaded page
         * will not be persisted when the application is switched. 
         */
        if(!netizenBrowser.getAttribute("history"))
        {
            this.saveCurrentWidgetState(netizenBrowser.contentWindow);
        }
        
        /* An application switch should purge the history flag since
         * the most recent view will be loaded.
         */
        netizenBrowser.removeAttribute("history");
        
        /* Set the current application property */
        netizenBrowser.setAttribute("application", applicationName);
        
        this.loadViewWithURI(applicationURL);
        
        return;
    },


    loadApplication : function()
    {
        var netizenAppController = MainAppWindow.NetizenApplicationController;
        
        netizenAppController.loadApplication();
        return;
    },


    unloadApplication : function()
    {
        var netizenAppController = MainAppWindow.NetizenApplicationController;
        
        netizenAppController.unloadApplication();
        return;
    },

    
    isPageFromHistoryView : function()
    {
        var netizenBrowser = this.getNetizenBrowser();
        if(!netizenBrowser)
        {
            return false;
        }
        
        if(netizenBrowser.getAttribute("history"))
        {
            return true;
        }

        return false;
    },


    setExternalProtocolPref : function(protocolScheme, prefValue)
    {
        if(protocolScheme == "http")
        {
            LocalPreferences.setBoolPref("network.protocol-handler.external.http", prefValue);
        }
        else if(protocolScheme == "https")
        {
            LocalPreferences.setBoolPref("network.protocol-handler.external.https", prefValue);
        }

        return;
    },

    loadExternalURL : function(externalURL)
    {
        var externalProtocolService = do_GetService(EXTERNAL_PROTOCOL_SERVICE            , 
                                                    Interfaces.nsIExternalProtocolService);
        if(!externalProtocolService)
        {
            throw Results.NS_ERROR_FAILURE;
        }
        
        var uriComponent = null;
        
//@line 699 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/base/content/netizenAppOverlay.js"

        if(isString(externalURL))
        {
            uriComponent = IOService.newURI(externalURL, null, null);            
        }
        else
        {
            uriComponent = do_QueryInterface(externalURL);
        }
        
        Assert.condition(uriComponent, "Illegal externalURL component");
        if(!uriComponent)
        {
            throw Results.NS_ERROR_FAILURE;
        }
        
        var protocolScheme = uriComponent.scheme;
        if(!protocolScheme)
        {
            throw Results.NS_ERROR_FAILURE;
        }

        /* Save the default preference for the HTTP/HTTPS protocol handlers */
        var allowExternalHTTP  = LocalPreferences.getBoolPref("network.protocol-handler.external.http");
        var allowExternalHTTPS = LocalPreferences.getBoolPref("network.protocol-handler.external.https");
        
        /* Check to see if external loads of the specified protocol scheme
         * should be allowed
         */
        this.setExternalProtocolPref(protocolScheme, true);
                
        /* Loading this URI through the external protocol service
         * will prompt the user with a warning unless the preference
         *   network.protocol-handler.warn-external.<scheme>
         *   network.protocol-handler.warn-external-default
         * 
         * is set to false.
         */
        externalProtocolService.loadUrl(uriComponent);

        /* Restore the protocol scheme preferences to the default */
        var allowExternal = protocolScheme == "http" ? allowExternalHTTP :
            (protocolScheme == "https") ? allowExternalHTTPS : false;
        
        window.setTimeout(NetizenWindowController.setExternalProtocolPref,
                          NETIZEN_CONTENT_LOAD_DELAY_IN_MS               , 
                          protocolScheme                                 ,
                          allowExternalHTTP                              );

        return;
    },


    applicationIsBusy : function()
    {
        var netizenThrobber = document.getElementById("netizenThrobber");
        if(netizenThrobber && netizenThrobber.getAttribute("busy") == "true")
        {
            return true;
        }
        
        return false;
    }    
};



var MainAppWindow          = NetizenWindowController.getMostRecentWindow(NETIZEN_WINDOW_TYPE_ID);
var NetizenFrameController =
{
    frameIdRefs        : [],
    frameReflowPadding : 8 ,

    addFrameReference : function(frameIdRef)
    {
        if(this.frameIdRefs)
        {
            this.frameIdRefs.push(frameIdRef);
        }
        
        return;
    },
    

    removeFrameReference : function(frameIdRef)
    {
        var internalFrame = document.getElementById(frameIdRef);
        if(internalFrame)
        {
            internalFrame.removeEventListener("load", this.onInternalFrameLoad, false);
        }
                
        for(var counter = 0; counter < this.frameIdRefs.length; counter++)
        {
            if(this.frameIdRefs[counter] == frameIdRef)
            {
                this.frameIdRefs.splice(counter, 1);
                break;
            }
        }
        
        return;
    },

    resizeInternalFrame : function(frameIdRef)
    {
        var internalFrame = document.getElementById(frameIdRef);
        if(!internalFrame)
        {
            return;
        }
        
        if(internalFrame.hidden)
        {
            internalFrame.height = 0;
            return;
        }
        
        if(internalFrame.contentDocument)
        {
            var offsetHeight = internalFrame.contentDocument.body.offsetHeight;
            if(offsetHeight)
            {
                internalFrame.height = offsetHeight + this.frameReflowPadding;
            }
        }
        else
        {
            internalFrame.height = this.frameReflowPadding;
        }
        
        internalFrame.addEventListener("load", this.onInternalFrameLoad, false);

        return;
    },


    onWindowResize : function()
    {
        for(var counter = 0; counter < this.frameIdRefs.length; counter++)
        {
            this.resizeInternalFrame(this.frameIdRefs[counter]);
        }
        
        return;
    },


    onWindowLoad : function(eventInfo)
    {
        for(var counter = 0; counter < NetizenFrameController.frameIdRefs.length; counter++)
        {
            NetizenFrameController.resizeInternalFrame(NetizenFrameController.frameIdRefs[counter]);
        }
        
        return;
    },
    
    onInternalFrameLoad : function(eventInfo)
    {
        var internalFrame = eventInfo.currentTarget;
        if(internalFrame)
        {
            NetizenFrameController.resizeInternalFrame(internalFrame.id);
        }
        
        return;
    }    
};



var NetizenWindowResizer =
{
    resizeCallbacks : [],

    addResizeCallback : function(callbackFunction)
    {
        if(this.resizeCallbacks)
        {
            this.resizeCallbacks.push(callbackFunction);
        }
        
        return;
    },
    

    removeResizeCallback : function(callbackFunction)
    {
        for(var counter = 0; counter < this.resizeCallbacks.length; counter++)
        {
            if(this.resizeCallbacks[counter] == callbackFunction)
            {
                this.resizeCallbacks.splice(counter, 1);
                break;
            }
        }
        
        return;
    },

    purgeCallbacks : function()
    {
        for(var counter = 0; counter < this.resizeCallbacks.length; counter++)
        {
            delete this.resizeCallbacks[counter];
        }
  
        return;
    },


    onWindowResize : function()
    {
        for(var counter = 0; counter < this.resizeCallbacks.length; counter++)
        {
            var callbackFunction = this.resizeCallbacks[counter];
            if(callbackFunction)
            {
                callbackFunction();
            }
        }
        
        return;
    }
};




var NetizenPropertyCache =
{
    cachePropertyValue : function(propertyName, propertyValue)
    {
        var oldPropertyValue = MainAppWindow.PropertyCache[propertyName];
        
//@line 938 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/base/content/netizenAppOverlay.js"

        MainAppWindow.PropertyCache[propertyName] = propertyValue;
        return oldPropertyValue;
    },


    removePropertyValue : function(propertyName)
    {
        var oldPropertyValue = MainAppWindow.PropertyCache[propertyName];
        
        MainAppWindow.PropertyCache[propertyName] = null;
        
        try
        {
//@line 955 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/base/content/netizenAppOverlay.js"
            delete MainAppWindow.PropertyCache[propertyName];
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }
        
        return oldPropertyValue;
    },


    getPropertyValue : function(propertyName)
    {
        return MainAppWindow.PropertyCache[propertyName];
    },


    purgeCache : function()
    {
        for(var propertyName in MainAppWindow.PropertyCache)
        {
            delete MainAppWindow.PropertyCache[propertyName];
        }
        
        return;
    }
};




var NetizenNetworkStatusObserver =
{
    statusElement : null,

    loadStringBundle : function()
    {
        var netizenStringBundle = document.getElementById("netizenStringBundle");
        if(!netizenStringBundle)
        {
            return;
        }

        this.disconnectedString = netizenStringBundle.getString("netizen.connect.status.disconnected");
        this.discoveringString  = netizenStringBundle.getString("netizen.connect.status.discovering" );
        this.connectingString   = netizenStringBundle.getString("netizen.connect.status.connecting"  );
        this.registeringString  = netizenStringBundle.getString("netizen.connect.status.joining"     );
        this.connectedString    = netizenStringBundle.getString("netizen.connect.status.connected"   );
        this.pingingString      = netizenStringBundle.getString("netizen.connect.status.pinging"     );
  
        return;
    },


    loadObservers : function()
    {
        this.statusElement = document.getElementById("netizenNetworkStatus");
        if(!this.statusElement)
        {
            return;
        }
        
        this.loadStringBundle();
        this.initStatus();
        
        ObserverService.addObserver(this, NETIZEN_NETWORK_STATUS_DISCONNECTED, false);
        ObserverService.addObserver(this, NETIZEN_NETWORK_STATUS_DISCOVERING , false);
        ObserverService.addObserver(this, NETIZEN_NETWORK_STATUS_CONNECTING  , false);
        ObserverService.addObserver(this, NETIZEN_NETWORK_STATUS_REGISTERING , false);
        ObserverService.addObserver(this, NETIZEN_NETWORK_STATUS_CONNECTED   , false);
        ObserverService.addObserver(this, NETIZEN_NETWORK_STATUS_PINGING     , false);

        return;
    },


    unloadObservers : function()
    {
        ObserverService.removeObserver(this, NETIZEN_NETWORK_STATUS_DISCONNECTED);
        ObserverService.removeObserver(this, NETIZEN_NETWORK_STATUS_DISCOVERING );
        ObserverService.removeObserver(this, NETIZEN_NETWORK_STATUS_CONNECTING  );
        ObserverService.removeObserver(this, NETIZEN_NETWORK_STATUS_REGISTERING );
        ObserverService.removeObserver(this, NETIZEN_NETWORK_STATUS_CONNECTED   );
        ObserverService.removeObserver(this, NETIZEN_NETWORK_STATUS_PINGING     );
        
        return;
    },
    

    initStatus : function()
    {
        if(!this.statusElement)
        {
            return;
        }

        var networkStatus = NetizenPropertyCache.getPropertyValue(NETIZEN_NETWORK_STATUS_PROPERTY);
        if(!networkStatus)
        {
            var tunnelProvider = do_GetService(NETIZEN_TUNNEL_PROVIDER_CONTRACTID ,
                                               Interfaces.ceITunnelProviderService);
            if(tunnelProvider)
            {
                networkStatus = tunnelProvider.networkStatusTopic;
            }
        }
        
        if(networkStatus)
        {
            this.observe(null, networkStatus, null);
        }
                
        return;
    },
    

    cacheStatus : function(statusTopic)
    {
        if(!this.statusElement)
        {
            return;
        }
        
        var networkStatus = NetizenPropertyCache.getPropertyValue(NETIZEN_NETWORK_STATUS_PROPERTY);
        if(networkStatus != statusTopic)
        {
            NetizenPropertyCache.cachePropertyValue(NETIZEN_NETWORK_STATUS_PROPERTY, statusTopic);
        }

        return;
    },


    QueryInterface : function(iid)
    {
        if(iid.equals(Interfaces.nsIObserver) ||
           iid.equals(Interfaces.nsISupports)  )
        {
            return this;
        }
        
        throw Results.NS_NOINTERFACE;
    },

                      
    observe : function(subject, topic, topicdata)
    {
        if(!this.statusElement)
        {
            return;
        }

        if(topic == NETIZEN_NETWORK_STATUS_DISCOVERING)
        {
            this.statusElement.setAttribute("label", this.discoveringString);
            this.statusElement.setAttribute("disableconnect"   , "true" );
            this.statusElement.setAttribute("disabledisconnect", "false");
        }
        else if(topic == NETIZEN_NETWORK_STATUS_CONNECTING)
        {
            this.statusElement.setAttribute("label", this.connectingString);
            this.statusElement.setAttribute("disableconnect"   , "true" );
            this.statusElement.setAttribute("disabledisconnect", "false");
        }
        else if(topic == NETIZEN_NETWORK_STATUS_REGISTERING)
        {
            this.statusElement.setAttribute("label", this.registeringString);
            this.statusElement.setAttribute("disableconnect"   , "true" );
            this.statusElement.setAttribute("disabledisconnect", "false");
        }
        else if(topic == NETIZEN_NETWORK_STATUS_CONNECTED)
        {
            this.statusElement.setAttribute("label", this.connectedString);
            this.statusElement.setAttribute("disableconnect"   , "true" );
            this.statusElement.setAttribute("disabledisconnect", "false");
        }
        else if(topic == NETIZEN_NETWORK_STATUS_PINGING)
        {
            this.statusElement.setAttribute("label", this.pingingString);
            this.statusElement.setAttribute("disableconnect"   , "true" );
            this.statusElement.setAttribute("disabledisconnect", "false");
        }
        else if(topic == NETIZEN_NETWORK_STATUS_DISCONNECTED)
        {
            this.statusElement.setAttribute("label", this.disconnectedString);
            this.statusElement.setAttribute("disableconnect"   , "false");
            this.statusElement.setAttribute("disabledisconnect", "true" );
        }

        this.statusElement.setAttribute("status", topic);
        this.cacheStatus(topic);
        
        return;
    }    
};



var NetizenNetworkAlertObserver =
{
    loadStringBundle : function()
    {
        var netizenStringBundle = document.getElementById("netizenStringBundle");
        if(!netizenStringBundle)
        {
            return;
        }

        this.disconnectedAlert = netizenStringBundle.getString("netizen.connect.alert.disconnected");
        this.firewalledAlert   = netizenStringBundle.getString("netizen.connect.alert.firewalled");
        this.learnMoreLinkText = netizenStringBundle.getString("netizen.connect.alert.linktext");
        
        return;
    },


    loadObservers : function()
    {
        this.loadStringBundle();
        this.initAlert();
        
        ObserverService.addObserver(this, NETIZEN_NETWORK_ALERT_FIREWALLED  , false);
        return;
    },


    unloadObservers : function()
    {
        ObserverService.removeObserver(this, NETIZEN_NETWORK_ALERT_FIREWALLED  );
        return;
    },


    initAlert : function()
    {
        var networkAlert = NetizenPropertyCache.getPropertyValue(NETIZEN_NETWORK_ALERT_PROPERTY);
        if(networkAlert)
        {
            this.observe(null, networkAlert, null);
        }
                
        return;
    },
    

    cacheAlert : function(alertTopic)
    {
        var networkAlert= NetizenPropertyCache.getPropertyValue(NETIZEN_NETWORK_ALERT_PROPERTY);
        if(networkAlert != alertTopic)
        {
            NetizenPropertyCache.cachePropertyValue(NETIZEN_NETWORK_ALERT_PROPERTY, alertTopic);
        }

        return;
    },


    QueryInterface : function(iid)
    {
        if(iid.equals(Interfaces.nsIObserver) ||
           iid.equals(Interfaces.nsISupports)  )
        {
            return this;
        }
        
        throw Results.NS_NOINTERFACE;
    },

    showFirewallAlert : function()
    {
        return;
    },


    observe : function(subject, topic, topicdata)
    {
        if(topic == NETIZEN_NETWORK_ALERT_FIREWALLED)
        {
            this.showFirewallAlert();
        }

        this.cacheAlert(topic);
        
        return;
    }
}
