/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is the Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2003 Civil Engines Research.  All Rights 
 * Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/

const APPLICATION_SERVICE_CONTRACTID        = "@civilengines.com/appshell/component/netizen;1";
const APPLICATION_WINDOW_SERVICE_CONTRACTID = "@civilengines.com/appshell/windowservice;1";

const NETIZEN_PRODUCT_NAME_PROPERTY         = "product";
const NETIZEN_PRODUCT_LOCALE_PROPERTY       = "locale";
const NETIZEN_PRODUCT_SVN_PROPERTY          = "svn";

var   ApplicationService                    = null;
var   ApplicationWindowService              = null;


var NetizenApplicationService =
{
    loadService : function()
    {
//@line 54 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/base/content/netizenMenuOverlay.js"

        try
        {
            ApplicationService       = do_GetService(APPLICATION_SERVICE_CONTRACTID  ,
                                                     Interfaces.ceIApplicationService);
            
            ApplicationWindowService = do_QueryInterface(ApplicationService                    ,
                                                         Interfaces.ceIApplicationWindowService);
            

            ApplicationService.startApplication();
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }

        return;
    },

    
    unloadService : function()
    {
//@line 80 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/base/content/netizenMenuOverlay.js"
        try
        {
            ApplicationService.stopApplication();
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }
        
        return;
    }
};



var NetizenHelpController =
{
    supportsCommand : function(commandId)
    {
        if(commandId == "cmd_checkForUpdates"    ||
           commandId == "cmd_checkUpdateWebsite" ||
           commandId == "cmd_checkStatusWebsite" ||
           commandId == "cmd_aboutNetizen"        )
        {
            return true;
        }
        
        return false;
    },


    isCommandEnabled : function(commandId)
    {
        if(commandId == "cmd_checkForUpdates"    ||
           commandId == "cmd_checkUpdateWebsite" ||
           commandId == "cmd_checkStatusWebsite" ||
           commandId == "cmd_aboutNetizen"        )
        {
            return true;
        }
        
        return false;
    },


    doCommand : function(commandId)
    {
        if(commandId == "cmd_checkForUpdates")
        {
            this.checkForUpdates();
        }
        else if(commandId == "cmd_checkUpdateWebsite")
        {
            this.checkUpdateWebsite();
        }
        else if(commandId == "cmd_checkStatusWebsite")
        {
            this.checkStatusWebsite();
        }
        else if(commandId == "cmd_aboutNetizen")
        {
            this.showAboutNetizen();
        }

        this.onCommandUpdate();            
            
        return;
    },
    

    onCommandUpdate : function()
    {
        var updateCheckerIsEnabled = this.isCommandEnabled("cmd_checkForUpdates");
        var updateNetizenIsEnabled = this.isCommandEnabled("cmd_checkUpdateWebsite");
        var netizenStatusIsEnabled = this.isCommandEnabled("cmd_checkStatusWebsite");
        var aboutIsEnabled         = this.isCommandEnabled("cmd_aboutNetizen");
        
        goSetCommandEnabled("cmd_checkForUpdates"   , updateCheckerIsEnabled);
        goSetCommandEnabled("cmd_checkUpdateWebsite", updateNetizenIsEnabled);
        goSetCommandEnabled("cmd_checkStatusWebsite", netizenStatusIsEnabled);
        goSetCommandEnabled("cmd_aboutNetizen"      , aboutIsEnabled        );
        
        return;
    },

    
    getProductVersionQueryString : function()
    {
        var versionParameters = new ceStandardURLQuery();

        try
        {
            var applicationName  = ApplicationService.applicationName;
            var localeInfo       = ApplicationService.interfaceLocale;
            var versionNumber    = ApplicationService.versionNumber;
            
            versionParameters.addQueryParameter(NETIZEN_PRODUCT_NAME_PROPERTY  , applicationName);
            versionParameters.addQueryParameter(NETIZEN_PRODUCT_LOCALE_PROPERTY, localeInfo);
            versionParameters.addQueryParameter(NETIZEN_PRODUCT_SVN_PROPERTY   , versionNumber);
        }
        catch(exception)
        {
            return "";
        }
        
        return versionParameters.toQueryString();
    },
    

    checkForUpdates : function()
    {
        var updateManager = do_GetService("@civilengines.com/updates/update-manager;1",
                                          Interfaces.nsIUpdateManager                 );
        
        var updatePrompt  = do_CreateInstance("@civilengines.com/updates/update-prompt;1",
                                              Interfaces.nsIUpdatePrompt                 );
        
        if(updateManager.activeUpdate && updateManager.activeUpdate.state == "pending")
        {
            updatePrompt.showUpdateDownloaded(updateManager.activeUpdate);            
        }
        else
        {
            updatePrompt.checkForUpdates();
        }
        
        return;
    },


    checkUpdateWebsite : function()
    {
        var updateWebsiteURL = NetizenApplication.updateURL;
        
        try
        {
            updateWebsiteURL    += "?" + this.getProductVersionQueryString();
            NetizenWindowController.loadExternalURL(updateWebsiteURL);
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }
        
        return;
    },


    checkStatusWebsite : function()
    {
        var statusWebsiteURL = NetizenApplication.statusURL;

        try
        {
            statusWebsiteURL    += "?" + this.getProductVersionQueryString();
            NetizenWindowController.loadExternalURL(statusWebsiteURL);
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }
        
        return;
    },


    showAboutNetizen : function()
    {
        window.openDialog("chrome://netizen/content/netizenAboutDialog.xul",
                          "netizenAboutDialog"                             ,
                          "dependent=yes,centerscreen,chrome,resizable=no" );
        
        return;
    }    
};





var NetizenNavigationController =
{
    canGoBack : function()
    {
        var navigationControl = NetizenApplication.getNavigationControl();
        if(!navigationControl || !navigationControl.canGoBack)
        {
            return false;
        }
        
        var previousPageIndex = NetizenWindowController.getCurrentPageIndex() - 1;
        var previousPageURI   = NetizenWindowController.getPageViewURIAtIndex(previousPageIndex, false);
        if(!previousPageURI)
        {
            return false;
        }
        
        return true;
    },


    canGoForward : function()
    {
        var navigationControl = NetizenApplication.getNavigationControl();
        if(!navigationControl || !navigationControl.canGoForward)
        {
            return false;
        }

        var nextPageIndex = NetizenWindowController.getCurrentPageIndex() + 1;
        var nextPageURI   = NetizenWindowController.getPageViewURIAtIndex(nextPageIndex, false);
        if(!nextPageURI)
        {
            return false;
        }

        return true;
    },


    supportsCommand : function(commandId)
    {
        if(commandId == "cmd_netizenBack" || commandId == "cmd_netizenForward")
        {
            return true;
        }
        
        return false;
    },


    isCommandEnabled : function(commandId)
    {
        if(commandId == "cmd_netizenBack")
        {
            return this.canGoBack();
        }
                
        if(commandId == "cmd_netizenForward")
        {
            return this.canGoForward();
        }

        return false;
    },


    doCommand : function(commandId)
    {
        if(commandId == "cmd_netizenBack")
        {
            this.netizenGoBack();
        }
        else if(commandId == "cmd_netizenForward")
        {
            this.netizenGoForward();
        }
        
        this.onCommandUpdate();
        return;
    },
    

    onCommandUpdate : function()
    {
        var backIsEnabled    = this.isCommandEnabled("cmd_netizenBack");
        var forwardIsEnabled = this.isCommandEnabled("cmd_netizenForward");
        
        goSetCommandEnabled("cmd_netizenBack"   , backIsEnabled   );
        goSetCommandEnabled("cmd_netizenForward", forwardIsEnabled);
        
        return;
    },


    netizenGoBack : function()
    {
        var netizenBrowser    = NetizenWindowController.getNetizenBrowser();
        var navigationControl = netizenBrowser.navigationControl;
        
        if(navigationControl)
        {
            try
            {
                /* Persist the current widget state only for the
                 * most recent view. Do not save widget state if
                 * the view was loaded from session history.
                 */
                if(!netizenBrowser.getAttribute("history"))
                {
                    NetizenWindowController.saveCurrentWidgetState(netizenBrowser.contentWindow);
                }
                
                navigationControl.goBack();
                netizenBrowser.setAttribute("history", "true");
                netizenBrowser.setAttribute("lastnav", "backward");
            }
            catch(exception)
            {
                Syslog.logException(exception);
            }
        }
        
        return;
    },


    netizenGoForward : function()
    {
        var netizenBrowser    = NetizenWindowController.getNetizenBrowser();
        var navigationControl = netizenBrowser.navigationControl;
        
        if(navigationControl)
        {
            try
            {
                /* Forward navigation must not persist any widget 
                 * state to preserve the pages loaded from session
                 * history.
                 */
                navigationControl.goForward();
                
                /* If there are more forward pages in session history
                 * then mark this page a previous view 
                 */
                if(this.canGoForward())
                {
                    netizenBrowser.setAttribute("history", "true");
                }

                netizenBrowser.setAttribute("lastnav", "forward");
            }
            catch(exception)
            {
                Syslog.logException(exception);
            }
        }
        
        return;
    },


    repeatNavigation : function()
    {
        var netizenBrowser    = NetizenWindowController.getNetizenBrowser();
        var navigationControl = netizenBrowser.navigationControl;
        if(navigationControl && netizenBrowser)
        {
            var lastNavAction = netizenBrowser.getAttribute("lastnav");
            if(lastNavAction && this.isCommandEnabled(lastNavAction))
            {
                this.doCommand(lastNavAction);
            }
            else
            {
                (lastNavAction == "forward") ? this.netizenGoBack() : this.netizenGoForward();
            }
        }
        
        return;
    }
};




var NetizenCommandController  =
{
    onLoad : function()
    {
        NetizenApplicationService.loadService();

        top.controllers.appendController(NetizenNavigationController);
        top.controllers.appendController(NetizenHelpController);
        return;
    },


    onUnload : function()
    {
        top.controllers.removeController(NetizenHelpController);
        top.controllers.removeController(NetizenNavigationController);

        NetizenApplicationService.unloadService();
        
        return;
    }
};



NetizenCommandController.onLoad();



/* update navigation commands */
function goUpdateNavigationCommands()
{
    goUpdateCommand("cmd_netizenBack"   );
    goUpdateCommand("cmd_netizenForward");

    return;
}


/* update menu items that rely on focus */
function goUpdateGlobalEditMenuItems()
{
    goUpdateCommand("cmd_undo"     );
    goUpdateCommand("cmd_redo"     );
    goUpdateCommand("cmd_cut"      );
    goUpdateCommand("cmd_copy"     );
    goUpdateCommand("cmd_paste"    );
    goUpdateCommand("cmd_selectAll");
    goUpdateCommand("cmd_delete"   );

    var textDirectionElement = document.getElementById('cmd_SwitchTextDirection');
    if(textDirectionElement)
    {
        textDirectionElement.setAttribute('disabled', !document.commandDispatcher.focusedElement);
    }
}


/* update menu items that rely on the current selection */
function goUpdateSelectEditMenuItems()
{
    goUpdateCommand("cmd_cut"      );
    goUpdateCommand("cmd_copy"     );
    goUpdateCommand("cmd_delete"   );
    goUpdateCommand("cmd_selectAll");
}



/* update menu items that relate to undo/redo */
function goUpdateUndoEditMenuItems()
{
    goUpdateCommand("cmd_undo");
    goUpdateCommand("cmd_redo");
}



/* update menu items that depend on clipboard contents */
function goUpdatePasteMenuItems()
{
    goUpdateCommand("cmd_paste");
}


/* update toolbar visibility */
function goToggleToolbar(id, elementId)
{
    var toolbar = document.getElementById(id);
    var element = document.getElementById(elementId);
    
    if(toolbar)
    {
        var isHidden   = toolbar.hidden;
        toolbar.hidden = !isHidden;
        document.persist(id, 'hidden');

        if(element) 
        {
            element.setAttribute("checked", isHidden ? "true" : "false");
            document.persist(elementId, 'checked');
        }
    }
}
