/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2005 Civil Engines Research.  All Rights 
 * Reserved.  
 *
 * Portions created by Netscape Copyright (C) 1998 Netscape 
 * Communications Corporation. All Rights Reserved.
 *
 * Portions created by the Mozilla Foundation are Copyright (C) 
 * 2003 Mozilla Foundation. All Rights Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/

const FILEPICKER_CONTRACTID                  = "@mozilla.org/filepicker;1";

const PARCEL_SLIP_DETAILS_VIEW_CHROME_URL    = "chrome://netizen/content/parcel/parcelSlipDetails.xul"
const PARCEL_UNPACK_DIALOG_CHROME_URL        = "chrome://netizen/content/parcel/parcelUnpackProgress.xul"
const PARCEL_DETAILS_VIEW_CHROME_URL         = "chrome://netizen/content/parcel/parcelDetails.xul";
const PARCEL_DETAILS_VIEWID_DEFAULT          = "default";

const PARCEL_APPLICATION_MODE_SEND           = 1;
const PARCEL_APPLICATION_MODE_PICKUP         = 2;
const PARCEL_APPLICATION_MODE_TRACK          = 4;
const PARCEL_APPLICATION_MODE_NONE           = 16;


var ParcelModeSelector =
{
    sendAppButton    : null,
    pickupAppButton  : null,
    trackerAppButton : null,
    currentMode      : null,

    onModeSelect : function(applicationMode)
    {
        if(!applicationMode || (this.currentMode == applicationMode))
        {
            return;
        }
        
        if(applicationMode == PARCEL_APPLICATION_MODE_SEND)
        {
            if(!this.sendAppButton)
            {
                this.sendAppButton = document.getElementById("parcelSendAppButton");
            }
            
            this.sendAppButton.setAttribute("checked", "true");
            this.currentMode = PARCEL_APPLICATION_MODE_SEND;
        }
        else if(applicationMode == PARCEL_APPLICATION_MODE_PICKUP)
        {
            if(!this.pickupAppButton)
            {
                this.pickupAppButton = document.getElementById("parcelPickupAppButton");
            }

            this.pickupAppButton.setAttribute("checked", "true");
            this.currentMode = PARCEL_APPLICATION_MODE_PICKUP;
        }
        else if(applicationMode == PARCEL_APPLICATION_MODE_TRACK)
        {
            if(!this.trackerAppButton)
            {
                this.trackerAppButton = document.getElementById("parcelTrackerAppButton");
            }

            this.trackerAppButton.setAttribute("checked", "true");
            this.currentMode = PARCEL_APPLICATION_MODE_TRACK;
        }

        return;
    },


    onModeUnselect : function(applicationMode)
    {
        if(!applicationMode)
        {
            return;
        }

        if(applicationMode == PARCEL_APPLICATION_MODE_SEND)
        {
            if(!this.sendAppButton)
            {
                this.sendAppButton = document.getElementById("parcelSendAppButton");
            }

            this.sendAppButton.setAttribute("checked", "false");
            this.currentMode = PARCEL_APPLICATION_MODE_NONE;            
        }
        else if(applicationMode == PARCEL_APPLICATION_MODE_PICKUP)
        {
            if(!this.pickupAppButton)
            {
                this.pickupAppButton = document.getElementById("parcelPickupAppButton");
            }

            this.pickupAppButton.setAttribute("checked", "false");
            this.currentMode = PARCEL_APPLICATION_MODE_NONE;
        }
        else if(applicationMode == PARCEL_APPLICATION_MODE_TRACK)
        {
            if(!this.trackerAppButton)
            {
                this.trackerAppButton = document.getElementById("parcelTrackerAppButton");
            }

            this.trackerAppButton.setAttribute("checked", "false");
            this.currentMode = PARCEL_APPLICATION_MODE_NONE;
        }

        return;
    }
};




var ParcelResourceController =
{
    initController : function()
    {
        if(this.unpackSelectFolderString)
        {
            return;
        }
        
        var parcelStringBundle = document.getElementById("parcelStringBundle");
        if(!parcelStringBundle)
        {
            return;
        }

        this.unpackSelectFolderString = parcelStringBundle.getString("parcel.dialog.select.folder");
        this.openPickupSlipString     = parcelStringBundle.getString("parcel.dialog.open.pickupslip.file");
        this.pickupSlipFilterString   = parcelStringBundle.getString("parcel.dialog.filter.pickupslip");
        
        return;
    },


    loadParcelDetailsView : function(resourceURL)
    {
        if(!this.parcelEntryExists(resourceURL))
        {
            return;
        }
        
        var currentPageURI        = NetizenWindowController.getCurrentPageViewURI();
        var referrerURISpec       = currentPageURI.spec;
        var currentMode           = ParcelModeSelector.currentMode;
        var detailsViewParameters = new ceStandardURLQuery();
        if(detailsViewParameters)
        {
            detailsViewParameters.addQueryParameter(NETIZEN_CONTENT_RESOURCE_PROPERTY, resourceURL);
            detailsViewParameters.addQueryParameter(NETIZEN_CONTENT_MODE_PROPERTY    , currentMode);
            detailsViewParameters.addQueryParameter(NETIZEN_CONTENT_REFERRER_PROPERTY, referrerURISpec); 
        }
        
        NetizenWindowController.scheduleApplicationViewLoad(PARCEL_DETAILS_VIEW_CHROME_URL       , 
                                                            PARCEL_DETAILS_VIEWID_DEFAULT        ,
                                                            detailsViewParameters.toQueryString());
        return;
    },


    loadPickupSlipDialog : function(resourceURL)
    {
        if(!this.parcelEntryExists(resourceURL))
        {
            return;
        }

        window.openDialog(PARCEL_SLIP_DETAILS_VIEW_CHROME_URL             ,
                          "viewPickupSlipDialog"                          ,
                          "dependent=yes,centerscreen,chrome,resizable=no",
                          resourceURL                                     );
    
        return;
    },



    loadUnpackParcelDialog : function(resourceURL, destinationDirectory)
    {
        if(!this.parcelEntryExists(resourceURL))
        {
            return;
        }

        try
        {
            var parcelManager    = do_GetService("@civilengines.com/apps/parcel-manager;1",
                                                 Interfaces.ceIParcelTrackerService       );
            
            var trackerInfo      = parcelManager.trackerMetaInfo;        
            var trackerResource  = trackerInfo.getResource(resourceURL);
            var trackerEntry     = do_QueryInterface(trackerResource                 , 
                                                     Interfaces.ceIParcelTrackerEntry);
            
            var parcelFileObject = parcelManager.getFileFromLocalPath(trackerEntry);
            var parcelName       = parcelFileObject.leafName.replace(/.parc/gi, "");
            if(parcelName)
            {
                destinationDirectory.append(parcelName);
                FSUtils.ensureUniqueness(destinationDirectory);
                
                destinationDirectory.create(Interfaces.nsIFile.DIRECTORY_TYPE, 0644);
            }
        }
        catch(exception)
        {
            Syslog.logException(exception);
            return;
        }
        

        window.openDialog(PARCEL_UNPACK_DIALOG_CHROME_URL                 ,
                          "unpackParcelProgressDialog"                    ,
                          "dependent=yes,centerscreen,chrome,resizable=no",
                          resourceURL                                     ,
                          destinationDirectory                            );
    
        return;
    },


    browseForPickupSlipFile : function()
    {
        this.initController();

        var pickupSlipFile = null;        
        var filePicker     = do_CreateInstance(FILEPICKER_CONTRACTID, Interfaces.nsIFilePicker);
        if(!filePicker)
        {
            return;
        }

        try
        {                
            filePicker.init(window                           , 
                            this.openPickupSlipString        , 
                            Interfaces.nsIFilePicker.modeOpen);
            
            filePicker.appendFilter(this.pickupSlipFilterString, "*.slip");
            
            var filePickerResult = filePicker.show();
            if(filePickerResult == Interfaces.nsIFilePicker.returnOK)
            {
                pickupSlipFile = do_QueryInterface(filePicker.file        ,
                                                   Interfaces.nsILocalFile);
                
            }
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }

        return pickupSlipFile;
    },
    

    unpackParcel : function(resourceURL)
    {
        this.initController();

        var destinationFolder = null;        
        var filePicker        = do_CreateInstance(FILEPICKER_CONTRACTID, Interfaces.nsIFilePicker);
        if(!filePicker)
        {
            return null;
        }

        try
        {                
            filePicker.init(window                                , 
                            this.unpackSelectFolderString         , 
                            Interfaces.nsIFilePicker.modeGetFolder);
            
            var filePickerResult = filePicker.show();
            if(filePickerResult == Interfaces.nsIFilePicker.returnOK)
            {
                destinationFolder = do_QueryInterface(filePicker.file        ,
                                                      Interfaces.nsILocalFile);
            }
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }
        
        
        if(!destinationFolder)
        {
            return null;
        }

        this.loadUnpackParcelDialog(resourceURL, destinationFolder);
        
        return FSUtils.convertPathToUrl(destinationFolder.path);
    },


    unpackParcelToDesktop : function(resourceURL)
    {
        var parcelManager = do_GetService("@civilengines.com/apps/parcel-manager;1",
                                          Interfaces.ceIParcelManager              );
        if(!parcelManager)
        {
            return null;
        }
        

        var destinationFolder = parcelManager.defaultDirectory;
        if(!destinationFolder)
        {
            Assert.condition(destinationFolder, "NULL default directory");
            return null;
        }

        this.loadUnpackParcelDialog(resourceURL, destinationFolder);
        
        return FSUtils.convertPathToUrl(destinationFolder.path);
    },



    unpackParcelToUserDocuments : function(resourceURL)
    {
        var parcelManager = do_GetService("@civilengines.com/apps/parcel-manager;1",
                                          Interfaces.ceIParcelManager              );
        if(!parcelManager)
        {
            return null;
        }
        

        var destinationFolder = parcelManager.extractDirectory;
        if(!destinationFolder)
        {
            Assert.condition(destinationFolder, "NULL default directory");
            return null;
        }

        this.loadUnpackParcelDialog(resourceURL, destinationFolder);
        
        return FSUtils.convertPathToUrl(destinationFolder.path);
    },


    parcelEntryExists : function(trackerResourceURI)
    {
        try
        {
            var parcelTracker = do_GetService("@civilengines.com/apps/parcel-manager;1",
                                              Interfaces.ceIParcelTrackerService       );

            return parcelTracker.trackerEntryExists(trackerResourceURI);
        }
        catch(exception)
        {
        }

        return false;
    },


    revealParcelLocation : function(locationURL)
    {
        var locationPath    = FSUtils.convertUrlToPath(locationURL);
        if(!locationPath)
        {
            return;
        }
        
        var parcelDirectory = FSUtils.createLocalFile(locationPath);
        if(parcelDirectory)
        {
            parcelDirectory.reveal();
        }

        return;
    }
};



var ParcelNetworkAlertObserver =
{
    loadStringBundle : function()
    {
        var parcelStringBundle = document.getElementById("parcelStringBundle");
        if(!parcelStringBundle)
        {
            return;
        }

        this.disconnectedAlert = parcelStringBundle.getString("parcel.connect.alert.disconnected");
        this.firewalledAlert   = parcelStringBundle.getString("parcel.connect.alert.firewalled");
        this.learnMoreLinkText = parcelStringBundle.getString("parcel.connect.alert.linktext");
        
        return;
    },


    loadObservers : function()
    {
        this.loadStringBundle();
        this.initAlert();
        
        ObserverService.addObserver(this, NETIZEN_NETWORK_ALERT_FIREWALLED  , false);
        ObserverService.addObserver(this, NETIZEN_NETWORK_ALERT_NATDETECTED , false);
        ObserverService.addObserver(this, NETIZEN_NETWORK_ALERT_OPENINTERNET, false);

        return;
    },


    unloadObservers : function()
    {
        ObserverService.removeObserver(this, NETIZEN_NETWORK_ALERT_FIREWALLED  );
        ObserverService.removeObserver(this, NETIZEN_NETWORK_ALERT_NATDETECTED );
        ObserverService.removeObserver(this, NETIZEN_NETWORK_ALERT_OPENINTERNET);

        return;
    },


    initAlert : function()
    {
        var networkAlert = NetizenPropertyCache.getPropertyValue(NETIZEN_NETWORK_ALERT_PROPERTY);
        if(networkAlert)
        {
            this.observe(null, networkAlert, null);
        }
                
        return;
    },
    

    cacheAlert : function(alertTopic)
    {
        var networkAlert= NetizenPropertyCache.getPropertyValue(NETIZEN_NETWORK_ALERT_PROPERTY);
        if(networkAlert != alertTopic)
        {
            NetizenPropertyCache.cachePropertyValue(NETIZEN_NETWORK_ALERT_PROPERTY, alertTopic);
        }

        return;
    },


    QueryInterface : function(iid)
    {
        if(iid.equals(Interfaces.nsIObserver) ||
           iid.equals(Interfaces.nsISupports)  )
        {
            return this;
        }
        
        throw Results.NS_NOINTERFACE;
    },


    showFirewallAlert : function()
    {
        var statusIcon         = "chrome://netizen/skin/parcel/expiration-status-yellow.png";
        var parcelControlPanel = document.getElementById("parcelControlPanel");
        if(parcelControlPanel)
        {
            parcelControlPanel.showStatusMessage(this.firewalledAlert            ,
                                                 null                            ,
                                                 null                            ,
                                                 NETIZEN_NETWORK_ALERT_FOLLOWLINK,
                                                 statusIcon                      );
        }

        return;
    },


    hideStatusAlert : function()
    {
        var parcelControlPanel = document.getElementById("parcelControlPanel");
        if(parcelControlPanel)
        {
            parcelControlPanel.hideStatusMessage();
        }
        
        return;
    },

    
    observe : function(subject, topic, topicdata)
    {
        if(topic == NETIZEN_NETWORK_ALERT_FIREWALLED )
        {
            this.showFirewallAlert();
        }
        else if(topic == NETIZEN_NETWORK_ALERT_NATDETECTED ||
                topic == NETIZEN_NETWORK_ALERT_OPENINTERNET )
        {
            this.hideStatusAlert();
        }
        
        this.cacheAlert(topic);
        
        return;
    }
}



function goUpdatePackageCommands()
{
    goUpdateCommand("cmd_parcelPackageTarget");
    goUpdateCommand("cmd_parcelPackageCancel");
    goUpdateCommand("cmd_parcelPackageNew");
    goUpdateCommand("cmd_parcelSelectPackage");

    goUpdateCommand("cmd_parcelSavePickupSlip");
    goUpdateCommand("cmd_parcelPackageViewSlip");
    goUpdateCommand("cmd_parcelEmailPickupSlip");

    goUpdateCommand("cmd_parcelViewExisting");
    goUpdateCommand("cmd_parcelViewNew");

    return;
}



function goUpdatePickupCommands()
{
    goUpdateCommand("cmd_parcelViewTextInput");
    goUpdateCommand("cmd_parcelViewFileInput");   
    goUpdateCommand("cmd_parcelValidateCancel");

    goUpdateCommand("cmd_parcelPickupTarget");
    goUpdateCommand("cmd_parcelPickupUnpack");

    goUpdateCommand("cmd_paste");
    goUpdateCommand("cmd_parcelPastePickupSlip");

    return;
}


function goUpdateTrackerCommands()
{
    goUpdateCommand("cmd_parcelTrackerDetails");
    goUpdateCommand("cmd_parcelConfirmDelete" );
    goUpdateCommand("cmd_parcelTrackerDelete" );
    goUpdateCommand("cmd_parcelTrackerUnpack" );
    
    return;
}



window.onresize = function()
{
    NetizenWindowResizer.onWindowResize();
    NetizenFrameController.onWindowResize();

    return;
};
