/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2005 Civil Engines Research.  All Rights 
 * Reserved.  
 *
 * Portions created by Netscape Copyright (C) 1998 Netscape 
 * Communications Corporation. All Rights Reserved.
 *
 * Portions created by the Mozilla Foundation are Copyright (C) 
 * 2003 Mozilla Foundation. All Rights Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/

const PARCEL_MANAGER_CONTRACTID              = "@civilengines.com/apps/parcel-manager;1";
const PARCEL_METAINFOSERVICE_CONTRACTID      = "@civilengines.com/metainf/loader-service;1";
const PARCEL_PICKUP_URL_CONTRACTID           = "@civilengines.com/protocol/pickup/url;1";

const PARCEL_PICKUPAPP_CHROME_URL            = "chrome://netizen/content/parcel/parcelAppPickup.xul";
const PARCEL_PICKUPAPP_STATE_DEFAULT         = "default";
const PARCEL_PICKUPAPP_STATE_TRANSFER        = "xfer";

const PARCEL_PICKUP_PROPERTY_TARGETURI       = "urn:parcel:property:pickup:targetURI";
const PARCEL_PICKUP_PROPERTY_METAINFOID      = "urn:parcel:property:pickup:metaInfoId";

const PARCEL_INFO_EVENT_FOLLOWLINK           = "urn:parcel:info:event:link";
const PARCEL_INFO_EVENT_EXECOMMAND           = "urn:parcel:info:event:command";

const PARCEL_PICKUP_EVENT_PICKUPANOTHER      = "urn:parcel:pickup:event:pickupanother";
const PARCEL_PICKUP_EVENT_UPDATEXFERTHROBBER = "urn:parcel:pickup:event:updatexferthrobber";

const PARCEL_PICKUP_SLIPDETAILS_VIEW_DELAY   = 500;

var   MetaInfoService                        = null;
var   ParcelManager                          = null;
var   TransferManager                        = null;


function loadApplication(loadRequestURL, isHistoryView)
{
    Syslog.logMessage("Loading Parcel Pickup Application");

    var applicationParams = loadRequestURL.getQueryParameters();

    ParcelPickupApplication.loadServices(applicationParams);
    ParcelPickupApplication.loadControllers();
    ParcelPickupApplication.loadHandlers();
    ParcelPickupApplication.loadStringBundles();
    ParcelPickupApplication.loadApplicationView(applicationParams, isHistoryView);
    
    return;
}



function unloadApplication(unloadRequestURL)
{
    var applicationParams = unloadRequestURL.getQueryParameters();

    if(TransferManager)
    {
        TransferManager.saveTransferState();
    }
    
    ParcelPickupApplication.unloadHandlers();
    ParcelPickupApplication.unloadControllers();
    ParcelPickupApplication.unloadServices(unloadRequestURL.getQueryParameters());

    ParcelPickupApplication.unloadApplicationView(applicationParams);

    return;
}



var ParcelPickupApplication =
{
    loadServices : function(applicationParams)
    {
        ParcelModeSelector.onModeSelect(PARCEL_APPLICATION_MODE_PICKUP);
        
        NetizenNetworkStatusObserver.loadObservers();

        ParcelNetworkAlertObserver.loadObservers();

        ParcelTransferObserver.loadObservers();
        
        ParcelPickupStatusObserver.loadObservers();
        
        try
        {
            TransferManager = do_GetService(TRANSFER_MANAGER_CONTRACTID  , 
                                            Interfaces.ceITransferManager);
            
            MetaInfoService = do_GetService(PARCEL_METAINFOSERVICE_CONTRACTID,
                                            Interfaces.ceIMetaInfoService    );
            
            ParcelManager   = do_GetService(PARCEL_MANAGER_CONTRACTID         , 
                                            Interfaces.ceIParcelTrackerService);
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }
        
        /* Set the transfer listener */
        var activeTransfers = TransferManager.activeTransfers;
        var pickupProgress  = new ceParcelPickupProgressListener(document, activeTransfers);
        if(pickupProgress)
        {
            TransferManager.progressListener = pickupProgress;
        }

        /* Set the retry event sink */
        var retryEventSink  = new ceParcelPickupRetryEventSink(document);
        if(retryEventSink)
        {
            TransferManager.retrySchedulerCallback = retryEventSink;
        }
        
        return;
    },
    

    unloadServices : function(applicationParams)
    {
        TransferManager.retrySchedulerCallback = null;

        TransferManager.progressListener = null;
        
        ParcelNetworkAlertObserver.unloadObservers();
        
        NetizenNetworkStatusObserver.unloadObservers();
        
        ParcelTransferObserver.unloadObservers();

        ParcelPickupStatusObserver.unloadObservers();

        ParcelModeSelector.onModeUnselect(PARCEL_APPLICATION_MODE_PICKUP);
    },
    

    loadControllers : function()
    {
        top.controllers.appendController(ParcelPickupController);
        ParcelPickupController.onCommandUpdate();
        
        return;
    },


    unloadControllers : function()
    {
        ParcelPickupController.onCommandUpdate();
        top.controllers.removeController(ParcelPickupController);
        
        return;
    },


    loadHandlers : function()
    {
        var pickupContentPanel  = document.getElementById("pickupTransferContentPanel");
        if(pickupContentPanel)
        {
            ParcelTransferEventHandler.registerListener(pickupContentPanel);
        }
        
        var parcelPickupDetails = document.getElementById("parcelPickupDetails");
        if(parcelPickupDetails)
        {
            ParcelInfoEventHandler.registerListener(parcelPickupDetails);
        }
        
        return;
    },


    unloadHandlers : function()
    {
        var pickupContentPanel = document.getElementById("pickupTransferContentPanel");
        if(pickupContentPanel)
        {
            ParcelTransferEventHandler.unregisterListener(pickupContentPanel);
        }
        
        var parcelPickupDetails = document.getElementById("parcelPickupDetails");
        if(parcelPickupDetails)
        {
            ParcelInfoEventHandler.unregisterListener(parcelPickupDetails);
        }
        
        return;
    },
        

    loadStringBundles : function()
    {
        var parcelStringBundle = document.getElementById("parcelStringBundle");
        if(!parcelStringBundle)
        {
            return;
        }

        this.desktopChoiceString   = parcelStringBundle.getString("parcel.unpack.desktop.choice");
        this.unpackChooserString   = parcelStringBundle.getString("parcel.unpack.otherfolder.choice");
        
//@line 238 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/components/parcel/content/parcelAppPickup.js"
        this.documentsChoiceString = parcelStringBundle.getString("parcel.unpack.documents.choice.win");
//@line 240 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/components/parcel/content/parcelAppPickup.js"
      
        return;
    },

    
    loadApplicationView : function(applicationParams, isHistoryView)
    {
        var viewid             = applicationParams.getParameterValue(NETIZEN_CONTENT_VIEWID_PROPERTY);
        var viewControl        = document.getElementById("parcelPickupContentPanels");
    
        if(viewControl && viewid == PARCEL_PICKUPAPP_STATE_TRANSFER)
        {
            this.loadTransferProgressView(applicationParams);
            ParcelPickupController.onCommandUpdate();
        }
        
        this.refreshApplicationView(isHistoryView);
        return;
    },


    unloadApplicationView : function(applicationParams)
    {
        return;
    },


    refreshApplicationView : function(isHistoryView)
    {
        var parcelPickupInfo = document.getElementById("parcelPickupInfo");
        var viewControl      = document.getElementById("parcelPickupContentPanels");
        
        if(viewControl && viewControl.selectedIndex == 1)
        {
            /* Restore the pickup viewstate */
            var transferDatasource = do_QueryInterface(TransferManager.transferMetaInfo,
                                                       Interfaces.ceIRDFMetaInfo       );

            var trackerId          = parcelPickupInfo.getAttribute("trackerId");
            var resourceEntry      = null;
            
            try
            {
                var trackerMetaInfo = ParcelManager.trackerMetaInfo;
                resourceEntry       = trackerMetaInfo.getResource(trackerId);

                /* Restore the transfer progress information */
                this.refreshTransferProgressInfo(trackerId);
        
                /* Attach the transfer datasource */
                parcelPickupInfo.addDataSource(transferDatasource.dataSource);
                parcelPickupInfo.rebuildContent();
            }
            catch(exception)
            {
                ParcelPickupController.cancelPickupSlipValidation();
                ParcelPickupController.onCommandUpdate();

                viewControl.selectedIndex = 0;
                return;
            }
        }


        ParcelPickupController.onCommandUpdate();

        return;
    },

    
    refreshTransferProgressInfo : function(trackerId)
    {
        var transferInfo = NetizenPropertyCache.getPropertyValue(trackerId);
        if(transferInfo                                                                   &&
           transferInfo.transferState == Interfaces.ceITransferInfo.TRANSFER_FINISHED     &&
           transferInfo.transferType  == Interfaces.ceITransferInfo.TRANSFER_TYPE_DOWNLOAD )
        {
            transferInfoEntry = do_QueryInterface(transferInfo, Interfaces.ceITransferInfoEntry);
            
            /* Restore transfer complete view state */
            this.showPickupCompleteLabel();
            this.hidePickupNotice();
            this.showUnpackParcelChoices(transferInfoEntry);
            this.showParcelUnpackConfirmation();                                  
        }
        else
        {
            /* Restore throbber status if needed */
            ParcelPickupStatusObserver.updateThrobberStatus();

            /* Restore transfer progress view state */
            this.showPickupNotice();
        }

        return;
    },
    

    loadTransferProgressView : function(applicationParams)
    {
        var windowController   = NetizenWindowController;
        var pickupTargetURI    = windowController.getPageViewProperty(PARCEL_PICKUP_PROPERTY_TARGETURI,
                                                                      applicationParams               );
        var registrationId     = windowController.getPageViewProperty(PARCEL_PICKUP_PROPERTY_METAINFOID,
                                                                      applicationParams                );
        try
        {
            var trackerSummaryInfo = ParcelManager.trackerSummaryInfo;
            var resourceEntry      = trackerSummaryInfo.getResource(registrationId);
            var summaryEntry       = do_QueryInterface(resourceEntry, Interfaces.ceIParcelTrackerSummary);
            
            var parcelPickupInfo   = document.getElementById("parcelPickupInfo");
            var viewControl        = document.getElementById("parcelPickupContentPanels");

            /* Set the proper target for the pickup info */
            parcelPickupInfo.setAttribute("trackerId", registrationId             );
            parcelPickupInfo.setAttribute("target"   , pickupTargetURI            );
            parcelPickupInfo.setAttribute("summary"  , summaryEntry.contentSummary);
            
            /* Set the transfer progress content panel to visible */
            viewControl.selectedIndex = 1;
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }
        
        return;
    },


    loadSummaryInfo : function(trackerResourceURI, parcelName)
    {
        if(!trackerResourceURI || !parcelName)
        {
            return;
        }

        try
        {
            var parcelDetailsElement = document.getElementById("parcelPickupDetails");
            var parcelDetailsBox     = document.getElementById("parcelPickupDetailsBox");            
         
            var trackerSummaryInfo   = ParcelManager.trackerSummaryInfo;
            var summaryResource      = trackerSummaryInfo.getResource(trackerResourceURI);
            var summaryEntry         = do_QueryInterface(summaryResource                   , 
                                                         Interfaces.ceIParcelTrackerSummary);

            parcelDetailsElement.setAttribute("name"   , parcelName                 );
            parcelDetailsElement.setAttribute("size"   , summaryEntry.contentSummary);
            parcelDetailsElement.setAttribute("created", summaryEntry.creationDate  );
            parcelDetailsElement.setAttribute("expires", summaryEntry.expirationDate);
            
            parcelDetailsBox.hidden = false;
        }
        catch(exception)
        {
            Syslog.logException(exception);
            return;
        }
                    
        ParcelPickupApplication.hidePickupSlipInputView();
        
                        
        return;
    },


    clearSummaryInfo : function()
    {
        var parcelDetailsElement = document.getElementById("parcelPickupDetails");
        if(parcelDetailsElement)
        {
            parcelDetailsElement.setAttribute("name"   , null);
            parcelDetailsElement.setAttribute("size"   , null);
            parcelDetailsElement.setAttribute("created", null);
            parcelDetailsElement.setAttribute("expires", null);
            
            var parcelDetailsBox = document.getElementById("parcelPickupDetailsBox");
            if(parcelDetailsBox)
            {
                parcelDetailsBox.hidden = true;
            }
        
            this.showPickupSlipInputView();
        }
                        
        return;
    },


    clearPickupSlipTextBox : function()
    {
        var pickupSlipData = document.getElementById("pickupSlipContentArea");
        if(pickupSlipData)
        {
            pickupSlipData.value = null;
        }
        
        return;
    },

    
    clearPickupSlipLocationInfo : function()
    {
        var pickupSlipLocationTextBox = document.getElementById("pickupSlipLocationTextBox");
        if(pickupSlipLocationTextBox)
        {
            pickupSlipLocationTextBox.value = null;
        }

        var browseButtonContainer = document.getElementById("browseButtonContainer");
        if(browseButtonContainer)
        {
            browseButtonContainer.hidden = false;
        }
        
        var loadButtonContainer = document.getElementById("loadButtonContainer");
        if(loadButtonContainer)
        {
            loadButtonContainer.hidden = true;
            loadButtonContainer.pickupSlipFile = null;
        }

        return;
    },
    
    
    showPickupSlipTextView : function()
    {        
        var pickupSlipTextView = document.getElementById("pickupSlipTextView");
        if(pickupSlipTextView)
        {
            pickupSlipTextView.hidden = false;
        }

        var pickupSlipFileView = document.getElementById("pickupSlipFileView");
        if(pickupSlipFileView)
        {
            pickupSlipFileView.hidden = true;
        }
        
        var pickupSelectionGroup = document.getElementById("pickupSelectionGroup");
        if(pickupSelectionGroup)
        {
            pickupSelectionGroup.selectedIndex = 0;
        }
        
        return;
    },

    
    showPickupSlipFileView : function()
    {
        var pickupSlipTextView = document.getElementById("pickupSlipTextView");
        if(pickupSlipTextView)
        {
            pickupSlipTextView.hidden = true;
        }

        var pickupSlipFileView = document.getElementById("pickupSlipFileView");
        if(pickupSlipFileView)
        {
            pickupSlipFileView.hidden = false;
        }

        var pickupSelectionGroup = document.getElementById("pickupSelectionGroup");
        if(pickupSelectionGroup)
        {
            pickupSelectionGroup.selectedIndex = 1;
        }

        return;
    },

    
    showPickupSlipInputView : function()
    {
        var pickupSlipSelectionView = document.getElementById("pickupSlipSelectionView");
        if(pickupSlipSelectionView)
        {
            pickupSlipSelectionView.hidden = false;
        }
        
        return;
    },
    

    hidePickupSlipInputView : function()
    {
        var pickupSlipSelectionView = document.getElementById("pickupSlipSelectionView");
        if(pickupSlipSelectionView)
        {
            pickupSlipSelectionView.hidden = true;
        }

        return;
    },


    showPickupNotice : function()
    {
        var packageUnpackSelectionGroup = document.getElementById("packageUnpackSelectionGroup");
        if(!packageUnpackSelectionGroup.hidden)
        {
            return;
        }
     
        var parcelPickupNoticeBox = document.getElementById("parcelPickupNoticeBox");
        if(parcelPickupNoticeBox)
        {
            parcelPickupNoticeBox.hidden = false;
        }
        
   
        var parcelPickupNoticeFrame = document.getElementById("parcelPickupNoticeFrame");
        if(parcelPickupNoticeFrame)
        {
            NetizenFrameController.addFrameReference("parcelPickupNoticeFrame");

            parcelPickupNoticeFrame.hidden = false;
        }

        NetizenFrameController.resizeInternalFrame("parcelPickupNoticeFrame");

        return;
    },
    
    
    hidePickupNotice : function()
    {
        var parcelPickupNoticeBox = document.getElementById("parcelPickupNoticeBox");
        if(parcelPickupNoticeBox)
        {
            parcelPickupNoticeBox.hidden = true;
        }
        
        var parcelPickupNoticeFrame = document.getElementById("parcelPickupNoticeFrame");
        if(parcelPickupNoticeFrame)
        {
            NetizenFrameController.removeFrameReference("parcelPickupNoticeFrame");

            parcelPickupNoticeFrame.hidden = true;
        }

        NetizenFrameController.resizeInternalFrame("parcelPickupNoticeFrame");

        return;
    },
    

    showPickupCompleteLabel : function()
    {
        var parcelPickupLabelDeck = document.getElementById("parcelPickupLabelDeck");
        if(parcelPickupLabelDeck)
        {
            parcelPickupLabelDeck.selectedIndex = 1;
        }
        
        return;
    },

    
    showUnpackParcelChoices : function(transferInfoEntry)
    {
        var parcelName = transferInfoEntry.filename;
        if(parcelName)
        {
            parcelName = parcelName.replace(/.parc/gi, "");
        }
        
        var packageUnpackSelectionGroup = document.getElementById("packageUnpackSelectionGroup");
        if(!packageUnpackSelectionGroup)
        {
            return;
        }
        
        var parcelManager = do_QueryInterface(ParcelManager, Interfaces.ceIParcelManager);
        if(!parcelManager)
        {
            return;
        }
                

        var parcelUnpackToDesktopChoice = document.getElementById("parcelUnpackToDesktopChoice");
        if(parcelUnpackToDesktopChoice)
        {
            var desktopDirectory = parcelManager.defaultDirectory;
            if(desktopDirectory)
            {
                desktopDirectory.append(parcelName);
                FSUtils.ensureUniqueness(desktopDirectory);
                
                parcelName = desktopDirectory.leafName;
                
                var unpackToDesktopString   = this.desktopChoiceString.substitute(parcelName);
                parcelUnpackToDesktopChoice.setAttribute("label", unpackToDesktopString);
            }
        }
        
        var parcelUnpackToDocumentsChoice = document.getElementById("parcelUnpackToDocumentsChoice");
        if(parcelUnpackToDocumentsChoice)
        {
            var documentsDirectory = parcelManager.extractDirectory;
            if(documentsDirectory)
            {
                documentsDirectory.append(parcelName);
                FSUtils.ensureUniqueness(documentsDirectory);
                
                parcelName = documentsDirectory.leafName;
                
                var unpackToDocumentsString = this.documentsChoiceString.substitute(parcelName);
                parcelUnpackToDocumentsChoice.setAttribute("label", unpackToDocumentsString);
            }
        }
        
        var parcelUnpackToFolderChoice = document.getElementById("parcelUnpackToFolderChoice");
        if(parcelUnpackToFolderChoice)
        {
            var chooseFolderString     = this.unpackChooserString;
            parcelUnpackToFolderChoice.setAttribute("label", chooseFolderString);
        }
        
        packageUnpackSelectionGroup.hidden = false;
        
        return;
    },


    showParcelUnpackConfirmation : function(trackerResourceURI, locationURL)
    {
        var parcelUnpackConfirmationBox = document.getElementById("parcelUnpackConfirmationBox");
        if(!parcelUnpackConfirmationBox)
        {
            return;
        }
        
        if(!trackerResourceURI)
        {
            trackerResourceURI = parcelUnpackConfirmationBox.getAttribute("parceluri");
        }
        
        if(!locationURL)
        {
            locationURL = parcelUnpackConfirmationBox.getAttribute("unpacklocation");
        }
        
        if(trackerResource == "null" || locationURL == "null")
        {
            return;
        }
        
        var trackerInfo = ParcelManager.trackerMetaInfo;
        if(!trackerInfo)
        {
            return;
        }

        var trackerResource  = null;
        var trackerEntry     = null;
        var parcelFileObject = null;
        
        try
        {
            trackerResource  = trackerInfo.getResource(trackerResourceURI);
            if(!trackerResource)
            {
                return;
            }
            
            trackerEntry     = do_QueryInterface(trackerResource, Interfaces.ceIParcelTrackerEntry);
            parcelFileObject = ParcelManager.getFileFromLocalPath(trackerEntry);
            if(!parcelFileObject)
            {
                return;
            }
        }
        catch(exception)
        {
            Syslog.logException(exception);
            return;
        }
        
        parcelUnpackConfirmationBox.setAttribute("parceluri"     , trackerResourceURI);
        parcelUnpackConfirmationBox.setAttribute("unpacklocation", locationURL);
        
        locationURL = "'" + locationURL + "'";
        
        var unpackConfirmationData =
        {
            parcelfile : {name: parcelFileObject.leafName, location: locationURL}
        };


        var parcelUnpackConfirmationFrame = document.getElementById("parcelUnpackConfirmationFrame");
        if(parcelUnpackConfirmationFrame)
        {
            var frameContent    = parcelUnpackConfirmationFrame.contentDocument;
            var templateContent = frameContent.getElementById("unpackConfirmationContent");

            NetizenFrameController.addFrameReference("parcelUnpackConfirmationFrame");
        
            parcelUnpackConfirmationFrame.hidden = false;
            
            templateContent.innerHTML = TemplateEngine.processTemplate(templateContent.innerHTML, 
                                                                       unpackConfirmationData   );
        }
        
        NetizenFrameController.resizeInternalFrame("parcelUnpackConfirmationFrame");

        return;
    }
};




var ParcelPickupController =
{
    supportsCommand : function(commandId)
    {
        switch(commandId)
        {
          case "cmd_parcelViewTextInput"  :
          case "cmd_parcelViewFileInput"  :
          case "cmd_parcelValidateCancel" :
          case "cmd_parcelPastePickupSlip":
          case "cmd_parcelBrowseForSlip"  :
          case "cmd_parcelOpenPickupSlip" :
          case "cmd_parcelResetSlipFile"  :
          case "cmd_parcelValidateText"   :
          case "cmd_parcelPickupTarget"   :
          case "cmd_parcelPickupNew"      :
          case "cmd_parcelPickupUnpack"   :
          case "cmd_parcelPickupCancel"   :
          case "cmd_parcelSelectUnpack"   :
          {      
              return true;
          }
        }
        
        return false;
    },


    isCommandEnabled : function(commandId)
    {
        if(commandId == "cmd_parcelViewTextInput")
        {
            var parcelDetailsBox = document.getElementById("parcelPickupDetailsBox");
            if(parcelDetailsBox && parcelDetailsBox.hidden == false)
            {
                var pickupSlipFileView = document.getElementById("pickupSlipFileView");
                if(pickupSlipFileView && pickupSlipFileView.hidden == false)
                {
                    return false;
                }
            }
            
            return true;
        }

        if(commandId == "cmd_parcelViewFileInput")
        {
            var parcelDetailsBox = document.getElementById("parcelPickupDetailsBox");
            if(parcelDetailsBox && parcelDetailsBox.hidden == false)
            {
                var pickupSlipTextView = document.getElementById("pickupSlipTextView");
                if(pickupSlipTextView && pickupSlipTextView.hidden == false)
                {
                    return false;
                }
            }

            return true;
        }
        
        if(commandId == "cmd_parcelPastePickupSlip")
        {
            var pickupSlipTextView = document.getElementById("pickupSlipTextView");
            if(!pickupSlipTextView)
            {
                return false;
            }
            
            var pickupSlipContentArea = document.getElementById("pickupSlipContentArea");
            if(!pickupSlipContentArea)
            {
                return false;
            }

            if(pickupSlipTextView.hidden == true)
            {
                return false;
            }
                        
            return true;
        }
        
        
        if(commandId == "cmd_parcelValidateText"   ||
           commandId == "cmd_parcelBrowseForSlip"  ||
           commandId == "cmd_parcelOpenPickupSlip" ||
           commandId == "cmd_parcelResetSlipFile"  ||
           commandId == "cmd_parcelPickupNew"       )
        {
            return true;
        }


        if(commandId == "cmd_parcelValidateCancel")
        {
            var pickupSlipFileView = document.getElementById("pickupSlipFileView");
            if(!pickupSlipFileView)
            {
                return false;
            }
            
            var pickupSlipData = document.getElementById("pickupSlipContentArea");
            if(!pickupSlipData)
            {
                return false;
            }

            var parcelDetailsBox = document.getElementById("parcelPickupDetailsBox");
            if(!parcelDetailsBox)
            {
                return false;
            }
            
            if(pickupSlipFileView.hidden == false   ||
               pickupSlipData.value                 ||
               parcelDetailsBox.hidden == false      )
            {
                return true;
            }
            
            return false;
        }


        if(commandId == "cmd_parcelPickupTarget")
        {
            var downloadButton = document.getElementById("pickupDownloadButton");
            if(!downloadButton)
            {
                return false;
            }


            if(downloadButton.getAttribute("metainfo"))
            {
                return true;
            }
                    
            return false;
        }


        if(commandId == "cmd_parcelPickupCancel")
        {
            return true;
        }


        if(commandId == "cmd_parcelSelectUnpack"   ||
           commandId == "cmd_parcelPickupUnpack"    )
        {
            return true;
        }

        return false;
    },


    doCommand : function(commandId)
    {
        switch(commandId)
        {
          case "cmd_parcelViewTextInput"  :
          {
              this.viewPickupSlipTextInput();
              break;
          }
          case "cmd_parcelViewFileInput"  :
          {
              this.viewPickupSlipFileInput();
              break;
          }
          case "cmd_parcelValidateCancel" :
          {
              this.cancelPickupSlipValidation();
              break;
          }
          case "cmd_parcelValidateText"   :
          {
              this.scheduleValidatePickupSlipText();
              break;
          }
          case "cmd_parcelPastePickupSlip":
          {
              this.pastePickupSlipText();
              break;
          }
          case "cmd_parcelBrowseForSlip"  :
          {
              this.browseForPickupSlip();
              break;
          }
          case "cmd_parcelOpenPickupSlip" :
          {
              this.validatePickupSlipFile();
              break;
          }
          case "cmd_parcelResetSlipFile"  :
          {
              this.resetPickupSlipBrowser();
              break;
          }
          case "cmd_parcelPickupTarget"   :
          {
              this.pickupTargetParcel();
              break;
          }
          case "cmd_parcelPickupNew"      :
          {
              this.pickupNewParcel();
              break;
          }
          case "cmd_parcelSelectUnpack"   :
          {
              this.selectUpackMethod();
              break;
          }
          case "cmd_parcelPickupUnpack"   :
          {
              this.unpackParcelResource();
              break;
          }
          case "cmd_parcelPickupCancel"   :
          {      
              this.cancelParcelPickup();
              break;
          }
        }

        this.onCommandUpdate();
        return;
    },
    

    onCommandUpdate : function()
    {
        var viewTextInputIsEnabled   = this.isCommandEnabled("cmd_parcelViewTextInput");
        var viewFileInputIsEnabled   = this.isCommandEnabled("cmd_parcelViewFileInput");
        var pastePickupSlipIsEnabled = this.isCommandEnabled("cmd_parcelPastePickupSlip");
        var cancelValidateIsEnabled  = this.isCommandEnabled("cmd_parcelValidateCancel");
        var cancelPickupIsEnabled    = this.isCommandEnabled("cmd_parcelPickupCancel");
        var newPickupIsEnabled       = this.isCommandEnabled("cmd_parcelPickupNew");
        var pickupTargetIsEnabled    = this.isCommandEnabled("cmd_parcelPickupTarget");
        var unpackTargetIsEnabled    = this.isCommandEnabled("cmd_parcelPickupUnpack");
       
 
        goSetCommandEnabled("cmd_parcelViewTextInput"  , viewTextInputIsEnabled  );
        goSetCommandEnabled("cmd_parcelViewFileInput"  , viewFileInputIsEnabled  );
        goSetCommandEnabled("cmd_parcelValidateCancel" , cancelValidateIsEnabled );
        goSetCommandEnabled("cmd_parcelPastePickupSlip", pastePickupSlipIsEnabled);
        goSetCommandEnabled("cmd_parcelPickupNew"      , newPickupIsEnabled      );
        goSetCommandEnabled("cmd_parcelPickupTarget"   , pickupTargetIsEnabled   );
        goSetCommandEnabled("cmd_parcelPickupCancel"   , cancelPickupIsEnabled   );
        goSetCommandEnabled("cmd_parcelPickupUnpack"   , unpackTargetIsEnabled   );
        
        return;
    },


    pickupNewParcel : function()
    {
        NetizenWindowController.scheduleApplicationViewLoad(PARCEL_PICKUPAPP_CHROME_URL   , 
                                                            PARCEL_PICKUPAPP_STATE_DEFAULT);

        return;
    },


    cancelPickupSlipValidation : function()
    {
        ParcelPickupApplication.clearSummaryInfo();
        ParcelPickupApplication.clearPickupSlipTextBox();
        ParcelPickupApplication.clearPickupSlipLocationInfo();
        ParcelPickupApplication.showPickupSlipTextView();
        
        var downloadButton = document.getElementById("pickupDownloadButton");
        if(downloadButton)
        {
            var metaInfoId   = downloadButton.getAttribute("metainfo");
            var trackerEntry = NetizenPropertyCache.getPropertyValue(metaInfoId);
            if(trackerEntry)
            {
                ParcelManager.discardTrackerEntry(trackerEntry);
            }

            downloadButton.removeAttribute("metainfo");
            downloadButton.removeAttribute("resource");
            downloadButton.removeAttribute("target"  );
            downloadButton.removeAttribute("name"    );
        }

        return;
    },


    pastePickupSlipText : function()
    {
        var pickupSlipContentArea = document.getElementById("pickupSlipContentArea");
        if(pickupSlipContentArea)
        {
            pickupSlipContentArea.focus();            
            goDoCommand("cmd_paste");
        }
        
        return;
    },

    
    browseForPickupSlip : function()
    {
        var pickupSlipFile = ParcelResourceController.browseForPickupSlipFile();
        if(!pickupSlipFile)
        {
            return;
        }
        
        var targetPath = pickupSlipFile.target;
        if(!targetPath)
        {
            return;
        }

        var pickupSlipLocationTextBox = document.getElementById("pickupSlipLocationTextBox");
        if(pickupSlipLocationTextBox)
        {
            pickupSlipLocationTextBox.value = targetPath;
        }
        
        var browseButtonContainer = document.getElementById("browseButtonContainer");
        if(browseButtonContainer)
        {
            browseButtonContainer.hidden = true;
        }
        
        var loadButtonContainer = document.getElementById("loadButtonContainer");
        if(loadButtonContainer)
        {
            loadButtonContainer.hidden = false;
            loadButtonContainer.pickupSlipFile = pickupSlipFile;
        }
        

        return;
    },
    
    
    resetPickupSlipBrowser : function()
    {
        ParcelPickupApplication.clearPickupSlipLocationInfo();
        return;
    },


    loadPickupSlip : function(metaInfoPointer)
    {
        var metaInfoId       = null;
        var resourceURLSpec  = null;
        var directoryURLSpec = null;
        var displayName      = null;
        var isExpired        = true;
        
        try
        {
            var trackerEntry    = ParcelManager.processPickupData(metaInfoPointer);
            if(!trackerEntry)
            {
                return;
            }
            
            isExpired           = ParcelManager.pickupDataIsExpired(metaInfoPointer);
            resourceURLSpec     = trackerEntry.activityTarget.spec;
            directoryURLSpec    = ParcelManager.getURLSpecFromLocalPath(trackerEntry);
            
            var activityTarget  = new ceStandardURL(resourceURLSpec);
            displayName         = activityTarget.getParameterValue("parc");                
            metaInfoId          = trackerEntry.descriptor;
        
            ParcelPickupApplication.loadSummaryInfo(metaInfoId, displayName);
        
            if(!isExpired)
            {
                NetizenPropertyCache.cachePropertyValue(metaInfoId, trackerEntry);
            }
        }
        catch(exception)
        {
            Syslog.logException(exception);
            return;
        }
                
        var downloadButton = document.getElementById("pickupDownloadButton");
        if(downloadButton && !isExpired)
        {
            downloadButton.setAttribute("metainfo", metaInfoId      );
            downloadButton.setAttribute("resource", resourceURLSpec );
            downloadButton.setAttribute("target"  , directoryURLSpec);
            downloadButton.setAttribute("name"    , displayName     );

            downloadButton.focus();
        }

        return;
    },
        
    
    validatePickupSlipFile : function()
    {
        var loadButtonContainer = document.getElementById("loadButtonContainer");
        if(!loadButtonContainer)
        {
            return;
        }
        
        try
        {
            var pickupSlipFile  = loadButtonContainer.pickupSlipFile;
            if(!pickupSlipFile)
            {
                return;
            }
            
            var metaInfoPointer = MetaInfoService.loadPointerFile(pickupSlipFile);
            if(metaInfoPointer)
            {
                this.loadPickupSlip(metaInfoPointer);
            }
        }
        catch(exception)
        {
            Syslog.logException(exception);
            return;
        }
                
        this.onCommandUpdate();

        return;
    },

    
    scheduleValidatePickupSlipText : function()
    {
        window.setTimeout(ParcelPickupController.validatePickupSlipText,
                          PARCEL_PICKUP_SLIPDETAILS_VIEW_DELAY         );

        return;
    },

    
    validatePickupSlipText : function()
    {
        var pickupSlipData   = document.getElementById("pickupSlipContentArea");
        if(!pickupSlipData.value || pickupSlipData.textLength == 0)
        {
            ParcelPickupController.cancelPickupSlipValidation();
            ParcelPickupController.onCommandUpdate();
            return;
        }

        try
        {
            var metaInfoPointer = MetaInfoService.loadMetafilePointer(pickupSlipData.value);
            if(metaInfoPointer)
            {
                ParcelPickupController.loadPickupSlip(metaInfoPointer);
            }
        }
        catch(exception)
        {
            Syslog.logException(exception);
            return;
        }
                
        ParcelPickupController.onCommandUpdate();
        
        return;
    },


    pickupTargetParcel : function()
    {
        var downloadButton  = document.getElementById("pickupDownloadButton");
        if(!downloadButton)
        {
            return;
        }
        
        var displayName     = downloadButton.getAttribute("name");
        var metaInfoId      = downloadButton.getAttribute("metainfo");
        var trackerEntry    = NetizenPropertyCache.getPropertyValue(metaInfoId);
        if(!trackerEntry)
        {
            Assert.condition(trackerEntry, "Cached tracker entry not found");
            return;
        }
        
        var resourceURLSpec = downloadButton.getAttribute("resource");
        var targetURLSpec   = downloadButton.getAttribute("target");                
        var targetURI       = IOService.newURI(targetURLSpec, null, null);
        
        try
        {
            var resourceURI = do_CreateInstance(PARCEL_PICKUP_URL_CONTRACTID, Interfaces.ceIPickupURL);
            if(!resourceURI)
            {
                return;
            }

            resourceURI.spec = resourceURLSpec;

            var transferComponent = do_CreateInstance(TRANSFER_COMPONENT_CONTRACTID, 
                                                      Interfaces.ceITransferInfo   );
            if(!transferComponent)
            {
                return;
            }


            var persistComponent = do_CreateInstance(TRANSFER_PERSIST_CONTRACTID    ,
                                                     Interfaces.nsIWebBrowserPersist);
            if(!persistComponent)
            {
                return;
            }

            transferComponent.initTransfer(resourceURI     ,
                                           targetURI       ,
                                           metaInfoId      ,
                                           displayName     ,
                                           null            ,
                                           persistComponent);
                        
            persistComponent.saveURI(resourceURI, null, null, null, null, targetURI);
        
            ParcelManager.commitTrackerEntry(trackerEntry);
        }
        catch(exception)
        {
            TransferManager.removeTransfer(targetURI.spec);
            ParcelManager.removeTrackerEntry(metaInfoId);

            Syslog.logException(exception);
            return;
        }
            
        
        var transferViewParameters = new ceStandardURLQuery();
        
        Assert.isNotNull(transferViewParameters);
        
        /* Create the transfer view properties */
        transferViewParameters.addQueryParameter(PARCEL_PICKUP_PROPERTY_TARGETURI , targetURI.spec);
        transferViewParameters.addQueryParameter(PARCEL_PICKUP_PROPERTY_METAINFOID, metaInfoId    );

        /* Update the cached properties for the recent viewstate */
        NetizenPropertyCache.cachePropertyValue(PARCEL_PICKUP_PROPERTY_TARGETURI , targetURI.spec);
        NetizenPropertyCache.cachePropertyValue(PARCEL_PICKUP_PROPERTY_METAINFOID, metaInfoId    );

        /* Clear cached download information from being reused */
        NetizenPropertyCache.removePropertyValue(metaInfoId);
        
        downloadButton.removeAttribute("metainfo");
        downloadButton.removeAttribute("resource");
        downloadButton.removeAttribute("target"  );
        downloadButton.removeAttribute("name"    );
        
        NetizenWindowController.scheduleApplicationViewLoad(PARCEL_PICKUPAPP_CHROME_URL           , 
                                                            PARCEL_PICKUPAPP_STATE_TRANSFER       ,
                                                            transferViewParameters.toQueryString());
        
        return;
    },


    cancelParcelPickup : function()
    {
        var parcelPickupInfo = document.getElementById("parcelPickupInfo");
        if(!parcelPickupInfo)
        {
            return;
        }

        var transferTargetURISpec = parcelPickupInfo.getAttribute("target");
        if(transferTargetURISpec)
        {
            TransferManager.cancelTransfer(transferTargetURISpec);
        }

        return;
    },


    retryParcelPickup : function()
    {
        var parcelPickupInfo = document.getElementById("parcelPickupInfo");
        if(!parcelPickupInfo)
        {
            return;
        }

        var transferTargetURISpec = parcelPickupInfo.getAttribute("target");
        if(transferTargetURISpec)
        {
            TransferManager.retryTransfer(transferTargetURISpec);
        }

        return;
    },

    selectUpackMethod : function()
    {
        var parcelUnpackChoiceGroup = document.getElementById("parcelUnpackChoiceGroup");
        if(!parcelUnpackChoiceGroup)
        {
            return;
        }

        var parcelUnpackActionDeck = document.getElementById("parcelUnpackActionDeck");
        if(!parcelUnpackActionDeck)
        {
            return;
        }

        parcelUnpackActionDeck.selectedIndex = parcelUnpackChoiceGroup.selectedIndex;
        return;
    },

    
    unpackParcelResource : function()
    {
        var resourceURI = NetizenWindowController.getPageViewProperty(PARCEL_PICKUP_PROPERTY_METAINFOID);
        if(resourceURI)
        {
            var parcelUnpackActionDeck = document.getElementById("parcelUnpackActionDeck");
            if(!parcelUnpackActionDeck)
            {
                return;
            }
            
            var parcelUnpackLocation = null;
            
            if(parcelUnpackActionDeck.selectedIndex == 0)
            {
                parcelUnpackLocation = ParcelResourceController.unpackParcelToDesktop(resourceURI);
            }
            else if(parcelUnpackActionDeck.selectedIndex == 1)
            {
                parcelUnpackLocation = ParcelResourceController.unpackParcelToUserDocuments(resourceURI);
            }
            else
            {
                parcelUnpackLocation = ParcelResourceController.unpackParcel(resourceURI);
            }

            ParcelPickupApplication.showParcelUnpackConfirmation(resourceURI         , 
                                                                 parcelUnpackLocation);
        }
                
        return;
    },


    viewPickupSlipTextInput : function()
    {
        ParcelPickupApplication.showPickupSlipTextView();
        return;
    },

    
    viewPickupSlipFileInput : function()
    {
        ParcelPickupApplication.showPickupSlipFileView();
        return;
    }
};





var ParcelTransferEventHandler =
{
    registerListener  : function(targetElement)
    {
        if(targetElement)
        {
            targetElement.addEventListener(TRANSFER_EVENT_RETRY  , this.onTransferRetry  , false);
            targetElement.addEventListener(TRANSFER_EVENT_CANCEL , this.onTransferCancel , false);
            targetElement.addEventListener(TRANSFER_EVENT_DETAILS, this.onTransferDetails, false);
        }
        
        return;
    },


    unregisterListener  : function(targetElement)
    {
        if(targetElement)
        {
            targetElement.removeEventListener(TRANSFER_EVENT_RETRY  , this.onTransferRetry  , false);
            targetElement.removeEventListener(TRANSFER_EVENT_CANCEL , this.onTransferCancel , false);
            targetElement.removeEventListener(TRANSFER_EVENT_DETAILS, this.onTransferDetails, false);
        }
        
        return;
    },
    

    onTransferDetails : function(eventInfo)
    {
        var parcelPickupInfo  = eventInfo.target;
        if(!parcelPickupInfo)
        {
            return;
        }

        var trackerResourceURI = parcelPickupInfo.getAttribute("trackerId");
        if(trackerResourceURI)
        {
            ParcelResourceController.loadParcelDetailsView(trackerResourceURI);
        }

        return;
    },


    onTransferCancel  : function(eventInfo)
    {
        var parcelPickupInfo  = eventInfo.target;
        if(!parcelPickupInfo)
        {
            return;
        }

        ParcelPickupController.cancelParcelPickup();
        return;
    },

    onTransferRetry  : function(eventInfo)
    {
        var parcelPickupInfo  = eventInfo.target;
        if(!parcelPickupInfo)
        {
            return;
        }

        ParcelPickupController.retryParcelPickup();
        return;
    }    
};



var ParcelInfoEventHandler =
{
    registerListener  : function(targetElement)
    {
        if(targetElement)
        {
            targetElement.addEventListener(PARCEL_INFO_EVENT_FOLLOWLINK, 
                                           this.onClearPickupInfo      , 
                                           false                       );
        }

        return;
    },


    unregisterListener  : function(targetElement)
    {
        if(targetElement)
        {
            targetElement.removeEventListener(PARCEL_INFO_EVENT_FOLLOWLINK, 
                                              this.onClearPickupInfo      , 
                                              false                       );        
        }
        
        return;
    },


    onClearPickupInfo : function(eventInfo)
    {
        ParcelPickupApplication.clearSummaryInfo();
        ParcelPickupApplication.clearPickupSlipTextBox();
        ParcelPickupApplication.clearPickupSlipLocationInfo();
        
        var downloadButton = document.getElementById("pickupDownloadButton");
        if(downloadButton)
        {
            downloadButton.removeAttribute("metainfo");
            downloadButton.removeAttribute("resource");
            downloadButton.removeAttribute("target"  );
            downloadButton.removeAttribute("name"    );
        }

        ParcelPickupController.onCommandUpdate();

        return;
    }
};



var ParcelPickupStatusObserver =
{
    loadObservers : function()
    {
        ObserverService.addObserver(this, TRANSFER_STATUS_RESOLVING   , false);
        ObserverService.addObserver(this, TRANSFER_STATUS_STARTED     , false);
        ObserverService.addObserver(this, TRANSFER_STATUS_TRANSFERRING, false);
        ObserverService.addObserver(this, TRANSFER_STATUS_FAILED      , false);
        ObserverService.addObserver(this, TRANSFER_STATUS_CANCELED    , false);
        ObserverService.addObserver(this, TRANSFER_STATUS_NOTFOUND    , false);

        ObserverService.addObserver(this, TRANSFER_STATUS_RESTARTING  , false);
        ObserverService.addObserver(this, TRANSFER_STATUS_RESTARTED   , false);
        ObserverService.addObserver(this, TRANSFER_STATUS_INTERRUPTED , false);

        return;
    },

    unloadObservers : function()
    {
        ObserverService.removeObserver(this, TRANSFER_STATUS_RESOLVING   );
        ObserverService.removeObserver(this, TRANSFER_STATUS_STARTED     );
        ObserverService.removeObserver(this, TRANSFER_STATUS_TRANSFERRING);
        ObserverService.removeObserver(this, TRANSFER_STATUS_FAILED      );
        ObserverService.removeObserver(this, TRANSFER_STATUS_CANCELED    );
        ObserverService.removeObserver(this, TRANSFER_STATUS_NOTFOUND    );

        ObserverService.removeObserver(this, TRANSFER_STATUS_RESTARTING  );
        ObserverService.removeObserver(this, TRANSFER_STATUS_RESTARTED   );
        ObserverService.removeObserver(this, TRANSFER_STATUS_INTERRUPTED );
        
        return;
    },

    QueryInterface : function(iid)
    {
        if(iid.equals(Interfaces.nsIObserver) ||
           iid.equals(Interfaces.nsISupports)  )
        {
            return this;
        }
        
        throw Results.NS_NOINTERFACE;
    },

    
    updateThrobberStatus : function()
    {
        var transferId = NetizenWindowController.getPageViewProperty(PARCEL_PICKUP_PROPERTY_TARGETURI);

        try
        {
            var transferMetaInfo  = TransferManager.transferMetaInfo;
            var metaInfoEntry     = transferMetaInfo.getResource(transferId);
            var transferInfoEntry = do_QueryInterface(metaInfoEntry                  , 
                                                      Interfaces.ceITransferInfoEntry);
            var transferState     = transferInfoEntry.transferStatus;
            
            if(transferState == Interfaces.ceITransferInfo.TRANSFER_NOTSTARTED   ||
               transferState == Interfaces.ceITransferInfo.TRANSFER_RESOLVING    ||
               transferState == Interfaces.ceITransferInfo.TRANSFER_STARTED      ||
               transferState == Interfaces.ceITransferInfo.TRANSFER_RESTART_WAIT ||
               transferState == Interfaces.ceITransferInfo.TRANSFER_RESTARTING   ||
               transferState == Interfaces.ceITransferInfo.TRANSFER_RESTARTED     )
            {
                ObserverService.notifyObservers(this                                  , 
                                                NETIZEN_APPLICATION_STATUS_BUSY       , 
                                                PARCEL_PICKUP_EVENT_UPDATEXFERTHROBBER);
            }
            else
            {
                ObserverService.notifyObservers(this                                  ,
                                                NETIZEN_APPLICATION_STATUS_IDLE       ,
                                                PARCEL_PICKUP_EVENT_UPDATEXFERTHROBBER);
            }
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }
        
        return;
    },
        

    observe : function(subject, topic, state)
    {
        switch(topic)
        {
          case TRANSFER_STATUS_RESOLVING    :
          case TRANSFER_STATUS_STARTED      :
          case TRANSFER_STATUS_RESTARTING   :
          case TRANSFER_STATUS_RESTARTED    :
          {
              ObserverService.notifyObservers(ParcelPickupApplication               ,
                                              NETIZEN_APPLICATION_STATUS_BUSY       ,
                                              PARCEL_PICKUP_EVENT_UPDATEXFERTHROBBER);
              
              break;
          }
          case TRANSFER_STATUS_TRANSFERRING :
          case TRANSFER_STATUS_CANCELED     :
          case TRANSFER_STATUS_NOTFOUND     :
          case TRANSFER_STATUS_FAILED       :
          case TRANSFER_STATUS_INTERRUPTED  :
          {
              ObserverService.notifyObservers(ParcelPickupApplication               ,
                                              NETIZEN_APPLICATION_STATUS_IDLE       ,
                                              PARCEL_PICKUP_EVENT_UPDATEXFERTHROBBER);

              break;
          }
        }
        
        return;
    }
};



/////////////////////////////////////////////////////////////////////////////////
//
// ceITransferProgressListener Implementation
//
/////////////////////////////////////////////////////////////////////////////////


function ceParcelPickupProgressListener(documentRef, activeTransfers)
{
    if(documentRef)
    {
        this.documentRef         = documentRef;
        var transferStringBundle = this.documentRef.getElementById("transferStringBundle");
        if(transferStringBundle)
        {
            this.loadStringBundle(transferStringBundle);
        }
    }
    
    if(activeTransfers)
    {
        this.initTransferInfoIndex(activeTransfers);
    }
    
    return;
}


ceParcelPickupProgressListener.prototype = new ceAbstractProgressListener();


ceParcelPickupProgressListener.prototype.constructor = ceParcelPickupProgressListener;


ceParcelPickupProgressListener.superclasss = ceAbstractProgressListener.prototype;



ceParcelPickupProgressListener.prototype.onProgressChange = function(webProgress         , 
                                                                     request             , 
                                                                     currentSelfProgress , 
                                                                     maxSelfProgress     , 
                                                                     currentTotalProgress, 
                                                                     maxTotalProgress    , 
                                                                     transferInfo        )
{
    if(!this.shouldUpdateProgress(transferInfo, currentTotalProgress, maxTotalProgress))
    {
        return;
    }
        
    var startingOffset = transferInfo.startingOffset;
    var transferRate   = 0;
    if(this.elapsedTime)
    {
        /* bytes/sec = currentTotalProgress[bytes] / (elapsedTime[ms] / 1000[ms_per_sec]) */
        transferRate = ((currentTotalProgress - startingOffset) * 1000) / this.elapsedTime;
    }
    
    /* Acquire the progress bar widget element to update */
    var transferDescriptor = transferInfo.target.spec;
    var transferElement    = this.documentRef.getElementById(transferDescriptor);
    
    /* 1. Update progress bar information */
    var transferPercent    = 0;
    if(maxTotalProgress > 0)
    {
        transferPercent = Math.floor((currentTotalProgress * 100.0)/maxTotalProgress);
        if(transferPercent > 100)
        {
            transferPercent = 100;
        }

        /* 1.1 Update progress bar percentage */
        if(transferElement)
        {
            transferElement.setAttribute("progress"    , transferPercent      );
            transferElement.setAttribute("progressmode", "normal"             );
        }
    }
    else
    {
        transferPercent = -1;
        if(transferElement)
        {
            transferElement.setAttribute("progressmode", "undetermined");
        }
    }
    
    /* 2. Update progress notification */
    var progressKBytes = parseInt(currentTotalProgress/1024 + 0.5);
    var totalKBytes    = parseInt(maxTotalProgress/1024 + 0.5);
    
    var progressString = this.formatProgressInfo(progressKBytes, totalKBytes);
    if(transferElement)
    {
        transferElement.setAttribute("progressinfo", progressString);
    }
    
    var statusMessage  = this.replaceInsert(this.statusFormat, 1, progressString);
    
    /* 3. Update transfer rate */        
    if(transferRate)
    {
        /* Convert to KBytes/sec from bytes/sec */
        var transferRateKBSec = transferRate / 1024;
        transferRateKBSec     = parseInt(transferRateKBSec * 10 + 0.5);
        if(transferRateKBSec != this.priorRate)
        {
            if(this.rateChanges++ == this.rateChangeLimit)
            {
                this.priorRate   = transferRateKBSec;
                this.rateChanges = 0;
            }
            else
            {
                transferRateKBSec = this.priorRate;
            }   
        }
        else
        {
            this.rateChanges = 0;
        }
        
        var fraction      = transferRateKBSec % 10;
        transferRateKBSec = parseInt((transferRateKBSec - fraction) / 10);
        if(transferRateKBSec < 100)
        {
            transferRateKBSec += "." + fraction;
        }
        
        statusMessage      = this.replaceInsert(statusMessage, 2, transferRateKBSec);
    }
    else
    {
        statusMessage      = this.replaceInsert(statusMessage, 2, "??.?");
    }
    
    
    /* 4. Update time remaining */
    if(transferRate && (maxTotalProgress > 0))
    {
        var timeRemaining  = (maxTotalProgress - currentTotalProgress) / transferRate;
        timeRemaining      = parseInt(timeRemaining + 0.5);
        
        var timeLeftString = this.formatVerboseTimeInfo(timeRemaining);
        if(timeLeftString)
        {
            timeLeftString = timeLeftString + " " + this.remaining;
        }
        else
        {
            timeLeftString = this.noneRemaining;
        }
        
        statusMessage      = this.replaceInsert(statusMessage, 3, timeLeftString);
    }
    else
    {
        statusMessage      = this.replaceInsert(statusMessage, 3, this.unknownFileSize);
    }
    
    /* 5. Set the status message */
    if(transferElement)
    {
        transferElement.setAttribute("status", statusMessage);
    }


    var parcelPickupInfo = document.getElementById("parcelPickupInfo");
    if(parcelPickupInfo)
    {
        parcelPickupInfo.setAttribute("status", statusMessage);
    }
    
    return;
}



ceParcelPickupProgressListener.prototype.onStateChange = function(webProgress , 
                                                                  request     , 
                                                                  stateFlags  , 
                                                                  statusCode  ,
                                                                  transferInfo)
{
    var transferInfoEntry = do_QueryInterface(transferInfo                   , 
                                              Interfaces.ceITransferInfoEntry);
            

    if(stateFlags & Interfaces.nsIWebProgressListener.STATE_STOP)
    {
        var transferDescriptor = transferInfo.target.spec;
        var transferElement    = this.documentRef.getElementById(transferDescriptor);
        if(transferElement)
        {
            /* Set the status message to an empty string */
            transferElement.setAttribute("status", "");
        }

        if(transferInfo.transferState == Interfaces.ceITransferInfo.TRANSFER_FINISHED     &&
           transferInfo.transferType  == Interfaces.ceITransferInfo.TRANSFER_TYPE_DOWNLOAD )
        {
            ParcelPickupApplication.showPickupCompleteLabel();
            ParcelPickupApplication.hidePickupNotice();
            ParcelPickupApplication.showUnpackParcelChoices(transferInfoEntry);            
        }
    }

    NetizenPropertyCache.cachePropertyValue(transferInfoEntry.metaInfoName, transferInfo);
    return;
}



/////////////////////////////////////////////////////////////////////////////////
//
// nsITimerCallback Implementation
//
/////////////////////////////////////////////////////////////////////////////////


function ceParcelPickupRetryEventSink(documentRef)
{
    if(documentRef)
    {
        this.documentRef         = documentRef;
        var transferStringBundle = this.documentRef.getElementById("transferStringBundle");
        if(transferStringBundle)
        {
            this.loadStringBundle(transferStringBundle);
        }
    }

    return;
}




ceParcelPickupRetryEventSink.prototype = new ceAbstractProgressListener();



ceParcelPickupRetryEventSink.prototype.constructor = ceParcelPickupRetryEventSink;


ceParcelPickupRetryEventSink.superclass  = ceAbstractProgressListener.prototype;



ceParcelPickupRetryEventSink.prototype.QueryInterface = function(iid)
{
    if(iid.equals(Interfaces.nsITimerCallback) ||
       iid.equals(Interfaces.nsISupports)       )
    {
        return this;
    }
    
    throw Results.NS_NOINTERFACE;
}



ceParcelPickupRetryEventSink.prototype.notify = function(timerObject)
{
    if(TransferManager.waitingTransferCount <= 0)
    {
        return;
    }
    
    var transferMetaInfo = TransferManager.transferMetaInfo;
    if(!transferMetaInfo)
    {
        return;
    }
    
    var transferEnumerator = transferMetaInfo.enumerateResources();
    if(!transferEnumerator)
    {
        return;
    }

    var currentDate = new Date();
    var timeNow     = currentDate.getTime();
    var isConnected = false;
    
    var networkStatus = NetizenPropertyCache.getPropertyValue(NETIZEN_NETWORK_STATUS_PROPERTY);
    if(networkStatus)
    {
        if(networkStatus == NETIZEN_NETWORK_STATUS_CONNECTED ||
           networkStatus == NETIZEN_NETWORK_STATUS_PINGING    )
        {
            isConnected = true;
        }
    }
    

    while(transferEnumerator.hasMoreElements())
    {
        var metaInfo = transferEnumerator.getNext();
        if(!metaInfo)
        {
            break;
        }
        
        var transferInfoEntry = do_QueryInterface(metaInfo, Interfaces.ceITransferInfoEntry);
        if(transferInfoEntry)
        {
            /* Acquire the progress widget element to update */
            var transferElement = this.documentRef.getElementById(transferInfoEntry.descriptor);
            if(transferElement)
            {
                this.updateRetryStatusForEntry(transferInfoEntry, transferElement, timeNow, isConnected);
                break;
            }
        }
    }
    
    return;
}


ceParcelPickupRetryEventSink.prototype.updateRetryStatusForEntry = function(transferInfoEntry,
                                                                            transferElement  ,
                                                                            timeNow          ,
                                                                            isConnected      )
{
    if(transferInfoEntry.transferStatus != Interfaces.ceITransferInfo.TRANSFER_RESOURCE_NOTFOUND &&
       transferInfoEntry.transferStatus != Interfaces.ceITransferInfo.TRANSFER_FAILED             )
    {
        return;
    }
     
    try
    {
        var transferInfo = TransferManager.getTransferInfo(transferInfoEntry.descriptor);
        if(transferInfo)
        {
            var timeLeft = ((transferInfo.nextRetryTime / 1000) - timeNow);
            if(timeLeft >= 0)
            {
                transferElement.setAttribute("retrytimer", this.formatVerboseTimeInfo(timeLeft / 1000));
            }
        }
    }
    catch(exception)
    {
        Syslog.logException(exception);
    }
    
    transferElement.setAttribute("isconnected", (isConnected == true ? "1" : "0"));
    
    return;
}
