/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2005 Civil Engines Research.  All Rights 
 * Reserved.  
 *
 * Portions created by Netscape Copyright (C) 1998 Netscape 
 * Communications Corporation. All Rights Reserved.
 *
 * Portions created by the Mozilla Foundation are Copyright (C) 
 * 2003 Mozilla Foundation. All Rights Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/

const PARCEL_CACHESERVICE_CONTRACTID         = "@civilengines.com/parc/cache-service;1";
const PARCEL_MANAGER_CONTRACTID              = "@civilengines.com/apps/parcel-manager;1";
const PARCEL_METAINFOSERVICE_CONTRACTID      = "@civilengines.com/metainf/loader-service;1";
const PARCEL_METAINFO_STUB_TYPE              = "http://civilengines.com/parcel/stub?version=1";
const PARCEL_METAINFO_EXPIRATION_DEFAULT     = 7;

const PARCEL_PACKAGE_EVENT_VIEWDETAILS       = "urn:parcel:package:event:details";
const PARCEL_PACKAGE_EVENT_CANCEL            = "urn:parcel:package:event:cancel";
const PARCEL_PACKAGE_EVENT_STORENTRY         = "urn:parcel:package:event:storentry";
const PARCEL_PACKAGE_EVENT_SELECTPARCEL      = "urn:parcel:tracker:event:select";
const PARCEL_PACKAGE_EVENT_UNSELECTPARCEL    = "urn:parcel:tracker:event:unselect";
const PARCEL_PACKAGE_EVENT_SENDSELECTED      = "urn:parcel:tracker:event:activate";
const PARCEL_PACKAGE_EVENT_SHOWDETAILS       = "urn:parcel:tracker:event:details";

const PARCEL_PACKAGE_PROPERTY_OLDRESOURCE    = "urn:parcel:property:package:oldresource";
const PARCEL_PACKAGE_PROPERTY_FILENAME       = "urn:parcel:property:package:filename";
const PARCEL_PACKAGE_PROPERTY_NEWRESOURCE    = "urn:parcel:property:package:newresource";

const PARCEL_SENDAPP_CHROME_URL              = "chrome://netizen/content/parcel/parcelAppSend.xul";
const PARCEL_SENDAPP_STATE_DEFAULT           = "default";
const PARCEL_SENDAPP_STATE_PACKAGE           = "package";
const PARCEL_SENDAPP_STATE_PACKAGEINFO       = "packageinfo";


var   ParcelCacheService                     = null;
var   MetaInfoService                        = null;
var   ParcelManager                          = null;


function loadApplication(loadRequestURL, isHistoryView)
{
    Syslog.logMessage("Loading Parcel Send Application");

    var applicationParams = loadRequestURL.getQueryParameters();
    
    ParcelSendApplication.loadServices(applicationParams);    
    ParcelSendApplication.loadControllers();
    ParcelSendApplication.loadHandlers();
    ParcelSendApplication.loadStringBundles();
    ParcelSendApplication.loadApplicationView(applicationParams, isHistoryView);

    return;
}



function unloadApplication(unloadRequestURL)
{
    var applicationParams = unloadRequestURL.getQueryParameters();

    ParcelSendApplication.unloadHandlers();
    ParcelSendApplication.unloadControllers();
    ParcelSendApplication.unloadServices(unloadRequestURL.getQueryParameters());

    ParcelSendApplication.unloadApplicationView(applicationParams);
        
    return;
}


var ParcelSendApplication = 
{
    loadServices : function(applicationParams)
    {
        ParcelModeSelector.onModeSelect(PARCEL_APPLICATION_MODE_SEND);
        
        NetizenNetworkStatusObserver.loadObservers();

        ParcelNetworkAlertObserver.loadObservers();

        ParcelTransferObserver.loadObservers();

        try
        {
            ParcelCacheService = do_GetService(PARCEL_CACHESERVICE_CONTRACTID  ,
                                               Interfaces.ceIParcelCacheService);
            
            MetaInfoService    = do_GetService(PARCEL_METAINFOSERVICE_CONTRACTID,
                                               Interfaces.ceIMetaInfoService    );
            
            ParcelManager      = do_GetService(PARCEL_MANAGER_CONTRACTID  , 
                                               Interfaces.ceIParcelManager);
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }

        ParcelCacheService.notificationCallback = new ceParcelPackageEventSink(document);

        return;
    },


    unloadServices : function(applicationParams)
    {
        ParcelCacheService.notificationCallback = null;
        
        ParcelTransferObserver.unloadObservers();

        ParcelNetworkAlertObserver.unloadObservers();

        NetizenNetworkStatusObserver.unloadObservers();

        ParcelModeSelector.onModeUnselect(PARCEL_APPLICATION_MODE_SEND);
        
        return;
    },


    loadControllers : function()
    {
        top.controllers.appendController(ParcelSendController);
        ParcelSendController.onCommandUpdate();
        
        return;
    },


    unloadControllers : function()
    {
        ParcelSendController.onCommandUpdate();
        top.controllers.removeController(ParcelSendController);
        return;
    },


    loadHandlers : function()
    {
        NetizenWindowResizer.addResizeCallback(this.onResizeApplication);

        var archiveTree = document.getElementById("parcelArchiveTreeView");
        if(archiveTree)
        {
            ParcelPackageEventHandler.registerListener(archiveTree);
        }

        var parcelPackageInfo = document.getElementById("parcelPackageProgressInfo");
        if(parcelPackageInfo)
        {
            ParcelPackageEventHandler.registerListener(parcelPackageInfo);
        }
      
        var packageExistingView = document.getElementById("packageExistingView");
        if(packageExistingView)
        {
            ParcelSelectionEventHandler.registerListener(packageExistingView);
        }
        
        return;
    },


    unloadHandlers : function()
    {
        NetizenWindowResizer.removeResizeCallback(this.onResizeApplication);

        var archiveTree = document.getElementById("parcelArchiveTreeView");
        if(archiveTree)
        {
            ParcelPackageEventHandler.unregisterListener(archiveTree);
        }

        var parcelPackageInfo = document.getElementById("parcelPackageProgressInfo");
        if(parcelPackageInfo)
        {
            ParcelPackageEventHandler.unregisterListener(parcelPackageInfo);
        }
        
        var packageExistingView = document.getElementById("packageExistingView");
        if(packageExistingView)
        {
            ParcelSelectionEventHandler.unregisterListener(packageExistingView);
        }

        return;
    },


    loadStringBundles : function()
    {
        var netizenStringBundle = document.getElementById("netizenStringBundle");
        if(!netizenStringBundle)
        {
            return;
        }

        var parcelStringBundle = document.getElementById("parcelStringBundle");
        if(!parcelStringBundle)
        {
            return;
        }

        this.ampersandString    = netizenStringBundle.getString("netizen.symbol.ampersand.text");
        this.activeCountString  = parcelStringBundle.getString("parcel.count.active");
        this.expiredCountString = parcelStringBundle.getString("parcel.count.expired");
        this.selectFilesString  = parcelStringBundle.getString("parcel.send.selectfiles");
        this.selectFolderString = parcelStringBundle.getString("parcel.send.selectfolder");
        
        return;
    },

    
    loadApplicationView : function(applicationParams, isHistoryView)
    {
        var viewid      = applicationParams.getParameterValue(NETIZEN_CONTENT_VIEWID_PROPERTY);
        var viewControl = document.getElementById("parcelSendContentPanels");
        if(!viewControl)
        {
            return;
        }

        if(viewid && viewid == PARCEL_SENDAPP_STATE_PACKAGE)
        {
            viewControl.selectedIndex = 1;
            ParcelSendController.onCommandUpdate();
        }
        else if(viewid && viewid == PARCEL_SENDAPP_STATE_PACKAGEINFO)
        {
            viewControl.selectedIndex = 1;
            this.loadSendInfoView(applicationParams);
            ParcelSendController.onCommandUpdate();
        }
     
        this.refreshApplicationView(isHistoryView);
        
        return;
    },


    unloadApplicationView : function(applicationParams)
    {
        return;
    },

    
    refreshApplicationView : function(isHistoryView)
    {
        var archiveTree = document.getElementById("parcelArchiveTreeView");
        if(archiveTree)
        {
            if(!isHistoryView)
            {
                archiveTree.purgeContents();
                archiveTree.setAttribute("mutable", "true");
            }
            else
            {
                if(archiveTree.isEmpty == false)
                {
                    archiveTree.hideOverlayText = true;
                    
                    archiveTree.rebuildContent();
                    archiveTree.toggleFolderOpenState(0);
                }
            }
        }

        if(this.parcelSummaryIsLoaded())
        {
            this.showSendPickupSlipSelection();
            this.showSendPickupSlipInstruction();
            this.showSavedPickupSlipConfirmation();
        }            

        this.updateParcelCount();

        ParcelSendController.onCommandUpdate();
        return;
    },
    

    loadSendInfoView : function(applicationParams)
    {
        var parcelPackageInfo  = document.getElementById("parcelPackageProgressInfo");

        var windowController   = NetizenWindowController;
        var trackerResourceURI = windowController.getPageViewProperty(PARCEL_PACKAGE_PROPERTY_OLDRESOURCE,
                                                                      applicationParams                  );
        var resourceFileName   = windowController.getPageViewProperty(PARCEL_PACKAGE_PROPERTY_FILENAME,
                                                                      applicationParams               );
        
        parcelPackageInfo.setAttribute("target", resourceFileName);
        parcelPackageInfo.setAttribute("state" , "7");

        this.loadParcelSummary(trackerResourceURI, parcelPackageInfo);

        return;
    },


    registerParcel : function(parcelArchive)
    {
        var parcelManager = do_GetService(PARCEL_MANAGER_CONTRACTID, Interfaces.ceIParcelManager);
        if(parcelManager)
        {
            var parcelFile = parcelArchive.parcelFile;
            
            parcelManager.createParcelStub(parcelArchive, PARCEL_METAINFO_EXPIRATION_DEFAULT);
            parcelManager.createParcelSlip(parcelArchive);
            
            return parcelManager.registerParcel(parcelArchive);
        }
        
        return null;
    },


    parcelSummaryIsLoaded : function()
    {
        var parcelPackageInfo  = document.getElementById("parcelPackageProgressInfo");
        if(parcelPackageInfo && parcelPackageInfo.getAttribute("state") == "7")
        {
            return true;
        }
        
        return false;
    },
    

    loadParcelSummary : function(trackerResourceURI, parcelPackageInfo)
    {
        try
        {
            var trackerSummaryInfo  = ParcelManager.trackerSummaryInfo;
            if(trackerResourceURI && trackerSummaryInfo)
            {
                var resourceEntry = trackerSummaryInfo.getResource(trackerResourceURI);
                var summaryEntry  = do_QueryInterface(resourceEntry, Interfaces.ceIParcelTrackerSummary);
                
                parcelPackageInfo.setAttribute("trackerId" , trackerResourceURI         );
                parcelPackageInfo.setAttribute("summary"   , summaryEntry.contentSummary);
                parcelPackageInfo.setAttribute("expiration", summaryEntry.expirySummary );
            }
        }
        catch(exception)
        {
            Syslog.logException(exception);
            return;
        }
                
        this.showPackageCompleteLabel();
        this.showSendPickupSlipSelection();
        this.showSendPickupSlipInstruction();
        
        return;
    },

    
    showPackageNewParcel : function()
    {
        this.updateParcelCount();
        
        var packageNewView = document.getElementById("packageNewView");
        if(packageNewView)
        {
            packageNewView.hidden = false;
        }        

        var packageExistingView = document.getElementById("packageExistingView");
        if(packageExistingView)
        {
            packageExistingView.hidden = true;
        }
        
        var packageActionDeck = document.getElementById("packageActionDeck");
        if(packageActionDeck)
        {
            packageActionDeck.selectedIndex = 0;
        }
        
        return;
    },


    showPackageExisting : function()
    {
        var packageNewView = document.getElementById("packageNewView");
        if(packageNewView)
        {
            packageNewView.hidden = true;
        }

        var packageExistingView = document.getElementById("packageExistingView");
        if(packageExistingView)
        {
            packageExistingView.hidden = false;
        }

        var packageActionDeck = document.getElementById("packageActionDeck");
        if(packageActionDeck)
        {
            packageActionDeck.selectedIndex = 1;
        }
        
        return;
    },


    showPackageCompleteLabel : function()
    {
        var parcelPackageLabelDeck = document.getElementById("parcelPackageLabelDeck");
        if(parcelPackageLabelDeck)
        {
            parcelPackageLabelDeck.selectedIndex = 1;
        }
        
        return;
    },

    
    showSendPickupSlipSelection : function()
    {
        var packageSendSelectionGroup = document.getElementById("packageSendSelectionGroup");
        if(packageSendSelectionGroup)
        {
            packageSendSelectionGroup.hidden = false;            
        }
        
        return;
    },


    showSendPickupSlipInstruction : function()
    {
        var packageSendSelectionGroup = document.getElementById("packageSendSelectionGroup");
        if(packageSendSelectionGroup.hidden)
        {
            return;
        }

        var parcelSendNoticeFrame = document.getElementById("parcelSendNoticeFrame");
        if(parcelSendNoticeFrame)
        {
            NetizenFrameController.addFrameReference("parcelSendNoticeFrame");
            
            parcelSendNoticeFrame.hidden = false;
        }

        NetizenFrameController.resizeInternalFrame("parcelSendNoticeFrame");

        return;
    },
    

    showSavedPickupSlipConfirmation : function()
    {
        var packageSendConfirmationBox = document.getElementById("packageSendConfirmationBox");
        if(!packageSendConfirmationBox)
        {
            return;
        }
        
        var pickupSlipName = packageSendConfirmationBox.getAttribute("lastsave");
        if(!pickupSlipName || pickupSlipName == "null")
        {
            return;
        }
        
        var pickupConfirmationData =
        {
            pickupslip : {name: pickupSlipName}
        };


        var parcelSaveConfirmationFrame = document.getElementById("parcelSaveConfirmationFrame");
        if(parcelSaveConfirmationFrame)
        {
            var frameContent    = parcelSaveConfirmationFrame.contentDocument;
            var templateContent = frameContent.getElementById("saveConfirmationContent");

            NetizenFrameController.addFrameReference("parcelSaveConfirmationFrame");

            parcelSaveConfirmationFrame.hidden = false;
            
            templateContent.innerHTML = TemplateEngine.processTemplate(templateContent.innerHTML, 
                                                                       pickupConfirmationData   );
        }
        
        NetizenFrameController.resizeInternalFrame("parcelSaveConfirmationFrame");

        return;
    },
    

    updateParcelCount : function()
    {
        var trackerService = do_QueryInterface(ParcelManager, Interfaces.ceIParcelTrackerService);
        if(!trackerService)
        {
            return;
        }
        
        var activeParcelsString  = this.activeCountString;
        var activeParcelsCount   = trackerService.activeParcels;        
        var activeParcelsStatus  = activeParcelsString.substitute(activeParcelsCount);
        if(activeParcelsStatus)
        {
            var activeParcelsCountLabel = document.getElementById("activeParcelsCountLabel");
            if(activeParcelsCountLabel)
            {
                activeParcelsCountLabel.setAttribute("value", activeParcelsStatus);
            }
        }
        
        var expiredParcelsString = this.expiredCountString;
        var expiredParcelsCount  = trackerService.expiredParcels;
        var expiredParcelsStatus = expiredParcelsString.substitute(expiredParcelsCount);
        if(expiredParcelsStatus)
        {
            var expiredParcelsCountLabel = document.getElementById("expiredParcelsCountLabel");
            if(expiredParcelsCountLabel)
            {
                expiredParcelsCountLabel.setAttribute("value", expiredParcelsStatus);
            }
        }
        
        return;
    },

    onResizeApplication : function()
    {
        var archiveTree = document.getElementById("parcelArchiveTreeView");
        archiveTree.onWindowResize();
        
        return;
    },


    loadFileEntries : function()
    {
        var archiveTree = document.getElementById("parcelArchiveTreeView");
        if(!archiveTree)
        {
            return;
        }
        
        var fileEntries = null;
        var filePicker  = do_CreateInstance(FILEPICKER_CONTRACTID, Interfaces.nsIFilePicker);
        if(!filePicker)
        {
            return;
        }

        try
        {                
            filePicker.init(window                                   , 
                            this.selectFilesString                   , 
                            Interfaces.nsIFilePicker.modeOpenMultiple);
            
            var filePickerResult = filePicker.show();
            if(filePickerResult == Interfaces.nsIFilePicker.returnOK)
            {
                fileEntries = do_QueryInterface(filePicker.files, Interfaces.nsISimpleEnumerator);

                while(fileEntries.hasMoreElements())
                {
                    try
                    {
                        var fileEntry  = fileEntries.getNext();
                        var fileObject = do_QueryInterface(fileEntry, Interfaces.nsIFile);
                        
                        archiveTree.addArchiveEntry(fileObject);
                    }
                    catch(exception)
                    {
                        Syslog.logException(exception);
                        break;
                    }
                }
            }
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }

        return;
    },


    loadFolderEntry : function()
    {
        var archiveTree = document.getElementById("parcelArchiveTreeView");
        if(!archiveTree)
        {
            return;
        }

        var folderEntry = null;
        var filePicker  = do_CreateInstance(FILEPICKER_CONTRACTID, Interfaces.nsIFilePicker);
        if(!filePicker)
        {
            return;
        }

        try
        {                
            filePicker.init(window                                , 
                            this.selectFolderString               , 
                            Interfaces.nsIFilePicker.modeGetFolder);
            
            var filePickerResult = filePicker.show();
            if(filePickerResult == Interfaces.nsIFilePicker.returnOK)
            {
                folderEntry = do_QueryInterface(filePicker.file, Interfaces.nsILocalFile);
                archiveTree.addArchiveEntry(folderEntry);
            }
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }

        return;
    }
};




var ParcelSendController =
{
    pickupSlipInfo     : null,
    trackerResourceURI : null,

    loadPickupSlipInfo : function(trackerResourceURI)
    {
        this.pickupSlipInfo     = new ceParcelPickupSlipInfo(trackerResourceURI);
        this.trackerResourceURI = trackerResourceURI;
            
        var parcelStringBundle  = document.getElementById("parcelStringBundle");
        if(parcelStringBundle)
        {
            this.pickupSlipInfo.loadStringBundle(parcelStringBundle);
        }     
        
        this.pickupSlipInfo.loadPickupSlipInfo(trackerResourceURI);

        return;
    },


    supportsCommand : function(commandId)
    {
        switch(commandId)
        {
          case "cmd_parcelViewExisting"   :
          case "cmd_parcelViewNew"        :
          case "cmd_parcelPackageTarget"  :
          case "cmd_parcelPackageCancel"  :
          case "cmd_parcelPackageNew"     :
          case "cmd_parcelSelectPackage"  :
          case "cmd_parcelSelectSlip"     :
          case "cmd_parcelPackageViewSlip":
          case "cmd_parcelEmailPickupSlip":
          case "cmd_parcelSavePickupSlip" :
          {      
              return true;
          }
        }
        
        return false;
    },


    isCommandEnabled : function(commandId)
    {
        if(commandId == "cmd_parcelViewNew"      ||
           commandId == "cmd_parcelSelectSlip"    )
        {
            return true;
        }

        if(commandId == "cmd_parcelViewExisting")
        {
            var trackerService = do_QueryInterface(ParcelManager, Interfaces.ceIParcelTrackerService);
            if(!trackerService)
            {
                return false;
            }
            
            return (trackerService.activeParcels > 0) ? true : false;
        }
        
        if(commandId == "cmd_parcelPackageNew")
        {
            var viewControl = document.getElementById("parcelSendContentPanels");
            if(!viewControl)
            {
                return false;
            }
            
            if(viewControl.selectedIndex == 1)
            {
                return (!this.isPackageCancelButtonEnabled());
            }

            return false;
        }
        
        if(commandId == "cmd_parcelPackageCancel")
        {
            var viewControl = document.getElementById("parcelSendContentPanels");
            if(!viewControl)
            {
                return false;
            }
            
            if(viewControl.selectedIndex == 0)
            {
                return this.isSelectionCancelButtonEnabled();
            }
            else if(viewControl.selectedIndex == 1)
            {
                return this.isPackageCancelButtonEnabled();
            }
            
            return false;
        }


        if(commandId == "cmd_parcelPackageTarget")
        {
            var archiveTree = document.getElementById("parcelArchiveTreeView");
            if(!archiveTree)
            {
                return false;
            }
            
            if(archiveTree.isEmpty)
            {
                return false;
            }
         
            var parcelNameField = document.getElementById("parcelNameTextBox");
            if(!parcelNameField || !parcelNameField.value)
            {
                return false;
            }
            
            return true;
        }
        

        if(commandId == "cmd_parcelSelectPackage")
        {
            var parcelSelectPackageButton = document.getElementById("parcelSelectPackageButton");
            if(!parcelSelectPackageButton)
            {
                return false;
            }
            
            var isExpired = parcelSelectPackageButton.getAttribute("expired");
            if(isExpired && isExpired != "")
            {
                return false;
            }
            
            var rowId = parcelSelectPackageButton.getAttribute("rowid");
            if(rowId && rowId != "")
            {
                return true;
            }

            return false;
        }
        
        if(commandId == "cmd_parcelEmailPickupSlip" ||
           commandId == "cmd_parcelPackageViewSlip" ||
           commandId == "cmd_parcelSavePickupSlip"   )

        {
            var parcelPackageInfo  = document.getElementById("parcelPackageProgressInfo");
            if(!parcelPackageInfo)
            {
                return false;
            }
            
            var trackerId    = parcelPackageInfo.getAttribute("trackerId");
            var packageState = parcelPackageInfo.getAttribute("state");
            if(trackerId                                                             && 
               parseInt(packageState) == Interfaces.ceIParcelArchive.STATUS_COMPLETE &&
               ParcelResourceController.parcelEntryExists(trackerId)                  )
            {
                return true;
            }
        }
        
        return false;
    },


    doCommand : function(commandId)
    {
        switch(commandId)
        {
          case "cmd_parcelPackageTarget"   :
          {
              this.packageParcelTarget();
              break;
          }
          case "cmd_parcelPackageCancel"   :
          {
              this.packageParcelCancel();
              break;
          }
          case "cmd_parcelPackageNew"      :
          {
              this.sendNewParcel();
              break;
          }
          case "cmd_parcelSelectPackage"   :
          {
              this.packageExistingParcel();
              break;
          }
          case "cmd_parcelSelectSlip" :
          {
              this.selectPickupSlip();
              break;
          }
          case "cmd_parcelViewExisting"    :
          {
              this.viewExistingParcels();
              break;
          }
          case "cmd_parcelViewNew"         :
          {
              this.viewPackageNewParcel();
              break;
          }
          case "cmd_parcelEmailPickupSlip" :
          {
              this.emailPickupSlip();
              break;
          }
          case "cmd_parcelPackageViewSlip" :
          {
              this.viewPickupSlip();
              break;
          }
          case "cmd_parcelSavePickupSlip"  :
          {
              this.savePickupSlipToFile();
              break;
          }
        }

        this.onCommandUpdate();
        return;
    },
    

    onCommandUpdate : function()
    {
        var packageTargetIsEnabled   = this.isCommandEnabled("cmd_parcelPackageTarget");
        var packageCancelIsEnabled   = this.isCommandEnabled("cmd_parcelPackageCancel");
        var sendNewIsEnabled         = this.isCommandEnabled("cmd_parcelPackageNew");
        var packageExistingIsEnabled = this.isCommandEnabled("cmd_parcelSelectPackage");
        var viewSlipIsEnabled        = this.isCommandEnabled("cmd_parcelPackageViewSlip");
        var emailSlipIsEnabled       = this.isCommandEnabled("cmd_parcelEmailPickupSlip");
        var saveAsFileIsEnabled      = this.isCommandEnabled("cmd_parcelSavePickupSlip");
        var viewExistingIsEnabled    = this.isCommandEnabled("cmd_parcelViewExisting");
        

        goSetCommandEnabled("cmd_parcelPackageTarget"  , packageTargetIsEnabled  );
        goSetCommandEnabled("cmd_parcelPackageCancel"  , packageCancelIsEnabled  );
        goSetCommandEnabled("cmd_parcelPackageNew"     , sendNewIsEnabled        );
        goSetCommandEnabled("cmd_parcelSelectPackage"  , packageExistingIsEnabled);
        goSetCommandEnabled("cmd_parcelPackageViewSlip", viewSlipIsEnabled       );
        goSetCommandEnabled("cmd_parcelEmailPickupSlip", emailSlipIsEnabled      );
        goSetCommandEnabled("cmd_parcelSavePickupSlip" , saveAsFileIsEnabled     );
        goSetCommandEnabled("cmd_parcelViewExisting"   , viewExistingIsEnabled   );
        
        return;
    },


    isSelectionCancelButtonEnabled : function()
    {
        var packageActionDeck = document.getElementById("packageActionDeck");
        if(!packageActionDeck)
        {
            return false;
        }
        
        if(packageActionDeck.selectedIndex == 0)
        {
            var archiveTree = document.getElementById("parcelArchiveTreeView");
            if(!archiveTree)
            {
                return false;
            }
        
            var parcelNameField = document.getElementById("parcelNameTextBox");
            if(!parcelNameField)
            {
                return false;
            }
            
            if(archiveTree.isEmpty && !parcelNameField.value)
            {
                return false;
            }
          
            var treeIsMutable = archiveTree.getAttribute("mutable");
            if(treeIsMutable == "false")
            {
                return false;
            }
            
            return true;
        }
        else if(packageActionDeck.selectedIndex == 1)
        {
            var parcelSelectPackageButton = document.getElementById("parcelSelectPackageButton");
            if(!parcelSelectPackageButton)
            {
                return false;
            }
            
            var rowId = parcelSelectPackageButton.getAttribute("rowid");
            if(rowId && rowId != "")
            {
                return true;
            }

            return false;
        }

        return false;
    },


    isPackageCancelButtonEnabled : function()
    {
        var archiveCacheURI = NetizenPropertyCache.getPropertyValue(PARCEL_PACKAGE_PROPERTY_NEWRESOURCE);
        if(!archiveCacheURI)
        {
            return false;
        }
        
        try
        {
            var currentArchive  = ParcelCacheService.getCachedArchive(archiveCacheURI);
            if(currentArchive && !currentArchive.isClosed)
            {
                return true;
            }
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }
        
        return false;
    },


    viewPackageNewParcel : function()
    {
        ParcelSendApplication.showPackageNewParcel();
        return;
    },


    viewExistingParcels : function()
    {
        ParcelSendApplication.showPackageExisting();
        return;
    },


    sendNewParcel : function()
    {
        NetizenWindowController.scheduleApplicationViewLoad(PARCEL_SENDAPP_CHROME_URL   , 
                                                            PARCEL_SENDAPP_STATE_DEFAULT);

        return;
    },


    packageExistingParcel : function()
    {
        var parcelSelectPackageButton = document.getElementById("parcelSelectPackageButton");
        if(parcelSelectPackageButton)
        {
            var trackerResourceURI = parcelSelectPackageButton.getAttribute("rowid");
            var selectedRow        = document.getElementById(trackerResourceURI);
            var resourceFileName   = null;
            
            if(selectedRow)
            {
                resourceFileName = selectedRow.getAttribute("filename");
            }
        }


        var packageViewParameters = new ceStandardURLQuery();
        
        Assert.isNotNull(packageViewParameters);
        
        packageViewParameters.addQueryParameter(PARCEL_PACKAGE_PROPERTY_OLDRESOURCE, trackerResourceURI);
        packageViewParameters.addQueryParameter(PARCEL_PACKAGE_PROPERTY_FILENAME   , resourceFileName  );

        NetizenPropertyCache.cachePropertyValue(PARCEL_PACKAGE_PROPERTY_OLDRESOURCE, trackerResourceURI);
        NetizenPropertyCache.cachePropertyValue(PARCEL_PACKAGE_PROPERTY_FILENAME   , resourceFileName);
        
        NetizenWindowController.scheduleApplicationViewLoad(PARCEL_SENDAPP_CHROME_URL            , 
                                                            PARCEL_SENDAPP_STATE_PACKAGEINFO     ,
                                                            packageViewParameters.toQueryString());

        return;
    },
    

    packageParcelTarget : function()
    {
        var archiveTree   = document.getElementById("parcelArchiveTreeView");
        if(archiveTree)
        {
            archiveTree.setAttribute("mutable", "false");
        }
        
        var parcelArchive = archiveTree.currentArchive;
        if(!parcelArchive)
        {
            Assert.condition(parcelArchive, "Archive UNAVAILABLE");
            return;
        }
                
        var parcelNameField = document.getElementById("parcelNameTextBox");
        var parcelName      = parcelNameField.value;
        if(parcelName && parcelName != "")
        {
            /* The `&` character must be stripped out of the parcel
             * name in order to not confuse the RDF XML serializer.
             * Until nsRDFService::GetResource escapes URL query strings
             * properly before querying a datasource, all ampersands
             * must be stripped out and replaced with the localized text
             * for its usual translation for "and"
             */
            var ampersandCharRegEx = new RegExp("[&]", "g");
            parcelName = parcelName.replace(ampersandCharRegEx, ParcelSendApplication.ampersandString);

            /* Prevent illegal characters from being entered into the filename */
            var illegalCharsRegEx  = new RegExp("[\/:><|*?]", "g");
            parcelName = parcelName.replace(illegalCharsRegEx, "");
            parcelName = parcelName.trim();            
        }
        
        try
        {
            var parcelWriter = parcelArchive.create(parcelName);
            parcelWriter.asyncFlush();
        }
        catch(exception)
        {
            this.cancelPackageSelection();

            Syslog.logException(exception);
            return;
        }
        
        var packageViewParameters = new ceStandardURLQuery();
        var archiveCacheURI       = archiveTree.getAttribute("ref");
        
        Assert.isNotNull(packageViewParameters);
        Assert.isNotNull(archiveCacheURI);
        
        packageViewParameters.addQueryParameter(PARCEL_PACKAGE_PROPERTY_NEWRESOURCE, archiveCacheURI);
        
        NetizenPropertyCache.cachePropertyValue(PARCEL_PACKAGE_PROPERTY_NEWRESOURCE, archiveCacheURI);

        NetizenWindowController.scheduleApplicationViewLoad(PARCEL_SENDAPP_CHROME_URL            , 
                                                            PARCEL_SENDAPP_STATE_PACKAGE         ,
                                                            packageViewParameters.toQueryString());
    },


    packageParcelCancel : function()
    {
        var viewControl = document.getElementById("parcelSendContentPanels");
        if(!viewControl)
        {
            return;
        }

        if(viewControl.selectedIndex == 0)
        {
            this.cancelPackageSelection();
        }
        else if(viewControl.selectedIndex == 1)
        {
            this.cancelPackageOperation();
        }

        return;
    },


    cancelPackageSelection : function()
    {
        var packageNewView       = document.getElementById("packageNewView");
        var packageExistingView  = document.getElementById("packageExistingView");

        Assert.isNotNull(packageNewView);
        Assert.isNotNull(packageExistingView);
        
        if(packageNewView.hidden == false)
        {
            var archiveTree = document.getElementById("parcelArchiveTreeView");
            if(!archiveTree)
            {
                return;
            }

            archiveTree.purgeContents();
            archiveTree.setAttribute("mutable", "true");
            
            var parcelNameField = document.getElementById("parcelNameTextBox");
            if(!parcelNameField)
            {
                return;
            }

            parcelNameField.value = "";
        }
        else if(packageExistingView.hidden == false)
        {
            var packageExistingView = document.getElementById("packageExistingView");
            if(!packageExistingView)
            {
                return;
            }
            
            packageExistingView.selected = null;

            var packageNewSelection = document.getElementById("packageNewSelection");
            if(packageNewSelection)
            {
                packageNewSelection.setAttribute("selected", "true");
            }

            var packageExistingSelection = document.getElementById("packageExistingSelection");
            if(packageExistingSelection)
            {
                packageExistingSelection.setAttribute("selected", "false");
            }

            this.viewPackageNewParcel();
        }
        
        return;
    },


    cancelPackageOperation : function()
    {
        var archiveCacheURI = NetizenPropertyCache.getPropertyValue(PARCEL_PACKAGE_PROPERTY_NEWRESOURCE);
        if(!archiveCacheURI)
        {
            return;
        }
        
        try
        {
            var currentArchive  = ParcelCacheService.getCachedArchive(archiveCacheURI);
            if(currentArchive && !currentArchive.isClosed)
            {
                var archiveRequest = do_QueryInterface(currentArchive, Interfaces.nsIRequest);
                if(archiveRequest)
                {
                    archiveRequest.cancel(Results.NS_ERROR_ABORT);
                }
            }
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }
        
        return;
    },
    

    onNameTextKeyEvent : function(keyEvent)
    {
        if(keyEvent.keyCode == Interfaces.nsIDOMKeyEvent.DOM_VK_ENTER  || 
           keyEvent.keyCode == Interfaces.nsIDOMKeyEvent.DOM_VK_RETURN  )
        {
            var packageTargetIsEnabled = this.isCommandEnabled("cmd_parcelPackageTarget");
            if(packageTargetIsEnabled)
            {
                this.packageParcelTarget();
            }
        }
        
        return;
    },


    selectPickupSlip : function()
    {
        var sendPickupSlipChoiceGroup = document.getElementById("sendPickupSlipChoiceGroup");
        if(!sendPickupSlipChoiceGroup)
        {
            return;
        }
        
        var packageSendActionDeck = document.getElementById("packageSendActionDeck");
        if(!packageSendActionDeck)
        {
            return;
        }
            
        packageSendActionDeck.selectedIndex = sendPickupSlipChoiceGroup.selectedIndex;
        return;
    },


    emailPickupSlip : function()
    {
        var parcelPackageInfo  = document.getElementById("parcelPackageProgressInfo");
        if(!parcelPackageInfo)
        {
            return;
        }

        var trackerResourceURI = parcelPackageInfo.getAttribute("trackerId");
        if(trackerResourceURI)
        {
            this.loadPickupSlipInfo(trackerResourceURI);
            this.pickupSlipInfo.emailPickupSlipText();
        }
        
        return;
    },


    viewPickupSlip : function()
    {
        var parcelPackageInfo  = document.getElementById("parcelPackageProgressInfo");
        if(!parcelPackageInfo)
        {
            return;
        }

        var trackerResourceURI = parcelPackageInfo.getAttribute("trackerId");
        if(trackerResourceURI)
        {
            ParcelResourceController.loadPickupSlipDialog(trackerResourceURI);
        }

        return;
    },


    savePickupSlipToFile : function()
    {
        var parcelPackageInfo  = document.getElementById("parcelPackageProgressInfo");
        if(!parcelPackageInfo)
        {
            return;
        }

        var trackerResourceURI = parcelPackageInfo.getAttribute("trackerId");
        if(trackerResourceURI)
        {
            this.loadPickupSlipInfo(trackerResourceURI);
            this.pickupSlipInfo.savePickupSlipTextToFile();            
            
            var packageSendConfirmationBox = document.getElementById("packageSendConfirmationBox");
            if(packageSendConfirmationBox)
            {
                packageSendConfirmationBox.setAttribute("lastsave"                        , 
                                                        this.pickupSlipInfo.pickupSlipName);
            
                ParcelSendApplication.showSavedPickupSlipConfirmation();
            }
            
        }

        return;
    }
};



var ParcelPackageEventHandler =
{
    registerListener  : function(targetElement)
    {
        if(targetElement)
        {
            targetElement.addEventListener(PARCEL_PACKAGE_EVENT_VIEWDETAILS, 
                                           this.onViewPackageDetails       , 
                                           false                           );

            targetElement.addEventListener(PARCEL_PACKAGE_EVENT_STORENTRY, 
                                           this.onStorePackageEntry      , 
                                           false                         );
        }

        return;
    },


    unregisterListener  : function(targetElement)
    {
        if(targetElement)
        {
            targetElement.removeEventListener(PARCEL_PACKAGE_EVENT_VIEWDETAILS, 
                                              this.onViewPackageDetails       , 
                                              false                           );
            
            targetElement.removeEventListener(PARCEL_PACKAGE_EVENT_STORENTRY, 
                                              this.onStorePackageEntry      , 
                                              false                         );        
        }
        
        return;
    },


    onViewPackageDetails : function(eventInfo)
    {
        var parcelPackageInfo  = eventInfo.target;
        if(!parcelPackageInfo)
        {
            return;
        }

        var trackerResourceURI = parcelPackageInfo.getAttribute("trackerId");
        if(trackerResourceURI)
        {
            ParcelResourceController.loadParcelDetailsView(trackerResourceURI);
        }
        
        return;
    },


    onStorePackageEntry : function(eventInfo)
    {
        var parcelNameField = document.getElementById("parcelNameTextBox");
        if(parcelNameField && !parcelNameField.value)
        {
            var archiveTree = document.getElementById("parcelArchiveTreeView");
            if(archiveTree && archiveTree.defaultName)
            {
                parcelNameField.value = archiveTree.defaultName;
                parcelNameField.select();
                parcelNameField.focus();
            }
        }
        
        ParcelSendController.onCommandUpdate();
                
        return;
    }
};



var ParcelSelectionEventHandler =
{
    registerListener  : function(targetElement)
    {
        if(targetElement)
        {
            targetElement.addEventListener(PARCEL_PACKAGE_EVENT_SELECTPARCEL, 
                                           this.onSelectExistingParcel      , 
                                           false                            );

            targetElement.addEventListener(PARCEL_PACKAGE_EVENT_UNSELECTPARCEL, 
                                           this.onUnselectExistingParcel      , 
                                           false                              );
        
            targetElement.addEventListener(PARCEL_PACKAGE_EVENT_SENDSELECTED,
                                           this.onPackageSelectedParcel     ,
                                           false                            );

            targetElement.addEventListener(PARCEL_PACKAGE_EVENT_SHOWDETAILS, 
                                           this.onShowPackageDetailsDelayed, 
                                           false                           );
        }
        
        return;
    },


    unregisterListener  : function(targetElement)
    {
        if(targetElement)
        {
            targetElement.removeEventListener(PARCEL_PACKAGE_EVENT_SELECTPARCEL, 
                                              this.onSelectExistingParcel      , 
                                              false                            );
        
            targetElement.removeEventListener(PARCEL_PACKAGE_EVENT_UNSELECTPARCEL, 
                                              this.onUnselectExistingParcel      , 
                                              false                              );
            
            targetElement.removeEventListener(PARCEL_PACKAGE_EVENT_SENDSELECTED,
                                              this.onPackageSelectedParcel     ,
                                              false                            );

            targetElement.removeEventListener(PARCEL_PACKAGE_EVENT_SHOWDETAILS, 
                                              this.onShowPackageDetailsDelayed, 
                                              false                           );
        }
        
        return;
    },


    onSelectExistingParcel : function(eventInfo)
    {
        var packageExistingView = eventInfo.target;
        if(packageExistingView)
        {
            var selectedEntryURI          = packageExistingView.getAttribute("rowid");
            var parcelSelectPackageButton = document.getElementById("parcelSelectPackageButton");
            if(parcelSelectPackageButton)
            {
                parcelSelectPackageButton.setAttribute("rowid", selectedEntryURI);
            }

            var selectedRowElement        = document.getElementById(selectedEntryURI);
            var isExpired                 = selectedRowElement.getAttribute("expired");
            if(selectedRowElement && isExpired)
            {
                parcelSelectPackageButton.setAttribute("expired", "true");
            }
        }

        ParcelSendController.onCommandUpdate();
        return;
    },


    onUnselectExistingParcel : function(eventInfo)
    {
        var parcelSelectPackageButton = document.getElementById("parcelSelectPackageButton");
        if(parcelSelectPackageButton)
        {
            parcelSelectPackageButton.removeAttribute("rowid");
            parcelSelectPackageButton.removeAttribute("expired");
        }

        ParcelSendController.onCommandUpdate();
        return;
    },


    onPackageSelectedParcel : function()
    {
        var parcelSelectPackageButton = document.getElementById("parcelSelectPackageButton");
        if(!parcelSelectPackageButton)
        {
            return;
        }
        
        var isExpired = parcelSelectPackageButton.getAttribute("expired");
        if(isExpired && isExpired != "")
        {
            return;
        }
        
        ParcelSendController.packageExistingParcel();
        ParcelSendController.onCommandUpdate();
        
        return;
    },

    
    onShowPackageDetails : function()
    {
        var parcelSelectPackageButton = document.getElementById("parcelSelectPackageButton");
        if(parcelSelectPackageButton)
        {
            var trackerResourceURI = parcelSelectPackageButton.getAttribute("rowid");
            if(trackerResourceURI)
            {
                ParcelResourceController.loadParcelDetailsView(trackerResourceURI);
            }
        }
        
        return;
    },


    onShowPackageDetailsDelayed : function()
    {
        window.setTimeout(ParcelSelectionEventHandler.onShowPackageDetails, 
                          NETIZEN_LINK_TRIGGER_DELAY_IN_MS                );
        return;
    }
};



/////////////////////////////////////////////////////////////////////////////////
//
// nsIProgressEventSink Implementation
//
/////////////////////////////////////////////////////////////////////////////////


function ceParcelPackageEventSink(documentRef)
{
    if(documentRef)
    {
        this.documentRef     = documentRef;
    }

    var parcelStringBundle   = documentRef.getElementById("parcelStringBundle");
    
    this.percentCompleteText = parcelStringBundle.getString("parcel.percent.complete");
    this.archiveStatusText   = parcelStringBundle.getString("parcel.archiving");

    return;
}


ceParcelPackageEventSink.prototype =
{
    documentRef : null,

    QueryInterface : function(iid)
    {
        if(!iid.equals(Interfaces.nsISupports)          &&
           !iid.equals(Interfaces.nsIInterfaceRequestor) )
        {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        
        return this;
    },
    
    getInterface : function(iid)
    {
        if(!iid.equals(Interfaces.nsISupports)         &&
           !iid.equals(Interfaces.nsIProgressEventSink) )
        {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }

        return this;
    },


    onProgress : function(requestObject, context, progressAmount, progressMax)
    {
        var parcelPackageInfo  = this.documentRef.getElementById("parcelPackageProgressInfo");
        if(parcelPackageInfo)
        {
            parcelPackageInfo.setAttribute("progress"  , progressAmount);
            parcelPackageInfo.setAttribute("completion", progressAmount + this.percentCompleteText);
        }

        return;
    },

    
    onStatus : function(requestObject, context, status, statusMessage)
    {
        var registrationId  = null;
        var parcelArchive   = do_QueryInterface(requestObject, Interfaces.ceIParcelArchive);
        if(!parcelArchive)
        {
            return;
        }
        
        var parcelPackageInfo  = this.documentRef.getElementById("parcelPackageProgressInfo");
        if(!parcelPackageInfo)
        {
            return;
        }
        
        if(status == Interfaces.ceIParcelArchive.STATUS_COMPRESSING ||
           status == Interfaces.ceIParcelArchive.STATUS_ARCHIVING    )
        {
            parcelPackageInfo.setAttribute("status", this.archiveStatusText);
        }
        else if(status == Interfaces.ceIParcelArchive.STATUS_IDLE)
        {
            var archiveTree = this.documentRef.getElementById("parcelArchiveTreeView");
            if(archiveTree)
            {
                archiveTree.rebuildContent();
                archiveTree.toggleFolderOpenState(0);
            }
        }

        var currentState = parcelPackageInfo.getAttribute("state");
        if(parseInt(currentState) == status)
        {
            return;
        }
        
        if(status == Interfaces.ceIParcelArchive.STATUS_COMPLETE)
        {
            var registrationId = ParcelSendApplication.registerParcel(parcelArchive);
            if(registrationId)
            {
                ParcelSendApplication.loadParcelSummary(registrationId, parcelPackageInfo);
            }
                    
            parcelArchive.close();
        }
        else if(status == Interfaces.ceIParcelArchive.STATUS_CANCELED)
        {
            ParcelSendController.sendNewParcel();
            parcelArchive.close();
        }

        parcelPackageInfo.setAttribute("target", parcelArchive.parcelName + ".parc");
        parcelPackageInfo.setAttribute("state" , status);

        if(status == Interfaces.ceIParcelArchive.STATUS_COMPLETE ||
           status == Interfaces.ceIParcelArchive.STATUS_CANCELED  )
        {
            ParcelSendController.onCommandUpdate();
        }
        
        return;
    }
};


