/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2005 Civil Engines Research.  All Rights 
 * Reserved.  
 *
 * Portions created by Netscape Copyright (C) 1998 Netscape 
 * Communications Corporation. All Rights Reserved.
 *
 * Portions created by the Mozilla Foundation are Copyright (C) 
 * 2003 Mozilla Foundation. All Rights Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/

const PARCEL_MANAGER_CONTRACTID              = "@civilengines.com/apps/parcel-manager;1";
const PARCEL_METAINFOSERVICE_CONTRACTID      = "@civilengines.com/metainf/loader-service;1";

const PARCEL_TRACKER_EVENT_VIEWDETAILS       = "urn:parcel:tracker:event:details";
const PARCEL_TRACKER_EVENT_SAFEDELETE        = "urn:parcel:tracker:event:safedelete";
const PARCEL_TRACKER_EVENT_DELETE            = "urn:parcel:tracker:event:delete";
const PARCEL_TRACKER_EVENT_SELECT            = "urn:parcel:tracker:event:select";
const PARCEL_TRACKER_EVENT_UNSELECT          = "urn:parcel:tracker:event:unselect";
const PARCEL_TRACKER_EVENT_RANGESELECT       = "urn:parcel:tracker:event:rangeselect";
const PARCEL_TRACKER_EVENT_RANGEUNSELECT     = "urn:parcel:tracker:event:rangeunselect";

const PARCEL_TRACKER_STATUS_UPDATED          = "urn:parcel:tracker:status:updated";
const PARCEL_TRACKER_STATUS_RELOADED         = "urn:parcel:tracker:status:reloaded";

const PARCEL_TRANSFER_EVENT_RETRY            = "urn:parcel:transfer:event:retry";
const PARCEL_TRANSFER_EVENT_CANCEL           = "urn:parcel:transfer:event:cancel";

var   ParcelManager                          = null;
var   TransferManager                        = null;


function loadApplication(loadRequestURL, isHistoryView)
{
    Syslog.logMessage("Loading Parcel Tracker Application");

    var applicationParams = loadRequestURL.getQueryParameters();
    
    ParcelTrackerApplication.loadServices(applicationParams);
    ParcelTrackerApplication.loadControllers();
    ParcelTrackerApplication.loadHandlers();
    ParcelTrackerApplication.loadStringBundles();
    ParcelTrackerApplication.loadApplicationView(applicationParams, isHistoryView);

    return;
}



function unloadApplication(unloadRequestURL)
{
    ParcelTrackerApplication.unloadHandlers();
    ParcelTrackerApplication.unloadControllers();
    ParcelTrackerApplication.unloadServices(unloadRequestURL.getQueryParameters());

    return;
}



var ParcelTrackerApplication = 
{
    loadServices : function(applicationParams)
    {
        ParcelModeSelector.onModeSelect(PARCEL_APPLICATION_MODE_TRACK);

        ParcelTransferObserver.loadObservers();

        ParcelTransferErrorObserver.loadObservers();
                
        NetizenNetworkStatusObserver.loadObservers();
        
        ParcelNetworkAlertObserver.loadObservers();

        ParcelTrackerStatusObserver.loadObservers();
        
        try
        {
            TransferManager = do_GetService(TRANSFER_MANAGER_CONTRACTID  , 
                                            Interfaces.ceITransferManager);   

            ParcelManager   = do_GetService(PARCEL_MANAGER_CONTRACTID         , 
                                            Interfaces.ceIParcelTrackerService);
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }

        /* Set the transfer listener */
        var activeTransfers  = TransferManager.activeTransfers;
        var transferListener = new ceParcelTrackerProgressListener(document, activeTransfers);
        if(transferListener)
        {
            TransferManager.progressListener = transferListener;
        }

        /* Set the retry event sink */
        var retryEventSink   = new ceParcelTrackerRetryEventSink(document);
        if(retryEventSink)
        {
            TransferManager.retrySchedulerCallback = retryEventSink;
        }
        
        return;
    },


    unloadServices : function(applicationParams)
    {
        TransferManager.retrySchedulerCallback = null;

        TransferManager.progressListener = null;

        ParcelTrackerStatusObserver.unloadObservers();

        ParcelTransferObserver.unloadObservers();

        ParcelTransferErrorObserver.unloadObservers();        

        ParcelNetworkAlertObserver.unloadObservers();

        NetizenNetworkStatusObserver.unloadObservers();

        ParcelModeSelector.onModeUnselect(PARCEL_APPLICATION_MODE_TRACK);        
    },


    loadControllers : function()
    {
        top.controllers.appendController(ParcelTrackerController);        
        ParcelTrackerController.onCommandUpdate();
        
        return;
    },


    unloadControllers : function()
    {
        top.controllers.removeController(ParcelTrackerController);            
        
        return;
    },


    loadHandlers : function()
    {
        var trackerGridBox = document.getElementById("parcelTrackerGridBox");
        if(trackerGridBox)
        {
            ParcelTrackerEventHandler.registerListener(trackerGridBox);
            ParcelTransferEventHandler.registerListener(trackerGridBox);

            trackerGridBox.focus();
        }
        
        return;
    },


    unloadHandlers : function()
    {
        var trackerGridBox = document.getElementById("parcelTrackerGridBox");
        if(trackerGridBox)
        {
            ParcelTrackerEventHandler.unregisterListener(trackerGridBox);
            ParcelTransferEventHandler.unregisterListener(trackerGridBox);

            trackerGridBox.blur();
        }

        return;
    },


    loadStringBundles : function()
    {
        var parcelStringBundle  = document.getElementById("parcelStringBundle");
        if(!parcelStringBundle)
        {
            return;
        }
        
        this.activeParcelsString  = parcelStringBundle.getString("parcel.count.active");
        this.expiredParcelsString = parcelStringBundle.getString("parcel.count.expired");
        this.pickupMultiCount     = parcelStringBundle.getString("parcel.count.pickup.multi");
        this.pickupSingleCount    = parcelStringBundle.getString("parcel.count.pickup.single");    
        this.pickupCountNone      = parcelStringBundle.getString("parcel.count.pickup.none");    

        return;
    },
    

    loadApplicationView : function(applicationParams, isHistoryView)
    {
        this.refreshApplicationView();
        return;
    },

    
    refreshApplicationView : function()
    {
        this.updateParcelCount();

        this.updateTransferStatus();

        this.updatePickupCounts();

        this.updateRetryStatus();
                
        this.updateParcelExpirationNotice();
        
        ParcelTrackerController.onCommandUpdate();
        
        return;
    },
    
    startBatchUpdate : function()
    {
        var rdfMetaInfo     = null;
        var trackerMetaInfo = ParcelManager.trackerMetaInfo;
        if(trackerMetaInfo)
        {
            rdfMetaInfo = do_QueryInterface(trackerMetaInfo, Interfaces.ceIRDFMetaInfo);
            if(rdfMetaInfo)
            {
                rdfMetaInfo.startBatchUpdate();
            }
        }

        var summaryMetaInfo = ParcelManager.trackerSummaryInfo;
        if(summaryMetaInfo)
        {
            rdfMetaInfo = do_QueryInterface(summaryMetaInfo, Interfaces.ceIRDFMetaInfo);
            if(rdfMetaInfo)
            {
                rdfMetaInfo.startBatchUpdate();
            }
        }


        var transferMetaInfo = TransferManager.transferMetaInfo;
        if(transferMetaInfo)
        {
            rdfMetaInfo = do_QueryInterface(transferMetaInfo, Interfaces.ceIRDFMetaInfo);
            if(rdfMetaInfo)
            {
                rdfMetaInfo.startBatchUpdate();
            }
        }
        
        return;
    },

    endBatchUpdate : function()
    {
        var rdfMetaInfo     = null;
        var trackerMetaInfo = ParcelManager.trackerMetaInfo;
        if(trackerMetaInfo)
        {
            rdfMetaInfo = do_QueryInterface(trackerMetaInfo, Interfaces.ceIRDFMetaInfo);
            if(rdfMetaInfo)
            {
                rdfMetaInfo.endBatchUpdate();
            }
        }

        var summaryMetaInfo = ParcelManager.trackerSummaryInfo;
        if(summaryMetaInfo)
        {
            rdfMetaInfo = do_QueryInterface(summaryMetaInfo, Interfaces.ceIRDFMetaInfo);
            if(rdfMetaInfo)
            {
                rdfMetaInfo.endBatchUpdate();
            }
        }


        var transferMetaInfo = TransferManager.transferMetaInfo;
        if(transferMetaInfo)
        {
            rdfMetaInfo = do_QueryInterface(transferMetaInfo, Interfaces.ceIRDFMetaInfo);
            if(rdfMetaInfo)
            {
                rdfMetaInfo.endBatchUpdate();
            }
        }

        return;
    },


    updateActiveParcelsView : function(activeParcelsCount)
    {
        var activeParcelsStatus  = this.activeParcelsString.substitute(activeParcelsCount);
        if(activeParcelsStatus)
        {
            var activeParcelsCountLabel = document.getElementById("activeParcelsCountLabel");
            if(activeParcelsCountLabel)
            {
                activeParcelsCountLabel.setAttribute("value", activeParcelsStatus);
            }
        }

        var activeParcelsGridDeck = document.getElementById("activeParcelsGridDeck");        
        if(!activeParcelsGridDeck)
        {
            return;
        }
        
        if(activeParcelsCount > 0)
        {
            activeParcelsGridDeck.selectedIndex = 1;
        }
        else
        {
            activeParcelsGridDeck.selectedIndex = 0;
        }

        return;
    },


    updateExpiredParcelsView : function(expiredParcelsCount)
    {

        var expiredParcelsStatus = this.expiredParcelsString.substitute(expiredParcelsCount);
        if(expiredParcelsStatus)
        {
            var expiredParcelsCountLabel = document.getElementById("expiredParcelsCountLabel");
            if(expiredParcelsCountLabel)
            {
                expiredParcelsCountLabel.setAttribute("value", expiredParcelsStatus);
            }
        }

        var expiredParcelsGridDeck = document.getElementById("expiredParcelsGridDeck");
        if(!expiredParcelsGridDeck)
        {
            return;
        }

        if(expiredParcelsCount > 0)
        {
            expiredParcelsGridDeck.selectedIndex = 1;
        }
        else
        {
            expiredParcelsGridDeck.selectedIndex = 0;
        }

        var keepExpiredCheckbox = document.getElementById("keepExpiredCheckbox");
        if(keepExpiredCheckbox)
        {
            var parcelManager = do_QueryInterface(ParcelManager, Interfaces.ceIParcelManager);
            if(parcelManager)
            {
                keepExpiredCheckbox.checked = parcelManager.keepExpired;    
            }
        }
        
        return;
    },

    
    updateParcelCount : function()
    {
        try
        {
            this.updateActiveParcelsView(ParcelManager.activeParcels);
            this.updateExpiredParcelsView(ParcelManager.expiredParcels);
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }
        
        return;
    },


    updatePickupCounts : function()
    {
        var transferMetaInfoIndex = NetizenPropertyCache.getPropertyValue(TRANSFER_PROPERTY_METAINFOINDEX);
        if(!transferMetaInfoIndex)
        {
            return;
        }
        
        for(var metaInfoName in transferMetaInfoIndex)
        {
            var transferBucket = transferMetaInfoIndex[metaInfoName];
            var transfers      = transferBucket.transfers;
            for(var counter = 0; counter < transfers.length; counter++)
            {
                this.updateTransferPickupCount(transfers[counter]);
            }
        }
        
        return;
    },
    

    updateTransferPickupCount : function(transferInfoEntry)
    {
        var transferMetaInfoIndex = NetizenPropertyCache.getPropertyValue(TRANSFER_PROPERTY_METAINFOINDEX);

        Assert.condition(transferMetaInfoIndex, "Transfer index not initialzied");
        Assert.condition(transferInfoEntry    , "NULL transferInfoEntry");

        var metaInfoName   = transferInfoEntry.metaInfoName;
        var transferBucket = transferMetaInfoIndex[metaInfoName];
        
        var parcelElement  = document.getElementById(transferInfoEntry.metaInfoName);
        if(!parcelElement)
        {
            return;
        }
        
        if(transferBucket)
        {
            var transferCount     = transferBucket.transfers.length;

            var pickupCountStatus = (transferCount >  1) ? this.pickupMultiCount.substitute(transferCount) : 
                                    (transferCount == 1) ? this.pickupSingleCount : this.pickupCountNone;
            
            var progressElement   = parcelElement.getElementsByAttribute("progressid", metaInfoName);
            if(progressElement[0])
            {
                progressElement[0].setAttribute("transferype"  , transferInfoEntry.transferType);
                progressElement[0].setAttribute("transfercount", transferCount    );
                progressElement[0].setAttribute("activepickups", pickupCountStatus);
            }
        }
        else
        {
            var progressElement   = parcelElement.getElementsByAttribute("progressid", metaInfoName);
            if(progressElement[0])
            {
                progressElement[0].setAttribute("transfertype" , transferInfoEntry.transferType);
                progressElement[0].setAttribute("transfercount", 0 );
                progressElement[0].setAttribute("activepickups", this.pickupCountNone);
            }
        }
        
        return;
    },


    updateRetryStatus : function()
    {
        var retryEventSink = TransferManager.retrySchedulerCallback;
        if(retryEventSink)
        {
            retryEventSink.notify(null);
        }
       
        return;
    },

    
    updateTransferStatus : function()
    {
        var transferMetaInfo = TransferManager.transferMetaInfo;
        if(!transferMetaInfo)
        {
            return;
        }
        
        var transferEnumerator = transferMetaInfo.enumerateResources();
        if(!transferEnumerator)
        {
            return;
        }
        
        while(transferEnumerator.hasMoreElements())
        {
            var metaInfo = transferEnumerator.getNext();
            if(!metaInfo)
            {
                break;
            }
                        
            var transferInfoEntry = do_QueryInterface(metaInfo, Interfaces.ceITransferInfoEntry);
            if(transferInfoEntry)
            {
                this.updateTransferInfoStatus(transferInfoEntry);
            }
        }

        return;
    },


    updateTransferInfoStatus : function(transferInfoEntry)
    {
        Assert.isNotNull(transferInfoEntry);
        
        var metaInfoName  = transferInfoEntry.metaInfoName;
        var parcelElement = document.getElementById(metaInfoName);
        if(!parcelElement)
        {
            return;
        }

        var progressElement = parcelElement.getElementsByAttribute("progressid", metaInfoName);
        if(progressElement[0])
        {
            var currentTotalProgress = transferInfoEntry.transferred;
            var maxTotalProgress     = transferInfoEntry.filesize;
            var progressPercent      = 0;
            
            if(maxTotalProgress != 0)
            {            
                progressPercent = Math.floor((currentTotalProgress * 100.0)/maxTotalProgress);
                if(progressPercent > 100)
                {
                    progressPercent = 100;
                }                
            }

            progressElement[0].setAttribute("transfertype" , transferInfoEntry.transferType);
            progressElement[0].setAttribute("transferstate", transferInfoEntry.transferStatus);
            progressElement[0].setAttribute("progress"     , progressPercent);
        }
        else
        {
            var expiryElement = parcelElement.getElementsByAttribute("expiryid", metaInfoName);
            if(expiryElement[0])
            {
                expiryElement[0].setAttribute("transfertype" , transferInfoEntry.transferType);
                expiryElement[0].setAttribute("transferstate", transferInfoEntry.transferStatus);
            }
        }
        
        return;
    },

    updateParcelExpirationNotice : function()
    {
        var keepNoticeVisibility   = "none";
        var emptyNoticeVisibility  = "none";
        var deleteNoticeVisibility = "none";

        var keepExpiredCheckbox = document.getElementById("keepExpiredCheckbox");
        if(keepExpiredCheckbox && keepExpiredCheckbox.checked)
        {
            keepNoticeVisibility   = "block";
        }
        else if(ParcelManager.expiredParcels > 0)
        {
            deleteNoticeVisibility = "block";
        }
        else
        {
            emptyNoticeVisibility  = "block";
        }
        
        var parcelExpirationNoticeFrame = document.getElementById("parcelExpirationNoticeFrame");
        if(parcelExpirationNoticeFrame)
        {
            var frameContent        = parcelExpirationNoticeFrame.contentDocument;
            var keepExpiredNotice   = frameContent.getElementById("keepExpiredNotice");
            var emptyExpiredNotice  = frameContent.getElementById("emptyDeleteExpiredNotice");
            var deleteExpiredNotice = frameContent.getElementById("deleteExpiredNotice");

            keepExpiredNotice.style.display   = keepNoticeVisibility;
            emptyExpiredNotice.style.display  = emptyNoticeVisibility;
            deleteExpiredNotice.style.display = deleteNoticeVisibility;
                        
            NetizenFrameController.addFrameReference("parcelExpirationNoticeFrame");
        }
        
        NetizenFrameController.resizeInternalFrame("parcelExpirationNoticeFrame");
        return;
    }
    
};



var ParcelTrackerController =
{
    supportsCommand : function(commandId)
    {
        if(commandId == "cmd_parcelTrackerDetails"       ||
           commandId == "cmd_parcelTrackerDelete"        ||
           commandId == "cmd_parcelConfirmDelete"        ||
           commandId == "cmd_parcelTrackerUnpack"        ||
           commandId == "cmd_parcelKeepExpired"          ||
           commandId == "cmd_parcelConfirmDeleteExpired" ||
           commandId == "cmd_parcelDeleteExpired"         )
        {      
            return true;
        }

        return false;
    },


    isCommandEnabled : function(commandId)
    {
        if(commandId == "cmd_parcelTrackerDetails")
        {    
            var viewDetailsElement = document.getElementById("trackerViewDetailsButton");
            if(!viewDetailsElement)
            {
                return false;
            }
            
            var parcelTrackerGridBox = document.getElementById("parcelTrackerGridBox");
            if(parcelTrackerGridBox && parcelTrackerGridBox.hasRangeSelection())
            {
                return false;
            }
            
            var rowId = viewDetailsElement.getAttribute("rowid");
            if(rowId && rowId != "")
            {
                return true;
            } 
        }
        else if(commandId == "cmd_parcelTrackerDelete" || 
                commandId == "cmd_parcelConfirmDelete"  )
        {
            var deleteElement = document.getElementById("trackerDeleteEntryButton");
            if(!deleteElement)
            {
                return false;
            }
            
            var rowId = deleteElement.getAttribute("rowid");
            if(rowId && rowId != "")
            {
                return true;
            }
        }
        else if(commandId == "cmd_parcelTrackerUnpack")
        {
            var unpackElement = document.getElementById("trackerUnpackButton");
            if(!unpackElement)
            {
                return false;
            }
            
            var parcelTrackerGridBox = document.getElementById("parcelTrackerGridBox");
            if(parcelTrackerGridBox && parcelTrackerGridBox.hasRangeSelection())
            {
                return false;
            }

            var rowId = unpackElement.getAttribute("rowid");
            if(rowId && rowId != "")
            {
                return this.isUnpackEnabled(rowId);
            } 
        }
        else if(commandId == "cmd_parcelKeepExpired")
        {
            return true;
        }
        else if(commandId == "cmd_parcelDeleteExpired"       ||
                commandId == "cmd_parcelConfirmDeleteExpired" )
        {
            try
            {
                if(ParcelManager.expiredParcels > 0)
                {
                    return true;
                }
            }
            catch(exception)
            {
                Syslog.logException(exception);
            }
        }
        
        
        return false;
    },


    doCommand : function(commandId)
    {
        if(commandId == "cmd_parcelTrackerDetails")
        {      
            this.trackerViewDetails();
        }
        else if(commandId == "cmd_parcelConfirmDelete")
        {
            this.trackerDeleteConfirm();
        }
        else if(commandId == "cmd_parcelTrackerDelete")
        {
            this.trackerDeleteSelection();
        }
        else if(commandId == "cmd_parcelTrackerUnpack")
        {
            this.trackerUnpackResource();
        }
        else if(commandId == "cmd_parcelKeepExpired")
        {
            this.checkKeepExpired();
        }
        else if(commandId == "cmd_parcelConfirmDeleteExpired")
        {
            this.trackerConfirmDeleteExpired();
        }
        else if(commandId == "cmd_parcelDeleteExpired")
        {
            this.trackerDeleteExpired();
        }
        
        this.onCommandUpdate();
        
        return;
    },
    

    onCommandUpdate : function()
    {
        var viewDetailsIsEnabled          = this.isCommandEnabled("cmd_parcelTrackerDetails");
        var deleteEntryIsEnabled          = this.isCommandEnabled("cmd_parcelTrackerDelete");
        var confirmDeleteIsEnabled        = this.isCommandEnabled("cmd_parcelConfirmDelete");
        var unpackEntryIsEnabled          = this.isCommandEnabled("cmd_parcelTrackerUnpack");
        var deleteExpiredIsEnabled        = this.isCommandEnabled("cmd_parcelDeleteExpired");
        var confirmDeleteExpiredIsEnabled = this.isCommandEnabled("cmd_parcelConfirmDeleteExpired");

        goSetCommandEnabled("cmd_parcelTrackerDetails"      , viewDetailsIsEnabled  );        
        goSetCommandEnabled("cmd_parcelTrackerDelete"       , deleteEntryIsEnabled  );
        goSetCommandEnabled("cmd_parcelConfirmDelete"       , confirmDeleteIsEnabled);
        goSetCommandEnabled("cmd_parcelTrackerUnpack"       , unpackEntryIsEnabled  );
        goSetCommandEnabled("cmd_parcelDeleteExpired"       , deleteExpiredIsEnabled);
        goSetCommandEnabled("cmd_parcelConfirmDeleteExpired", confirmDeleteExpiredIsEnabled);
        
        return;
    },


    isUnpackEnabled : function(rowId)
    {
        var statusElement = document.getElementById(rowId);
        if(!statusElement)
        {
            return true;
        }
        
        var progressElement = statusElement.getElementsByAttribute("progressid", rowId);
        if(progressElement[0])
        {
            var entryType     = progressElement[0].getAttribute("entry");
            var transferState = progressElement[0].getAttribute("transferstate");
            var entryTypeStub = Interfaces.ceIParcelTrackerEntry.ENTRY_TYPE_LOCAL_STUB_INFO;
            
            if(parseInt(transferState) == Interfaces.ceITransferInfo.TRANSFER_FINISHED)
            {
                return true;
            }
            else if(parseInt(entryType) == entryTypeStub)
            {
                return true;
            }

            return false;                        
        }

        try
        {
            var trackerMetaInfo    = ParcelManager.trackerMetaInfo;
            var transferMetaInfo   = TransferManager.transferMetaInfo;
            
            var trackerEntry       = trackerMetaInfo.getResource(rowId);
            var transferTargetSpec = ParcelManager.getURLSpecFromLocalPath(trackerEntry);
            
            var transferInfo       = transferMetaInfo.getResource(transferTargetSpec);
            var transferInfoEntry  = do_QueryInterface(transferInfo                   ,
                                                       Interfaces.ceITransferInfoEntry);
            if(transferInfoEntry)
            {
                var transferStatus = transferInfoEntry.transferStatus;
                var transferType   = transferInfoEntry.transferType;
                
                if(transferType   == Interfaces.ceITransferInfo.TRANSFER_TYPE_DOWNLOAD &&
                   transferStatus != Interfaces.ceITransferInfo.TRANSFER_FINISHED       )
                {
                    return false;
                }                                
            }
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }                
                    
        return true;
    },

    
    trackerDeleteConfirm : function()
    {
        var deleteCount = 1;
        var parcelTrackerGridBox = document.getElementById("parcelTrackerGridBox");
        if(parcelTrackerGridBox)
        {
            for(var entryId in parcelTrackerGridBox.selectedEntries)
            {
                var selectedRow = parcelTrackerGridBox.selectedEntries[entryId];
                if(selectedRow && isObject(selectedRow))
                {
                    deleteCount++;
                }
            }
        }
        
        window.openDialog("chrome://netizen/content/parcel/parcelConfirmDelete.xul",
                          "confirmDeleteEntryDialog"                               ,
                          "centerscreen,modal,chrome,resizable=no"                 ,
                          deleteCount                                              ,
                          false                                                    );
    
        return;
    },


    trackerDeleteTransfer : function(metaInfoId)
    {
        var transferMetaInfo = TransferManager.transferMetaInfo;
        if(!transferMetaInfo)
        {
            return;
        }
        
        var transferEnumerator = transferMetaInfo.enumerateResources();
        if(!transferEnumerator)
        {
            return;
        }
        
        while(transferEnumerator.hasMoreElements())
        {
            var metaInfo = transferEnumerator.getNext();
            if(!metaInfo)
            {
                break;
            }
                        
            var transferInfoEntry = do_QueryInterface(metaInfo, Interfaces.ceITransferInfoEntry);
            if(transferInfoEntry.transferType != Interfaces.ceITransferInfo.TRANSFER_TYPE_DOWNLOAD)
            {
                continue;
            }
            
            if(transferInfoEntry.metaInfoName != metaInfoId)
            {
                continue;
            }
            
            var metaInfoEntry = do_QueryInterface(transferInfoEntry, Interfaces.ceIMetaInfoEntry);
            if(metaInfoEntry && metaInfoEntry.descriptor)
            {
                try
                {
                    TransferManager.cancelTransfer(metaInfoEntry.descriptor);
                }
                catch(exception)
                {
                    Syslog.logException(exception);
                }
                
                TransferManager.removeTransfer(metaInfoEntry.descriptor);
            }            
        }

        return;
    },

    
    trackerDeleteSelection : function()
    {
        var targetElement = document.getElementById("trackerDeleteEntryButton");
        if(!targetElement)
        {
            return;
        }

        var rowId = targetElement.getAttribute("rowid");
        if(rowId)
        {
            Syslog.logMessage("tackerDeleteEntry => " + rowId);

            this.trackerDeleteTransfer(rowId);

            try
            {
                ParcelManager.removeTrackerEntry(rowId);
                targetElement.removeAttribute("rowid");
            }
            catch(exception)
            {
                Syslog.logException(exception);
            }
        }
        

        var parcelTrackerGridBox = document.getElementById("parcelTrackerGridBox");
        if(parcelTrackerGridBox && parcelTrackerGridBox.hasRangeSelection())
        {
            ParcelTrackerApplication.startBatchUpdate();
            
            for(var entryId in parcelTrackerGridBox.selectedEntries)
            {
                var selectedRow = parcelTrackerGridBox.selectedEntries[entryId];
                if(selectedRow && isObject(selectedRow))
                {
                    Syslog.logMessage("tackerDeleteEntry => " + entryId);

                    this.trackerDeleteTransfer(rowId);

                    try
                    {
                        ParcelManager.removeTrackerEntry(entryId);
                    }
                    catch(exception)
                    {
                        Syslog.logException(exception);
                    }
                }
            }

            ParcelTrackerApplication.endBatchUpdate();
        }
        
        ParcelTrackerApplication.refreshApplicationView();

        return;
    },
    

    trackerViewDetails : function()
    {
        var targetElement = document.getElementById("trackerViewDetailsButton");
        if(!targetElement)
        {
            return;
        }
        
        var rowId = targetElement.getAttribute("rowid");
        if(rowId)
        {
            ParcelResourceController.loadParcelDetailsView(rowId);
        }
        
        return;
    },


    trackerCheckDeleteConfirm : function(event)
    {
        if(event && event.altKey)
        {
            if(this.isCommandEnabled("cmd_parcelTrackerDelete"))
            {
                this.trackerDeleteSelection();
            }
        }
        else
        {
            if(this.isCommandEnabled("cmd_parcelConfirmDelete"))
            {
                this.trackerDeleteConfirm();
            }
        }
        
        return;
    },


    trackerUnpackResource : function()
    {
        var targetElement = document.getElementById("trackerUnpackButton");
        if(!targetElement)
        {
            return;
        }
        
        var rowId = targetElement.getAttribute("rowid");
        if(rowId)
        {
            ParcelResourceController.unpackParcelToDesktop(rowId);
        }

        return;
    },


    checkKeepExpired : function()
    {
        var keepExpiredCheckbox = document.getElementById("keepExpiredCheckbox");
        if(keepExpiredCheckbox)
        {
            var parcelManager = do_QueryInterface(ParcelManager, Interfaces.ceIParcelManager);
            if(parcelManager)
            {
                parcelManager.keepExpired = keepExpiredCheckbox.checked;
            }

            ParcelTrackerApplication.updateParcelExpirationNotice();
        }
        
        return;
    },


    trackerDeleteExpired : function()
    {
        var parcelManager = do_QueryInterface(ParcelManager, Interfaces.ceIParcelManager);
        if(parcelManager)
        {
            try
            {
                parcelManager.deleteExpiredParcels();
            }
            catch(exception)
            {
                Syslog.logMessage(exception);
            }
        }
        
        ParcelTrackerApplication.refreshApplicationView();

        return;
    },


    trackerConfirmDeleteExpired : function()
    {
        try
        {
            window.openDialog("chrome://netizen/content/parcel/parcelConfirmDelete.xul",
                              "confirmDeleteEntryDialog"                               ,
                              "centerscreen,modal,chrome,resizable=no"                 ,
                              ParcelManager.expiredParcels                             ,
                              true                                                     );
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }
        
        return;
    },


    trackerCheckDeleteExpired : function(event)
    {
        if(event && event.altKey)
        {
            if(this.isCommandEnabled("cmd_parcelDeleteExpired"))
            {
                this.trackerDeleteExpired();
            }
        }
        else
        {
            if(this.isCommandEnabled("cmd_parcelConfirmDeleteExpired"))
            {
                this.trackerConfirmDeleteExpired();
            }
        }
        
        return;
    }
};



var ParcelTrackerEventHandler =
{
    registerListener  : function(targetElement)
    {
        if(targetElement)
        {
            targetElement.addEventListener(PARCEL_TRACKER_EVENT_VIEWDETAILS, 
                                           this.onTrackerViewDetails       , 
                                           false                           );

            targetElement.addEventListener(PARCEL_TRACKER_EVENT_SAFEDELETE , 
                                           this.onTrackerEntryConfirmDelete, 
                                           false                           );

            targetElement.addEventListener(PARCEL_TRACKER_EVENT_DELETE, 
                                           this.onTrackerEntryDelete  , 
                                           false                      );

            targetElement.addEventListener(PARCEL_TRACKER_EVENT_UNSELECT, 
                                           this.onTrackerEntryUnselect  , 
                                           false                        );

            targetElement.addEventListener(PARCEL_TRACKER_EVENT_SELECT, 
                                           this.onTrackerEntrySelect  , 
                                           false                      );

            targetElement.addEventListener(PARCEL_TRACKER_EVENT_RANGESELECT, 
                                           this.onTrackerEntrySelect       , 
                                           false                           );

            targetElement.addEventListener(PARCEL_TRACKER_EVENT_RANGEUNSELECT, 
                                           this.onTrackerEntryUnselect       , 
                                           false                             );
        }
        
        return;
    },


    unregisterListener  : function(targetElement)
    {
        if(targetElement)
        {
            targetElement.removeEventListener(PARCEL_TRACKER_EVENT_VIEWDETAILS, 
                                              this.onTrackerViewDetails       , 
                                              false                           );

            targetElement.removeEventListener(PARCEL_TRACKER_EVENT_SAFEDELETE , 
                                              this.onTrackerEntryConfirmDelete, 
                                              false                           );

            targetElement.removeEventListener(PARCEL_TRACKER_EVENT_DELETE, 
                                              this.onTrackerEntryDelete  , 
                                              false                      );

            targetElement.removeEventListener(PARCEL_TRACKER_EVENT_UNSELECT, 
                                              this.onTrackerEntryUnselect  , 
                                              false                        );

            targetElement.removeEventListener(PARCEL_TRACKER_EVENT_SELECT, 
                                              this.onTrackerEntrySelect  , 
                                              false                      );

            targetElement.removeEventListener(PARCEL_TRACKER_EVENT_RANGESELECT, 
                                              this.onTrackerEntrySelect       , 
                                              false                           );

            targetElement.removeEventListener(PARCEL_TRACKER_EVENT_RANGEUNSELECT, 
                                              this.onTrackerEntryUnselect       , 
                                              false                             );
        }
        
        return;
    },

    onTrackerViewDetails : function(eventInfo)
    {
        ParcelTrackerController.doCommand("cmd_parcelTrackerDetails");        
        return;
    },

    
    onTrackerEntryConfirmDelete : function(eventInfo)
    {
        ParcelTrackerController.doCommand("cmd_parcelConfirmDelete");
        return;
    },


    onTrackerEntryDelete : function(eventInfo)
    {
        ParcelTrackerController.doCommand("cmd_parcelTrackerDelete");
        return;
    },


    onTrackerEntrySelect : function(eventInfo)
    {
        ParcelTrackerController.onCommandUpdate();
        return;
    },


    onTrackerEntryUnselect : function(eventInfo)
    {
        ParcelTrackerController.onCommandUpdate();
        return;
    }
};



var ParcelTrackerStatusObserver =
{
    loadObservers : function()
    {
        ObserverService.addObserver(this, PARCEL_TRACKER_STATUS_UPDATED , false);
        ObserverService.addObserver(this, PARCEL_TRACKER_STATUS_RELOADED, false);

        return;
    },

    unloadObservers : function()
    {
        ObserverService.removeObserver(this, PARCEL_TRACKER_STATUS_UPDATED );
        ObserverService.removeObserver(this, PARCEL_TRACKER_STATUS_RELOADED);
        return;
    },

    QueryInterface : function(iid)
    {
        if(iid.equals(Interfaces.nsIObserver) ||
           iid.equals(Interfaces.nsISupports)  )
        {
            return this;
        }
        
        throw Results.NS_NOINTERFACE;
    },

    observe : function(subject, topic, state)
    {
        switch(topic)
        {
          case PARCEL_TRACKER_STATUS_UPDATED  :
          case PARCEL_TRACKER_STATUS_RELOADED :
          {
              var activeParcelsGridRows = document.getElementById("activeParcelsGridRows");
              if(activeParcelsGridRows)
              {
                  activeParcelsGridRows.builder.rebuild();
              }

              var expiredParcelsGridRows = document.getElementById("expiredParcelsGridRows");
              if(expiredParcelsGridRows)
              {
                  expiredParcelsGridRows.builder.rebuild();
              }
              
              ParcelTrackerApplication.updateRetryStatus();

              ParcelTrackerApplication.updateParcelCount();

              ParcelTrackerApplication.updateTransferStatus();

              ParcelTrackerApplication.updatePickupCounts();
              
              ParcelTrackerApplication.updateParcelExpirationNotice();
              
              ParcelTrackerController.onCommandUpdate();

              break;
          }
        }
        
        return;
    }
};




var ParcelTransferErrorObserver =
{
    loadObservers : function()
    {
        ObserverService.addObserver(this, TRANSFER_STATUS_RESOLVING   , false);
        ObserverService.addObserver(this, TRANSFER_STATUS_STARTED     , false);
        ObserverService.addObserver(this, TRANSFER_STATUS_TRANSFERRING, false);
        ObserverService.addObserver(this, TRANSFER_STATUS_COMPLETE    , false);
        ObserverService.addObserver(this, TRANSFER_STATUS_FAILED      , false);
        ObserverService.addObserver(this, TRANSFER_STATUS_CANCELED    , false);
        ObserverService.addObserver(this, TRANSFER_STATUS_NOTFOUND    , false);

        ObserverService.addObserver(this, TRANSFER_STATUS_RESTARTING  , false);
        ObserverService.addObserver(this, TRANSFER_STATUS_RESTARTED   , false);
        ObserverService.addObserver(this, TRANSFER_STATUS_INTERRUPTED , false);

        return;
    },

    unloadObservers : function()
    {
        ObserverService.removeObserver(this, TRANSFER_STATUS_RESOLVING   );
        ObserverService.removeObserver(this, TRANSFER_STATUS_STARTED     );
        ObserverService.removeObserver(this, TRANSFER_STATUS_TRANSFERRING);
        ObserverService.removeObserver(this, TRANSFER_STATUS_COMPLETE    );
        ObserverService.removeObserver(this, TRANSFER_STATUS_FAILED      );
        ObserverService.removeObserver(this, TRANSFER_STATUS_CANCELED    );
        ObserverService.removeObserver(this, TRANSFER_STATUS_NOTFOUND    );

        ObserverService.removeObserver(this, TRANSFER_STATUS_RESTARTING  );
        ObserverService.removeObserver(this, TRANSFER_STATUS_RESTARTED   );
        ObserverService.removeObserver(this, TRANSFER_STATUS_INTERRUPTED );
        
        return;
    },

    QueryInterface : function(iid)
    {
        if(iid.equals(Interfaces.nsIObserver) ||
           iid.equals(Interfaces.nsISupports)  )
        {
            return this;
        }
        
        throw Results.NS_NOINTERFACE;
    },

    observe : function(subject, topic, state)
    {
        switch(topic)
        {
          case TRANSFER_STATUS_STARTED     :
          case TRANSFER_STATUS_COMPLETE    :
          case TRANSFER_STATUS_CANCELED    :
          case TRANSFER_STATUS_NOTFOUND    :
          case TRANSFER_STATUS_FAILED      :
          case TRANSFER_STATUS_RESTARTING  :
          case TRANSFER_STATUS_RESTARTED   :
          case TRANSFER_STATUS_INTERRUPTED :
          {
              var transferInfo = do_QueryInterface(subject, Interfaces.ceITransferInfoEntry);              
              if(!transferInfo)
              {
                  Assert.isNotNull(transferInfo);
                  break;
              }
              
              if(transferInfo.transferType == Interfaces.ceITransferInfo.TRANSFER_TYPE_DOWNLOAD)
              {
                  ParcelTrackerApplication.updateTransferInfoStatus(transferInfo);
              }
              else
              {
                  ParcelTrackerApplication.updateTransferPickupCount(transferInfo);
              }

              ParcelTrackerApplication.updateRetryStatus();

              ParcelTrackerController.onCommandUpdate();

              break;
          }
        }
        
        return;
    }
};



var ParcelTransferEventHandler =
{
    registerListener  : function(targetElement)
    {
        if(targetElement)
        {
            targetElement.addEventListener(PARCEL_TRANSFER_EVENT_RETRY,
                                           this.onTransferRetry       , 
                                           false                      );

            targetElement.addEventListener(PARCEL_TRANSFER_EVENT_CANCEL, 
                                           this.onTransferCancel       , 
                                           false                       );        
        }
        
        return;
    },


    unregisterListener  : function(targetElement)
    {
        if(targetElement)
        {
            targetElement.removeEventListener(PARCEL_TRANSFER_EVENT_RETRY, 
                                              this.onTransferRetry       , 
                                              false                      );
        
            targetElement.removeEventListener(PARCEL_TRANSFER_EVENT_CANCEL, 
                                              this.onTransferCancel       , 
                                              false                       );
        }
        
        return;
    },


    retryTransfer : function(shouldRetry)
    {
        var targetElement = document.getElementById("trackerViewDetailsButton");
        if(!targetElement)
        {
            return;
        }

        var metaInfoId = targetElement.getAttribute("rowid");
        if(!metaInfoId)
        {
            return;
        }
        
        var transferMetaInfo = TransferManager.transferMetaInfo;
        if(!transferMetaInfo)
        {
            return;
        }
        
        var transferEnumerator = transferMetaInfo.enumerateResources();
        if(!transferEnumerator)
        {
            return;
        }
        
        while(transferEnumerator.hasMoreElements())
        {
            var metaInfo = transferEnumerator.getNext();
            if(!metaInfo)
            {
                break;
            }
                        
            var transferInfoEntry = do_QueryInterface(metaInfo, Interfaces.ceITransferInfoEntry);
            if(transferInfoEntry.transferType != Interfaces.ceITransferInfo.TRANSFER_TYPE_DOWNLOAD)
            {
                continue;
            }
            
            if(transferInfoEntry.metaInfoName != metaInfoId)
            {
                continue;
            }
            
            var metaInfoEntry = do_QueryInterface(transferInfoEntry, Interfaces.ceIMetaInfoEntry);
            if(metaInfoEntry && metaInfoEntry.descriptor)
            {
                try
                {
                    if(shouldRetry)
                    {
                        TransferManager.retryTransfer(metaInfoEntry.descriptor);
                    }
                    else
                    {
                        TransferManager.cancelTransfer(metaInfoEntry.descriptor);
                    }
                }
                catch(exception)
                {
                    Syslog.logException(exception);
                }
            }            
        }

        return;
    },


    onTransferRetry : function(eventInfo)
    {
        window.setTimeout(ParcelTransferEventHandler.retryTransfer,
                          NETIZEN_LINK_TRIGGER_DELAY_IN_MS        ,
                          true                                    );
        return;
    },


    onTransferCancel : function(eventInfo)
    {
        window.setTimeout(ParcelTransferEventHandler.retryTransfer, 
                          NETIZEN_LINK_TRIGGER_DELAY_IN_MS        ,
                          false                                   );
        return;
    }    
};



/////////////////////////////////////////////////////////////////////////////////
//
// ceITransferProgressListener Implementation
//
/////////////////////////////////////////////////////////////////////////////////


function ceParcelTrackerProgressListener(documentRef, activeTransfers)
{
    if(documentRef)
    {
        this.documentRef         = documentRef;
        var transferStringBundle = this.documentRef.getElementById("transferStringBundle");
        if(transferStringBundle)
        {
            this.loadStringBundle(transferStringBundle);
        }
    }
    
    if(activeTransfers)
    {
        this.initTransferInfoIndex(activeTransfers);
    }

    return;
}



ceParcelTrackerProgressListener.prototype = new ceAbstractProgressListener();



ceParcelTrackerProgressListener.prototype.constructor = ceParcelTrackerProgressListener;


ceParcelTrackerProgressListener.superclass  = ceAbstractProgressListener.prototype;



ceParcelTrackerProgressListener.prototype.onProgressChange = function(webProgress         , 
                                                                      request             , 
                                                                      currentSelfProgress , 
                                                                      maxSelfProgress     , 
                                                                      currentTotalProgress, 
                                                                      maxTotalProgress    , 
                                                                      transferInfo        )
{
    if(!this.shouldUpdateProgress(transferInfo, currentTotalProgress, maxTotalProgress))
    {
        return;
    }

    if(maxTotalProgress <= 0)
    {
        return;
    }
    
    var transferEntry = do_QueryInterface(transferInfo, Interfaces.ceITransferInfoEntry);
    if(!transferEntry)
    {
        return;
    }                

    var metaInfoName  = transferEntry.metaInfoName;
    var parcelElement = document.getElementById(metaInfoName);
    if(parcelElement)
    {
        var progressElement = parcelElement.getElementsByAttribute("progressid", metaInfoName);
        if(progressElement[0])
        {
            var transferred     = transferEntry.transferred;
            var fileSize        = transferEntry.filesize;
            var progressPercent = Math.floor((transferred / fileSize) * 100);

            progressElement[0].setAttribute("transfertype" , transferEntry.transferType);
            progressElement[0].setAttribute("transferstate", transferEntry.transferStatus);
            progressElement[0].setAttribute("progress"     , progressPercent);
        }
    }
    
    return;
}



ceParcelTrackerProgressListener.prototype.onStateChange = function(webProgress , 
                                                                   request     , 
                                                                   stateFlags  , 
                                                                   statusCode  ,
                                                                   transferInfo)
{    

    if(stateFlags & Interfaces.nsIWebProgressListener.STATE_START ||
       stateFlags & Interfaces.nsIWebProgressListener.STATE_STOP   )
    {
        var transferInfoEntry = do_QueryInterface(transferInfo, Interfaces.ceITransferInfoEntry);
        if(transferInfoEntry)
        {
            if(transferInfoEntry.transferType == Interfaces.ceITransferInfo.TRANSFER_TYPE_UPLOAD)
            {
                ParcelTrackerApplication.updateTransferPickupCount(transferInfoEntry);
            }

            NetizenPropertyCache.cachePropertyValue(transferInfoEntry.metaInfoName, transferInfo);
        }
    }

    return;
}





/////////////////////////////////////////////////////////////////////////////////
//
// nsITimerCallback Implementation
//
/////////////////////////////////////////////////////////////////////////////////


function ceParcelTrackerRetryEventSink(documentRef)
{
    if(documentRef)
    {
        this.documentRef         = documentRef;
        var transferStringBundle = this.documentRef.getElementById("transferStringBundle");
        if(transferStringBundle)
        {
            this.loadStringBundle(transferStringBundle);
        }
    }

    return;
}




ceParcelTrackerRetryEventSink.prototype = new ceAbstractProgressListener();



ceParcelTrackerRetryEventSink.prototype.constructor = ceParcelTrackerRetryEventSink;


ceParcelTrackerRetryEventSink.superclass  = ceAbstractProgressListener.prototype;



ceParcelTrackerRetryEventSink.prototype.QueryInterface = function(iid)
{
    if(iid.equals(Interfaces.nsITimerCallback) ||
       iid.equals(Interfaces.nsISupports)       )
    {
        return this;
    }
    
    throw Results.NS_NOINTERFACE;
}



ceParcelTrackerRetryEventSink.prototype.notify = function(timerObject)
{
    if(TransferManager.waitingTransferCount <= 0)
    {
        return;
    }
    
    var transferMetaInfo = TransferManager.transferMetaInfo;
    if(!transferMetaInfo)
    {
        return;
    }
    
    var transferEnumerator = transferMetaInfo.enumerateResources();
    if(!transferEnumerator)
    {
        return;
    }

    var currentDate = new Date();
    var timeNow     = currentDate.getTime();
    var isConnected = false;
    
    var networkStatus = NetizenPropertyCache.getPropertyValue(NETIZEN_NETWORK_STATUS_PROPERTY);
    if(networkStatus)
    {
        if(networkStatus == NETIZEN_NETWORK_STATUS_CONNECTED ||
           networkStatus == NETIZEN_NETWORK_STATUS_PINGING    )
        {
            isConnected = true;
        }
    }
    
    while(transferEnumerator.hasMoreElements())
    {
        var metaInfo = transferEnumerator.getNext();
        if(!metaInfo)
        {
            break;
        }
        
        var transferInfoEntry = do_QueryInterface(metaInfo, Interfaces.ceITransferInfoEntry);
        if(transferInfoEntry)
        {
            this.updateRetryStatusForEntry(transferInfoEntry, timeNow, isConnected);
        }
    }
    
    return;
}


ceParcelTrackerRetryEventSink.prototype.updateRetryStatusForEntry = function(transferInfoEntry,
                                                                             timeNow          ,
                                                                             isConnected      )
{
    if(transferInfoEntry.transferType   != Interfaces.ceITransferInfo.TRANSFER_TYPE_DOWNLOAD)
    {
        return;
    }
    
    if(transferInfoEntry.transferStatus != Interfaces.ceITransferInfo.TRANSFER_RESOURCE_NOTFOUND &&
       transferInfoEntry.transferStatus != Interfaces.ceITransferInfo.TRANSFER_FAILED            &&
       transferInfoEntry.transferStatus != Interfaces.ceITransferInfo.TRANSFER_INTERRUPTED        )
    {
        return;
    }
     
    var metaInfoName  = transferInfoEntry.metaInfoName;
    var parcelElement = this.documentRef.getElementById(metaInfoName);
    if(!parcelElement)
    {
        return;
    }
    
    var progressElement = parcelElement.getElementsByAttribute("progressid", metaInfoName);
    if(progressElement[0])
    {        
        try
        {
            var transferInfo = TransferManager.getTransferInfo(transferInfoEntry.descriptor);
            if(transferInfo)
            {
                var timeLeft = ((transferInfo.nextRetryTime / 1000) - timeNow);
                if(timeLeft >= 0)
                {
                    progressElement[0].setAttribute("retrytimer", this.formatTimeInfo(timeLeft / 1000));
                }
            }
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }

        progressElement[0].setAttribute("isconnected", (isConnected == true ? "1" : "0"));
    }
    
    return;
}
