/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2005 Civil Engines Research.  All Rights 
 * Reserved.  
 *
 * Portions created by Netscape Copyright (C) 1998 Netscape 
 * Communications Corporation. All Rights Reserved.
 *
 * Portions created by the Mozilla Foundation are Copyright (C) 
 * 2003 Mozilla Foundation. All Rights Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/

const PARCEL_MANAGER_CONTRACTID              = "@civilengines.com/apps/parcel-manager;1";
const PARCEL_TRANSFERMANAGER_CONTRACTID      = "@civilengines.com/runtime/transfer-manager;1";
const PARCEL_INFO_EVENT_FOLLOWLINK           = "urn:parcel:info:event:link";
const PARCEL_INFO_EVENT_EXECOMMAND           = "urn:parcel:info:event:command";


var   ParcelManager                          = null;
var   TransferManager                        = null;


function loadApplication(loadRequestURL, isHistoryView)
{
    var applicationParams = loadRequestURL.getQueryParameters();
    
    ParcelDetailsView.loadServices(applicationParams);
    ParcelDetailsView.loadControllers();
    ParcelDetailsView.loadHandlers();
    ParcelDetailsView.loadStringBundles();
    ParcelDetailsView.loadApplicationView(applicationParams, isHistoryView);

    return;
}



function unloadApplication(unloadRequestURL)
{
    if(TransferManager)
    {
        try
        {
            TransferManager.saveTransferState();
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }
        
    }
    
    ParcelDetailsView.unloadHandlers();
    ParcelDetailsView.unloadControllers();
    ParcelDetailsView.unloadServices(unloadRequestURL.getQueryParameters());

    return;
}



var ParcelDetailsView =
{
    loadServices : function(applicationParams)
    {
        var applicationMode = applicationParams.getParameterValue(NETIZEN_CONTENT_MODE_PROPERTY);
        if(applicationMode)
        {
            ParcelModeSelector.onModeSelect(applicationMode);
        }
        
        NetizenNetworkStatusObserver.loadObservers();

        ParcelNetworkAlertObserver.loadObservers();

        ParcelTransferObserver.loadObservers();
        
        try
        {
            TransferManager = do_GetService(PARCEL_TRANSFERMANAGER_CONTRACTID, 
                                            Interfaces.ceITransferManager    );
            
            ParcelManager   = do_GetService(PARCEL_MANAGER_CONTRACTID         , 
                                            Interfaces.ceIParcelTrackerService);
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }

        /* Set the transfer listener */
        var activeTransfers         = TransferManager.activeTransfers;
        var progressDetailsListener = new ceParcelDetailedProgressListener(document, activeTransfers);
        if(progressDetailsListener)
        {
            TransferManager.progressListener = progressDetailsListener;
        }
        
        return;
    },


    unloadServices : function(applicationParams)
    {
        ParcelTransferObserver.unloadObservers();

        ParcelNetworkAlertObserver.unloadObservers();

        NetizenNetworkStatusObserver.unloadObservers();
        
        var applicationMode = applicationParams.getParameterValue(NETIZEN_CONTENT_MODE_PROPERTY);
        if(applicationMode)
        {
            ParcelModeSelector.onModeUnselect(applicationMode);
        }
        
        return;
    },
    

    loadControllers : function()
    {
        top.controllers.appendController(ParcelDetailsViewController);        
        ParcelDetailsViewController.onCommandUpdate();
        
        return;
    },


    unloadControllers : function()
    {
        top.controllers.removeController(ParcelDetailsViewController);            
        
        return;
    },


    loadHandlers : function()
    {
        var parcelDetailsElement = document.getElementById("parcelDetailsSummary");
        if(parcelDetailsElement)
        {
            ParcelInfoEventHandler.registerListener(parcelDetailsElement);
        }

        return;
    },


    unloadHandlers : function()
    {
        var parcelDetailsElement = document.getElementById("parcelDetailsSummary");
        if(parcelDetailsElement)
        {
            ParcelInfoEventHandler.unregisterListener(parcelDetailsElement);
        }
        
        return;
    },


    loadStringBundles : function()
    {
        var parcelStringBundle = document.getElementById("parcelStringBundle");
        if(!parcelStringBundle)
        {
            return;
        }

        this.uploadCountSingle   = parcelStringBundle.getString("parcel.details.upload.count.single");
        this.uploadCountMulti    = parcelStringBundle.getString("parcel.details.upload.count.multi");
        this.downloadCountSingle = parcelStringBundle.getString("parcel.details.download.count.single");
        this.downloadCountMulti  = parcelStringBundle.getString("parcel.details.download.count.multi");
        
        return;
    },

    
    loadApplicationView : function(applicationParams, isHistoryView) 
    {
        var viewid             = applicationParams.getParameterValue(NETIZEN_CONTENT_VIEWID_PROPERTY);
        var trackerResourceURI = applicationParams.getParameterValue(NETIZEN_CONTENT_RESOURCE_PROPERTY);
        if(!trackerResourceURI)
        {
            return;
        }
        
        var parcelDetailsSummary = document.getElementById("parcelDetailsSummary");
        if(parcelDetailsSummary)
        {
            parcelDetailsSummary.setAttribute("trackerId", trackerResourceURI);
        }
        
        var referringURI       = applicationParams.getParameterValue(NETIZEN_CONTENT_REFERRER_PROPERTY);
        if(!referringURI)
        {
            return;
        }
        
        var referrerURL        = new ceStandardURL(referringURI);
        var referrerURISpec    = referrerURL.baseSpec;
        var deleteButton       = document.getElementById("parcelDeleteButton");
        if(deleteButton)
        {
            deleteButton.setAttribute("referrer", referrerURISpec);
        }

        try
        {
            var trackerInfo        = ParcelManager.trackerMetaInfo;
            var trackerSummaryInfo = ParcelManager.trackerSummaryInfo;
            if(!trackerInfo || !trackerSummaryInfo)
            {
                return;
            }
            
            
            var trackerResource = trackerInfo.getResource(trackerResourceURI);
            var trackerEntry    = do_QueryInterface(trackerResource                 , 
                                                    Interfaces.ceIParcelTrackerEntry);
            
            
            var summaryResource = trackerSummaryInfo.getResource(trackerResourceURI);
            var summaryEntry    = do_QueryInterface(summaryResource                   , 
                                                    Interfaces.ceIParcelTrackerSummary);
            
            this.loadSummaryView(trackerEntry, summaryEntry);
            
            if(TransferManager.canCleanUp)
            {
                TransferManager.cleanUp();
            }

            var transferList  = TransferManager.activeTransfers;
            if(!transferList)
            {
                return;
            }
            
            this.loadActivityView(trackerResourceURI, transferList);
            
            if(this.shouldLoadArchiveTree(trackerResourceURI))
            {
                this.loadArchiveTree(trackerEntry);
            }
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }

        ParcelDetailsViewController.onCommandUpdate();
        
        return;
    },

    
    loadSummaryView : function(trackerEntry, summaryEntry)
    {
        var parcelDetailsElement = document.getElementById("parcelDetailsSummary");
        if(!parcelDetailsElement)
        {
            return;
        }
        
        if(trackerEntry)
        {
            var activityTarget = new ceStandardURL(trackerEntry.activityTarget.spec);
            var parcelName     = activityTarget.getParameterValue("parc");                
            
            parcelDetailsElement.setAttribute("name"  , parcelName);
            parcelDetailsElement.setAttribute("folder", trackerEntry.localPath);

            var trackerMetaInfoEntry = do_QueryInterface(trackerEntry               , 
                                                         Interfaces.ceIMetaInfoEntry);
            if(trackerMetaInfoEntry)
            {
                parcelDetailsElement.setAttribute("resource", trackerMetaInfoEntry.descriptor);
            }
        }
        
        if(summaryEntry)
        {
            parcelDetailsElement.setAttribute("size"   , summaryEntry.contentSummary);
            parcelDetailsElement.setAttribute("created", summaryEntry.creationDate  );
            parcelDetailsElement.setAttribute("expires", summaryEntry.expirationDate);
        }
        
        return;
    },


    loadActivityView : function(trackerResourceURI, transferList)
    {
        if(transferList)
        {
            this.createTooltipText(transferList);
            this.loadActivityGrids(trackerResourceURI, transferList);
        }
                
        return;
    },
    

    createTooltipText : function(transferList)
    {
        Assert.condition(transferList, "NULL or emptry active transfer list");
        
        var transferTooltipSet = document.getElementById("transferActivityTooltipSet");
        var transferCount      = transferList.Count();
        
        for(var index = 0; index < transferCount; index++)
        {
            var transferInfo = transferList.QueryElementAt(index, Interfaces.ceITransferInfo);
            if(!transferInfo)
            {
                continue;
            }

            var transferInfoEntry = do_QueryInterface(transferInfo, Interfaces.ceITransferInfoEntry);
            if(!transferInfoEntry)
            {
                continue;
            }

            var remoteURL = transferInfoEntry.remoteFileURL;
            if(!remoteURL)
            {
                continue;
            }
            
            var remoteURLSpec = remoteURL.spec;
            var tooltip       = document.createElement("tooltip");
            var hostLabel     = document.createElement("label");
            var statusLabel   = document.createElement("label");

            tooltip.id = remoteURLSpec;

            tooltip.setAttribute("orient"    , "vertical");
            tooltip.setAttribute("noautohide", "true");
            
            statusLabel.setAttribute("hostid"  , transferInfo.target.spec);
            statusLabel.setAttribute("statusid", transferInfo.target.spec);
            
            tooltip.appendChild(hostLabel);
            tooltip.appendChild(statusLabel);

            transferTooltipSet.appendChild(tooltip);
        }
        
        return;
    },
    

    loadActivityGrids : function(trackerResourceURI, transferList)
    {
        Assert.condition(trackerResourceURI, "NULL tracker resource URI");
        Assert.condition(transferList      , "NULL or emptry active transfer list");

        var uploadCount   = 0;
        var downloadCount = 0;
        var transferCount = transferList.Count();

        for(var index = 0; index < transferCount; index++)
        {
            var transferInfo = transferList.QueryElementAt(index, Interfaces.ceITransferInfo);
            if(!transferInfo)
            {
                continue;
            }

            var transferInfoEntry = do_QueryInterface(transferInfo, Interfaces.ceITransferInfoEntry);
            if(!transferInfoEntry)
            {
                continue;
            }
            
            if(transferInfoEntry.metaInfoName == trackerResourceURI)
            {
                if(transferInfo.transferType == Interfaces.ceITransferInfo.TRANSFER_TYPE_DOWNLOAD)
                {
                    downloadCount++;
                }
                else if(transferInfo.transferType == Interfaces.ceITransferInfo.TRANSFER_TYPE_UPLOAD)
                {
                    uploadCount++;
                }
            }
        }

        var transferDatasource = do_QueryInterface(TransferManager.transferMetaInfo,
                                                   Interfaces.ceIRDFMetaInfo       );

        if(uploadCount > 0)
        {
            var uploadSelectionRule = document.getElementById("uploadSelectionRule");
            if(uploadSelectionRule)
            {
                uploadSelectionRule.setAttribute("object", trackerResourceURI);            
            }
            
            var uploadCountString = (uploadCount > 1) ? 
                this.uploadCountMulti.substitute(uploadCount) : this.uploadCountSingle;

            var uploadActivityGrid  = document.getElementById("uploadActivityGrid");
            if(uploadActivityGrid)
            {
                uploadActivityGrid.hidden = false;
                uploadActivityGrid.database.AddDataSource(transferDatasource.dataSource);
                uploadActivityGrid.builder.rebuild();            

                uploadActivityGrid.setAttribute("activity", uploadCountString);
            }
        }
        
        if(downloadCount > 0)
        {
            var downloadSelectionRule = document.getElementById("downloadSelectionRule");
            if(downloadSelectionRule)
            {
                downloadSelectionRule.setAttribute("object", trackerResourceURI);            
            }
            
            var downloadCountString = (downloadCount > 1) ? 
                this.downloadCountMulti.substitute(downloadCount) : this.downloadCountSingle;

            var downloadActivityGrid  = document.getElementById("downloadActivityGrid");
            if(downloadActivityGrid)
            {
                if(uploadCount > 0)
                {
                    downloadActivityGrid.setAttribute("hideheader", "true" );
                }
                
                downloadActivityGrid.hidden = false;
                downloadActivityGrid.database.AddDataSource(transferDatasource.dataSource);
                downloadActivityGrid.builder.rebuild();

                downloadActivityGrid.setAttribute("activity", downloadCountString);
            }
        }
        
        if(uploadCount || downloadCount)
        {
            var transferActivityBox = document.getElementById("transferActivityBox");
            if(transferActivityBox)
            {
                transferActivityBox.hidden = false;
            }
        }
        
        return;
    },

        
    shouldLoadArchiveTree : function(trackerResourceURI)
    {
        var transferMetaInfo = TransferManager.transferMetaInfo;
        if(!transferMetaInfo)
        {
            return false;
        }
        
        var transferEnumerator = transferMetaInfo.enumerateResources();
        if(!transferEnumerator)
        {
            return false;
        }
        
        while(transferEnumerator.hasMoreElements())
        {
            var metaInfo = transferEnumerator.getNext();
            if(!metaInfo)
            {
                break;
            }
                        
            var transferInfoEntry = do_QueryInterface(metaInfo, Interfaces.ceITransferInfoEntry);
            if(transferInfoEntry)
            {           
                if(transferInfoEntry.metaInfoName   == trackerResourceURI                                &&
                   transferInfoEntry.transferType   == Interfaces.ceITransferInfo.TRANSFER_TYPE_DOWNLOAD &&
                   transferInfoEntry.transferStatus != Interfaces.ceITransferInfo.TRANSFER_FINISHED       )
                {
                    return false;
                }
            }
        }

        return true;
    },
    

    loadArchiveTree : function(trackerEntry)
    {
        var archiveTree = document.getElementById("parcelDetailsArchiveView");
        if(archiveTree && trackerEntry)
        {
            try
            {
                var archiveURISpec = ParcelManager.getURLSpecFromLocalPath(trackerEntry);
                if(archiveURISpec)
                {
                    archiveTree.setAttribute("ref", archiveURISpec);
                }
            
                archiveTree.rebuildContent();
                archiveTree.toggleFolderOpenState(0);
            }
            catch(exception)
            {
                archiveTree.setAttribute("ref", "null");
                Syslog.logException(exception);
            }
        }

        return;
    },


    unloadArchiveTree : function()
    {
        var archiveTree = document.getElementById("parcelDetailsArchiveView");
        if(archiveTree)
        {
            archiveTree.setAttribute("ref", "null");
        }

        return;
    },


    closeArchive : function()
    {
        var archiveTree = document.getElementById("parcelDetailsArchiveView");
        if(archiveTree)
        {
            try
            {
                var parcelArchive = archiveTree.currentArchive;
                if(parcelArchive && !parcelArchive.isClosed())
                {
                    parcelArchive.close();
                }        
            }                
            catch(exception)
            {
                Syslog.logException();
            }            
        }

        return;
    }
};


/////////////////////////////////////////////////////////////////////////////////
//
// ceITransferProgressListener Implementation
//
/////////////////////////////////////////////////////////////////////////////////


function ceParcelDetailedProgressListener(documentRef, activeTransfers)
{
    if(documentRef)
    {
        this.documentRef         = documentRef;
        var transferStringBundle = this.documentRef.getElementById("transferStringBundle");
        if(transferStringBundle)
        {
            this.loadStringBundle(transferStringBundle);
        }     
    }

    if(activeTransfers)
    {
        this.initTransferInfoIndex(activeTransfers);
    }
    
    return;
}


ceParcelDetailedProgressListener.prototype = new ceAbstractProgressListener();

ceParcelDetailedProgressListener.prototype.constructor = ceParcelDetailedProgressListener;

ceParcelDetailedProgressListener.prototype.onProgressChange = function(webProgress         , 
                                                                       request             , 
                                                                       currentSelfProgress , 
                                                                       maxSelfProgress     , 
                                                                       currentTotalProgress, 
                                                                       maxTotalProgress    , 
                                                                       transferInfo        )
{
    if(!this.shouldUpdateProgress(transferInfo, currentTotalProgress, maxTotalProgress))
    {
        return;
    }
    
    var startingOffset = transferInfo.startingOffset;
    var transferRate   = 0;
    if(this.elapsedTime)
    {
        /* bytes/sec = currentTotalProgress[bytes] / (elapsedTime[ms] / 1000[ms_per_sec]) */
        transferRate = ((currentTotalProgress - startingOffset) * 1000) / this.elapsedTime;
    }
    
    /* Acquire the progress bar widget element to update */
    var transferDescriptor = transferInfo.target.spec;
    var transferElement    = this.documentRef.getElementsByAttribute("xferid", transferDescriptor)[0];
    
    /* 1. Update progress bar information */
    var transferPercent    = 0;
    if(maxTotalProgress > 0)
    {
        transferPercent = Math.floor((currentTotalProgress * 100.0)/maxTotalProgress);
        if(transferPercent > 100)
        {
            transferPercent = 100;
        }
        
        var progressPercentString = this.replaceInsert(this.statusFormatProgress, 
                                                       1                        ,
                                                       transferPercent          );            
        /* 1.1 Update progress bar percentage */
        if(transferElement)
        {
            transferElement.setAttribute("progress"    , transferPercent      );
            transferElement.setAttribute("progressmode", "normal"             );
            transferElement.setAttribute("progresstext", progressPercentString);
        }
    }
    else
    {
        transferPercent = -1;
        if(transferElement)
        {
            transferElement.setAttribute("progressmode", "undetermined");
        }
    }
    
    /* 2. Update progress notification */
    var progressKBytes = parseInt(currentTotalProgress/1024 + 0.5);
    var totalKBytes    = parseInt(maxTotalProgress/1024 + 0.5);
    
    var progressString = this.formatProgressInfo(progressKBytes, totalKBytes);
    if(transferElement)
    {
        transferElement.setAttribute("progressinfo", progressString);
    }
    
    var statusMessage      = this.replaceInsert(this.statusFormatTooltip, 1, progressString);
    var transferRateString = null;
    
    /* 3. Update transfer rate */        
    if(transferRate)
    {
        /* Convert to KBytes/sec from bytes/sec */
        var transferRateKBSec = transferRate / 1024;
        transferRateKBSec     = parseInt(transferRateKBSec * 10 + 0.5);
        if(transferRateKBSec != this.priorRate)
        {
            if(this.rateChanges++ == this.rateChangeLimit)
            {
                this.priorRate   = transferRateKBSec;
                this.rateChanges = 0;
            }
            else
            {
                transferRateKBSec = this.priorRate;
            }   
        }
        else
        {
            this.rateChanges = 0;
        }
        
        var fraction      = transferRateKBSec % 10;
        transferRateKBSec = parseInt((transferRateKBSec - fraction) / 10);
        if(transferRateKBSec < 100)
        {
            transferRateKBSec += "." + fraction;
        }
        
        transferRateString = this.replaceInsert(this.statusFormatRate, 1, transferRateKBSec);
    }
    else
    {
        transferRateString = this.replaceInsert(this.statusFormatRate, 1, "??.?"); 
    }
    
    
    /* 4. Update time remaining */
    if(transferRate && (maxTotalProgress > 0))
    {
        var timeRemaining  = (maxTotalProgress - currentTotalProgress) / transferRate;
        timeRemaining      = parseInt(timeRemaining + 0.5);
        
        var timeLeftString = this.formatTimeInfo(timeRemaining);
        if(timeLeftString)
        {
            timeLeftString = timeLeftString + " " + this.remaining;
        }
        else
        {
            timeLeftString = this.noneRemaining;
        }
        
        statusMessage      = this.replaceInsert(statusMessage, 2, timeLeftString);
    }
    else
    {
        statusMessage      = this.replaceInsert(statusMessage, 2, this.unknownFileSize);
    }
    
    
    /* 5. Set the status message */
    if(transferElement)
    {
        transferElement.setAttribute("rate", transferRateString);
    }


    /* 6. Set the tooltip status text */
    var statusLabel = document.getElementsByAttribute("statusid", transferDescriptor)[0];
    if(statusLabel)
    {
        statusLabel.setAttribute("value", statusMessage);
    }
    
    /* 7. Set the transfer rate information */
    var rateLabel = document.getElementsByAttribute("rateid", transferDescriptor)[0];
    if(rateLabel)
    {
        rateLabel.setAttribute("value", transferRateString);
    }
    
    return;
}



ceParcelDetailedProgressListener.prototype.onStateChange = function(webProgress , 
                                                                    request     , 
                                                                    stateFlags  , 
                                                                    statusCode  ,
                                                                    transferInfo)
{
    if(stateFlags & Interfaces.nsIWebProgressListener.STATE_START && transferInfo)
    {
        var transferInfoEntry  = do_QueryInterface(transferInfo, Interfaces.ceITransferInfoEntry);
        if(!transferInfoEntry)
        {
            return;
        }

        try
        {
            if(TransferManager.canCleanUp)
            {
                TransferManager.cleanUp();
            }
        
            var trackerResourceURI   = transferInfoEntry.metaInfoName;
            var transferList         = TransferManager.activeTransfers;
            
            var parcelDetailsSummary = document.getElementById("parcelDetailsSummary");
            if(!parcelDetailsSummary)
            {
                return;
            }
            
            var trackerId = parcelDetailsSummary.getAttribute("trackerId");
            if(trackerResourceURI != trackerId)
            {
                return;
            }
            
            NetizenPropertyCache.cachePropertyValue(trackerResourceURI, transferInfo);
            
            ParcelDetailsView.loadActivityView(trackerResourceURI, transferList);
        }
        catch(exception)
        {
            Syslog.logException(exception);
            return;
        }
        
    }
    else if(stateFlags & Interfaces.nsIWebProgressListener.STATE_STOP && transferInfo)
    {
        if(transferInfo.transferType == Interfaces.ceITransferInfo.TRANSFER_TYPE_DOWNLOAD)
        {
            try
            {
                var trackerInfo = ParcelManager.trackerMetaInfo;
                if(!trackerInfo)
                {
                    return;
                }
                
                var transferInfoEntry  = do_QueryInterface(transferInfo                   , 
                                                           Interfaces.ceITransferInfoEntry);
                if(!transferInfoEntry)
                {
                    return;
                }
                
                var trackerResourceURI = transferInfoEntry.metaInfoName;
                if(!trackerResourceURI)
                {
                    return;
                }
                
                var parcelDetailsSummary = document.getElementById("parcelDetailsSummary");
                if(!parcelDetailsSummary)
                {
                    return;
                }
                
                var trackerId = parcelDetailsSummary.getAttribute("trackerId");
                if(trackerResourceURI != trackerId)
                {
                    return;
                }
                
                NetizenPropertyCache.cachePropertyValue(trackerResourceURI, transferInfo);
                
                var trackerResource = trackerInfo.getResource(trackerResourceURI);
                var trackerEntry    = do_QueryInterface(trackerResource                 , 
                                                        Interfaces.ceIParcelTrackerEntry);
                if(trackerEntry)
                {
                    if(ParcelDetailsView.shouldLoadArchiveTree(trackerResourceURI))
                    {
                        ParcelDetailsView.loadArchiveTree(trackerEntry);            
                    }
                    
                    ParcelDetailsViewController.onCommandUpdate();
                }
            }
            catch(exception)
            {
                Syslog.logException(exception);
                return;
            }
        }
    }
    
    return;
}





var ParcelDetailsViewController =
{
    supportsCommand : function(commandId)
    {
        if(commandId == "cmd_parcelDetailsViewSlip"      ||
           commandId == "cmd_parcelDetailsDelete"        ||
           commandId == "cmd_parcelDetailsConfirmDelete" ||
           commandId == "cmd_parcelDetailsUnpack"         )
        {      
            return true;
        }

        return false;
    },


    isCommandEnabled : function(commandId)
    {
        if(commandId == "cmd_parcelDetailsUnpack")
        {
            var archiveTree = document.getElementById("parcelDetailsArchiveView");
            if(!archiveTree)
            {
                return false;
            }
            
            var archiveResourceURI = archiveTree.getAttribute("ref");
            if(archiveResourceURI && archiveResourceURI != "null")
            {
                return true;
            }
        }

        if(commandId == "cmd_parcelDetailsViewSlip"     ||
           commandId == "cmd_parcelDetailsDelete"       ||
           commandId == "cmd_parcelDetailsConfirmDelete" )
        {      
            return true;
        }

        return false;
    },


    doCommand : function(commandId)
    {
        switch(commandId)
        {
          case "cmd_parcelDetailsViewSlip"      :
          {
              this.trackerViewPickupSlip();
              break;
          }
          case "cmd_parcelDetailsDelete"        :
          {
              this.deleteParcelResource();
              break;
          }
          case "cmd_parcelDetailsConfirmDelete" :
          {
              this.confirmDeleteParcelResource();
              break;
          }
          case "cmd_parcelDetailsUnpack"        :
          {
              this.unpackParcelResource();
              break;
          }
        }

        this.onCommandUpdate();
        return;
    },
    

    onCommandUpdate : function()
    {
        var viewSlipIsEnabled = this.isCommandEnabled("cmd_parcelDetailsViewSlip");
        var deleteIsEnabled   = this.isCommandEnabled("cmd_parcelDetailsDelete");
        var unpackIsEnabled   = this.isCommandEnabled("cmd_parcelDetailsUnpack");
        
        goSetCommandEnabled("cmd_parcelDetailsViewSlip", viewSlipIsEnabled);        
        goSetCommandEnabled("cmd_parcelDetailsDelete"  , deleteIsEnabled  );
        goSetCommandEnabled("cmd_parcelDetailsUnpack"  , unpackIsEnabled  );

        return;
    },



    trackerViewPickupSlip : function()
    {
        var parcelDetailsElement = document.getElementById("parcelDetailsSummary");
        if(!parcelDetailsElement)
        {
            return;
        }
        
        var trackerResourceURI   = parcelDetailsElement.getAttribute("resource");
        if(trackerResourceURI)
        {
            ParcelResourceController.loadPickupSlipDialog(trackerResourceURI);
        }
        
        return;
    },

    
    unpackParcelResource : function()
    {
        var parcelDetailsElement = document.getElementById("parcelDetailsSummary");
        if(!parcelDetailsElement)
        {
            return;
        }
        
        var trackerResourceURI   = parcelDetailsElement.getAttribute("resource");
        if(trackerResourceURI)
        {
            ParcelResourceController.unpackParcel(trackerResourceURI);
        }
        
        return;
    },


    deleteParcelResource : function()
    {
        var parcelDetailsElement = document.getElementById("parcelDetailsSummary");
        if(!parcelDetailsElement)
        {
            return;
        }
        
        var trackerResourceURI   = parcelDetailsElement.getAttribute("resource");
        if(trackerResourceURI)
        {
            Syslog.logMessage("deleteParcelResource => " + trackerResourceURI);

            try
            {                
                ParcelManager.removeTrackerEntry(trackerResourceURI);
            
                ParcelDetailsView.closeArchive();
                ParcelDetailsView.unloadArchiveTree();
            }
            catch(exception)
            {
                Syslog.logException(exception);
            }
        }

        var deleteButton = document.getElementById("parcelDeleteButton");
        if(!deleteButton)
        {
            return;
        }

        var referrer = deleteButton.getAttribute("referrer");
        if(referrer)
        {
            goDoCommand("cmd_netizenBack");
        }
        
        return;
    },


    confirmDeleteParcelResource : function()
    {
        var parcelDetailsElement = document.getElementById("parcelDetailsSummary");
        if(!parcelDetailsElement)
        {
            return;
        }
        
        var trackerResourceURI   = parcelDetailsElement.getAttribute("resource");
        if(trackerResourceURI)
        {
            window.openDialog("chrome://netizen/content/parcel/parcelConfirmDelete.xul",
                              "confirmDeleteEntryDialog"                               ,
                              "centerscreen,modal,chrome,resizable=no"                 ,
                              1                                                        );  
        }

        return;
    },


    checkDeleteConfirm : function(event)
    {
        if(event && event.altKey)
        {
            if(this.isCommandEnabled("cmd_parcelDetailsDelete"))
            {
                this.deleteParcelResource();
            }
        }
        else
        {
            if(this.isCommandEnabled("cmd_parcelDetailsConfirmDelete"))
            {
                this.confirmDeleteParcelResource();
            }
        }
        
        return;
    },


    cancelTransfer : function(eventInfo)
    {
        var transferElement = eventInfo.target;
        if(!transferElement)
        {
            return;
        }

        var transferTargetURISpec = transferElement.getAttribute("xferid");
        if(transferTargetURISpec)
        {
            try
            {
                TransferManager.cancelTransfer(transferTargetURISpec);
            }
            catch(exception)
            {
                Syslog.logException(exception);
            }
        }
        
        return;
    }
};




var ParcelInfoEventHandler =
{
    registerListener  : function(targetElement)
    {
        if(targetElement)
        {
            targetElement.addEventListener(PARCEL_INFO_EVENT_FOLLOWLINK, 
                                           this.onShowParcelFolder     , 
                                           false                       );

            targetElement.addEventListener(PARCEL_INFO_EVENT_EXECOMMAND, 
                                           this.onShowParcelFolder     , 
                                           false                       );
        }

        return;
    },


    unregisterListener  : function(targetElement)
    {
        if(targetElement)
        {
            targetElement.removeEventListener(PARCEL_INFO_EVENT_FOLLOWLINK, 
                                              this.onShowParcelFolder     , 
                                              false                       );
            
            targetElement.removeEventListener(PARCEL_INFO_EVENT_EXECOMMAND, 
                                              this.onShowParcelFolder     , 
                                              false                       );        
        }
        
        return;
    },


    onShowParcelFolder : function(eventInfo)
    {
        var parcelDetailsSummary = document.getElementById("parcelDetailsSummary");
        if(!parcelDetailsSummary)
        {
            return;
        }
        
        var resourceURLSpec = parcelDetailsSummary.getAttribute("resource");
                
        try
        {
            var trackerInfo     = ParcelManager.trackerMetaInfo;
            if(!trackerInfo)
            {
                return;
            }
            
            var trackerResource = trackerInfo.getResource(resourceURLSpec);
            var trackerEntry    = do_QueryInterface(trackerResource                 , 
                                                    Interfaces.ceIParcelTrackerEntry);
            
            var resourceFile    = ParcelManager.getFileFromLocalPath(trackerEntry);
            if(!resourceFile)
            {
                return;
            }

            resourceFile = do_QueryInterface(resourceFile, Interfaces.nsILocalFile);
            if(!resourceFile)
            {
                return;
            }

            resourceFile.reveal();
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }
                
        return;
    }
};


