/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2005 Civil Engines Research.  All Rights 
 * Reserved.  
 *
 * Portions created by Netscape Copyright (C) 1998 Netscape 
 * Communications Corporation. All Rights Reserved.
 *
 * Portions created by the Mozilla Foundation are Copyright (C) 
 * 2003 Mozilla Foundation. All Rights Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/

const PARCEL_TRACKER_CONTRACTID              = "@civilengines.com/apps/parcel-manager;1";
const PARCEL_METAINFOLOADER_CONTRACTID       = "@civilengines.com/metainf/loader-service;1";

const PARCEL_TEXT_NEWLINES                   = "\r\n\r\n";
const PARCEL_TEXT_NEWLINE                    = "\r\n";

var   ParcelManager                          = null;
var   MetaInfoService                        = null;



function ceParcelPickupSlipInfo(trackerResourceURI)
{
    try
    {
        MetaInfoService = do_GetService(PARCEL_METAINFOLOADER_CONTRACTID,
                                        Interfaces.ceIMetaInfoService   );
        
        ParcelManager   = do_GetService(PARCEL_TRACKER_CONTRACTID         , 
                                        Interfaces.ceIParcelTrackerService);
    }
    catch(exception)
    {
        Syslog.logException(exception);
    }

    if(trackerResourceURI)
    {
        this.loadPickupSlipInfo(trackerResourceURI);
    }
    
    return;
}


ceParcelPickupSlipInfo.prototype = 
{
    parcelFileName     : null,
    pickupSlipName     : null,
    stubDescriptorSpec : null,
    metaInfoPointer    : null,

    
    loadStringBundle : function(stringBundle)
    {
        if(!stringBundle)
        {
            return;
        }
        
        this.expirationString   = stringBundle.getString("parcel.dialog.viewslip.expiration");
        this.sizeFormatKB       = stringBundle.getString("parcel.status.sizeFormatKB");
        this.sizeFormatMB       = stringBundle.getString("parcel.status.sizeFormatMB");
        this.sizeFormatGB       = stringBundle.getString("parcel.status.sizeFormatGB");
        
        this.subjectHeader      = stringBundle.getString("parcel.pickupslip.email.subject");        
        this.instructionHeader  = stringBundle.getString("parcel.pickupslip.email.text1");
        this.instruction1String = stringBundle.getString("parcel.pickupslip.email.text2");
        this.instruction2String = stringBundle.getString("parcel.pickupslip.email.text3");
        this.instruction3String = stringBundle.getString("parcel.pickupslip.email.text4");
        
        return;
    },


    formatSizeString : function(sizeInBytes)
    {
        var sizeInKBytes = parseInt(sizeInBytes/1024 + 0.5);
        var sizeInMBytes = parseInt(sizeInKBytes/1024);
        var sizeHasMB    = parseInt(sizeInKBytes/1024) > 0;
        var sizeHasGB    = parseInt(sizeInMBytes/1024) > 0;
        var sizeFormat   = "";

        if(sizeHasGB)
        {
            sizeFormat = this.sizeFormatGB.substitute((sizeInMBytes / 1024).toFixed(2));
        }
        else if(sizeHasMB)
        {
            sizeFormat = this.sizeFormatMB.substitute((sizeInKBytes / 1024).toFixed(1));
        }
        else
        {
            if(sizeInKBytes == 0 && sizeInBytes > 0)
            {
                sizeInKBytes = 1;
            }
            
            sizeFormat = this.sizeFormatKB.substitute((sizeInKBytes));
        }
        
        return sizeFormat;
    },
    

    loadPickupSlipInfo : function(trackerResourceURI)
    {
        var trackerInfo        = ParcelManager.trackerMetaInfo;
        if(!trackerInfo)
        {
            return;
        }

        var trackerResource    = trackerInfo.getResource(trackerResourceURI);
        var trackerEntry       = do_QueryInterface(trackerResource                 , 
                                                   Interfaces.ceIParcelTrackerEntry);
        if(!trackerEntry)
        {
            Assert.condition(trackerEntry, "Pickup Slip Entry Unavailable");
            return;
        }
        
        var activityTargetURL   = new ceStandardURL(trackerEntry.activityTarget.spec);

        this.parcelFileName     = activityTargetURL.getParameterValue("parc");
        this.pickupSlipName     = this.parcelFileName.replace(/.parc/gi, ".slip");
        this.stubDescriptorSpec = activityTargetURL.baseSpec;

        try
        {
            var stubInfo = MetaInfoService.loadMetadata(this.stubDescriptorSpec);
            if(!stubInfo)
            {
                Assert.condition(stubInfo, "Stub Info Load Failure");
                return;
            }

            this.metaInfoPointer = stubInfo.getPointer();
            if(!this.metaInfoPointer)
            {
                Assert.condition(this.metaInfoPointer, "MetaInfoPointer Load Failure");
                return;
            }
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }

        return;
    },


    emailPickupSlipText : function()
    {
        try
        {
            var stubInfo = MetaInfoService.loadMetadata(this.stubDescriptorSpec);
            if(!stubInfo)
            {
                Assert.condition(stubInfo, "Stub Info Load Failure");
                return;
            }

            var resourceEnumerator = stubInfo.enumerateResources();
            var resourceInfo       = null;
            if(resourceEnumerator)
            {
                while(resourceEnumerator.hasMoreElements())
                {
                    resourceInfo = resourceEnumerator.getNext();
                    if(resourceInfo)
                    {
                        break;
                    }
                }
            }
            
            var mailtoURL   = "mailto:?";
            var subjectText = this.subjectHeader;
            var parcelName  = null;
            var parcelSize  = 0;

            resourceInfo    = do_QueryInterface(resourceInfo, Interfaces.ceIStubInfoEntry);
            if(resourceInfo)
            {
                parcelName = resourceInfo.filename;
                parcelSize = resourceInfo.filesize;
            }
            
            var pickupSlipContent = new Object();
            this.metaInfoPointer.flushToString(pickupSlipContent);

            if(parcelName)
            {
                subjectText = subjectText.substitute(parcelName, this.formatSizeString(parcelSize));
                mailtoURL  += "subject=" + encodeURIComponent(subjectText);
            }
             
            if(pickupSlipContent.value)
            {
                mailtoURL += "&body="                                                            + 
                              encodeURIComponent(PARCEL_TEXT_NEWLINES)                           + 
                              encodeURIComponent(pickupSlipContent.value + PARCEL_TEXT_NEWLINES) + 
                              encodeURIComponent(this.instructionHeader  + PARCEL_TEXT_NEWLINE ) +
                              encodeURIComponent(this.instruction1String + PARCEL_TEXT_NEWLINE ) +
                              encodeURIComponent(this.instruction2String + PARCEL_TEXT_NEWLINE ) +
                              encodeURIComponent(this.instruction3String + PARCEL_TEXT_NEWLINE );
            }
            
            NetizenWindowController.loadExternalURL(mailtoURL);
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }
        

        return;
    },


    savePickupSlipTextToFile : function()
    {
        var parcelManager        = do_QueryInterface(ParcelManager, Interfaces.ceIParcelManager);
        var destinationDirectory = parcelManager.defaultDirectory;
        if(!destinationDirectory)
        {
            Assert.condition(destinationDirectory, "NULL default directory");
            return;
        }

        try
        {
            var destinationFile = do_QueryInterface(destinationDirectory, Interfaces.nsIFile);
            
            destinationFile.append(this.pickupSlipName);
 
            this.metaInfoPointer.flush(destinationDirectory);
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }
        
        return;
    }    
};



var ParcelSlipDetailsDialog =
{
    trackerResourceURI    : null,
    pickupSlipInfo        : null,
    windowResizeIncrement : 62  ,

    loadDialog : function(eventInfo)
    {
        this.trackerResourceURI = window.arguments[0];
        if(!this.trackerResourceURI)
        {
            Assert.condition(this.trackerResourceURI, "Pickup Slip Resource URL Unavailable");
            return;
        }

        this.pickupSlipInfo    = new ceParcelPickupSlipInfo(this.trackerResourceURI);
        if(!this.pickupSlipInfo)
        {
            Assert.condition(this.pickupSlipInfo, "Pickup Slip Resource URL Unavailable");
            return;
        }
        
        var dialogStringBundle = document.getElementById("parcelStringBundle");
        if(dialogStringBundle)
        {
            this.pickupSlipInfo.loadStringBundle(dialogStringBundle);
        }     
        
        this.loadDialogContent(this.trackerResourceURI);
        
        return;
    },
    

    unloadDialog : function(eventInfo)
    {
        this.trackerResourceURI = null;
        this.pickupSlipInfo     = null;
        
        return;
    },

    
    loadDialogContent : function(trackerResourceURI)
    {
        if(this.pickupSlipInfo.parcelFileName)
        {
            var parcelNameLabel = document.getElementById("parcelNameLabel");
            if(!parcelNameLabel)
            {
                return;
            }

            parcelNameLabel.setAttribute("value", this.pickupSlipInfo.parcelFileName);
        }
        
        var expirationLabel = document.getElementById("parcelExpirationLabel");
        if(!expirationLabel)
        {
            return;
        }

        var trackerSummaryInfo = ParcelManager.trackerSummaryInfo;
        if(!trackerSummaryInfo)
        {
            return;
        }
                
        var summaryResource = trackerSummaryInfo.getResource(trackerResourceURI);
        var summaryEntry    = do_QueryInterface(summaryResource                   , 
                                                Interfaces.ceIParcelTrackerSummary);        
        if(summaryEntry)
        {
            var expirationString = this.pickupSlipInfo.expirationString;
            var expirationDate   = expirationString.substitute(summaryEntry.expirationDate);
            expirationLabel.setAttribute("value", expirationDate);
        }


        try
        {
            var slipContentArea = document.getElementById("pickupSlipContentArea");
            if(!slipContentArea)
            {
                return;
            }
        
            var metaInfoPointer = this.pickupSlipInfo.metaInfoPointer;
            if(metaInfoPointer)
            {
                metaInfoPointer.flushToString(slipContentArea);
            }
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }

        return;
    },


    toggleSelectionContent : function(selectionContent)
    {
        var dialogCopyDescription = document.getElementById("dialogCopySelectionDescription");
        if(dialogCopyDescription)
        {
            dialogCopyDescription.collapsed = (selectionContent == "copy") ? false : true;
        }
        

        var dialogSaveDescription = document.getElementById("dialogSavedPickupSlipDescription");
        if(dialogSaveDescription)
        {
            dialogSaveDescription.collapsed = (selectionContent == "save") ? false : true;
        }
        

        var dialogEmailDescription = document.getElementById("dialogEmailPickupSlipDescription");
        if(dialogEmailDescription)
        {
            dialogEmailDescription.collapsed = (selectionContent == "email") ? false : true;
        }
        
        return;
    },


    toggleNetizenNotice : function()
    {
        var downloadNetizenDescription = document.getElementById("downloadNetizenDescription");
        if(downloadNetizenDescription)
        {
            downloadNetizenDescription.collapsed = false;
        }

        var downloadNetizenLink = document.getElementById("downloadNetizenLink");
        if(downloadNetizenLink)
        {
            downloadNetizenLink.collapsed = false;
        }

//@line 426 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/components/parcel/content/parcelSlipDetails.js"
        window.focus();
        window.resizeTo(window.outerWidth, window.outerHeight + this.windowResizeIncrement);
        this.windowResizeIncrement = 0;
//@line 430 "/cygdrive/d/projects/cers/netizen.source/devel/mozilla/civil/netizen/components/parcel/content/parcelSlipDetails.js"

        return;
    },
    

    copyPickupSlipText : function(eventInfo)
    {
        var slipContentArea = document.getElementById("pickupSlipContentArea");
        if(slipContentArea)
        {
            slipContentArea.focus();
            slipContentArea.select();
            
            goDoCommand("cmd_copy");
        }

        this.toggleSelectionContent("copy");
        this.toggleNetizenNotice();
        
        return;
    },



    savePickupSlipTextToFile : function(eventInfo)
    {
        this.pickupSlipInfo.savePickupSlipTextToFile();
        this.toggleSelectionContent("save");
        this.toggleNetizenNotice();
        
        return;
    },


    emailPickupSlipText : function(eventInfo)
    {
        this.pickupSlipInfo.emailPickupSlipText();
        this.toggleSelectionContent("email");
        this.toggleNetizenNotice();

        return;
    }
};


