/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is the Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2003 Civil Engines Research.  All Rights 
 * Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/



/////////////////////////////////////////////////////////////////////////////////
//
// nsIObserver Implementation
//
/////////////////////////////////////////////////////////////////////////////////


function ceParcelTransferObserver()
{
    return;
}


ceParcelTransferObserver.prototype = new ceAbstractTransferObserver();

ceParcelTransferObserver.prototype.constructor = ceParcelTransferObserver();

ceParcelTransferObserver.superclass  = ceAbstractTransferObserver.prototype;


ceParcelTransferObserver.prototype.loadObservers = function()
{
    /* For application pages that do not have any progress listeners
     * to trigger the initialization of the transfer info index, 
     * ensure that the index is created, cached and ready for use.
     */
    var transferMetaInfoIndex = NetizenPropertyCache.getPropertyValue(TRANSFER_PROPERTY_METAINFOINDEX);
    if(!transferMetaInfoIndex)
    {
        transferMetaInfoIndex = new Object();
        NetizenPropertyCache.cachePropertyValue(TRANSFER_PROPERTY_METAINFOINDEX, 
                                                transferMetaInfoIndex          );
    }

    /* Execute the superclass method by using "call" to invoke
     * the superclass function using this subclass as the execution
     * context.
     */
    ceParcelTransferObserver.superclass.loadObservers.call(this);
    
    return;
}


ceParcelTransferObserver.prototype.onTransferStarted = function(transferInfoEntry)
{
    var transferMetaInfoIndex = NetizenPropertyCache.getPropertyValue(TRANSFER_PROPERTY_METAINFOINDEX);
    var metaInfoName          = transferInfoEntry.metaInfoName;

    Assert.condition(transferMetaInfoIndex, "TransferMetaInfoIndex not initialized");
    if(!transferMetaInfoIndex)
    {
        return;
    }
    
    /* Update the transfer index bucket */
    var transferBucket = transferMetaInfoIndex[metaInfoName];
    if(!transferBucket)
    {
        transferBucket = new Object();
        
        transferBucket.meanProgress    = 0;
        transferBucket.activeTransfers = 0;
        transferBucket.transfers       = [];
        
        transferMetaInfoIndex[metaInfoName] = transferBucket;
    }
    
    transferBucket.transfers.push(transferInfoEntry);
    
    return;
}



ceParcelTransferObserver.prototype.onTransferComplete = function(transferInfoEntry)
{
    var transferMetaInfoIndex = NetizenPropertyCache.getPropertyValue(TRANSFER_PROPERTY_METAINFOINDEX);
    var metaInfoName          = transferInfoEntry.metaInfoName;
    
    Assert.condition(transferMetaInfoIndex, "TransferMetaInfoIndex not initialized");
    if(!transferMetaInfoIndex)
    {
        return;
    }

    /* Remove the transferInfo entry from the transfer index */
    var transferBucket = transferMetaInfoIndex[metaInfoName];
    if(transferBucket)
    {
        for(var counter = 0; counter < transferBucket.transfers.length; counter++)
        {
            if(transferBucket.transfers[counter].metaInfoName == metaInfoName)
            {
                transferBucket.transfers.splice(counter, 1);
                break;
            }
        }
    }
        
    return;
}



ceParcelTransferObserver.prototype.observe = function(subject, topic, state)
{
    switch(topic)
    {
      case TRANSFER_STATUS_COMPLETE    :
      case TRANSFER_STATUS_CANCELED    :
      case TRANSFER_STATUS_FAILED      :
      {
          var transferInfo = do_QueryInterface(subject, Interfaces.ceITransferInfoEntry);
          if(transferInfo)
          {
              this.onTransferComplete(transferInfo);
          }
          
          break;
      }
      case TRANSFER_STATUS_STARTED     :
      case TRANSFER_STATUS_RESTARTED   :
      {
          var transferInfo = do_QueryInterface(subject, Interfaces.ceITransferInfoEntry);
          if(transferInfo)
          {
              this.onTransferStarted(transferInfo);
          }
          
          break;
      }
    }
    
    return;
}


var ParcelTransferObserver  = new ceParcelTransferObserver();
