/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2005 Civil Engines Research.  All Rights 
 * Reserved.  
 *
 * Portions created by Netscape Copyright (C) 1998 Netscape 
 * Communications Corporation. All Rights Reserved.
 *
 * Portions created by the Mozilla Foundation are Copyright (C) 
 * 2003 Mozilla Foundation. All Rights Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Paolo de Dios <paolo.dedios@civilengines.com> (original author)
 *
 *
 ***** END License Block *****/


const PARCEL_CACHESERVICE_CONTRACTID         = "@civilengines.com/parc/cache-service;1";
const PARCEL_MANAGER_CONTRACTID              = "@civilengines.com/apps/parcel-manager;1";

var   ParcelCacheService                     = null;
var   ParcelManager                          = null;


var ParcelUnpackProgressDialog = 
{
    extractDirectory : null,

    loadDialog : function(eventInfo)
    {
        var dialogStringBundle = document.getElementById("parcelStringBundle");
        if(dialogStringBundle)
        {
            this.loadStringBundle(dialogStringBundle);
        }

        try
        {
            ParcelCacheService = do_GetService(PARCEL_CACHESERVICE_CONTRACTID  ,
                                               Interfaces.ceIParcelCacheService);

            ParcelManager      = do_GetService(PARCEL_MANAGER_CONTRACTID         , 
                                               Interfaces.ceIParcelTrackerService);
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }
        
        var trackerResourceURI = window.arguments[0];
        var unpackDirectory    = window.arguments[1];
        
        Assert.condition(trackerResourceURI, "trackerURI Unavailable");
        Assert.condition(unpackDirectory   , "Destination Directory Unavailable");

        this.extractDirectory = unpackDirectory;
        this.unpackParcel(trackerResourceURI, unpackDirectory);
        
        return;
    },


    unloadDialog : function(eventInfo)
    {
        this.unpackCancel();        
        return;
    },


    loadStringBundle : function(dialogStringBundle)
    {
        if(!dialogStringBundle)
        {
            return;
        }
                
        this.unpackItemString   = dialogStringBundle.getString("parcel.dialog.unpack.item");
        this.unpackTitleString  = dialogStringBundle.getString("parcel.dialog.unpack.title");

        return;
    },
    

    unpackParcel : function(trackerResourceURI, unpackDirectory)
    {
        try
        {
            var trackerInfo      = ParcelManager.trackerMetaInfo;
            var trackerResource  = trackerInfo.getResource(trackerResourceURI);
            var trackerEntry     = do_QueryInterface(trackerResource                 , 
                                                     Interfaces.ceIParcelTrackerEntry);
            
            var parcelFileObject = ParcelManager.getFileFromLocalPath(trackerEntry);
            if(!parcelFileObject)
            {
                return;
            }

            var parcelReader = ParcelCacheService.getParcelArchive(parcelFileObject);
            if(!parcelReader)
            {
                return;
            }
            
            var parcelArchive = do_QueryInterface(parcelReader, Interfaces.ceIParcelArchive);
            if(parcelArchive)
            {
                parcelArchive.setNotificationCallback(ParcelUnpackEventSink);
            }
            
            parcelReader.asyncExtractAll(unpackDirectory);
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }
        

        return;
    },
    

    unpackCancel : function(eventInfo)
    {
        if(ParcelUnpackEventSink.parcelArchive)
        {
            ParcelUnpackEventSink.parcelArchive.setNotificationCallback(null);
            ParcelUnpackEventSink.parcelArchive.cancel(Results.NS_ERROR_ABORT);
        }
        
        return;
    },


    showParcelContents : function()
    {
        try
        {
            this.extractDirectory.reveal();
        }
        catch(exception)
        {
            Syslog.logException(exception);
        }

        return;
    }
};



var ParcelUnpackEventSink =
{
    parcelArchive : null,

    QueryInterface : function(iid)
    {
        if(!iid.equals(Interfaces.nsISupports)          &&
           !iid.equals(Interfaces.nsIInterfaceRequestor) )
        {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        
        return this;
    },
    
    getInterface : function(iid)
    {
        if(!iid.equals(Interfaces.nsISupports)         &&
           !iid.equals(Interfaces.nsIProgressEventSink) )
        {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }

        return this;
    },


    onProgress : function(requestObject, context, progressAmount, progressMax)
    {
        var unpackProgressMeter = document.getElementById("parcelUnpackProgressMeter");
        if(!unpackProgressMeter)
        {
            return;
        }
        
        unpackProgressMeter.setAttribute("value", progressAmount);

        var progressTextLabel = document.getElementById("parcelUnpackProgressTextLabel");
        if(progressTextLabel)
        {
            progressTextLabel.setAttribute("value", (progressAmount + "%"));
        }
     
        return;
    },

    
    onStatus : function(requestObject, context, status, statusMessage)
    {
        if(!this.parcelArchive)
        {
            this.parcelArchive    = do_QueryInterface(requestObject, Interfaces.ceIParcelArchive);
            var unpackTitleString = ParcelUnpackProgressDialog.unpackTitleString;
            var archiveName       = this.parcelArchive.parcelName + ".parc";
            
            document.title        = unpackTitleString.substitute(archiveName);
        }
        
        if(status == Interfaces.ceIParcelArchive.STATUS_EXTRACTING)
        {
            var pathIndex = statusMessage.lastIndexOf('\\');
            var fileName  = (pathIndex >= 0) ? statusMessage.substring(pathIndex + 1) : statusMessage;

            var unpackItemLabel = document.getElementById("parcelUnpackItemLabel");
            if(unpackItemLabel)
            {
                var unpackItemText = ParcelUnpackProgressDialog.unpackItemString;
                if(unpackItemText)
                {
                    unpackItemText = unpackItemText.substitute(fileName);        
                    unpackItemLabel.setAttribute("value", unpackItemText);
                }
            }
        }
        else if(status == Interfaces.ceIParcelArchive.STATUS_COMPLETE ||
                status == Interfaces.ceIParcelArchive.STATUS_IDLE      )
        {
            var openUnpackedCheckbox = document.getElementById("parcelOpenUnpackedCheckbox");
            if(openUnpackedCheckbox.checked)
            {
                ParcelUnpackProgressDialog.showParcelContents();
            }
            
            this.parcelArchive.close();
            this.parcelArchive = null;

            window.close();
        }
        else if(status == Interfaces.ceIParcelArchive.STATUS_CANCELED)
        {
            if(this.parcelArchive)
            {
                this.parcelArchive.close();
                this.parcelArchive = null;
            }
            
            window.close();
        }

        return;
    }
};
