/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is the Mozilla Update Service.
 *
 * The Initial Developer of the Original Code is Ben Goodger
 *
 * Portions created by the Mozilla Foundation are Copyright (C) 
 * 2006 Mozilla Foundation. All Rights Reserved.
 *
 * Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2006 Civil Engines Research.  All Rights 
 * Reserved.  
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s):
 *   Ben Goodger <ben@mozilla.org> (Original Author)
 *   Paolo de Dios <paolo.dedios@civilengines.com>
 *
 *
 ***** END License Block *****/

const PREF_APP_UPDATE_MANUAL_URL                   = "app.update.url.manual";
const PREF_APP_UPDATE_NAGTIMER_DOWNLOAD            = "app.update.nagTimer.download";
const PREF_APP_UPDATE_NAGTIMER_RESTART             = "app.update.nagTimer.restart";
const PREF_APP_UPDATE_LOG_BRANCH                   = "app.update.log.";
const PREF_APP_UPDATE_TEST_LOOP                    = "app.update.test.loop";

const NETIZEN_BRAND_PROPERTIES_URI                 = "chrome://global/locale/brand.properties";
const NETIZEN_UPDATES_PROPERTIES_URI               = "chrome://netizen/locale/updates.properties";

const UPDATE_SERVICE_TEST_LOOP_INTERVAL            = 2000;

const UPDATE_SERVICE_STATE_NONE                    = null;
const UPDATE_SERVICE_STATE_DOWNLOADING             = "downloading";
const UPDATE_SERVICE_STATE_PENDING                 = "pending";
const UPDATE_SERVICE_STATE_APPLYING                = "applying";
const UPDATE_SERVICE_STATE_SUCCEEDED               = "succeeded";
const UPDATE_SERVICE_STATE_DOWNLOAD_FAILED         = "download-failed";
const UPDATE_SERVICE_STATE_FAILED                  = "failed";

const UPDATE_SERVICE_EVENT_FOREGROUND              = 1;
const UPDATE_SERVICE_EVENT_BACKGROUND              = 2;


function tryToClose() 
{
    var currentPage = UpdateWizard.wizard.currentPage;

    if(currentPage.pageid != "updatesFinishedPage" && currentPage.pageid != "updatesDownloadedPage")
    {
        UpdateWizard.onWizardCancel();
    }
    
    return true;
}


/* Callback for the Update Prompt to set the current page if an Update Wizard
 * window is already found to be open.
 * @param   pageid
 *          The ID of the page to switch to
 */
function setCurrentPage(wizardPage)
{
    UpdateWizard.wizard.currentPage = document.getElementById(wizardPage);
    return;
}



/* An object implementing nsITimerCallback that uses the Update Prompt
 * component to notify the user about some event relating to app update
 * that they should take action on.
 * @param   update
 *          The nsIUpdate object in question
 * @param   methodName
 *          The name of the method on the Update Prompter that should be 
 *          called
 * @constructor
 */
function ceNagTimerCallback(update, methodName) 
{
    this.update     = update;
    this.methodName = methodName;
    this.prompter   = do_CreateInstance("@civilengines.com/updates/update-prompt;1",
                                        Interfaces.nsIUpdatePrompt                 );      
}
 

ceNagTimerCallback.prototype = 
{
    update     : null,
    prompter   : null,
    methodName : ""  ,
      
    QueryInterface : function(iid)
    {
        if(!iid.equals(Interfaces.nsITimerCallback) && 
           !iid.equals(Interfaces.nsISupports)       )
        {
            throw Results.NS_ERROR_NO_INTERFACE;
        }
        
        return this;
    },

    /* Called when the timer fires. Notifies the user about whichever event 
     * they need to be nagged about (e.g. update available, please restart,
     * etc).
     */
    notify : function(timerCallback) 
    {
        if(methodName in this.prompter)
        { 
            this.prompter[methodName](null, this.update);
        }
        
        return;
    }
}



var UpdateWizard = 
{
    update       : null,
    stringBundle : null,
    brandName    : null,
    wizard       : null,
    pages        : { } ,
    
    /* The checking process that spawned this update UI. There are two types:
     *
     * UPDATE_SERVICE_EVENT_FOREGROUND:
     *   Some user-generated event caused this UI to appear, e.g. the Help
     *   menu item or the button in preferences. When in this mode, the UI
     *   should remain active for the duration of the download. 
     *
     * UPDATE_SERVICE_EVENT_BACKGROUND:
     *   A background update check caused this UI to appear, probably because
     *   incompatibilities in Extensions or other addons were discovered and
     *   the user's consent to continue was required. When in this mode, the
     *   UI will disappear after the user's consent is obtained.
     */
    sourceEvent  : UPDATE_SERVICE_EVENT_FOREGROUND,

    /* The global error message - the reason the update failed. This is human
     * readable text, used to initialize the error page.
     */
    errorMessage : "",


    onWizardLoad : function()
    {
        this.wizard       = document.documentElement;

        var brandStrings  = document.getElementById("brandStringBundle");
        this.stringBundle = document.getElementById("updateStringBundle");
        this.brandName    = brandStrings.getString("brandShortName");

        var pages         = UpdateWizard.wizard.childNodes;

        for(var i = 0; i < pages.length; ++i) 
        {
            var page = pages[i];
            if(page.localName == "wizardpage") 
            {
                /* Load the specified page object */
                this.pages[page.pageid] = eval(page.getAttribute("object"));
            }
        }
        
  
        /* Cache the standard button labels in case we need to restore them */
        this.backButtonLabel   = this.wizard.getButton("back").label;
        this.nextButtonLabel   = this.wizard.getButton("next").label;
        this.finishButtonLabel = this.wizard.getButton("finish").label;
        this.cancelButtonLabel = this.wizard.getButton("cancel").label;

        this.hideButtonLabel   = this.stringBundle.getString("updates.hideButtonLabel");

        this.wizard.getButton("cancel").label = this.hideButtonLabel;
        
        /* Advance to the Start page */ 
        UpdateWizard.wizard.currentPage = this.startPage;

        return;        
    },


    /* Called when the user presses the "Finish" button on the wizard, dispatches
     * the function call to the selected page.
     */
    onWizardFinish : function() 
    {
        var pageid = document.documentElement.currentPage.pageid;
        if("onWizardFinish" in this.pages[pageid])
        {
            this.pages[pageid].onWizardFinish();
        }
        
        return;
    },


    /* Called when the user presses the "Cancel" button on the wizard, dispatches
     * the function call to the selected page.
     */
    onWizardCancel : function() 
    {
        var pageid = document.documentElement.currentPage.pageid;
        if("onWizardCancel" in this.pages[pageid])
        {
            this.pages[pageid].onWizardCancel();
        }
        
        return;
    },


    /* Called when the user presses the "Next" button on the wizard, dispatches
     * the function call to the selected page.
     */
    onWizardNext : function() 
    {
        var currentPage = document.documentElement.currentPage;
        if(!currentPage)
        {  
            return;
        }

        var pageid = currentPage.pageid;

        if("onWizardNext" in this.pages[pageid])
        {
            this.pages[pageid].onWizardNext();
        }
        
        return;
    },

    /* Set the state for the Wizard's control buttons (labels and disabled
     * state).
     * @param   backButtonLabel
     *          The label to put on the Back button, or null for default.
     * @param   backButtonDisabled
     *          true if the Back button should be disabled, false otherwise
     * @param   nextButtonLabel
     *          The label to put on the Next button, or null for default.
     * @param   nextButtonDisabled
     *          true if the Next button should be disabled, false otherwise
     * @param   finishButtonLabel
     *          The label to put on the Finish button, or null for default.
     * @param   finishButtonDisabled
     *          true if the Finish button should be disabled, false otherwise
     * @param   cancelButtonLabel
     *          The label to put on the Cancel button, or null for default.
     * @param   cancelButtonDisabled
     *          true if the Cancel button should be disabled, false otherwise
     * @param   hideBackAndCancelButtons
     *          true if the Cancel button should be hidden, false otherwise
     * @param   extraButtonLabel
     *          The label to put on the Extra button, if present, or null for 
     *          default.
     * @param   extraButtonDisabled
     *          true if the Extra button should be disabled, false otherwise
     */
    setButtons: function(backButtonLabel         , 
                         backButtonDisabled      , 
                         nextButtonLabel         , 
                         nextButtonDisabled      ,
                         finishButtonLabel       ,
                         finishButtonDisabled    ,
                         cancelButtonLabel       , 
                         cancelButtonDisabled    ,
                         hideBackAndCancelButtons,
                         extraButtonLabel        , 
                         extraButtonDisabled     ) 
    {
        var backButton   = this.wizard.getButton("back");
        var nextButton   = this.wizard.getButton("next");
        var finishButton = this.wizard.getButton("finish");
        var cancelButton = this.wizard.getButton("cancel");
        var extra1Button = this.wizard.getButton("extra1");
        
        backButton.label    = backButtonLabel   || this.backButtonLabel;
        nextButton.label    = nextButtonLabel   || this.nextButtonLabel;
        finishButton.label  = finishButtonLabel || this.finishButtonLabel;
        cancelButton.label  = cancelButtonLabel || this.hideButtonLabel;
        extra1Button.label  = extraButtonLabel  || this.hideButtonLabel;
        
        /* update button state using the wizard commands */
        this.wizard.canRewind = !backButtonDisabled;

        /* The Finish and Next buttons are never exposed at the same time */
        if(this.wizard.onLastPage)
        {
            this.wizard.canAdvance = !finishButtonDisabled;
        }
        else
        {
            this.wizard.canAdvance = !nextButtonDisabled;
        }
        
        finishButton.disabled = finishButtonDisabled;
        cancelButton.disabled = cancelButtonDisabled;
        extra1Button.disabled = extraButtonDisabled;
        
        /* Show or hide the cancel and back buttons, since the Extra 
         * button does this job.
         */
        cancelButton.hidden = hideBackAndCancelButtons;  
        backButton.hidden   = hideBackAndCancelButtons;
        
        /* Show or hide the extra button */
        extra1Button.hidden = extraButtonLabel == null;

        return;
    },


    /* Return the <wizardpage> object that should be displayed first.
     *
     * This is determined by how we were called by the update prompt:
     *
     * U'Prompt Method:     Arg0:      Update State: Src Event:  p'Failed: Result:
     * ---------------------------------------------------------------------------
     * showUpdateAvailable  nsIUpdate  --            background  --        updatesfound
     * showUpdateDownloaded nsIUpdate  pending       background  --        finishedBackground
     * showUpdateInstalled  nsIUpdate  succeeded     either      --        installed
     * showUpdateError      nsIUpdate  failed        either      partial   errorpatching
     * showUpdateError      nsIUpdate  failed        either      complete  errors
     * checkForUpdates      null       --            foreground  --        checking
     * checkForUpdates      null       downloading   foreground  --        downloading
     */
    get startPage() 
    {
        if(window.arguments) 
        {
            var firstArgument = window.arguments[0];

            if(firstArgument instanceof Interfaces.nsIUpdate) 
            {
                /* If the first argument is a nsIUpdate object, we are notifying the
                 * user that the background checking found an update that requires
                 * their permission to install, and it's ready for download.
                 */

                this.setUpdate(firstArgument);

                var patch = this.update.selectedPatch;
                if(patch) 
                {
                    var state = patch.state;

                    if(state == UPDATE_SERVICE_STATE_DOWNLOADING) 
                    {
                        var patchFailed = false;
                        
                        try 
                        {
                            patchFailed = this.update.getProperty("patchingFailed");
                        }
                        catch(exception) 
                        {
                        }

                        if(patchFailed == "partial") 
                        {
                            /* If the system failed to apply the partial patch, show the 
                             * screen which best describes this condition, which is triggered
                             * by the |STATE_FAILED| state.
                             */
                            state = UPDATE_SERVICE_STATE_FAILED; 
                        }
                        else if(patchFailed == "complete") 
                        {
                            /* Otherwise, if the complete patch failed, which is far less 
                             * likely, show the error text held by the update object in the
                             * generic errors page, triggered by the |STATE_DOWNLOAD_FAILED|
                             * state.
                             */
                            state = UPDATE_SERVICE_STATE_DOWNLOAD_FAILED;
                        }
                    }
                    
                    /* Now select the best page to start with, given the current state of
                     * the Update.
                     */
                    switch(state) 
                    {
                      case UPDATE_SERVICE_STATE_PENDING         :
                      {
                          this.sourceEvent = UPDATE_SERVICE_EVENT_BACKGROUND;
                          return document.getElementById("updatesDownloadedPage");
                      }
                      case UPDATE_SERVICE_STATE_SUCCEEDED       :
                      {                          
                          return document.getElementById("updatesInstalledPage");
                      }
                      case UPDATE_SERVICE_STATE_DOWNLOADING     :
                      {                          
                          return document.getElementById("updatesDownloadingPage");
                      }
                      case UPDATE_SERVICE_STATE_FAILED          :
                      {                          
                          window.getAttention();            
                          return document.getElementById("updatesPatchingError");
                      }
                      case UPDATE_SERVICE_STATE_DOWNLOAD_FAILED :
                      case UPDATE_SERVICE_STATE_APPLYING        :
                      {                          
                          return document.getElementById("updatesErrorPage");
                      }
                    }
                }

                return document.getElementById("updatesAvailablePage");
            }
        }
        else 
        {
            var updateManager = do_GetService("@civilengines.com/updates/update-manager;1",
                                              Interfaces.nsIUpdateManager                 );
            if(updateManager.activeUpdate) 
            {
                this.setUpdate(updateManager.activeUpdate);
                return document.getElementById("updatesDownloadingPage");
            }
        }

        return document.getElementById("updatesCheckPage");
    },


    setUpdate : function(update) 
    {
        this.update = update;
        if(this.update)
        {
            this.update.QueryInterface(Interfaces.nsIWritablePropertyBag);
        }

        return;
    },


    /* Registers a timer to nag the user about something relating to update
     * @param   timerID
     *          The ID of the timer to register, used for persistence
     * @param   timerInterval
     *          The interval of the timer
     * @param   methodName
     *          The method to call on the Update Prompter when the timer fires
     */
    registerNagTimer : function(timerID, timerInterval, methodName) 
    {
        var timerManager = do_GetService("@civilengines.com/updates/timer-manager;1",
                                         Interfaces.nsIUpdateTimerManager           );
    

        timerManager.registerTimer(timerID                                                  , 
                                   (new ceNagTimerCallback(UpdateWizard.update, methodName)), 
                                   timerInterval                                            );
        return;
    }
    
    
};


var UpdateCheckListener =
{
    QueryInterface : function(iid) 
    {
        if(!iid.equals(Interfaces.nsIUpdateCheckListener) &&
           !iid.equals(Interfaces.nsISupports)             )
        {
            throw Results.NS_ERROR_NO_INTERFACE;
        }

        return this;
    },

    onProgress : function(request, position, totalSize) 
    {
        var progressMeter = document.getElementById("checkingProgress");

        progressMeter.setAttribute("mode", "normal");
        progressMeter.setAttribute("value", Math.floor(100 * (position/totalSize)));
    
        return;
    },


    onCheckComplete: function(request, updates, updateCount) 
    {
        updateService = do_GetService("@civilengines.com/updates/update-service;1",
                                      Interfaces.nsIUpdateService                 );

        UpdateWizard.setUpdate(updateService.selectUpdate(updates, updates.length));
        if(!UpdateWizard.update) 
        {
            Syslog.logMessage("Update Wizard : Could not select an update");
            
            var checking = document.getElementById("updatesCheckPage");
            checking.setAttribute("next", "updatesNotFoundPage");
        }

        UpdateWizard.wizard.canAdvance = true;
        UpdateWizard.wizard.advance();
        
        return;
    },


    onError : function(request, update) 
    {
        Syslog.logMessage("Update Wizard : Update Check Error");

        UpdateWizard.setUpdate(update);

        UpdateWizard.wizard.currentPage = document.getElementById("updatesErrorPage");

        return;
    }
};



var UpdatesCheckPage = 
{
    updateChecker : null,

    onPageShow : function() 
    {
        UpdateWizard.setButtons(null                          ,
                                true                          ,
                                null                          ,
                                true                          ,
                                null                          , 
                                true                          , 
                                UpdateWizard.cancelButtonLabel,
                                false                         ,
                                false                         ,
                                null                          , 
                                false                         );

        this.updateChecker = do_CreateInstance("@civilengines.com/updates/update-checker;1",
                                               Interfaces.nsIUpdateChecker                 );

        this.updateChecker.checkForUpdates(UpdateCheckListener, true);
        
        return;
    },


    /* The user has closed the window, either by pressing cancel or using a Window
     * Manager control, so stop checking for updates.
     */
    onWizardCancel : function() 
    {
        if(this.updateChecker) 
        {
            this.updateChecker.stopChecking(Interfaces.nsIUpdateChecker.CURRENT_CHECK);
        }
        
        return;
    }    
};



var UpdatesNotFoundPage = 
{
    onPageShow : function() 
    {
        UpdateWizard.setButtons(null, true, null, true, null, false, null, true, false, null, false);
        UpdateWizard.wizard.getButton("finish").focus();
    
        return;
    }
};


/* The "Updates Are Available" page provides the user information
 * about the available update, extensions it might make incompatible,
 * and a means to continue downloading and installing it.
 */
var UpdatesAvailablePage =
{
    onPageShow : function()
    {
        var productName    = UpdateWizard.brandName;
        var productVersion = UpdateWizard.update.version;
        
        var updateName     = UpdateWizard.update.name;

        var updateNameElement   = document.getElementById("updateName");
        updateNameElement.value = updateName;

        var severity          = UpdateWizard.update.isSecurityUpdate ? "minor" : "major";
        var displayType       = UpdateWizard.stringBundle.getString("updates.updateType." + severity);

        var updateTypeElement = document.getElementById("updateType");        
        updateTypeElement.setAttribute("severity", severity);
        
        var intro = UpdateWizard.stringBundle.getFormattedString("updates.introType." + severity,
                                                                 [productName]                  );


        while(updateTypeElement.hasChildNodes())
        {
            updateTypeElement.removeChild(updateTypeElement.firstChild);
        }

        updateTypeElement.appendChild(document.createTextNode(intro));
    
        var updateMoreInfoURL = document.getElementById("updateMoreInfoURL");
        updateMoreInfoURL.href = UpdateWizard.update.detailsURL;


        /* If we were invoked from a background update check, automatically show 
         * the additional details the user may need to make this decision since 
         * they did not consciously make the decision to check. 
         * if (UpdateWizard.sourceEvent == UPDATE_SERVICE_EVENT_BACKGROUND)
         */
        this.onShowMoreDetails();
        
        try 
        {
            UpdateWizard.update.getProperty("licenseAccepted");
        }
        catch(exception) 
        {
            UpdateWizard.update.QueryInterface(Interfaces.nsIWritablePropertyBag);
            UpdateWizard.update.setProperty("licenseAccepted", "false");
        }

        var downloadNowLabel   = UpdateWizard.wizard.currentPage.getAttribute("downloadNowLabel");
        var downloadLaterLabel = UpdateWizard.wizard.currentPage.getAttribute("downloadLaterLabel");

        UpdateWizard.setButtons(null              , 
                                false             ,
                                downloadNowLabel  ,
                                false             ,
                                null              ,
                                false             ,
                                null              , 
                                false             ,
                                true              ,
                                downloadLaterLabel,
                                false             );

        UpdateWizard.wizard.getButton("next").focus();

        return;
    },


    onShowMoreDetails: function() 
    {
        var detailsDeck = document.getElementById("detailsDeck");
        detailsDeck.selectedIndex = 1;
    
        return;
    },
  
    /* User said they wanted to install now, so advance to the License or 
     * Downloading page, whichever is appropriate.
     */
    onInstallNow : function() 
    {
        var nextPage = "updatesDownloadingPage";

        if(UpdateWizard.update.licenseURL) 
        {
            try 
            {
                var licenseAccepted = gUpdates.update.getProperty("licenseAccepted");
            }
            catch(exception) 
            {
            }

            if(licenseAccepted != "true")
            {
                nextPage = "updatesLicensePage";
            }
            
        }

        UpdateWizard.wizard.currentPage = document.getElementById(nextPage);

        return;
    },

    /* User said that they would install later. Register a timer to remind them
     * after a day or so.
     */
    onInstallLater : function() 
    {
        var interval = LocalPreferences.getIntPref(PREF_APP_UPDATE_NAGTIMER_DOWNLOAD, 86400);

        UpdateWizard.registerNagTimer("download-nag-timer" ,
                                      interval             , 
                                      "showUpdateAvailable");
        
        /* The user said "Later", so stop all update checks for this session
         * so that we don't bother them again.
         */
        var updateService = do_GetService("@civilengines.com/updates/update-service;1",
                                          Interfaces.nsIUpdateService                 );
                
        close();

        return;
    },


    showIncompatibleItems : function() 
    {
        return;
    }   
};


/* The page which shows the user a license associated with an update.
 * The user must agree to the terms of the license before continuing to
 * install the update.
 */
var UpdatesLicensePage =
{
    licenseContent : null,

    onPageShow : function() 
    {
        this.licenseContent = document.getElementById("licenseContent");
    
        var IAgree      = UpdateWizard.stringBundle.getString("update.IAgreeLabel");
        var IDoNotAgree = UpdateWizard.stringBundle.getString("update.IDoNotAgreeLabel");

        UpdateWizard.setButtons(null       ,
                                true       ,
                                IAgree     ,
                                true       ,
                                null       ,
                                true       ,
                                IDoNotAgree, 
                                false      ,
                                false      ,
                                null       ,
                                false      );

        this.licenseContent.addEventListener("load", this.onLicenseLoad, false);
        this.licenseContent.url = UpdateWizard.update.licenseURL;
    
        return;
    },
    
    onLicenseLoad : function() 
    {
        UpdateWizard.wizard.getButton("next").disabled = false;
        UpdateWizard.wizard.getButton("next").focus();
        
        return;
    },

    onWizardNext : function() 
    {
        UpdateWizard.update.setProperty("licenseAccepted", "true");
        
        var updateManager = do_GetService("@civilengines.com/updates/update-manager;1",
                                          Interfaces.nsIUpdateManager                 );

        updateManager.saveUpdates();
        
        return;
    },

    onWizardCancel : function() 
    {
        /* If the license was downloading, stop it. */
        this.licenseContent.stopDownloading();
    
        /* The user said "Do Not Agree", so stop all update checks for this session
         * so that we don't bother them again.
         */
        var updateService = do_GetService("@civilengines.com/updates/update-service;1",
                                          Interfaces.nsIUpdateService                 );

        updateService.backgroundChecker.stopChecking(Interfaces.nsIUpdateChecker.CURRENT_SESSION);
    
        return;
    }
};



function ceDownloadStatusFormatter()
{
    var updateStringBundle       = UpdateWizard.stringBundle;

    this.startTime               = Math.floor((new Date()).getTime() / 1000);
    this.elapsed                 = 0;
  
    this.statusFormat            = updateStringBundle.getString("updates.statusFormat");
    
    this.progressFormat          = updateStringBundle.getString("updates.progressFormat");
    this.progressFormatKBMB      = updateStringBundle.getString("updates.progressFormatKBMB");
    this.progressFormatKBKB      = updateStringBundle.getString("updates.progressFormatKBKB");
    this.progressFormatMBMB      = updateStringBundle.getString("updates.progressFormatMBMB");
    this.progressFormatUnknownMB = updateStringBundle.getString("updates.progressFormatUnknownMB");
    this.progressFormatUnknownKB = updateStringBundle.getString("updates.progressFormatUnknownKB");
    
    this.rateFormat              = updateStringBundle.getString("updates.rateFormat");
    this.rateFormatKBSec         = updateStringBundle.getString("updates.rateFormatKBSec");
    this.rateFormatMBSec         = updateStringBundle.getString("updates.rateFormatMBSec");
    
    this.timeFormat              = updateStringBundle.getString("updates.timeFormat");
    this.longTimeFormat          = updateStringBundle.getString("updates.longTimeFormat");
    this.shortTimeFormat         = updateStringBundle.getString("updates.shortTimeFormat");
    
    this.remain                  = updateStringBundle.getString("updates.remain");
    this.unknownFilesize         = updateStringBundle.getString("updates.unknownFilesize");

    return;
}


ceDownloadStatusFormatter.prototype =
{
    startTime              : 0 ,
    elapsed                : -1,
    rate                   : 0 ,
    rateFormatted          : "",
    rateFormattedContainer : "",
    progress               : "",


    /* Format a human-readable status message based on the current download
     * progress.
     * @param   currSize
     *          The current number of bytes transferred
     * @param   finalSize
     *          The total number of bytes to be transferred
     * @returns A human readable status message, e.g.
     *          "3.4 of 4.7MB; 01:15 remain"
     */
    formatStatus : function(currSize, finalSize) 
    {
        var now           = Math.floor((new Date()).getTime() / 1000);
    
        /* 1. Determine the Download Progress in Kilobytes */
        var total         = parseInt(finalSize/1024 + 0.5);
        this.progress     = this.formatKBytes(parseInt(currSize/1024 + 0.5), total);
    
        var progress      = this.replaceInsert(this.progressFormat, 1, this.progress);
        var rateFormatted = "";
    
        /* 2. Determine the Transfer Rate */
        var oldElapsed    = this.elapsed;
        this_elapsed      = now - this.startTime;
        
        if(oldElapsed != this.elapsed) 
        {
            this.rate = this.elapsed ? Math.floor((currSize / 1024) / this.elapsed) : 0;

            var isKB = true;
            if(parseInt(this.rate / 1024) > 0) 
            {
                this.rate = (this.rate / 1024).toFixed(1);
                isKB = false;
            }

            if(this.rate > 100)
            {
                this.rate = Math.round(this.rate);
            }
            
            if(this.rate) 
            {
                var format = isKB ? this.rateFormatKBSec : this.rateFormatMBSec;

                this.rateFormatted = this.replaceInsert(format, 1, this.rate);

                this.rateFormattedContainer = this.replaceInsert(" " + this.rateFormat,
                                                                 1                    ,
                                                                 this.rateFormatted   );
            }
        }

        progress = this.replaceInsert(progress, 2, this.rateFormattedContainer);
    

        /* 3. Determine the Time Remaining */
        var remainingTime = "";

        if(this.rate && (finalSize > 0)) 
        {
            remainingTime = Math.floor(((finalSize - currSize) / 1024) / this.rate);
            remainingTime = this.formatSeconds(remainingTime);
            remainingTime = this.replaceInsert(this.timeFormat, 1, remainingTime);
            remainingTime = this.replaceInsert(remainingTime, 2, this.remain);
        }


        var status = this.statusFormat;
        status     = this.replaceInsert(status, 1, progress);
        status     = this.replaceInsert(status, 2, remainingTime);
     
        return status;
    },

    /* Inserts a string into another string at the specified index, e.g. for
     * the format string var foo ="#1 #2 #3", |_replaceInsert(foo, 2, "test")|
     * returns "#1 test #3";
     * @param   format
     *          The format string
     * @param   index
     *          The Index to insert into
     * @param   value
     *          The value to insert
     * @returns The string with the value inserted. 
     */  
    replaceInsert : function(format, index, value) 
    {
        return format.replace(new RegExp("#" + index), value);
    },
    
    /* Formats progress in the form of kilobytes transfered vs. total to 
     * transfer.
     * @param   currentKB
     *          The current amount of data transfered, in kilobytes.
     * @param   totalKB
     *          The total amount of data that must be transfered, in kilobytes.
     * @returns A string representation of the progress, formatted according to:
     * 
     *            KB           totalKB           returns
     *            x, < 1MB     y < 1MB           x of y KB
     *            x, < 1MB     y >= 1MB          x KB of y MB
     *            x, >= 1MB    y >= 1MB          x of y MB
     */
    formatKBytes : function(currentKB, totalKB) 
    {
        var progressHasMB = parseInt(currentKB / 1024) > 0;
        var totalHasMB    = parseInt(totalKB / 1024) > 0;
    
        var format = "";

        if(!progressHasMB && !totalHasMB)
        {
            if(!totalKB) 
            {
                format = this.progressFormatUnknownKB;
                format = this.replaceInsert(format, 1, currentKB);
            } 
            else 
            {
                format = this.progressFormatKBKB;
                format = this.replaceInsert(format, 1, currentKB);
                format = this.replaceInsert(format, 2, totalKB);
            }
        }
        else if(progressHasMB && totalHasMB)
        {
            format = this.progressFormatMBMB;
            format = this.replaceInsert(format, 1, (currentKB / 1024).toFixed(1));
            format = this.replaceInsert(format, 2, (totalKB / 1024).toFixed(1));
        }
        else if(totalHasMB && !progressHasMB)
        {
            format = this.progressFormatKBMB;
            format = this.replaceInsert(format, 1, currentKB);
            format = this.replaceInsert(format, 2, (totalKB / 1024).toFixed(1));
        }
        else if(progressHasMB && !totalHasMB) 
        {
            format = this.progressFormatUnknownMB;
            format = this.replaceInsert(format, 1, (currentKB / 1024).toFixed(1));
        }

        return format;  
    },

    /* Formats a time in seconds into something human readable.
     * @param   seconds
     *          The time to format
     * @returns A human readable string representing the date.
     */
    formatSeconds : function(seconds) 
    {
        /* Determine number of hours/minutes/seconds */
        var hours = (seconds - (seconds % 3600)) / 3600;
        seconds -= hours * 3600;

        var minutes = (seconds - (seconds % 60)) / 60;
        seconds -= minutes * 60;
        
        /* Pad single digit values */
        if(hours < 10)
        {
            hours = "0" + hours;
        }
        
        if(minutes < 10)
        {
            minutes = "0" + minutes;
        }

        if(seconds < 10)
        {
            seconds = "0" + seconds;
        }
        
        /* Insert hours, minutes, and seconds into result string. */
        var result = parseInt(hours) ? this.longTimeFormat : this.shortTimeFormat;

        result = this.replaceInsert(result, 1, hours);
        result = this.replaceInsert(result, 2, minutes);
        result = this.replaceInsert(result, 3, seconds);
        
        return result;
    }
};



/* The "Update is Downloading" page - provides feedback for the download 
 * process plus a pause/resume UI.
 */
var UpdatesDownloadingPage =
{
    downloadName        : null,
    downloadStatus      : null,
    downloadProgress    : null,
    downloadThrobber    : null,
    pauseButton         : null,
    downloadStatusLabel : null,
    progressFormatter   : null,
    paused              : false,

    get statusFormatter() 
    {
        if(!this.progressFormatter) 
        {
            this.progressFormatter = new ceDownloadStatusFormatter();
        }
        
        return this.progressFormatter;
    },


    onPageShow : function() 
    {
        this.downloadName        = document.getElementById("downloadName");
        this.downloadStatus      = document.getElementById("downloadStatus");
        this.downloadProgress    = document.getElementById("downloadProgress");
        this.downloadThrobber    = document.getElementById("downloadThrobber");
        this.pauseButton         = document.getElementById("pauseButton");
        this.downloadStatusLabel = this.downloadStatus.textContent;
        
        var updateService = do_GetService("@civilengines.com/updates/update-service;1",
                                          Interfaces.nsIUpdateService                 );

        var updateManager = do_GetService("@civilengines.com/updates/update-manager;1",
                                          Interfaces.nsIUpdateManager                 );

        var activeUpdate  = updateManager.activeUpdate;
        if(activeUpdate)
        {
            UpdateWizard.setUpdate(activeUpdate);
        }
        
        if(!UpdateWizard.update) 
        {
            Syslog.logMessage("Update Wizard : No Updates to Download");
            return;
        }
    
        /* Say that this was a foreground download, not a background download, 
         * since the user cared enough to look in on this process.
         */
        UpdateWizard.update.QueryInterface(Interfaces.nsIWritablePropertyBag);
        UpdateWizard.update.setProperty("foregroundDownload", "true");
  
        /* Pause any active background download and restart it as a foreground
         * download.
         */
        updateService.pauseDownload();

        var state = updateService.downloadUpdate(UpdateWizard.update, false);

        if(state == "failed") 
        {
            /* We've tried as hard as we could to download a valid update - 
             * we fell back from a partial patch to a complete patch and even
             * then we couldn't validate. Show a validation error with instructions
             * on how to manually update.
             */
            this.showVerificationError();
        }
        else 
        {
            /* Add this UI as a listener for active downloads */
            updateService.addDownloadListener(this);
        }
    
        if(activeUpdate)
        {
            this.setUIState(!updateService.isDownloading);
        }
        
        var link  = document.getElementById("detailsLink");
        link.href = UpdateWizard.update.detailsURL;

        UpdateWizard.setButtons(null, true, null, true, null, true, null, false, false, null, false);

        return;
    },
  
    /* Updates the text status message */
    setStatus: function(status)
    {
        /* Don't bother setting the same text more than once. This can happen
         * due to the asynchronous behavior of the downloader.
         */
        if(this.downloadStatus.textContent == status)
        {
            return;
        }
        
        while(this.downloadStatus.hasChildNodes())
        {
            this.downloadStatus.removeChild(this.downloadStatus.firstChild);
        }
        
        this.downloadStatus.appendChild(document.createTextNode(status));
    
        return;
    },


    /* Adjust UI to suit a certain state of paused-ness
     * @param   paused
     *          Whether or not the download is paused
     */
    setUIState : function(paused) 
    {

        var update = UpdateWizard.update;

        if(paused) 
        {
            if(this.downloadThrobber.hasAttribute("state"))
            {
                this.downloadThrobber.removeAttribute("state");
            }

            if(this.downloadProgress.mode != "normal")
            {
                this.downloadProgress.mode = "normal";
            }

            this.downloadName.value = UpdateWizard.stringBundle.getFormattedString("updates.pausedName",
                                                                                   [update.name]       );

            this.pauseButton.label  = UpdateWizard.stringBundle.getString("updates.resumeButtonLabel");

            var propertyBag = update.selectedPatch.QueryInterface(Interfaces.nsIPropertyBag);
            var status      = propertyBag.getProperty("status");
            if(status)
            {
                this.setStatus(status);
            }
            
        }
        else 
        {
            if(!(this.downloadThrobber.hasAttribute("state")                &&
                 (this.downloadThrobber.getAttribute("state") == "loading")) )
            {
                this.downloadThrobber.setAttribute("state", "loading");
            }

            if(this.downloadProgress.mode != "undetermined")
            {
                this.downloadProgress.mode = "undetermined";
            }

            this.downloadName.value = 
                UpdateWizard.stringBundle.getFormattedString("updates.downloadingPrefix", [update.name]);

            this.pauseButton.label  = UpdateWizard.stringBundle.getString("updates.pauseButtonLabel");

            this.setStatus(this.downloadStatusLabel);
        }

        return;
    },

    
    onPause: function() 
    {
        var updateService = do_GetService("@civilengines.com/updates/update-service;1",
                                          Interfaces.nsIUpdateService                 );

        if(this.paused)
        {
            updateService.downloadUpdate(UpdateWizard.update, false);
        }
        else 
        {
            var patch  = UpdateWizard.update.selectedPatch;
            var status = UpdateWizard.stringBundle.getFormattedString("updates.pausedStatus"         ,
                                                                      [this.statusFormatter.progress]);
            
            patch.QueryInterface(Interfaces.nsIWritablePropertyBag);
            patch.setProperty("status", status);
            
            updateService.pauseDownload();
        }

        this.paused = !this.paused;
        
        /* Update the UI */
        this.setUIState(this.paused);
    
        return;
    },


    onWizardCancel : function() 
    {
        /* Remove ourself as a download listener so that we don't continue to be 
         * fed progress and state notifications after the UI we're updating has 
         * gone away.
         */
        var updateService = do_GetService("@civilengines.com/updates/update-service;1",
                                          Interfaces.nsIUpdateService                 );

        updateService.removeDownloadListener(this);
    
        var updateManager = do_GetService("@civilengines.com/updates/update-manager;1",
                                          Interfaces.nsIUpdateManager                 );

        updateManager.activeUpdate = UpdateWizard.update;
    
        /* If the download was paused by the user, ask the user if they want to 
         * have the update resume in the background. 
         */
        var downloadInBackground = true;
        
        if(this.paused) 
        {
            var title   = UpdateWizard.stringBundle.getString("updates.resumePausedAfterCloseTitle");
            var message = 
                UpdateWizard.stringBundle.getFormattedString("updates.resumePausedAfterCloseMessage",
                                                             [UpdateWizard.brandName]               );

            var promptService = do_GetService("@mozilla.org/embedcomp/prompt-service;1",
                                              Interfaces.nsIPromptService              );

            var promptFlags   = promptService.STD_YES_NO_BUTTONS;

            var result        = promptService.confirmEx(window     ,
                                                        title      ,
                                                        message    ,
                                                        promptFlags,
                                                        null       ,
                                                        null       ,
                                                        null       ,
                                                        null       ,
                                                        { }        );
            if(result == 1) 
            {
                downloadInBackground = false;
            }
        }
        
        if(downloadInBackground) 
        {
            /* Cancel the download and start it again in the background. */
            Syslog.logMessage("Update Wizard : Resuming Download in the background");

            updateService.pauseDownload();
            updateService.downloadUpdate(UpdateWizard.update, true);
        }

        return;
    },


    QueryInterface : function(iid) 
    {
        if(!iid.equals(Interfaces.nsIRequestObserver)   &&
           !iid.equals(Interfaces.nsIProgressEventSink) &&
           !iid.equals(Interfaces.nsISupports)           )
        {
            throw Results.NS_ERROR_NO_INTERFACE;
        }

        return this;
    },


    /* When the data transfer begins
     * @param   request
     *          The nsIRequest object for the transfer
     * @param   context
     *          Additional data
     */
    onStartRequest : function(request, context) 
    {
        request.QueryInterface(Interfaces.nsIIncrementalDownload);

        Syslog.logMessage("Update Wizard : Starting Request " + request.URI.spec);
    
        /* This !paused test is necessary because onStartRequest may fire after
         * the download was paused.
         */
        if(this.paused)
        {
            return;
        }
        
        if(!(this.downloadThrobber.hasAttribute("state")                 &&
             (this.downloadThrobber.getAttribute("state") == "loading"))  )
        { 
            this.downloadThrobber.setAttribute("state", "loading");
        }

        if(this.downloadProgress.mode != "undetermined")
        {
            this.downloadProgress.mode = "undetermined";
        }
        
        this.setStatus(this.downloadStatusLabel);
        
        return;
    },


    /* When new data has been downloaded
     * @param   request
     *          The nsIRequest object for the transfer
     * @param   context
     *          Additional data
     * @param   progress
     *          The current number of bytes transferred
     * @param   maxProgress
     *          The total number of bytes that must be transferred
     */
    onProgress : function(request, context, progress, maxProgress) 
    {
        request.QueryInterface(Interfaces.nsIIncrementalDownload);
    
        Syslog.logMessage("Update Wizard : ", " " + 
                          request.URI.spec        +
                          ", "                    +
                          progress                + 
                          "/" + maxProgress       );

        var name     = UpdateWizard.stringBundle.getFormattedString("updates.downloadingPrefix",
                                                                    [UpdateWizard.update.name] );

        var status   = this.statusFormatter.formatStatus(progress, maxProgress);
        var progress = Math.round(100 * (progress/maxProgress));

        var patch    = UpdateWizard.update.selectedPatch;

        patch.QueryInterface(Interfaces.nsIWritablePropertyBag);
        patch.setProperty("progress", progress);
        patch.setProperty("status", status);
    
        /* This !paused test is necessary because onProgress may fire after
         * the download was paused.
         */
        if(this.paused)
        {
            return;
        }
        
        if(!(this.downloadThrobber.hasAttribute("state")                &&
             (this.downloadThrobber.getAttribute("state") == "loading")) )
        {
            this.downloadThrobber.setAttribute("state", "loading");
        }

        if(this.downloadProgress.mode != "normal")
        {
            this.downloadProgress.mode = "normal";
        }
        
        this.downloadProgress.value = progress;
        this.pauseButton.disabled   = false;
        this.downloadName.value     = name;
        
        this.setStatus(status);

        return;
    },


    /* When we have new status text
     * @param   request
     *          The nsIRequest object for the transfer
     * @param   context
     *          Additional data
     * @param   status
     *          A status code
     * @param   statusText
     *          Human readable version of |status|
     */
    onStatus : function(request, context, status, statusText) 
    {
        request.QueryInterface(Interfaces.nsIIncrementalDownload);

        Syslog.logMessage("Update Wizard : " +
                          request.URI.spec   +
                          " status = "       + 
                          status             +
                          ", text = "        +
                          statusText         );

        this.setStatus(statusText);
        
        return;
    },

    /* When data transfer ceases
     * @param   request
     *          The nsIRequest object for the transfer
     * @param   context
     *          Additional data
     * @param   status
     *          Status code containing the reason for the cessation.
     */
    onStopRequest : function(request, context, status) 
    {
        request.QueryInterface(Interfaces.nsIIncrementalDownload);

        Syslog.logMessage("Update Wizard : "          +
                          request.URI.spec            + 
                          ", stopping with status = " +
                          status                      );
    
        if(this.downloadThrobber.hasAttribute("state"))
        {
            this.downloadThrobber.removeAttribute("state");
        }
        
        if(this.downloadProgress.mode != "normal")
        {
            this.downloadProgress.mode = "normal";
        }
        
        var update = UpdateWizard.update;
        
        const NS_BINDING_ABORTED = 0x804b0002;
        
        switch(status) 
        {
          case Components.results.NS_ERROR_UNEXPECTED :
          {              
              if(update.isCompleteUpdate)
              {
                  Syslog.logMessage("Update Wizard : Verification of Complete Patch Failed");

                  /* Verification error of complete patch, informational text is held in 
                   * the update object.
                   */
                  UpdateWizard.wizard.currentPage = document.getElementById("updatesErrorPage");
              }
              else 
              {
                  Syslog.logMessage("Update Wizard : Verification of Partial Patch Failed");

                  /* Verification failed for a partial patch, complete patch is now
                   * downloading so return early and do NOT remove the download listener!
                   *
                   * Reset the progress meter to "undertermined" mode so that we don't 
                   * show old progress for the new download of the "complete" patch.
                   */
                  this.downloadProgress.mode = "undetermined";
                  this.pauseButton.disabled  = true;
                  
                  var verificationFailed = document.getElementById("verificationFailed");

                  verificationFailed.hidden = false;                  
                  this.progressFormatter    = null;

                  return;
              }

              break;
          }
          case NS_BINDING_ABORTED :
          {
              Syslog.logMessage("Update Wizard : Pausing Download");

              /* Return early, do not remove UI listener since the user may resume
               * downloading again.
               */
              return;
          }
          case Results.NS_OK:
          {
              Syslog.logMessage("Update Wizard : Patch Verification Succeeded");

              UpdateWizard.wizard.canAdvance = true;
              UpdateWizard.wizard.advance();

              break;
          }
          default :
          {              
              Syslog.logMessage("Update Wizard : Transfer Failed");


              UpdateWizard.wizard.currentPage = document.getElementById("updatesErrorPage");
              break;
          }
        }
        
        
        var updateService = do_GetService("@civilengines.com/updates/update-service;1",
                                          Interfaces.nsIUpdateService                 );

        updateService.removeDownloadListener(this);
    
        return;
    }
    
};


/* The "There was an error during the update" page. */
var UpdatesErrorPage =
{
    onPageShow : function() 
    {
        UpdateWizard.setButtons(null, true, null, true, null, false, null, true, false, null, false);
        UpdateWizard.wizard.getButton("finish").focus();
    
        var errorReason   = document.getElementById("errorReason");
        errorReason.value = UpdateWizard.update.statusText;

        var manualURL      = LocalPreferences.getCharPref(PREF_APP_UPDATE_MANUAL_URL, "");
        var errorLinkLabel = document.getElementById("errorLinkLabel");

        errorLinkLabel.value = manualURL;
        errorLinkLabel.href  = manualURL;
    
        return;
    },

    onPageShowPatching : function() 
    {
        UpdateWizard.wizard.getButton("back").disabled   = true;
        UpdateWizard.wizard.getButton("cancel").disabled = true;
        UpdateWizard.wizard.getButton("next").focus();

        return;
    }
};


/* The "Update has been downloaded" page. Shows information about 
 * what was downloaded.
 */
var UpdatesDownloadedPage =
{
    /* Re-enables and focuses the Restart Application button */
    enableRestartButton : function() 
    {
        UpdateWizard.wizard.canAdvance = true;

        var finishButton      = UpdateWizard.wizard.getButton("finish");
        finishButton.disabled = false;
        finishButton.focus();
        
        return;
    },

    /* Called to initialize the Wizard Page.
     * @param   delayRestart
     *          true if the Restart Application button should be disabled for a
     *          short delay
     */
    onPageShow : function(delayRestart) 
    {  
        var restart = UpdateWizard.stringBundle.getFormattedString("updates.restartButton" ,
                                                                   [UpdateWizard.brandName]);

        var later   = UpdateWizard.stringBundle.getString("updates.laterButton");
        
        UpdateWizard.setButtons(null        ,
                                true        ,
                                null        ,
                                true        ,
                                restart     ,
                                delayRestart,
                                later       ,
                                false       ,
                                false       ,
                                null        ,
                                false       );

        if(delayRestart)
        {
            setTimeout(this.enableRestartButton, 2000);
        }
        else
        {
            UpdateWizard.wizard.getButton("finish").focus();
        }
    
        return;
    },

    /* Called to initialize the Wizard Page. (Background Source Event) */
    onPageShowBackground : function() 
    {
        var finishedBackground = document.getElementById("updatesDownloadedPage");
        var installReadyHeader = 
            UpdateWizard.stringBundle.getFormattedString("updates.updateReadyToInstallHeader",
                                                         [UpdateWizard.update.name]          );
        
                                                                              

        finishedBackground.setAttribute("label", installReadyHeader);
        
        UpdateWizard.wizard._adjustWizardHeader();

        var updateFinishedName   = document.getElementById("updateFinishedName");
        updateFinishedName.value = UpdateWizard.update.name;
        
        var link  = document.getElementById("finishedBackgroundLink");
        link.href = UpdateWizard.update.detailsURL;
        
        this.onPageShow(true);
        
        if(LocalPreferences.getBoolPref(PREF_APP_UPDATE_TEST_LOOP, false)) 
        {
            window.restart = function () 
            {
                UpdateWizard.wizard.getButton("finish").click();
            }

            setTimeout("restart();", UPDATE_SERVICE_TEST_LOOP_INTERVAL);
        }

        return;
    },
  
    /* Called when the wizard finishes, i.e. the "Restart Now" button is clicked. */
    onWizardFinish : function()
    {
        Syslog.logMessage("Update Wizard : Restarting Application...");
        
        /* This process is *extremely* retarded. There should be some nice 
         * integrated system for determining whether or not windows are allowed
         * to close or not, and what happens when that happens. We need to 
         * jump through all these hoops (duplicated from globalOverlay.js) to
         * ensure that various window types (browser, downloads, etc) all 
         * allow the app to shut down. 
         */

        /* Notify all windows that an application quit has been requested */
        var cancelQuit = do_CreateInstance("@mozilla.org/supports-PRBool;1",
                                           Interfaces.nsISupportsPRBool    );

        ObserverService.notifyObservers(cancelQuit, "quit-application-requested", null);
        
        if(cancelQuit.data)
        {
            return;
        }
        
        ObserverService.notifyObservers(null, "quit-application-granted", null);
        
        /* Enumerate all windows and call shutdown handlers */
        var windowMediator   = do_GetService("@mozilla.org/appshell/window-mediator;1",
                                           Interfaces.nsIWindowMediator             );

        var windowEnumerator = windowMediator.getEnumerator(null);
        while(windowEnumerator.hasMoreElements()) 
        {
            var domWindow = windowEnumerator.getNext();
            if(("tryToClose" in domWindow) && !domWindow.tryToClose())
            {
                return;
            }
        }
        
        var appStartup = do_GetService("@civilengines.com/runtime/app-startup;1", 
                                       Interfaces.ceIAppStartup                 );
        
        appStartup.quit(Interfaces.ceIAppStartup.ATTEMPT_QUIT | Interfaces.ceIAppStartup.RESTART);

        return;
    },

    /* Called when the wizard is canceled, i.e. when the "Later" button is clicked */
    onWizardCancel : function() 
    {
        var promptService = do_GetService("@mozilla.org/embedcomp/prompt-service;1",
                                          Interfaces.nsIPromptService              );
        var promptMessage = UpdateWizard.stringBundle.getFormattedString("updates.restartLaterMessage",
                                                                         [UpdateWizard.brandName]     );

        promptService.alert(window                                                          , 
                            UpdateWizard.stringBundle.getString("updates.restartLaterTitle"), 
                            promptMessage                                                   );

        var interval = LocalPreferences.getIntPref(PREF_APP_UPDATE_NAGTIMER_RESTART, 18000000);

        UpdateWizard.registerNagTimer("restart-nag-timer", interval, "showUpdateInstalled");

        return;
    }
};


/* The "Update was Installed Successfully" page */
var UpdatesInstalledPage =
{
    onPageShow : function()
    {
        var applicationInfo = do_GetService("@mozilla.org/xre/app-info;1"      ,
                                            Components.interfaces.nsIXULAppInfo);
    
        var brandStrings    = document.getElementById("brandStringBundle");

        try 
        {
            var whatsNewURL     = brandStrings.getFormattedString("brandUpdatesWebsite"    , 
                                                                  [applicationInfo.version]);
            var whatsNewLink    = document.getElementById("whatsnewLink");

            whatsnewLink.href   = whatsNewURL;
            whatsnewLink.hidden = false;
        }
        catch(exception) 
        {
            Syslog.logMessage("Update Wizard : What's New Link URL Not Found");
        }
        
        UpdateWizard.setButtons(null, true, null, true, null, false, null, true, false, null, false);
        UpdateWizard.wizard.getButton("finish").focus();
        
        return;
    }
};

