/***** BEGIN License Block *****
 * Version 1.1 MPL
 *
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License").  You may not use this 
 * file except in compliance with the License. You may obtain a 
 * copy of the License at http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an 
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express  
 * or implied. See the License for the specific language governing 
 * rights and limitations under the License.
 *
 * The Original Code is Civil Netizen/Civil Servant.
 *
 * The Initial Developer of the Original Code is Civil Engines 
 * Research. Portions created by Civil Engines Research LLC are 
 * Copyright (C) 2001-2005 Civil Engines Research.  All Rights 
 * Reserved.  
 *
 * Portions created by Netscape Copyright (C) 1998 Netscape 
 * Communications Corporation. All Rights Reserved.
 *
 * Portions created by the Mozilla Foundation are Copyright (C) 
 * 2003 Mozilla Foundation. All Rights Reserved.
 *
 * Redistribution of the Original Code or portions of the Original
 * Code or Software under this agreement must retain the above 
 * copyright notice, this redistribution statement and the following 
 * disclaimer.  Neither the name of Civil Engines Research nor the 
 * names of its contributors may be used to endorse or promote any
 * products derived from this software without specific prior 
 * written permission from Civil Engines Research.
 *
 *
 * Contributor(s): 
 *   Ben Goodger <ben@bengoodger.com>
 *   Blake Ross <blakeross@telocity.com>
 *   Paolo de Dios <paolo.dedios@civilengines.com>
 *
 ***** END License Block *****/


const TRANSFER_COMPONENT_CONTRACTID             = "@civilengines.com/runtime/transfer;1";
const TRANSFER_MANAGER_CONTRACTID               = "@civilengines.com/runtime/transfer-manager;1";
const TRANSFER_PERSIST_CONTRACTID               = "@civilengines.com/runtime/transfer-persist;1";

const TRANSFER_PROPERTY_METAINFOINDEX           = "urn:transfer:property:metainfo:index";
const TRANSFER_PROGRESS_UPDATE_INTERVAL         = 500;
var   TransferMetaInfoIndex                     = null;


/////////////////////////////////////////////////////////////////////////////////
//
// ceITransferProgressListener Implementation
//
/////////////////////////////////////////////////////////////////////////////////


function ceAbstractProgressListener(documentRef)
{
    if(documentRef)
    {
        this.documentRef = documentRef;
    }

    return;
}


ceAbstractProgressListener.prototype =
{
    documentRef       : null,
    metaListeners     : []  ,
    elapsedTime       : 0   ,
    rateChanges       : 0   ,
    rateChangeLimit   : 0   ,
    priorRate         : 0   ,
    lastUpdateTime    : -1  ,
    

    initTransferInfoIndex : function(activeTransfers)
    {
        /* Ensure that a singleton transfer metainfo index is available */
        TransferMetaInfoIndex = NetizenPropertyCache.getPropertyValue(TRANSFER_PROPERTY_METAINFOINDEX);
        if(!TransferMetaInfoIndex)
        {
            TransferMetaInfoIndex = new Object();
            NetizenPropertyCache.cachePropertyValue(TRANSFER_PROPERTY_METAINFOINDEX, 
                                                    TransferMetaInfoIndex          );
        }
        
        if(activeTransfers)
        {
            /* Index missing transfer info objects by metainfo name. 
             * Existing entries in the index should be populated
             * dynamically as transfers are started and stopped via
             * the AbstractTransferObserver.
             */
            var transferCount = activeTransfers.Count();
            for(var counter = 0; counter < transferCount; counter++)
            {
                var transferInfo = activeTransfers.QueryElementAt(counter, Interfaces.ceITransferInfo);
                if(!transferInfo)
                {
                    continue;
                }

                var metaInfoName = transferInfo.metaInfoName;
                if(!metaInfoName)
                {
                    continue;
                }

                var transferBucket = TransferMetaInfoIndex[metaInfoName];
                if(!transferBucket)
                {
                    transferBucket = new Object();
                    
                    transferBucket.meanProgress    = 0;
                    transferBucket.activeTransfers = 0;
                    transferBucket.transfers       = [];
                    
                    TransferMetaInfoIndex[metaInfoName] = transferBucket;
                
                    /* Insert transfer info into the proper index bucket */
                    transferBucket.transfers.push(transferInfo);
                }
            }
        }

        return;
    },


    loadStringBundle : function(transferStringBundle)
    {
        if(!transferStringBundle)
        {
            return;
        }
        
        this.statusFormatKBKB      = transferStringBundle.getString("transfer.statusFormatKBKB");
        this.statusFormatKBMB      = transferStringBundle.getString("transfer.statusFormatKBMB");
        this.statusFormatMBMB      = transferStringBundle.getString("transfer.statusFormatMBMB");
        this.statusFormatUnknownKB = transferStringBundle.getString("transfer.statusFormatUnknownKB");
        this.statusFormatUnknownMB = transferStringBundle.getString("transfer.statusFormatUnknownMB");
        this.unknownFileSize       = transferStringBundle.getString("transfer.unknownFileSize");
        this.remaining             = transferStringBundle.getString("transfer.remain");
        this.noneRemaining         = transferStringBundle.getString("transfer.remain.none");
        this.statusFormat          = transferStringBundle.getString("transfer.statusFormat");
        this.statusFormatTooltip   = transferStringBundle.getString("transfer.statusFormatTooltip");

        this.statusFormatRate      = transferStringBundle.getString("transfer.statusFormatRate");
        this.statusFormatProgress  = transferStringBundle.getString("transfer.statusFormatProgress");
        
        this.longTimeFormat        = transferStringBundle.getString("transfer.longTimeFormat");
        this.shortTimeFormat       = transferStringBundle.getString("transfer.shortTimeFormat");
                
        this.verboseTimeMultiHour  = transferStringBundle.getString("transfer.verboseTimeMultiHour");
        this.verboseTimeOneHour    = transferStringBundle.getString("transfer.verboseTimeOneHour");
        this.verboseTimeMultiMins  = transferStringBundle.getString("transfer.verboseTimeMultiMins");
        this.verboseTimeOneMin     = transferStringBundle.getString("transfer.verboseTimeOneMin");
        this.verboseTimeMultiSecs  = transferStringBundle.getString("transfer.verboseTimeMultiSecs");
        this.verboseTimeOneSec     = transferStringBundle.getString("transfer.verboseTimeOneSec");
        
        return;
    },

    
    shouldUpdateProgress : function(transferInfo, currentTotalProgress, maxTotalProgress)
    {
        var currentDate = new Date();
        var timeNow     = currentDate.getTime();
        
        if(timeNow - this.lastUpdate < TRANSFER_PROGRESS_UPDATE_INTERVAL &&
           maxTotalProgress != -1                                        &&
           currentTotalProgress < maxTotalProgress                        )
        {
            return false;
        }
        
        this.lastUpdateTime = timeNow;
        this.elapsedTime    = timeNow - (transferInfo.startTime / 1000);

        return true;
    },

    
    addMetaProgressListener : function(metaListener)
    {
        if(this.metaListeners)
        {
            this.metaListeners.push(metaListener);
        }
        
        return;
    },
    

    removeMetaProgressListener : function(metaListener)
    {
        for(var counter = 0; counter < this.metaListeners.length; counter++)
        {
            if(this.metaListeners[counter] == metaListener)
            {
                this.metaListeners.splice(counter, 1);
                break;
            }
        }
        
        return;
    },
    

    updateProgressAverage : function(transferInfo)
    {
        var transferEntry   = do_QueryInterface(transferInfo, Interfaces.ceITransferInfoEntry);
        if(!transferEntry)
        {
            return;
        }
        
        var metaInfoName    = transferInfo.metaInfoName;
        if(!metaInfoName)
        {
            return;
        }
        
        var transferBucket  = TransferMetaInfoIndex[metaInfoName];
        if(!transferBucket)
        {
            Syslog.logMessage("Transfer Bucket not found => " + metaInfoName);
            return;
        }

        var activeTransfers = transferBucket.transfers.length;
        if(activeTransfers == 0)
        {
            transferBucket.meanProgress    = -1;
            transferBucket.activeTransfers =  0;
            
            Syslog.logMessage("No active transfers found");
            return;
        }
        
        var averageProgress = 0;
        var aggregateSize   = 0;
        var transferData    = null;
        
        for(var counter = 0; counter < activeTransfers; counter++)
        {
            transferData  = transferBucket.transfers[counter];
            
            transferEntry = do_QueryInterface(transferData, Interfaces.ceITransferInfoEntry);
            if(!transferEntry)
            {
                continue;
            }
                        
            if(transferData.filesize > 0 && transferEntry.transferred < transferData.filesize)
            {
                averageProgress += transferEntry.transferred;
                aggregateSize   += transferData.filesize;
            }            
        }
        
        if(aggregateSize == 0)
        {
            averageProgress = 100;
        }
        else
        {
            averageProgress = Math.floor((averageProgress / aggregateSize) * 100);
        }

        if(averageProgress != transferBucket.meanProgress    || 
           transferBucket.activeTransfers != activeTransfers  )
        {
            /* Cache the current progress and transfer count info */
            transferBucket.meanProgress    = averageProgress;
            transferBucket.activeTransfers = activeTransfers;
            

            /* Notify all metaListeners of updated aggregate progress */
            for(var observerCount = 0; observerCount < this.metaListeners.length; observerCount++)
            {
                var metaListener = this.metaListeners[observerCount];
                if(metaListener)
                {
                    metaListener.onAverageProgressChange(transferEntry  , 
                                                         averageProgress, 
                                                         activeTransfers);
                }
            }
        }
        
        
        return;
    },


    replaceInsert: function(text, index, value) 
    {
        return (text) ? text.replaceIndex(index, value) : null;
    },


    /* progressKBytes    totalKBytes   result
     * --------------------------------------
     *   x < 1MB          y < 1MB      x of y KB
     *   x < 1MB          y >= 1MB     x KB of y MB
     *   x >= 1MB         y >= 1MB     x of y MB
     */
    formatProgressInfo: function(progressKBytes, totalKBytes)
    {
        var progressHasMB = parseInt(progressKBytes/1024) > 0;
        var totalHasMB    = parseInt(totalKBytes/1024) > 0;
     
        var format = "";
        if(!progressHasMB && !totalHasMB) 
        {
            if(!totalKBytes) 
            {
                format = this.statusFormatUnknownKB;
                format = this.replaceInsert(format, 1, progressKBytes);
            } 
            else 
            {
                format = this.statusFormatKBKB;
                format = this.replaceInsert(format, 1, progressKBytes);
                format = this.replaceInsert(format, 2, totalKBytes);
            }
        }
        else if(progressHasMB && totalHasMB) 
        {
            format = this.statusFormatMBMB;
            format = this.replaceInsert(format, 1, (progressKBytes / 1024).toFixed(1));
            format = this.replaceInsert(format, 2, (totalKBytes / 1024).toFixed(1));
        }
        else if (totalHasMB && !progressHasMB) 
        {
            format = this.statusFormatKBMB;
            format = this.replaceInsert(format, 1, progressKBytes);
            format = this.replaceInsert(format, 2, (totalKBytes / 1024).toFixed(1));
        }
        else if(progressHasMB && !totalHasMB) 
        {
            format = this.statusFormatUnknownMB;
            format = this.replaceInsert(format, 1, (progressKBytes / 1024).toFixed(1));
        }

        return format;  
    },
    

    formatTimeInfo: function(secs)
    {
        /* Round the number of seconds to remove fractions */
        secs = parseInt(secs + 0.5);
        
        var hours = parseInt(secs/3600);
        secs -= hours * 3600;

        var mins = parseInt(secs/60);
        secs -= mins * 60;

        var result = hours ? this.longTimeFormat : this.shortTimeFormat;
 
        if(hours < 10)
        {
            hours = "0" + hours;
        }
        
        if(mins < 10)
        {
            mins = "0" + mins;
        }
        
        if(secs < 10)
        {
            secs = "0" + secs;
        }
        
        result = this.replaceInsert(result, 1, hours);
        result = this.replaceInsert(result, 2, mins );
        result = this.replaceInsert(result, 3, secs );
 
        return result;
    },


    formatVerboseTimeInfo: function(secs)
    {
        /* Round the number of seconds to remove fractions */
        secs = parseInt(secs + 0.5);
        
        var hours = parseInt(secs/3600);
        secs -= hours * 3600;

        var mins = parseInt(secs/60);
        secs -= mins * 60;

        var resultHours = (hours > 1) ? this.verboseTimeMultiHour : 
                          (hours > 0) ? this.verboseTimeOneHour   : null;

        var resultMins  = (mins  > 1) ? this.verboseTimeMultiMins :
                          (mins  > 0) ? this.verboseTimeOneMin    : null;

        var resultSecs  = (secs  > 1 || secs == 0) ? this.verboseTimeMultiSecs :
                          (secs  == 1) ? this.verboseTimeOneSec    : null;
        
        
        if(resultHours)
        {
            resultHours = this.replaceInsert(resultHours, 1, hours);
            resultHours = (resultMins || resultSecs) ? (resultHours + ", ") : resultHours;
        }

        if(resultMins)
        {
            resultMins  = this.replaceInsert(resultMins , 2, mins);
            resultMins  = (resultSecs) ? (resultMins + ", ") : resultMins;
        }
        
        if(resultSecs)
        {
            resultSecs  = this.replaceInsert(resultSecs , 3, secs);
        }
        
        var result = "";
        if(resultHours)
        {
            result += resultHours;
        }
        
        if(resultMins)
        {
            result += resultMins;
        }
        
        if(resultSecs)
        {
            result += resultSecs;
        }
        
        return result;
    },


    QueryInterface : function(iid)
    {
        if(iid.equals(Interfaces.ceITransferProgressListener) ||
           iid.equals(Interfaces.nsISupports)                  )
        {
            return this;
        }
        
        throw Results.NS_NOINTERFACE;
    },

    
    onProgressChange : function(webProgress         , 
                                request             , 
                                currentSelfProgress , 
                                maxSelfProgress     , 
                                currentTotalProgress, 
                                maxTotalProgress    , 
                                transferInfo        )
    {
        return;
    },


    onStateChange : function(webProgress, request, stateFlags, statusCode, transferInfo)
    {
        return;
    },

    
    onLocationChange : function(webProgress, request, location, transferInfo)
    {
        return;
    },

    
    onStatusChange : function(webProgress, request, status, statusMessage, transferInfo)
    {
        return;
    },

  
    onSecurityChange : function(webProgress, request, state, transferInfo)
    {
        return;
    }
};
